package edu.byu.deg.OntologyEditor;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;


public class Arrow implements Shape
{
    protected boolean m_bRightArrowVisible = true;
    protected boolean m_bLeftArrowVisible = false;
    
    public final static int ARROW_SEGMENT = 10;
    private GeneralPath m_arrow = new GeneralPath();
    
    public Arrow()
    {
        recompute();
    }
    
    public void recompute()
    {
        m_arrow.reset();
        // Initialize Directional Arrow
        // --->
        
        // First draw line
        if (isRightArrowVisible() || isLeftArrowVisible())
        {
            m_arrow.moveTo(-(2 * ARROW_SEGMENT), 0);
            m_arrow.lineTo(2 * ARROW_SEGMENT, 0);
        }
        
        // Draw Right Arrow if needed
        if (isRightArrowVisible())
        {
            m_arrow.moveTo(2 * ARROW_SEGMENT, 0);
            m_arrow.lineTo(ARROW_SEGMENT, -0.24f * ARROW_SEGMENT);
            m_arrow.lineTo(ARROW_SEGMENT,  0.24f * ARROW_SEGMENT);
            m_arrow.closePath();
        }
        
        // Draw Left Arrow if needed
        if (isLeftArrowVisible())
        {
            m_arrow.moveTo(-(2 * ARROW_SEGMENT), 0);
            m_arrow.lineTo(-ARROW_SEGMENT, -0.24f * ARROW_SEGMENT);
            m_arrow.lineTo(-ARROW_SEGMENT,  0.24f * ARROW_SEGMENT);
            m_arrow.closePath();
        }
    }
    
    
    public boolean contains(double x, double y, double w, double h)
    {
        return m_arrow.contains(x,y,w,h);
    }
    
    public boolean contains(Rectangle2D r)
    {
        return m_arrow.contains(r);
    }
    
    public boolean contains(Point2D p)
    {
        return m_arrow.contains(p);
    }
    
    public boolean contains(double x, double y)
    {
        return m_arrow.contains(x,y);
    }
    
    public Rectangle getBounds()
    {
        return m_arrow.getBounds();
    }
    
    public Rectangle2D getBounds2D()
    {
        return m_arrow.getBounds2D();
    }
    
    public PathIterator getPathIterator(AffineTransform at, double flatness)
    {
        return m_arrow.getPathIterator(at, flatness);
    }
    
    public PathIterator getPathIterator(AffineTransform at)
    {
        return m_arrow.getPathIterator(at);
    }
    
    public boolean intersects(double x, double y, double w, double h)
    {
        return m_arrow.intersects(x,y,w,h);
    }
    
    public boolean intersects(Rectangle2D r)
    {
        return m_arrow.intersects(r);
    }
    
    public boolean isRightArrowVisible()
    {
        return m_bRightArrowVisible;
    }
    
    public void setRightArrowVisible(boolean propValue)
    {
        m_bRightArrowVisible = propValue;
    }
    
    public boolean isLeftArrowVisible()
    {
        return m_bLeftArrowVisible;
    }
    
    public void setLeftArrowVisible(boolean propValue)
    {
        m_bLeftArrowVisible = propValue;
    }
}