package edu.byu.deg.OntologyEditor;

import java.awt.Color;
import java.awt.Font;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.PathIterator;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.StringTokenizer;

/**
 * Title:        Ontology Editor<p>
 * Description:  Project description goes here.<p>
 * Copyright:    Copyright (c) Kimball Hewett & Friends<p>
 * Company:      Brigham Young University<p>
 * @author David Lewis
 * @version 1.0
 */


public class PSWriter extends OutputStreamWriter
{
    private Color m_currColor = null;
    private int m_lineWidth = -1;
    private String m_fontName = null;
    private int m_fontSize = -1;
    private int m_fontStyle = -1;
    private double m_fontHeight = -1;
    
    
    public PSWriter(OutputStream out, String enc) throws UnsupportedEncodingException
    {
        super(out,enc);
    }
    
    public void translate(int x, int y) throws IOException
    {
        write(System.getProperty("line.separator") + "" + (x) + " " + (y) + " translate");
    }
    
    public void translate(double x, double y) throws IOException
    {
        write(System.getProperty("line.separator") + "" + (x) + " " + (y) + " translate");
    }
    
    public void rotate(double ang) throws IOException
    {
        write(System.getProperty("line.separator") + "" + 180*ang/Math.PI + " rotate");
    }
    
    public void setColor(Color color) throws IOException
    {
        if (m_currColor!=color)
        {
            write(System.getProperty("line.separator") + "" + (color.getRed()/255.0) + " "  + (color.getGreen()/255.0)
            + " "  + (color.getBlue()/255.0) + " setrgbcolor");
            m_currColor = color;
        }
    }
    
    public void setLineWidth(int width) throws IOException
    {
        if (width != m_lineWidth)
        {
            m_lineWidth = width;
            write(System.getProperty("line.separator") + "" + (double)m_lineWidth + " setlinewidth");
        }
    }
    
    public void fillRect(int x, int y, int width, int height) throws IOException
    {
        write(System.getProperty("line.separator") + "" + (x) + " " + (y) + " " + width + " " + height + " rectfill");
    }
    
    public void drawRect(int x, int y, int width, int height) throws IOException
    {
        write(System.getProperty("line.separator") + "" + (x) + " " + (y) + " " + width + " " + height + " rectstroke");
    }
    
    public void drawLine(int x1, int y1, int x2, int y2) throws IOException
    {
        write(System.getProperty("line.separator") + "newpath " + (x1) + " " + (y1) + " moveto " + (x2) + " " + (y2) + " lineto stroke");
    }
    
    public void drawRoundRect(int x, int y, int w, int h, double radius) throws IOException
    {
        drawRoundRectPath(x,y,w,h,radius);
        write(" stroke");
    }
    
    public void fillRoundRect(int x, int y, int w, int h, double radius) throws IOException
    {
        drawRoundRectPath(x,y,w,h,radius);
        write(" fill");
    }
    
    protected void drawRoundRectPath(int x, int y, int w, int h, double radius) throws IOException
    {
        if (radius > (double)w/2)
            radius = (double)w/2;
        if (radius > (double)h/2)
            radius = (double)h/2;
        double xCenter = x+(double)w/2;
        double yCenter = y+(double)h/2;
        write(System.getProperty("line.separator") + "newpath " + (xCenter) + " " + (y) + " moveto");
        write(System.getProperty("line.separator") + "" + (x+w) + " " + (y) + " " + (x+w) + " "
        + (yCenter) + " " + radius + " arct");
        write(System.getProperty("line.separator") + "" + (x+w) + " " + (y+h) + " " + (xCenter)
        + " " + (y+h) + " " + radius + " arct");
        write(System.getProperty("line.separator") + "" + (x) + " " + (y+h) + " " + (x) + " "
        + (yCenter) + " " + radius + " arct");
        write(System.getProperty("line.separator") + "" + (x) + " " + (y) + " " + (xCenter)
        + " " + (y) + " " + radius + " arct closepath");
    }
    
    private void execPolygon(Polygon p, String command) throws IOException
    {
        if (p.npoints>1)
        {
            write(System.getProperty("line.separator") + "newpath " + (p.xpoints[0]) + " " + (p.ypoints[0]) + " moveto" + System.getProperty("line.separator"));
            for (int i=1; i<p.npoints; i++)
            {
                write(System.getProperty("line.separator") + "" + (p.xpoints[i]) + " " + (p.ypoints[i]) + " lineto");
            }
            write(" closepath " + command);
        }
    }
    
    public void drawPolygon(Polygon p) throws IOException
    {
        execPolygon(p, "stroke");
    }
    
    public void fillPolygon(Polygon p) throws IOException
    {
        execPolygon(p, "fill");
    }
    
    public void fillCircle(double x, double y, double r) throws IOException
    {
        write(System.getProperty("line.separator") + "newpath " + (x) + " " + (y) + " " + r + " 0 360 arc closepath fill");
    }
    
    
    public void drawCircle(double x, double y, double r) throws IOException
    {
        write(System.getProperty("line.separator") + "newpath " + (x) + " " + (y) + " " + (r) + " 0 360 arc closepath stroke");
    }
    
    public void draw(Shape s) throws IOException
    {
        drawPath(s);
        write(" stroke");
    }
    
    public void fill(Shape s) throws IOException
    {
        drawPath(s);
        write(" fill");
    }
    
    protected void drawPath(Shape s) throws IOException
    {
        PathIterator p = s.getPathIterator(null);
        int type;
        double [] coords = new double[6];
        
        write(System.getProperty("line.separator") + "newpath");
        
        while (!p.isDone())
        {
            type = p.currentSegment(coords);
            if (type==PathIterator.SEG_MOVETO)
                write(" " + (coords[0]) + " " + (coords[1]) + " moveto");
            else if (type==PathIterator.SEG_LINETO)
                write(" " + (coords[0]) + " " + (coords[1]) + " lineto");
            else if (type==PathIterator.SEG_QUADTO)
                write(" " + (coords[0]) + " " + (coords[1]) + " " + (coords[0]) + " "
                + (coords[1]) + " " + (coords[2]) + " " + (coords[3]) + " curveto");
            else if (type==PathIterator.SEG_CUBICTO)
                write(" " + (coords[0]) + " " + (coords[1]) + " " + (coords[2]) + " "
                + (coords[3]) + " " + (coords[4]) + " " + (coords[5]) + " curveto");
            else if (type==PathIterator.SEG_CLOSE)
                write(" closepath");
            p.next();
        }
    }
    
    public void saveTransform() throws IOException
    {
        write(System.getProperty("line.separator") + "gsave");
    }
    
    public void restoreTransform() throws IOException
    {
        write(System.getProperty("line.separator") + "grestore");
    }
    
    public void setFont(Font font) throws IOException
    {
        if (!font.getName().equalsIgnoreCase(m_fontName) ||
        font.getSize() != m_fontSize ||
        font.getStyle() != m_fontStyle)
        {
            m_fontName = font.getName();
            m_fontSize = font.getSize();
            m_fontStyle = font.getStyle();
            
            write(System.getProperty("line.separator") + "/");
            if (m_fontName.equalsIgnoreCase("monospaced"))
            {
                switch (m_fontStyle)
                {
                    case Font.BOLD:
                        write("Courier-Bold");
                        break;
                    case Font.ITALIC:
                        write("Courier-Oblique");
                        break;
                    case Font.BOLD + Font.ITALIC:
                        write("Courier-BoldOblique");
                        break;
                    default:
                        write("Courier");
                        break;
                }
            } else if (m_fontName.equalsIgnoreCase("serif"))
            {
                switch (m_fontStyle)
                {
                    case Font.BOLD:
                        write("Times-Bold");
                        break;
                    case Font.ITALIC:
                        write("Times-Italic");
                        break;
                    case Font.BOLD + Font.ITALIC:
                        write("Times-BoldItalic");
                        break;
                    default:
                        write("Times-Roman");
                        break;
                }
            } else
            {
                m_fontName = "SansSerif";
                switch (m_fontStyle)
                {
                    case Font.BOLD:
                        write("Helvetica-Bold");
                        break;
                    case Font.ITALIC:
                        write("Helvetica-Oblique");
                        break;
                    case Font.BOLD + Font.ITALIC:
                        write("Helvetica-BoldOblique");
                        break;
                    default:
                        write("Helvetica");
                        break;
                }
            }
            write(" findfont");
            write(System.getProperty("line.separator") + "" +
            font.getSize() + " scalefont setfont");
            LineMetrics met = font.getLineMetrics("HELLO",
            new FontRenderContext(null, false, false));
            m_fontHeight = met.getHeight();
        }
    }
    
    public void displayText(int x, int y, String text) throws IOException
    {
        int                ix;
        double                line_no = 0.0;
        String                tok;
        String                line_sep = System.getProperty("line.separator");
        StringTokenizer        st = new StringTokenizer(text, line_sep);
        
        while (st.hasMoreTokens())
        {
            write(line_sep + (x) + " " +
            (y + (line_no + 0.75d) * m_fontHeight) + " moveto");
            tok = st.nextToken();
            while ((ix = tok.indexOf("(")) >= 0)
            {
                if (ix > 0)
                    tok = tok.substring(0, ix - 1) + "\\" + tok.substring(ix);
                else
                    tok = "\\" + tok;
            }
            while ((ix = tok.indexOf(")")) >= 0)
            {
                if (ix > 0)
                    tok = tok.substring(0, ix - 1) + "\\" + tok.substring(ix);
                else
                    tok = "\\" + tok;
            }
            while ((ix = tok.indexOf("\\")) >= 0)
            {
                if (ix > 0)
                    tok = tok.substring(0, ix - 1) + "\\" + tok.substring(ix);
                else
                    tok = "\\" + tok;
            }
            write(line_sep + "1 -1 scale (" + tok + ") show 1 -1 scale");
            line_no = line_no + 1.0;
        }
    }
    
}
