/*
 * ToolbarLoader.java
 *
 * Created on May 13, 2003, 12:42 PM
 */

package edu.byu.deg.OntologyEditor;

import java.io.IOException;
import java.net.URL;
import java.util.Map;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

/**
 *
 * @author  Alan Wessman
 */
public class ToolbarLoader {
  
  private static final String NAME_ATTR_NAME = "name";
  private static final String TOOLBAR_ELEM_NAME = "Toolbar";
  private static final String BUTTON_GROUP_ELEM_NAME = "ButtonGroup";
  private static final String BUTTON_ELEM_NAME = "Button";
  private static final String DIVIDER_ELEM_NAME = "Divider";
  private static final String ACTION_ATTR_NAME = "action";
  private static final String MUTEX_ATTR_NAME = "mutex";
  
  /** Creates a new instance of ToolbarLoader */
  protected ToolbarLoader() { /* protected to avoid instantiation */ }
  
  public static void loadToolbars(URL toolbarResourceFile, Map toolbarMap, Map toolbarButtonGroups, Map actions) throws IOException, SAXException {
    if (toolbarResourceFile == null) return;
    
    try {
      // parse the file
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(toolbarResourceFile.toString());
      
      NodeList toolbarNodes = doc.getDocumentElement().getChildNodes();
      for (int i = 0; i < toolbarNodes.getLength(); i++) {
        Node n = toolbarNodes.item(i);
        if (n.getNodeType() != Node.ELEMENT_NODE) continue;
        if (!n.getNodeName().equals(TOOLBAR_ELEM_NAME)) continue;
        try {
          String toolbarName = ((Attr) n.getAttributes().getNamedItem(NAME_ATTR_NAME)).getValue();
          JToolBar newBar = createToolbar(n, toolbarButtonGroups, actions);
          if (newBar != null) toolbarMap.put(toolbarName, newBar);
        }
        catch (NullPointerException npe)
        { /* REVISIT: handle better? */ } // skip to the next one
      } // for loop on menuNodes
            
    } // try block
    catch (ParserConfigurationException pce)
    { /* REVISIT: handle better? */ } // fail quietly
    
  }
  
  public static JToolBar createToolbar(Node n, Map toolbarButtonGroups, Map actions) {
    JToolBar bar = new JToolBar();
    NodeList childNodes = n.getChildNodes();
    
    int buttonGroupCount = 0;
    for (int i = 0; i < childNodes.getLength(); i++) {
      Node btnGroup = childNodes.item(i);
      if (btnGroup.getNodeType() != Node.ELEMENT_NODE) continue;
      if (btnGroup.getNodeName().equals(BUTTON_GROUP_ELEM_NAME)) {
        buttonGroupCount++;
        if (buttonGroupCount > 1) bar.addSeparator();
        createButtonGroup(btnGroup, bar, toolbarButtonGroups, actions);
      } // check on element name == ButtonGroup
    } // for loop over child nodes (ButtonGroup elements)
    return bar;
  }
  
  public static void createButtonGroup(Node n, JToolBar bar, Map toolbarButtonGroups, Map actions) {
    Attr mutex = (Attr) n.getAttributes().getNamedItem(MUTEX_ATTR_NAME);
    boolean isMutex = (mutex != null && mutex.getValue().equalsIgnoreCase("true"));
    
    Attr btnGroupName = (Attr) n.getAttributes().getNamedItem(NAME_ATTR_NAME);
    if (isMutex && btnGroupName != null) {
      if (toolbarButtonGroups.get(btnGroupName.getValue()) == null) {
        toolbarButtonGroups.put(btnGroupName.getValue(), new ButtonGroup());
      }
    }
    
    NodeList buttons = n.getChildNodes();
    
    for (int j = 0; j < buttons.getLength(); j++) {
      Node groupItem = buttons.item(j);
      if (groupItem.getNodeType() != Node.ELEMENT_NODE) continue;
      
      if (groupItem.getNodeName().equals(DIVIDER_ELEM_NAME)) {
        bar.addSeparator();
      }
      else if (groupItem.getNodeName().equals(BUTTON_ELEM_NAME)) {
        Attr actionRef = (Attr) groupItem.getAttributes().getNamedItem(ACTION_ATTR_NAME);
        if (actionRef != null) {
          Action a = (Action) actions.get(actionRef.getValue());
          if (a != null) {
            String toggle = (String) a.getValue(ActionLoader.ACTION_TOGGLE_ATTR_NAME);
            boolean isToggle = false;
            if (toggle != null) isToggle = toggle.equalsIgnoreCase("true");
            if (!isToggle) bar.add(a);
            else {
              String toggleState = (String) a.getValue(ActionLoader.ACTION_TOGGLE_STATE_KEY);
              ImageIcon actionIcon = (ImageIcon) a.getValue(Action.SMALL_ICON);
              JToggleButton tog;
              if (isMutex) {
                tog = new JToggleButton(actionIcon);
                ButtonGroup mutexGroup = (ButtonGroup) toolbarButtonGroups.get(btnGroupName.getValue());
                mutexGroup.add(tog);
                JMenuItem menuItem = (JMenuItem) a.getValue(ActionLoader.MENU_ITEM_KEY_NAME);
                if (menuItem != null) {
                  tog.setModel(menuItem.getModel());
                }
              }
              else {
                tog = new JCheckBox(actionIcon);
              }
              tog.setSelected(toggleState != null && toggleState.equals(ActionLoader.ACTION_TOGGLE_STATE_ON));
              tog.setAction(a);
              tog.setText(null);
              bar.add(tog);
            }
          }
        }
      }
    } // for loop over children of btnGroup (Button and Divider elements)
  }
  
}
