/*
 * OpenAction.java
 *
 * Created on May 20, 2003, 4:37 PM
 */

package edu.byu.deg.OntologyEditor.actions;

import edu.byu.deg.OntologyEditor.ExtFileChooser;
import edu.byu.deg.OntologyEditor.OSMFileFilter;
import edu.byu.deg.OntologyEditor.OntologyCanvas;
import edu.byu.deg.OntologyEditor.OntologyCanvasWindow;
import edu.byu.deg.OntologyEditor.OntologyEditor;
import edu.byu.deg.OntologyEditor.ServerFileChooser;
import edu.byu.deg.osmx.OSMXDocument;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.xml.bind.JAXBException;


/**
 *
 * @author  Alan Wessman
 */
public class OpenAction extends GeneralAction {
  private ServerFileChooser file_chooser;
  
  public static final String KEY = "OpenAction";
  
  /**
   * Opens a saved OntologyDocument and creates an OntologyFrame for the document.
   *
   * @param evt
   */
  public void actionPerformed(ActionEvent evt) {
    OntologyEditor editor = getEditor();
    
    if (!editor.isApplet() ||
    evt.getActionCommand().indexOf("_local") != -1) {
      final ExtFileChooser chooser = new ExtFileChooser();
      chooser.setFileFilter(new OSMFileFilter());
      int returnVal = chooser.showOpenDialog(editor);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        openDocument(chooser.getSelectedFile());
        editor.enableActions();
      }
    }
    else {
      //open up the url chooser here.
      file_chooser = new ServerFileChooser(this, editor.osm_files);
      file_chooser.setSelectedIndex(editor.current_ont_index);
      editor.insert_component_at(file_chooser, 2);
      file_chooser.ready = true;
    }
  }
  
  public void close_chooser() {
    /*
    OntologyEditor editor = this.getEditor();
    JToolBar toolbar = editor.getMainToolbar();
    file_chooser.ready = false;
    toolbar.remove(file_chooser);
    toolbar.updateUI();
     */
  }
  
  public void openDocument(Object file) {
    OntologyEditor editor = this.getEditor();
    
    try {
      if (file instanceof File) {
        OSMXDocument doc = OSMXDocument.openDocument((File) file);
        OntologyCanvasWindow window = new OntologyCanvasWindow(doc, editor);
        doc.setTitle(((File)file).getName());
        window.updateTitle();
        editor.addFrame(window);
        window.toFront();
        window.setVisible(true);
        editor.setCurrentShape(OntologyCanvas.SELECTION_SHAPE_KEY);
      }
    } catch (JAXBException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}
