/*
 * EpsDrawState.java
 *
 * Created on December 26, 2003, 2:58 PM
 */

package edu.byu.deg.OntologyEditor.epsgraphics;

import java.awt.Color;

/**
 *
 * @author  Liddle
 */
public class EpsDrawState {
    // Constants
    private static final float  EQUAL_RANGE = 0.000001f;

    // Attributes
    private Color   color = null;
    private String  fontName = null;
    private int     fontSize = -1;
    private int     fontStyle = -1;
    private int     lineJoin = -1;
    private int     lineCap = -1;
    private float   lineWidth = -1.0f;
    private float   miterLimit = -1.0f;
    private float[] dashPattern = null;
    
    // Constructors
    /** Creates a new instance of EpsDrawState */
    public EpsDrawState() { /* default constructor */ }

    /** Copy constructor */
    public EpsDrawState(EpsDrawState e) {
        color = e.color;
        fontName = e.fontName;
        fontSize = e.fontSize;
        fontStyle = e.fontStyle;
        lineJoin = e.lineJoin;
        lineCap = e.lineCap;
        lineWidth = e.lineWidth;
        miterLimit = e.miterLimit;
        dashPattern = e.dashPattern;
    }

    // Accessor methods
    public Color getColor()             { return color; }
    public String getFontName()         { return fontName; }
    public int getFontSize()            { return fontSize; }
    public int getFontStyle()           { return fontStyle; }
    public int getLineJoin()            { return lineJoin; }
    public int getLineCap()             { return lineCap; }
    public float getLineWidth()         { return lineWidth; }
    public float getMiterLimit()        { return miterLimit; }
    public float[] getDashPattern()     { return dashPattern; }

    // Mutator methods
    public void setColor(Color c)        { color = c; }
    public void setFontName(String n)    { fontName = n; }
    public void setFontSize(int s)       { fontSize = s; }
    public void setFontStyle(int s)      { fontStyle = s; }
    public void setLineJoin(int j)       { lineJoin = j; }
    public void setLineCap(int c)        { lineCap = c; }
    public void setLineWidth(float w)    { lineWidth = w; }
    public void setMiterLimit(float l)   { miterLimit = l; }
    public void setDashPattern(float[] p){ dashPattern = p; }

    // Facilitator methods
    public boolean isDifferentColor(Color c)        { return !c.equals(color); }
    public boolean isDifferentFontName(String n)    { return !n.equalsIgnoreCase(fontName); }
    public boolean isDifferentFontSize(int s)       { return fontSize != s; }
    public boolean isDifferentFontStyle(int s)      { return fontStyle != s; }
    public boolean isDifferentLineJoin(int j)       { return lineJoin != j; }
    public boolean isDifferentLineCap(int c)        { return lineCap != c; }
    public boolean isDifferentLineWidth(float w)    { return Math.abs(lineWidth - w) > EQUAL_RANGE; }
    public boolean isDifferentMiterLimit(float l)   { return Math.abs(miterLimit - l) > EQUAL_RANGE; }

    public boolean isDifferentDashPattern(float[] p)
    {
        if (dashPattern == null && p == null) {
            return (false);
        }
        if (dashPattern == null || p == null || dashPattern.length != p.length) {
            return (true);
        }

        for (int i = 0; i < dashPattern.length; i++) {
            if (Math.abs(dashPattern[i] - p[i]) > EQUAL_RANGE) {
                return (true);
            }
        }
        return (false);
    }

} /* class EpsDrawState */
