/*
 * EPSExporter.java
 *
 * Created on November 19, 2003, 10:42 AM
 */

package edu.byu.deg.OntologyEditor.export;

import java.io.*;
import java.awt.*;

import java.awt.font.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.text.*;


/**
 *
 * @author  Glenn
 */
public class EPSExporter extends BaseExporter {
    
    private Color color = null;
    private Color background = null;
    private Paint paint = null;
    private BasicStroke stroke = null;
    private AffineTransform transform = null;
    private Font font = null;
    private boolean clipSet = false;
    private StringBuffer buffer;
    static private double minX, maxX, minY, maxY;
    
    private OutputStreamWriter outputWriter;
    /** Creates a new instance of EPSExporter */
    public EPSExporter() {
        buffer = new StringBuffer();
        transform = new AffineTransform();
        minX = minY = Float.POSITIVE_INFINITY;
        maxX = maxY = Float.NEGATIVE_INFINITY;
        setColor(Color.black);
        setPaint(Color.black);
        setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        setFont(Font.decode(null));
        setStroke(new BasicStroke());
    }
    
    public EPSExporter(EPSExporter g) {
        outputWriter = g.outputWriter;
        clip = g.clip;
        color = g.color;
        stroke = g.stroke;
        transform = g.transform;
        paint = g.paint;
        background = g.background;
        font = g.font;
        buffer = g.buffer;
        minX = EPSExporter.minX;
        maxX = EPSExporter.maxX;
        minY = EPSExporter.minY;
        maxY = EPSExporter.maxY;
        
    }
    public void addRenderingHints(java.util.Map hints) { /* do nothing */ }
    
    public void clearRect(int x, int y, int width, int height) { /* do nothing */ }
    
    public void clip(java.awt.Shape s) {
/*        if (clip == null) {
            setClip(s);
        }
        else {
            //draw(s, "clip");
            Area area = new Area(clip);
            area.intersect(new Area(s));
            clip = area;
        }
 */
        setClip(s);
    }
    
    public void clipRect(int x, int y, int width, int height) {
        clip(new java.awt.Rectangle(x,y,width,height));
    }
    
    public void copyArea(int x, int y, int width, int height, int dx, int dy) { /* do nothing */ }
    
    public java.awt.Graphics create() {
        return new EPSExporter(this);
    }
    
    public Graphics create(int x, int y, int width, int height) {
	Graphics g = create();
	g.translate(x, y);
	g.clipRect(0, 0, width, height);
	return g;
    }
    
    public void dispose() { /* do nothing */ }
    
    public void draw(java.awt.Shape s) {
        draw(s, "stroke");
    }
    
    public void drawArc(int x, int y, int width, int height, int startAngle, int arcAngle) {
        Shape shape = new Arc2D.Float(x, y, width, height, startAngle, arcAngle, Arc2D.OPEN);
        draw(shape);
    }
    
    public void drawGlyphVector(java.awt.font.GlyphVector g, float x, float y) {
        Shape shape = g.getOutline(x, y);
        fill(shape);
        
    }
    
    public boolean drawImage(java.awt.Image img, java.awt.geom.AffineTransform xform, java.awt.image.ImageObserver obs) {
        return false;
    }
    
    public void drawImage(java.awt.image.BufferedImage img, java.awt.image.BufferedImageOp op, int x, int y) { /* do nothing */ }
    
    public boolean drawImage(java.awt.Image img, int x, int y, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public boolean drawImage(java.awt.Image img, int x, int y, java.awt.Color bgcolor, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public boolean drawImage(java.awt.Image img, int x, int y, int width, int height, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public boolean drawImage(java.awt.Image img, int x, int y, int width, int height, java.awt.Color bgcolor, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public boolean drawImage(java.awt.Image img, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public boolean drawImage(java.awt.Image img, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2, java.awt.Color bgcolor, java.awt.image.ImageObserver observer) {
        return false;
    }
    
    public void drawLine(int x1, int y1, int x2, int y2) {
        Shape shape = new Line2D.Float(x1, y1, x2, y2);
        draw(shape);
    }
    
    public void drawOval(int x, int y, int width, int height) {
        Shape shape = new Ellipse2D.Float(x, y, width, height);
        draw(shape);
    }
    
    protected void draw(Shape s, String command) {
        PathIterator p = s.getPathIterator(null);
        int type;
        double [] coords = new double[6];
        // Update the bounds.
        if (!command.equals("clip")) {
            Rectangle2D shapeBounds = s.getBounds2D();
            Rectangle2D visibleBounds = shapeBounds;
            if (clip != null) {
                Rectangle2D clipBounds = clip.getBounds2D();
                visibleBounds = shapeBounds.createIntersection(clipBounds);
            }
            float lineRadius = stroke.getLineWidth() / 2;
            float minX = (float) visibleBounds.getMinX() - lineRadius;
            float minY = (float) visibleBounds.getMinY() - lineRadius;
            float maxX = (float) visibleBounds.getMaxX() + lineRadius;
            float maxY = (float) visibleBounds.getMaxY() + lineRadius;
            updateBounds(minX, -minY);
            updateBounds(maxX, -maxY);
        }
        write("newpath");
        
        while (!p.isDone()) {
            type = p.currentSegment(coords);
            if (type==PathIterator.SEG_MOVETO)
                write((coords[0]) + " " + (-coords[1]) + " moveto" );
            else if (type==PathIterator.SEG_LINETO)
                write((coords[0]) + " " + (-coords[1]) + " lineto" );
            else if (type==PathIterator.SEG_QUADTO)
                write((coords[0]) + " " + (-coords[1]) + " " + (coords[0]) + " "
                + (-coords[1]) + " " + (coords[2]) + " " + (-coords[3]) + " curveto");
            else if (type==PathIterator.SEG_CUBICTO)
                write((coords[0]) + " " + (coords[1]) + " " + (coords[2]) + " "
                + (-coords[3]) + " " + (coords[4]) + " " + (-coords[5]) + " curveto" );
            else if (type==PathIterator.SEG_CLOSE)
                write(" closepath");
            p.next();
        }
        write(command);
        write("newpath");
    }
    
    public void drawPolygon(Polygon p) {
        draw(p);
    }
    
    public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints) {
        Polygon p = new Polygon(xPoints, yPoints, nPoints);
        drawPolygon(p);
    }
    
    public void drawPolyline(int[] xPoints, int[] yPoints, int nPoints) {
        if (nPoints > 0) {
            GeneralPath path = new GeneralPath();
            path.moveTo(xPoints[0], yPoints[0]);
            for (int i = 1; i < nPoints; i++) {
                path.lineTo(xPoints[i], yPoints[i]);
            }
            draw(path);
        }
    }
    
    public void drawRect(int x, int y, int width, int height) {
        Shape shape = new Rectangle(x, y, width, height);
        draw(shape);
    }
    
    public void drawRenderableImage(java.awt.image.renderable.RenderableImage img, java.awt.geom.AffineTransform xform) { /* do nothing */ }
    
    public void drawRenderedImage(java.awt.image.RenderedImage img, java.awt.geom.AffineTransform xform) { /* do nothing */ }
    
    public void drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight) {
        Shape shape = new RoundRectangle2D.Float(x, y, width, height, arcWidth, arcHeight);
        draw(shape);
    }
    
    public void drawString(java.text.AttributedCharacterIterator iterator, int x, int y) {
        drawString(iterator, (float) x, (float) y);
    }
    
    public void drawString(java.text.AttributedCharacterIterator iterator, float x, float y) {
        write( "newpath");
        Point2D location = transform(x, y);
        write( location.getX() + " " + location.getY() + " moveto");
        StringBuffer buffer = new StringBuffer();
        for (char ch = iterator.first(); ch != CharacterIterator.DONE; ch = iterator.next()) {
            if (ch == '(' || ch == ')') {
                buffer.append('\\');
            }
            buffer.append(ch);
        }
        write( "(" + buffer.toString() + ") show");
    }
    
    public void drawString(String s, float x, float y) {
        if (s != null && s.length() > 0) {
            AttributedString as = new AttributedString(s);
            as.addAttribute(TextAttribute.FONT, getFont());
            drawString(as.getIterator(), x, y);
        }
    }
    
    public void drawString(String str, int x, int y) {
        drawString(str,(float)x, (float)y);
    }
    
    public void fill(java.awt.Shape s) {
        draw(s, "fill");
    }
    
    public void fillArc(int x, int y, int width, int height, int startAngle, int arcAngle) {
        Shape shape = new Arc2D.Float(x, y, width, height, startAngle, arcAngle, Arc2D.OPEN);
        fill(shape);
    }
    
    public void fillOval(int x, int y, int width, int height) {
        Shape shape = new Ellipse2D.Float(x, y, width, height);
        fill(shape);
    }
    
    public void fillPolygon(Polygon p) {
        fill(p);
    }
    
    public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints) {
        Polygon p = new Polygon(xPoints, yPoints, nPoints);
        fillPolygon(p);
    }
    
    public void fillRect(int x, int y, int width, int height) {
        Shape shape = new Rectangle(x, y, width, height);
        fill(shape);
    }
    
    public void fillRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight) {
        Shape shape = new RoundRectangle2D.Float(x, y, width, height, arcWidth, arcHeight);
        fill(shape);
    }
    
    public java.awt.Color getBackground() {
        return background;
    }
    
    public Rectangle getBounds() {
        return new Rectangle((int)minX,(int)minY,(int)(maxX-minX),(int)(maxY-minY));
    }
    
    public java.awt.Shape getClip() {
        return clip;
    }
    
    public java.awt.Rectangle getClipBounds() {
        if (clip == null) {
            return null;
        }
        Rectangle rect = clip.getBounds();
        return rect;
    }
    
    public java.awt.Color getColor(){
        return color;
    }
    public java.awt.Composite getComposite() {
        return null;
    }
    
    public java.awt.GraphicsConfiguration getDeviceConfiguration() {
        return null;
    }
    
    public java.awt.Font getFont() {
        return font;
    }
    
    public java.awt.FontMetrics getFontMetrics(java.awt.Font f) {
        BufferedImage image = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);
        Graphics g = image.getGraphics();
        return g.getFontMetrics(f);
    }
    
    public java.awt.font.FontRenderContext getFontRenderContext() {
        return null;
    }
    
    public java.awt.Paint getPaint() {
        return paint;
    }
    
    public Object getRenderingHint(java.awt.RenderingHints.Key hintKey) {
        return null;
    }
    
    public java.awt.RenderingHints getRenderingHints() {
        return null;
    }
    
    public java.awt.Stroke getStroke() {
        return stroke;
    }
    
    public java.awt.geom.AffineTransform getTransform() {
        return new AffineTransform(transform);
    }
    
    public boolean hit(java.awt.Rectangle rect, java.awt.Shape s, boolean onStroke) {
        return s.intersects(rect);
    }
    
    public boolean isClipSet() {
        return clipSet;
    }
     
    public void rotate(double theta) {
        write(System.getProperty("line.separator") + "" + 180*theta/Math.PI + " rotate");
    }
    
    public void rotate(double theta, double x, double y) { /* do nothing */ }
    
    public void scale(double sx, double sy) {
        transform(AffineTransform.getScaleInstance(sx, sy));
    }
    
    public void setBackground(java.awt.Color color) {
        if (color == null){
            color = Color.black;
        }
        background = color;
    }
    
    public void setClip(java.awt.Shape newClip) {
        if (newClip != null) {
            if (isClipSet()) {
                write("grestore");
                write("gsave");
            }
            else {
                setClipSet(true);
                write("gsave");
            }
            clip = newClip;
            draw(clip, "clip");
        }
        else {
            if (isClipSet()) {
                write("grestore");
                setClipSet(false);
            }
            clip = null;
        }
    }
    
    public void setClip(int x, int y, int width, int height) {
        setClip(new Rectangle(x, y, width, height));
    }
    
    public void setClipSet(boolean newVal) {
        clipSet = newVal;
    }
    
    public void setColor(java.awt.Color c) {
        
        if(c == null) c = Color.black;
        if (color!=c) {
            write((c.getRed()/255.0) + " "  + (c.getGreen()/255.0) + " "  + (c.getBlue()/255.0) + " setrgbcolor");
            color = c;
        }
    }
    
    public void setComposite(java.awt.Composite comp) { /* do nothing */ }
    
    public void setFont(java.awt.Font newFont) {
        if (newFont == null) {
            newFont = Font.decode(null);
        }
        if(!(newFont.equals(font))) {
            font = newFont;
            write(System.getProperty("line.separator")+ "/" + font.getPSName() + " findfont " + (font.getSize()) + " scalefont setfont");
        }
    }
    
    public void setPaint(java.awt.Paint newPaint) {
        if (newPaint instanceof Color) {
            if(!(newPaint.equals(paint))){
                paint = newPaint;
                setColor((Color) newPaint);
            }
        }
    }
    
    public void setPaintMode() { /* do nothing */ }
    
    public void setRenderingHint(java.awt.RenderingHints.Key hintKey, Object hintValue) { /* do nothing */ }
    
    public void setRenderingHints(java.util.Map hints) { /* do nothing */ }
    
    public void setStroke(java.awt.Stroke s) {
        if (s instanceof BasicStroke) {
            if(!(s.equals(stroke))){
                stroke = (BasicStroke) s;
                
                write(stroke.getLineWidth() + " setlinewidth");
                write(stroke.getMiterLimit() + " setmiterlimit");
                write(stroke.getLineJoin() + " setlinejoin");
                write(stroke.getEndCap() + " setlinecap");
                
                StringBuffer dashes = new StringBuffer();
                dashes.append("[ ");
                float[] dashArray = stroke.getDashArray();
                if (dashArray != null) {
                    for (int i = 0; i < dashArray.length; i++) {
                        dashes.append((dashArray[i]) + " ");
                    }
                }
                dashes.append("]");
                write(dashes.toString() + " 0 setdash");
            }
        }
    }
    
    public void setTransform(java.awt.geom.AffineTransform Tx) {
        if (Tx == null) {
            transform = new AffineTransform();
        }
        else {
            transform = new AffineTransform(Tx);
        }
        // Need to update the stroke and font so they know the scale changed
        setStroke(getStroke());
        setFont(getFont());
    }
    
    public void setXORMode(java.awt.Color c1) { /* do nothing */ }
    
    public void shear(double shx, double shy) {
        transform(AffineTransform.getShearInstance(shx, shy));
    }
    
    public void transform(java.awt.geom.AffineTransform Tx) {
        transform.concatenate(Tx);
        setTransform(getTransform());
    }
    
    private Point2D transform(float x, float y) {
        Point2D result = new Point2D.Float(x, y);
        result = transform.transform(result, result);
        result.setLocation(result.getX(), -result.getY());
        return result;
    }
    
    public void translate(double tx, double ty) {
        transform(AffineTransform.getTranslateInstance(tx, ty));
    }
    
    public void translate(int x, int y) {
        translate((double) x, (double) y);
    }
    
    private void updateBounds(double x, double y) {
        if(x < minX) minX = x;
        if(x > maxX) maxX = x;
        if(y < minY) minY = y;
        if(y > maxY) maxY = y;
    }
    private void write(String stringToWrite) {
        buffer.append(System.getProperty("line.separator") + stringToWrite);
    }
    
    public String getFooter() {
        StringBuffer footer = new StringBuffer();
        footer.append(System.getProperty("line.separator")+"%%EOF");
        return footer.toString();
    }
    
    public String getContents() {
        return buffer.toString();
    }
    public String getHeader() {
        StringBuffer header = new StringBuffer();
        header.append(System.getProperty("line.separator")+"%!PS-Adobe-3.0 EPSF-3.0");
        Rectangle boundingBox = getBounds();
        header.append(System.getProperty("line.separator")+"%%BoundingBox: " + (int)Math.floor(.75d*boundingBox.x-2) + " " + (int)Math.floor(.75d*boundingBox.y-2) + " "
        + (int)Math.ceil(.75d*(boundingBox.x+boundingBox.width+2)) + " " + (int)Math.ceil(.75d*(boundingBox.y+boundingBox.height)+2));
        header.append(System.getProperty("line.separator")+"%%HiResBoundingBox: " + (.75d*boundingBox.x-2) + " " + (.75d*boundingBox.y-2) + " "
        + (.75d*(boundingBox.x+boundingBox.width+2)) + " " + (.75d*(boundingBox.y+boundingBox.height)+2));
        header.append(System.getProperty("line.separator")+"%%EndComments");
        return header.toString();
    }
    
    public String toString() {
        StringBuffer result = new StringBuffer();
        result.append(getHeader());
        result.append(getContents());
        result.append(getFooter());
        return result.toString();
    }
    
}
