package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.epsgraphics.EpsGraphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.beans.PropertyChangeEvent;
import java.util.*;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.AggregationType;
import edu.byu.deg.osmx.binding.GenSpecType;
import edu.byu.deg.osmx.binding.ParentRelSetConnection;

import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;

public class AggregationShape extends ConnectorShape{
	 // Constants
    protected static final float    TRIANGLE_WIDTH = 30;
    protected static final float    MITER_BUFFER = 16;
    protected static final float    PLUS_WIDTH = 6.0f,
                                    U_WIDTH = 9.0f;

    protected AggregationType 		aggregation;
    protected ParentRelSetConnectionShape	parentConnection;
    protected List					childConnections;
    protected TrianglePanel         triangleShape;
  
    /** Creates a new instance of GenSpecShape */
    public AggregationShape(Container c, OSMXElement elem) {
        super(c, elem);
        init();
        //triangleShape = new TrianglePanel();
        add(triangleShape, 0);
        repositionTriangle();
    }
    /*public void addConnection(OSMXRelSetConnectionType conn) {
    String objSetID = conn.getObjectSet();
    OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
    OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
    ConnectableShape connectedShape = null;
    if (objSet == null) {
        connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
    } else {
        connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
    }
    if (connectedShape != null) {
      if (!aggregation.getRelSetConnection().contains(conn)) {
    	  aggregation.getRelSetConnection().add(conn);
      }
      RelSetConnectionShape connShape = new RelSetConnectionShape(this, (ObjConnectableShape) connectedShape, this, conn);
      connections.add(connShape);
    } else {
        System.out.println("WARNING: null connection");
    }
  } */  
    public void addParentConnection(OSMXParentRelSetConnection conn){
    	String objSetID = conn.getObjectSet();
    	OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
        OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
        ConnectableShape connectedShape = null;
        if (objSet == null) {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
        } else {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
        }
        if (connectedShape != null) {
          if (!aggregation.getParentConnection().equals(conn)) {
        	  aggregation.setParentConnection(conn);
          }
           parentConnection = new ParentRelSetConnectionShape(this, (ObjConnectableShape) connectedShape, this, conn);
        } else {
            System.out.println("WARNING: null connection");
        }
    }
    public void addChildConnection(OSMXChildRelSetConnection conn) {
    	String objSetID = conn.getObjectSet();
        OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
        OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
        ConnectableShape connectedShape;
        if (objSet ==  null) {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
        } else {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
        }
        if (connectedShape != null) {
            if (!aggregation.getChildConnection().contains(conn)) {
                aggregation.getChildConnection().add(conn);
            }
            ConnectionShape childConn = new ChildRelSetConnectionShape(this, (ObjConnectableShape) connectedShape, this, conn);
            childConnections.add(childConn);
        }
    }
    protected void centerPointMoved() {
        super.centerPointMoved();
        repositionTriangle();
    }
  
    public boolean contains(int x, int y) {
        return contains(new Point(x, y));
    }
  
    public boolean contains(Point localPoint) {
    	boolean result = false;
    	for(Iterator i = childConnections.iterator(); i.hasNext();){
    		ConnectionShape conn = (ConnectionShape)i.next();
    		if (conn.contains(SwingUtilities.convertPoint(this,localPoint,conn))){
    			return true;
    		}
    	}
    	ConnectionShape conn = (ConnectionShape)parentConnection;
    	if (conn.contains(SwingUtilities.convertPoint(this,localPoint,conn))){
			return true;
    	}
        return masksConnectionsAt(localPoint);
    }
  
    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        //if (OSMXAggregationType.CONSTRAINT_PROPERTY.equals(evt.getPropertyName())) {
          //  repaint();
        //}
    }
  
    /**
     * this method need to be tested. the print function is based on it.
     */
    public Rectangle getBoundingBox() {
        Rectangle bounds = triangleShape.getBounds();
        bounds.union(parentConnection.getBounds());
        
        for (Iterator it = childConnections.iterator(); it.hasNext(); ) {
            ConnectionShape conn = (ConnectionShape) it.next();
            if (bounds == null) {
                bounds = conn.getBoundingBox();
            } else {
                bounds = bounds.union(conn.getBoundingBox());
            }
        }

        return (bounds);
    }    

  
    public Point2D.Double getParentConnectionPoint() {
        Point centerPt = getCenterPoint();
        double theta = getShapeOrientation() + Math.PI / 2;
        double dx = TRIANGLE_WIDTH * Math.cos(theta) / Math.sqrt(3);
        double dy = TRIANGLE_WIDTH * Math.sin(theta) / Math.sqrt(3);
        return (new Point2D.Double(centerPt.x - dx, centerPt.y - dy));
    }

    public ConnectionShape getHeadConnection() {
        //if (connections.size() == 0) return null;
        //return (ConnectionShape) connections.get(0);
    	if (parentConnection == null) return null;
        return (ConnectionShape)parentConnection;
    }

    private double getShapeOrientation() {
        ConnectionShape head = getHeadConnection();
        ConnectionShape tail = getTailConnection();
        if (head != null && tail != null) {
            Point2D.Double ctrPt = PointUtilities.hiResPoint(getCenterPoint());
            if (ctrPt == null) return 0.0;
            Point2D.Double tmp = PointUtilities.convertPoint(this, ctrPt, head);
            Point2D.Double headPt = head.getObjectIntersectionPoint(tmp);
            double w = headPt.x - ctrPt.x;
            double h = headPt.y - ctrPt.y;
            return Math.atan2(h, w) + Math.PI / 2;
        }
        return 0.0;
    }
    
    public ConnectionShape getTailConnection() {
        //if (connections.size() < 2) return null;
        //return (ConnectionShape) connections.get(1);
    	if (childConnections.size() < 1) return null;
    	return (ConnectionShape)childConnections.get(0);
    }

    protected void initConnections() {
        init();
        if (aggregation == null) return;
        
        List childList = aggregation.getChildConnection();
        for(Iterator i = childList.iterator(); i.hasNext();){
        	OSMXChildRelSetConnection conn = (OSMXChildRelSetConnection) i.next();
        	//TODO:
        	addChildConnection(conn);
        }
        //TODO:
        addParentConnection((OSMXParentRelSetConnection) aggregation.getParentConnection());
    }
    
    protected void init() {
        if (aggregation == null) {
        	aggregation = (AggregationType)element;
        	triangleShape = new TrianglePanel();
        }
        if (childConnections == null){
        	childConnections = new ArrayList();
        }
    }

    public boolean masksConnectionsAt(Point pt) {
        if (childConnections.size() < 1 || parentConnection == null) return false;
        return triangleShape.contains(SwingUtilities.convertPoint(this, pt, triangleShape));
    }
  
    protected void repositionTriangle() {
        Point ctrPt = getCenterPoint();
        //test
        //if (ctrPt == null) 	System.err.println("crtPt is null");
        //end test
        if (ctrPt != null) {
        	//test
        	if (triangleShape == null) System.err.println("triangleShape is null");
        	//if (triangleShape.getWidth() == null) System.out.println("triangleShape.getwidth is null");
        	//end test
            int x = (int)(ctrPt.x - triangleShape.getWidth() / 2.0 + 0.5);
            int y = (int)(ctrPt.y - triangleShape.getHeight() / 2.0 + 0.5);
            triangleShape.setLocation(x, y);
        }
    }
  
    TrianglePanel getTriangleShape(){
    	return triangleShape;
    }
    protected void updateSelectionState() {
        super.updateSelectionState();
        triangleShape.updateSelectionState(selected);
    }
    
    public class TrianglePanel extends JPanel {
        private GeneralPath             triangle;
        private GeneralPath             typeShape;
        private float                   triangleWidth = TRIANGLE_WIDTH;
        private boolean                 triangleSelected = false;
        private final float             PANEL_WIDTH = triangleWidth + MITER_BUFFER;
        private final AffineTransform   TRANS_TO_CENTER;

        public TrianglePanel() {
          TRANS_TO_CENTER = AffineTransform.getTranslateInstance(
                                PANEL_WIDTH / 2, PANEL_WIDTH / 2);
          setOpaque(false);
          setLayout(null);
          setSize((int)(PANEL_WIDTH + 0.5), (int)(PANEL_WIDTH + 0.5));
          initTriangle();
        }

        private void initTriangle() {
          triangle = new GeneralPath();
          typeShape = new GeneralPath();
          float offset = triangleWidth / (float)Math.sqrt(3);
          triangle.moveTo(0, -offset);
          triangle.lineTo(triangleWidth / 2, offset / 2);
          triangle.lineTo(-triangleWidth / 2, offset / 2);
          triangle.closePath();
          triangle.transform(TRANS_TO_CENTER);
          
        }

        
        private AffineTransform getRotationTransform() {
          return AffineTransform.getRotateInstance(getShapeOrientation(), PANEL_WIDTH/2, PANEL_WIDTH/2);
        }

        private void paintTriangle(Graphics2D g) {
          try {
            AffineTransform trans = getRotationTransform();
            AffineTransform inverseTrans = trans.createInverse();
            triangle.transform(trans);
            g.setStroke(getStroke());
            g.setColor(Color.BLACK);
            g.fill(triangle);
            g.setColor(getLineColor());
            g.draw(triangle);
            if (triangleSelected) {
                // draw selection handles at triangle vertices
                g.setColor(Color.lightGray);
                PathIterator    it = triangle.getPathIterator(new AffineTransform());
                double[]        coord = new double[6];
                double          handleWidth = 6;
                int             seg_type;

                while (!it.isDone()) {
                    seg_type = it.currentSegment(coord);
                    if (seg_type == PathIterator.SEG_MOVETO ||
                                            seg_type == PathIterator.SEG_LINETO) {
                        g.fill(new Rectangle2D.Double(coord[0] - handleWidth / 2,
                                                      coord[1] - handleWidth / 2,
                                                      handleWidth, handleWidth));
                    }
                    it.next();
                }
            }
            triangle.transform(inverseTrans);
          } catch (Exception e) {
            // do nothing
          }
        }

        /*private void paintTypeShape(Graphics2D g) {
          try {
            AffineTransform trans = getRotationTransform();
            AffineTransform inverseTrans = trans.createInverse();
            updateTypeShape();
            typeShape.transform(trans);
            g.setStroke(getStroke());
            g.setColor(((OSMXStyleType) getStyle()).getLineColor());
            g.draw(typeShape);
            typeShape.transform(inverseTrans);
          } catch (Exception e) {
            // do nothing
          }
        }*/

        public void paintComponent(Graphics g) {
            /*if (g instanceof EpsGraphics2D) {
                ((EpsGraphics2D)g).addComment("");
                ((EpsGraphics2D)g).addComment("GenSpec<" + genSpec.getGenSpecConstraint() + ">");
            }*/

            Graphics2D g2d = (Graphics2D) g.create();
            paintTriangle(g2d);
            //paintTypeShape(g2d);
        }

        public boolean contains(int x, int y) {
            Point pt = new Point(x,y);
            return contains(pt);
        }

        public boolean contains(Point point) {
            boolean result = false;
            try {
                AffineTransform trans = getRotationTransform();
                AffineTransform inverseTrans = trans.createInverse();
                triangle.transform(trans);
                result = triangle.contains(point);
                triangle.transform(inverseTrans);
            } catch (Exception e) {
                // do nothing
            }
            return result;
        }

        public void processMouseEvent(MouseEvent evt) {
            evt = SwingUtilities.convertMouseEvent(this, evt, AggregationShape.this);
            super.processMouseEvent(evt);
        }

        protected void updateSelectionState(boolean selected) {
            triangleSelected = selected;
        }

  } /* nested class TrianglePanel */
}
