package edu.byu.deg.OntologyEditor.shapes;

import java.awt.Container;
import java.awt.Point;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;

import javax.swing.SwingUtilities;

import edu.byu.deg.osmx.OSMXChildRelSetConnection;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.OSMXPositionedText;
import edu.byu.deg.osmx.binding.BasicConnection;
import edu.byu.deg.osmx.binding.ChildRelSetConnection;
import edu.byu.deg.osmx.binding.ParticipationConstraintType;

public class ChildRelSetConnectionShape extends ConnectionShape{
	protected AggregationShape aggregationShape;
	
    protected ChildRelSetConnection          childRelSetConn;
    protected ParticipationConstraintShape  parentSidePartConst;
    protected ParticipationConstraintShape  childSidePartConst;

    protected static final int              ARROWHEAD_LENGTH = 10;
    protected static final int              OPTIONAL_RADIUS = 3;
    
	public ChildRelSetConnectionShape(Container c, ConnectableShape connectable, AggregationShape connector, ChildRelSetConnection conn) {
		super(c, connectable, connector, conn);
		aggregationShape = connector;
		childRelSetConn = conn;
		this.initParticipationConstraint();
	}
	protected void initParticipationConstraint() {
        boolean p_functional = childRelSetConn.isParentSideFunctional();
        boolean c_functional = childRelSetConn.isChildSideFunctional();
        boolean p_optional = childRelSetConn.isParentSideOptional();
        boolean c_optional = childRelSetConn.isParentSideOptional();
        ParticipationConstraintType p_pc = childRelSetConn.getParentSideParticipationConstraint();
        ParticipationConstraintType c_pc = childRelSetConn.getChildSideParticipationConstraint();
  
        if (p_pc != null) {
        	//create the view shape based on the model
            parentSidePartConst = new ParticipationConstraintShape(this, (OSMXElement) p_pc, true);
            parentSidePartConst.setSize(parentSidePartConst.getPreferredSize());
            this.add(parentSidePartConst, 0);
            if (childRelSetConn.isParentSideFunctional() || childRelSetConn.isParentSideOptional()) { //why ????
            	parentSidePartConst.setVisible(false);
            }
            //repositionParticipationConstraint();
        }
        if (c_pc != null) {
        	//create the view shape based on the model
            childSidePartConst = new ParticipationConstraintShape(this, (OSMXElement) c_pc, false);
            childSidePartConst.setSize(childSidePartConst.getPreferredSize());
            this.add(childSidePartConst, 0);
            if (childRelSetConn.isChildSideFunctional() || childRelSetConn.isChildSideOptional()) { //why ????
            	childSidePartConst.setVisible(false);
            }
            //repositionParticipationConstraint();
        }        
        repositionParticipationConstraint();
        childRelSetConn.setParentSideFunctional(p_functional);
        childRelSetConn.setChildSideFunctional(c_functional);
        childRelSetConn.setParentSideOptional(p_optional);
        childRelSetConn.setChildSideOptional(c_optional);
    }
	public void repositionParticipationConstraint() {
		OSMXPositionedText p_pc = (OSMXPositionedText)childRelSetConn.getParentSideParticipationConstraint();
		OSMXPositionedText c_pc = (OSMXPositionedText)childRelSetConn.getChildSideParticipationConstraint();
		if (p_pc==null || c_pc==null) return;//test
		Point ctrPt = getRealObjectCenterPoint();
		if (ctrPt == null) return;
		if (!c_pc.isSetPosition()) {
			childSidePartConst.setPosition(-connectedShape.getWidth(),
					-connectedShape.getHeight());
		} else {
			childSidePartConst.setLocationFromPosition();
		}
		if (!p_pc.isSetPosition()){
			//System.out.println("TODO: set parent side position");
			//TODO:
			//System.out.println( ((AggregationShape)this.getParent()).getTriangleShape().getWidth() +
					//" " +((AggregationShape)this.getParent()).getTriangleShape().getHeight());
			parentSidePartConst.setPosition(-(int)(AggregationShape.TRIANGLE_WIDTH), -(int)(AggregationShape.TRIANGLE_WIDTH));
		}
		else{
			//System.out.println("TODO: set parent side position");
			parentSidePartConst.setLocationFromPosition();
			//TODO:
		}
	}	
	
	public boolean contains(Point pt){
        boolean result = super.contains(pt);
        if (result) return result;
        if (parentSidePartConst == null && childSidePartConst == null) return result;
        Point p_Pt = SwingUtilities.convertPoint(this, pt, parentSidePartConst);
        Point c_Pt = SwingUtilities.convertPoint(this, pt, childSidePartConst);
        if (parentSidePartConst!=null){
        	if(parentSidePartConst.isVisible() && parentSidePartConst.contains(p_Pt)) 
        		return true;
        } 
        if (childSidePartConst != null){ 
        	if(c_Pt!=null && childSidePartConst.isVisible() && childSidePartConst.contains(c_Pt))
        		return true;
        }
        return false;
	}
    public Rectangle getBoundingBox() {
        Rectangle bounds = super.getBoundingBox();
        if (parentSidePartConst != null && parentSidePartConst.isVisible()) {
            bounds = bounds.union(parentSidePartConst.getBoundingBox());
        }
        if (childSidePartConst != null && childSidePartConst.isVisible()){
        	bounds = bounds.union(childSidePartConst.getBoundingBox());
        }
        return (bounds);
    }	
    
    protected void notifyConnectorShape(OSMXElement elem) {
        /*if (isHeadConnectionPoint(elem) && partConst != null) {
            partConst.setLocationFromPosition();
            partConst.repaint();
        }*/
        if (parentSidePartConst != null) {
        	parentSidePartConst.setLocationFromPosition();
        	parentSidePartConst.repaint();
        }
        if (childSidePartConst != null) {
        	childSidePartConst.setLocationFromPosition();
        	childSidePartConst.repaint();
        }
        super.notifyConnectorShape(elem);
    }
    
    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        if (OSMXChildRelSetConnection.CHILD_SIDE_PARTICIPATION_CONSTRAINT_PROPERTY.
                                            equals(evt.getPropertyName())) {
        	ParticipationConstraintType pct = (ParticipationConstraintType)evt.getNewValue();
        	if (pct != null){
        		childSidePartConst = new ParticipationConstraintShape(this, (OSMXElement) pct, false);
        		childSidePartConst.setSize(childSidePartConst.getPreferredSize());
        		this.repositionParticipationConstraint();
        	}
        	if (childSidePartConst != null){
        		childSidePartConst.setVisible(evt.getNewValue()!=null);
        	}
            
        }else if (OSMXChildRelSetConnection.PARENT_SIDE_PARTICIPATION_CONSTRAINT_PROPERTY.
                                            equals(evt.getPropertyName())){
        	ParticipationConstraintType pct = (ParticipationConstraintType)evt.getNewValue();
        	if (pct != null){
        		parentSidePartConst = new ParticipationConstraintShape(this,(OSMXElement) pct, true);
        		parentSidePartConst.setSize(parentSidePartConst.getPreferredSize());
        		this.repositionParticipationConstraint();
        	}
        	if(parentSidePartConst != null){
        		parentSidePartConst.setVisible(evt.getNewValue() != null);
        	}
        }
        
    }
  
}
