package edu.byu.deg.OntologyEditor.shapes;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;

public class ConnectableContainerLayout implements LayoutManager {
  public final static String CENTER = "center";
  public final static String TOP_CENTER = "topCenter";
  public final static String BOTTOM_CENTER = "bottomCenter";
  public final static String TOP_RIGHT = "topRight";
  public final static String TOP_LEFT = "topLeft";
  public final static String ABSOLUTE = "absolute";
  
  private Component m_centerComp = null;
  private Component m_topCenterComp = null;
  private Component m_bottomCenterComp = null;
  private Component m_upperRightComp = null;
  private Component m_upperLeftComp = null;
  private int m_centerRightGap = 3;//3
  private int m_centerLeftGap = 3;//3
  private int m_upperRightPadding = 4;//4
  private int m_upperLeftPadding = 4;//4
  private int m_topBottomPadding = 3;//3
  
  public void addLayoutComponent(String name,Component comp) {
    if (name == CENTER) {
      m_centerComp=comp;
    }
    else if (name == TOP_RIGHT) {
      m_upperRightComp=comp;
    }
    else if (name == TOP_LEFT) {
      m_upperLeftComp=comp;
    }
    else if (name == TOP_CENTER) {
      m_topCenterComp=comp;
    }
    else if (name == BOTTOM_CENTER) {
      m_bottomCenterComp=comp;
    }
    comp.setSize(comp.getPreferredSize());
  }
  
  public void layoutContainer(Container parent) {
    Component comp;
    Component components[] = parent.getComponents();
    int x,y;
    Dimension minSize = new Dimension(minimumLayoutSize(parent));
    Dimension size = new Dimension(parent.getSize());
    if (size.width < minSize.width)
      size.width = minSize.width;
    if (size.height < minSize.height)
      size.height = minSize.height;
    parent.setSize(size);
    
    
    for (int i=0; i<components.length; i++) {
      comp = components[i];
      
      if (comp.getWidth()==0)
        comp.setSize(comp.getPreferredSize());
      if (comp==m_centerComp) {
        m_centerComp.setSize(m_centerComp.getPreferredSize());
        x = (parent.getWidth() - comp.getWidth()) / 2;
        if (m_upperRightComp != null)
          x -= m_upperRightComp.getWidth() / 2;
        if (m_upperLeftComp != null)
          x += m_upperLeftComp.getWidth() / 2;
        y = (parent.getHeight() - comp.getHeight())/2;
        comp.setLocation(x,y);
      }
      else if (comp == m_upperLeftComp) {
        Insets insets = parent.getInsets();
        m_upperLeftComp.setSize(m_upperLeftComp.getPreferredSize());
        x = insets.left+1;
        y = insets.top+1;
        comp.setLocation(x,y);
      }
      else if (comp == m_upperRightComp) {
        Insets insets = parent.getInsets();
        x = parent.getWidth() - comp.getWidth() - insets.right;
        y = insets.top;
        comp.setLocation(x,y);
      }
      else if ((comp==m_topCenterComp || comp==m_bottomCenterComp) && m_bottomCenterComp!=null && m_topCenterComp!=null) {
        comp.setSize(comp.getPreferredSize());
        Dimension top = m_topCenterComp.getPreferredSize();
        Dimension bottom = m_bottomCenterComp.getPreferredSize();
        int middle;
        if (top.width > bottom.width)
          x = (parent.getWidth() - top.width)/2;
        else
          x = (parent.getWidth() - bottom.width)/2;
        
        middle = getMiddle(parent);
        if (comp == m_topCenterComp)
          y = (middle - comp.getHeight())/2;
        else
          y = (parent.getHeight() + middle - comp.getHeight())/2;
        
        comp.setLocation(x,y);
      }
    }
  }
  
  public Dimension minimumLayoutSize(Container parent) {
    Dimension size = new Dimension(0,0);
    
    if (m_centerComp!=null) {
      size.setSize(m_centerComp.getPreferredSize());
      if (m_upperRightComp != null) {
        m_upperRightComp.setSize(m_upperRightComp.getPreferredSize());
        size.setSize(size.width + m_centerRightGap + m_upperRightComp.getWidth(),size.height+m_upperRightPadding);
      }
      if (m_upperLeftComp != null) {
        m_upperLeftComp.setSize(m_upperLeftComp.getPreferredSize());
        size.setSize(size.width + m_centerLeftGap + m_upperLeftComp.getWidth(), size.height + m_upperLeftPadding);
      }
    }
    
    else if (m_upperLeftComp!=null) {
      size.setSize(m_upperLeftComp.getPreferredSize());
      
      if (m_upperRightComp != null) {
        //y = size.height;
        
        m_upperRightComp.setSize(m_upperRightComp.getPreferredSize());
        size.setSize(size.width + m_centerRightGap + m_upperRightComp.getWidth(),size.height);
      }
    }
     
    else if (m_topCenterComp!=null && m_bottomCenterComp!=null) {
      size.setSize(m_topCenterComp.getPreferredSize());
      Dimension size2 = new Dimension(m_bottomCenterComp.getPreferredSize());
      if (size2.width>size.width)
        size.width = size2.width;
      
      size.height += m_topBottomPadding + size2.height;
    }
    
    Insets insets = parent.getInsets();
    size.setSize(size.width+insets.left+insets.right,size.height+insets.top+insets.bottom);
    return size;
  }
  
  
  public Dimension preferredLayoutSize(Container parent) {
    return minimumLayoutSize(parent);
  }
  
  public void removeLayoutComponent(Component comp) {
    if (comp == m_centerComp)
      m_centerComp = null;
    if (comp == m_topCenterComp)
      m_topCenterComp = null;
    if (comp == m_bottomCenterComp)
      m_bottomCenterComp = null;
    if (comp == m_upperRightComp)
      m_upperRightComp = null;
    if (comp == m_upperLeftComp)
      m_upperLeftComp = null;
  }
  
  public int getMiddle(Container parent) {
    if (m_topCenterComp!=null && m_bottomCenterComp!=null) {
      double ratio = (double)m_topCenterComp.getHeight()/(double)(m_bottomCenterComp.getHeight()+m_topCenterComp.getHeight());
      
      return (int)(parent.getHeight()*ratio+0.5001);
    }
    else
      return parent.getHeight()/2;
  }
}