/*
 * ConnectableContainerShape.java
 *
 * Created on June 23, 2003, 9:09 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.Canvas;
import edu.byu.deg.OntologyEditor.EditableText;
import edu.byu.deg.OntologyEditor.ObjBorder;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.OSMType;
import edu.byu.deg.osmx.binding.impl.ObjectSetImpl;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.beans.PropertyChangeEvent;
import java.awt.Font;

/**
 *
 * @author  Alan
 */
public abstract class ConnectableContainerShape extends ObjConnectableShape 
implements Canvas, DocumentListener {
  
  protected static final Color HIGHLEVEL_COLOR = new Color(230,230,230,255);
  protected JTextArea nameText = null;
  protected boolean suppressDocEvents = false;
  protected ConnectableContainerLayout ccl;
    
  /** Creates a new instance of ConnectableContainerShape */
  public ConnectableContainerShape(Container c, OSMXElement elem) {
    super(c, elem);
    ObjBorder objBorder = (ObjBorder) shapeBorder;
    objBorder.setRectDraw(true);
    setBorder(objBorder);
    FlowLayout fl = new FlowLayout();
    ccl = new ConnectableContainerLayout();
    fl.setHgap(0);
    fl.setVgap(0);
    setLayout(ccl);
    setBackground(getFillColor());
    setForeground(getFontColor());
    nameText = new EditableText();
    nameText.addMouseListener(this.getRootCanvas());
    nameText.addMouseMotionListener(this.getRootCanvas());
    nameText.setForeground(getFontColor());
    nameText.setBackground(getFillColor());
    nameText.setFont(new Font(getFontFamily(), OSMXFontType.styleToInt(getFontStyle()), (int) getFontSize()));
    //nameText.setFont(new Font(getFontFamily(), OSMXFontType.styleToInt(getFontStyle()), 18));
    positionNameText();
    nameText.getDocument().addDocumentListener(this);
    suppressDocEvents = true;
    String shapeName = getShapeName();
    nameText.setText(shapeName);

    int preferHeight = this.getPreferredSize().height;
    int preferWidth = this.getPreferredSize().width;
    int height = getContainerHeight();
    int width = getContainerWidth();
    
    if (height < preferHeight)
    	height = preferHeight;
    if (width < preferWidth) 
    	width = preferWidth;
    Dimension size = new Dimension(width, height);
    this.setSize(size);
    suppressDocEvents = false;
  }
    
  public void positionNameText()
  {
    if (element != null && element instanceof TextAlignableElement) {
      TextAlignableElement tae = (TextAlignableElement) element;
      remove(nameText);
      if (tae.isCentered()) {
        add(ConnectableContainerLayout.CENTER, nameText);
      } else {
        add(ConnectableContainerLayout.TOP_LEFT, nameText);
        //add(ConnectableContainerLayout.CENTER, nameText);
      }
    }
  }
  public void updateNameTextPosition(){
	  ccl.layoutContainer(this);
	  
  }
  /** Gives notification that an attribute or set of attributes changed.
   *
   * @param e the document event
   *
   */
  public void changedUpdate(DocumentEvent e) { /* do nothing */ }
  
  public void dragSelectedObjects(int x, int y) { /* do nothing */ }
  
  public int getNumberSelected(boolean bWithSubObjs) {
    return 0; // TODO
  }
  public int getContainerHeight(){
	  if (element instanceof ObjectSetImpl) {
		  return ((ObjectSetImpl)element).getHeight();
	  }
	  else return DEFAULT_HEIGHT;
	  
}

  public int getContainerWidth(){
	  if (element instanceof ObjectSetImpl) {
		  return ((ObjectSetImpl)element).getWidth();
	  }
	  else return DEFAULT_WIDTH;
  }
  
  public abstract OSMType getOSM();
  
  public abstract String getShapeName();
  
  protected abstract void setShapeName(String s);
  
  public int getState() {
    return 0; // TODO
  }
  
  private Dimension getChoosedSize() {
	  Dimension currentSize, preferSize, size;
	  int w, h;
	  preferSize = getPreferredSize();
	  currentSize = getSize();
	  if (preferSize.width > currentSize.width)
		  w = preferSize.width;
	  else
		  w = currentSize.width;
	  if (preferSize.height > currentSize.height)
		  h = preferSize.height;
	  else
		  h = currentSize.height;
	  size = new Dimension(w,h);
	  return size;
  }
  /** Gives notification that there was an insert into the document.  The
   * range given by the DocumentEvent bounds the freshly inserted region.
   *
   * @param e the document event
   *
   */
  public void insertUpdate(DocumentEvent e) {
    if (!suppressDocEvents) {
    	//System.out.println("insertUpdate");
      //setSize(getPreferredSize());
    	setSize(getChoosedSize());
      setShapeName(nameText.getText());
    }
  }
  
  /** Gives notification that a portion of the document has been
   * removed.  The range is given in terms of what the view last
   * saw (that is, before updating sticky positions).
   *
   * @param e the document event
   *
   */
  public void removeUpdate(DocumentEvent e) {
    if (!suppressDocEvents) {
    	//System.out.println("removeUpdate");
      //setSize(getPreferredSize());
    	setSize(getChoosedSize());
      setShapeName(nameText.getText());
    }
  } 
  
  public void setState(int state) { /* do nothing */ }
  
  public void toBack(Component comp) { /* do nothing */ }
  
  public void toFront(Component comp) { /* do nothing */ }
  
  public void unselectAll() { /* do nothing */ }
  
    protected void updateSelectionState() {
        super.updateSelectionState();
        if (!selected) {
            // de-select the text in the nested text component
            nameText.select(0, 0);
        }
    }

    public DrawShape findShapeForElement(OSMXElement elem) {
        return null; // TODO
    }
  
    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        if (OSMXStyleType.STYLE_PROPERTY.equals(evt.getPropertyName())) {
            nameText.setFont(new Font(getFontFamily(),
                                      OSMXFontType.styleToInt(getFontStyle()),
                                      (int)getFontSize()));
            nameText.setForeground(getFontColor());
            nameText.setBackground(getFillColor());
        }
        else if (OSMXObjectSetType.CENTER_NAME_PROPERTY.equals(evt.getPropertyName())) {
        	this.positionNameText();
        	this.updateNameTextPosition(); // update the positon on the shape
        	repaint();
        }
        
        
    }
  
} /* class ConnectableContainerShape */
