/*
 * ConnectionShape.java
 *
 * Created on July 5, 2003, 7:46 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.ObjBorder;
import edu.byu.deg.osmx.OSMXAnchorType;
import edu.byu.deg.osmx.OSMXBasicConnection;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.PositionListener;
import edu.byu.deg.osmx.PositionedElement;
import edu.byu.deg.osmx.binding.AnchorType;
import edu.byu.deg.osmx.binding.BasicConnection;
import edu.byu.deg.osmx.binding.GenSpecType;
import edu.byu.deg.osmx.binding.RelSetConnectionType;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.geom.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;


/**
 *
 * @author  Alan
 * Add handle point
 */
public class ConnectionShape extends DrawShape {

    protected ConnectableShape  connectedShape = null;
    protected ConnectorShape    connectorShape = null;
    protected BasicConnection   connection = null;
    protected List              anchors = new ArrayList();
    private ComponentAdapter    parentComponentListener;
    //private OSMXAnchorType handleType;
    //private AnchorShape handleShape;

    public static final int     CLICKRANGE = 4;
    protected static final int  DEFAULT_WIDTH = 5;
    protected static final int  GAP_RADIUS = 12;
  
    /** Creates a new instance of ConnectorShape */
    public ConnectionShape(Container c, ConnectableShape connectable, ConnectorShape connector, BasicConnection conn) {
        super(c, (OSMXElement) conn);
        setLayout(null);
        setOpaque(false);
        connectedShape = connectable;
        initConnectedShapePositionListener();
        connectorShape = connector;
        connection = conn;
        initConnectionPathListener();
        initSize();
        initAnchors();
        setLocation(0, 0);
        initHandles();//zonghui
    }
  public int getAngle() {
	  /*int ag = 0;
	  if (connection instanceof RelSetConnectionType)
		  ag =( (RelSetConnectionType)connection ).getAngle();
	  else if (connection instanceof GenSpecType){
		  //TODO:add angle attribute to genspecType
		  //return ( (GenSpecType)connection ).getAngle();
		  return 0;
	  }
	  return ag;*/
	  return connection.getAngle();
  }
  public void setAngle(int angle) {
	  //if (connection instanceof RelSetConnectionType)
		//  ( (RelSetConnectionType)connection ).setAngle(angle);
	  //TODO: Add anle attribute to genSpecType
	  connection.setAngle(angle);
  }
  protected void initConnectedShapePositionListener() {
    connectedShape.addComponentListener(
      new ComponentAdapter() {
        public void componentMoved(ComponentEvent evt) {
          notifyConnectorShape(connectedShape.getElement());
        }
        
        public void componentResized(ComponentEvent evt) {
          notifyConnectorShape(connectedShape.getElement());
        }
      }
    );
    PositionedElement pe = (PositionedElement) connectedShape.getElement();
    pe.addPositionListener(
      new PositionListener() {
        public void updateX(OSMXElement src, int x) {
          notifyConnectorShape(src);
        }
        
        public void updateY(OSMXElement src, int y) {
          notifyConnectorShape(src);
        }
        
        public void updateOrder(OSMXElement src, int order) {
          notifyConnectorShape(src);
        }
      }
    );
  }
  
  protected void notifyConnectorShape(OSMXElement elem) {
    if (isTailConnectionPoint(elem)) {
      connectorShape.centerPointMoved();
    }
    repaint();
  }
  
  protected void initConnectionPathListener() {
    if (connection != null) {
      ((OSMXBasicConnection) connection).addPropertyChangeListener(
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getSource() == connection) {
              if (OSMXBasicConnection.CONNECTION_PATH_PROPERTY.equals(evt.getPropertyName())) {
                OSMXElement movedElement = (OSMXElement) evt.getOldValue();
                if (evt.getNewValue() == null || isTailConnectionPoint(movedElement)) {
                  connectorShape.centerPointMoved();
                }
                repaint();
              }
            }
          }
        }
      );
    }
  }
  
  protected boolean isTailConnectionPoint(OSMXElement elem) {
    if (elem instanceof AnchorType) {
      return (connection.getAnchor().size() > 0 && connection.getAnchor().get(0) == elem);
    } else if (elem == connectedShape.getElement()) {
      return true;
    } else return false;
  }
  
  protected boolean isHeadConnectionPoint(OSMXElement elem) {
    if (elem instanceof AnchorType) {
      int numAnchors = connection.getAnchor().size();
      return (numAnchors > 0 && connection.getAnchor().get(numAnchors - 1) == elem);
    } else if (elem == connectedShape.getElement()) {
      return true;
    } else return false;
  }
  
  protected void initSize() {
    parentComponentListener = new ComponentAdapter() {
      public void componentResized(ComponentEvent evt) {
        setSize(getParent().getSize());
      }
      
      public void componentMoved(ComponentEvent evt) {
        setLocation(0, 0);
      }
    };
    getParent().addComponentListener(parentComponentListener);
    setSize(getParent().getSize());
  }
  //test
  protected void initHandles() {
	  final int threshold = 2;
	  
	  AnchorShape anchorShape;
	  OSMXAnchorType handleType;
	  if (connection == null)  return;
	  Point2D.Double dest;
	  dest = PointUtilities.hiResPoint(connectedShape.getConnectionPointAccordingToAngle(this.getAngle()));
	  dest = PointUtilities.convertPoint(connectedShape, dest, this);
	  handleType = new OSMXAnchorType();
	  handleType.setX((int)dest.getX());
	  handleType.setY((int)dest.getY());
	  if (anchors.size() > 0) {//already have 1 or more anchors
		  anchorShape = (AnchorShape) anchors.get(anchors.size() - 1);
		  Point anchorPt = new Point();
		  anchorPt.x = anchorShape.getX();
		  anchorPt.y = anchorShape.getY();
		  
		  if ( (anchorPt.x - (handleType.getX()- AnchorShape.ANCHOR_WIDTH/2) < threshold) &&
			   (anchorPt.x - (handleType.getX()- AnchorShape.ANCHOR_WIDTH/2) > -threshold) &&  
			   (anchorPt.y -  (handleType.getY() - AnchorShape.ANCHOR_WIDTH/2) < threshold) &&
			   (anchorPt.y -  (handleType.getY() - AnchorShape.ANCHOR_WIDTH/2) < -threshold	 )) {
			  addAnchor(anchors.size(), handleType);
			  //System.out.println("handle added");//test
		  }
	  }
	  else {// new relConn
		  addAnchor(anchors.size(), handleType);
	  }
	  ((AnchorShape)anchors.get(anchors.size() -1)).isHandle(true);
  }
  //end test
  
  protected void initAnchors() {
    if (connection == null) return;
    List anchorElems = connection.getAnchor();
    for (Iterator i = anchorElems.iterator(); i.hasNext();) {
      OSMXAnchorType anchorElem = (OSMXAnchorType) i.next();
      addAnchor(anchorElem);
    }
  }
  
  public void addAnchor(AnchorType anchor) {
    addAnchor(anchors.size(), anchor);
  }
  
  public void addAnchor(int index, AnchorType anchor) {
    final AnchorShape anchorShape = new AnchorShape(this, (OSMXElement) anchor);
    anchorShape.setVisible(this.isSelected());
    //anchorShape.setVisible(true);
    anchors.add(index, anchorShape);
    if (!connection.getAnchor().contains(anchor)) {
      connection.getAnchor().add(index, anchor);
    }
    if (connectorShape != null) {
        connectorShape.centerPointMoved();
        connectorShape.invalidate();
    }
    anchorShape.addComponentListener(
      new ComponentAdapter() {
        public void componentMoved(ComponentEvent evt) {
          notifyConnectorShape(anchorShape.getElement());
        }
      }
    );
  }
  
  public AnchorShape getNearestAnchor(Point pt) {
    if (anchors.size() == 0) return null;
    double minDist = Double.MAX_VALUE;
    AnchorShape nearest = null;
    for (Iterator i = anchors.iterator(); i.hasNext();) {
      AnchorShape a = (AnchorShape) i.next();
      AnchorType at = (AnchorType) a.getElement();
      int x = at.getX();
      int y = at.getY();
      double dist = Point.distance(pt.x, pt.y, x, y);
      if (dist < minDist) {
        minDist = dist;
        nearest = a;
      }
    }
    return nearest;
  }
  
  public int getAnchorCount() {
    return anchors.size();
  }

  public int getAnchorInsertionIndex(Point pt) {
    int index = 0, minIndex = 0;
    double minDistance = Double.MAX_VALUE;
    Point2D.Double origin = getConnectionPoint();
    Point2D.Double dest;
    List anchorElems = connection.getAnchor();
    for (Iterator i = anchorElems.iterator(); i.hasNext();) {
      AnchorType anchor = (AnchorType) i.next();
      dest = new Point2D.Double(anchor.getX(), anchor.getY());
      double dist = Line2D.ptSegDist(origin.x, origin.y, dest.x, dest.y, pt.x, pt.y);
      if (dist < minDistance) {
        minIndex = index;
        minDistance = dist;
      }
      origin = dest;
      index++;
    }
    //dest = PointUtilities.hiResPoint(connectedShape.getCenterPoint());
    dest = PointUtilities.hiResPoint(connectedShape.getConnectionPointAccordingToAngle(this.getAngle()));//test
    dest = PointUtilities.convertPoint(connectedShape, dest, this);
    
    if (dest == null) dest = new Point2D.Double(0, 0);
    
    double dist = Line2D.ptSegDist(origin.x, origin.y, dest.x, dest.y, pt.x, pt.y);
    if (dist < minDistance) {
      minIndex = index;
      minDistance = dist;
    }
    return minIndex;
  }
  
  protected void updateSelectionState() { /* do nothing */ }
  
  public Point getCenterPoint() {
    if (connectorShape != null) 
      return SwingUtilities.convertPoint(connectorShape, connectorShape.getCenterPoint(), this);
    else return super.getCenterPoint();
  }
  
  public void setLocation(Point pt) {
    super.setLocation(pt);
  }
  
  public void setLocation(int x, int y) {
    super.setLocation(x, y);
  }

    private double distance(double x1, double y1, double x2, double y2) {
        return (Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)));
    }
    
    private Point2D.Double computeShortPoint(double x1, double y1, double x2, double y2, double gap) {
        if (distance(x1, y1, x2, y2) <= gap) {
            return (new Point2D.Double(x1, y1));
        }

        double theta = Math.atan2(y2 - y1, x2 - x1);
        return (new Point2D.Double(x2 - gap * Math.cos(theta), y2 - gap * Math.sin(theta)));
    }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    if (connectorShape != null && connectedShape != null && connection != null) {
      Graphics2D g2d = (Graphics2D) g;
      
      Color originalColor = g2d.getColor();
      Stroke originalStroke = g2d.getStroke();
      
      int numVertices = connection.getAnchor().size() + 2;
      
      double[] xPts = new double[numVertices];
      double[] yPts = new double[numVertices];
      int indx = 0;
      Point2D.Double origin;
      origin = getConnectionPoint();
      //System.out.println("origin 1: " + origin);//test
      xPts[indx] = origin.x;
      yPts[indx] = origin.y;
      indx++;
      Point2D.Double dest;
      List anchorElems = connection.getAnchor();
      
      /*for (Iterator i = anchorElems.iterator(); i.hasNext();) {
        AnchorType anchor = (AnchorType) i.next();
        origin = new Point2D.Double(anchor.getX(), anchor.getY());
        xPts[indx] = origin.x;
        yPts[indx] = origin.y;
        indx++;
        //g2d.drawLine(origin.x, origin.y, dest.x, dest.y);
        //origin = dest;
      }*/
      
      //test
      for (int i = 0; i < anchorElems.size()-1; i++) {
          AnchorType anchor = (AnchorType) anchorElems.get(i);
          origin = new Point2D.Double(anchor.getX(), anchor.getY());
          xPts[indx] = origin.x;
          yPts[indx] = origin.y;
          indx++;
          //g2d.drawLine(origin.x, origin.y, dest.x, dest.y);
          //origin = dest;
      }
      //end test
      
      /*dest = PointUtilities.hiResPoint(connectedShape.getConnectionPointAccordingToAngle(this.getAngle()));
      dest = PointUtilities.convertPoint(connectedShape, dest, this);
      if (dest == null) dest = new Point2D.Double(0, 0);
      xPts[indx] = dest.x;
      yPts[indx] = dest.y;*/
      
      //test
      xPts[indx] = ((AnchorShape) anchors.get(anchors.size()-1)).getX();
      yPts[indx] = ((AnchorShape) anchors.get(anchors.size()-1)).getY();
      
      Point2D.Double revisedPt = onBorderCheck(xPts[indx],yPts[indx]);
      if (xPts[indx] != revisedPt.x || yPts[indx] != revisedPt.y){
	      xPts[indx] = revisedPt.x;
	      yPts[indx] = revisedPt.y;
	      ((AnchorShape) anchors.get(anchors.size()-1)).setPosition((int)xPts[indx],(int)yPts[indx]);
	      Point tempPt= new Point();
	      tempPt.x = (int) xPts[indx];
	      tempPt.y = (int) yPts[indx];
	      ((AnchorShape) anchors.get(anchors.size()-1)).setPoint(tempPt);
	      //re-draw center point
	      updateAngle(revisedPt);
	      
	      origin = getConnectionPoint();
	      xPts[0] = origin.x;
	      yPts[0] = origin.y;
      }
      //System.out.println("handle x: " + xPts[indx]);
      //System.out.println("handle y: " + yPts[indx]);
      
      //end test
      
      GeneralPath connLine = new GeneralPath();
      connLine.moveTo((float)xPts[0], (float)yPts[0]);
      for (int i = 1; i < numVertices-1; i++) {//numVertices-1 'coz the last anchor is handlepoint
          connLine.lineTo((float)xPts[i], (float)yPts[i]);
      }
      
      /*if (distance(xPts[0], yPts[0], xPts[numVertices - 1], yPts[numVertices - 1]) > GAP_RADIUS * 2 &&
          !(this instanceof GenConnectionShape) && !(this instanceof SpecConnectionShape)) {
          GeneralPath gapLine = new GeneralPath();
          Point2D.Double newPt = computeShortPoint(xPts[numVertices - 2], yPts[numVertices - 2],
                                                   xPts[numVertices - 1], yPts[numVertices - 1],
                                                   GAP_RADIUS);
          gapLine.moveTo((float)newPt.x, (float)newPt.y);
          for (int i = numVertices - 2; i > 0; i--) {
              gapLine.lineTo((float)xPts[i], (float)yPts[i]);
          }
          newPt = computeShortPoint(xPts[1], yPts[1], xPts[0], yPts[0], GAP_RADIUS);
          gapLine.lineTo((float)newPt.x, (float)newPt.y);

          g2d.setColor(Color.white);
          g2d.setStroke( new BasicStroke(getLineWidth() + 4, BasicStroke.CAP_BUTT,
                                         BasicStroke.JOIN_MITER, 10.0f, null, 0));
          g2d.draw(gapLine);
      }*/
      
      g2d.setColor(getLineColor());
      g2d.setStroke(getStroke());
      g2d.draw(connLine);
      // g2d.drawPolyline(xPts, yPts, numVertices);

      g2d.setColor(originalColor);
      g2d.setStroke(originalStroke);
    }
  }
  //test
  protected void updateAngle(Point2D.Double pt){
	  int angle = connectedShape.getAngleAccordingToHandle(this, pt );
	  //System.out.println("updated angle " + angle);
	  this.setAngle(angle);
  }
  //test
  protected Point2D.Double onBorderCheck(double xPt, double yPt){
	  Point2D.Double revisedPt = new Point2D.Double();
	  double objLineWidth = connectedShape.getStyle().getLine().getWidth();
	  
	  int width = connectedShape.getWidth();
	  int height = connectedShape.getHeight();
	  Point2D.Double topLeft,topRight,bottomLeft,bottomRight;
	  //Point2D.Double[] pos = new Point2D.Double[4]; //indx: 0-topleft, 1-topright, 2-bottomleft, 3-bottomright
	  //consider the anchorshape's width
	  double x = xPt + AnchorShape.ANCHOR_WIDTH/2, y = yPt + AnchorShape.ANCHOR_WIDTH/2;
	  
	  Point tempPt = new Point();
	 	  
	  //left top
	  tempPt.x = 0;
	  tempPt.y = 0;
	  topLeft = PointUtilities.hiResPoint(tempPt);
	  topLeft = PointUtilities.convertPoint(connectedShape, topLeft, this);
	  
	  //right top
	  tempPt.x = width;
	  topRight = PointUtilities.hiResPoint(tempPt);
	  topRight = PointUtilities.convertPoint(connectedShape, topRight, this);

	  //left bottom
	  tempPt.x = 0;
	  tempPt.y = height;
	  bottomLeft = PointUtilities.hiResPoint(tempPt);
	  bottomLeft = PointUtilities.convertPoint(connectedShape, bottomLeft, this);
	  
	  //right bottom
	  tempPt.x = width;
	  tempPt.y = height;
	  bottomRight = PointUtilities.hiResPoint(tempPt);
	  bottomRight = PointUtilities.convertPoint(connectedShape, bottomRight, this);
	  
	  /*System.out.println("********************");
	  System.out.println("width and height: [" + width + ", " + height + "]");
	  System.out.println("Handle: " + "[" + xPt + "," + yPt + "]");
	  System.out.println("top: " + topLeft + " " + topRight);
	  System.out.println("bottom: " + bottomLeft + " " + bottomRight);*/
	  
	  double minDist = Double.MAX_VALUE; 
	  //handle is in the objectsetShape
	  if (x > topLeft.x && x < topRight.x && y > topLeft.y && y < bottomLeft.y){
		  //System.out.println("handle is in obj!!!!!!!");
		  //top side
		  if (Point.distance(x,y,x,topLeft.y)<minDist){
			  minDist = Point.distance(x,y,x,topLeft.y);
			  revisedPt.x = x;
			  revisedPt.y = topLeft.y;
		  }
		  //bottom side
		  if (Point.distance(x,y,x,bottomLeft.y)<minDist){
			  minDist = Point.distance(x,y,x,bottomLeft.y);
			  revisedPt.x = x;
			  revisedPt.y = bottomLeft.y;
		  }
		  //left side
		  if (Point.distance(x,y,topLeft.x,y)<minDist){
			  minDist = Point.distance(x,y,topLeft.x,y);
			  revisedPt.x = topLeft.x;
			  revisedPt.y = y;
		  }
		  //right side
		  if (Point.distance(x,y,bottomRight.x,y)<minDist){
			  minDist = Point.distance(x,y,bottomRight.x,y);
			  revisedPt.x = bottomRight.x;
			  revisedPt.y = y;
		  }
	  }
	  //handle is out of the objectsetShape, in the range of x 
	  else if(x > topLeft.x && x < topRight.x){
		  revisedPt.x = x;
		  if(Point.distance(x,y,x,topLeft.y) < Point.distanceSq(x,y,x,bottomLeft.y)){
			  revisedPt.y = topLeft.y;
		  }
		  else {
			  revisedPt.y = bottomLeft.y;
		  }
	  }
	  //handle is out of the objectsetShape, in the range of y
	  else if(y > topLeft.y && y < bottomLeft.y){
		  revisedPt.y =y;
		  if(Point.distanceSq(x,y,topLeft.x,y) < Point.distanceSq(x,y,topRight.x,y)){
			  revisedPt.x = topLeft.x;
		  }
		  else{
			  revisedPt.x = topRight.x;
		  }
	  }
	  //handle is not in range of neighter x nor y
	  else {
		  minDist = Point.distanceSq(x,y,topLeft.x,topLeft.y);
		  revisedPt.x = topLeft.x;
		  revisedPt.y = topLeft.y;
		  if (Point.distance(x,y,topRight.x, topRight.y) < minDist){
			  minDist = Point.distance(x,y,topRight.x, topRight.y);
			  revisedPt.x = topRight.x;
			  revisedPt.y = topRight.y;
		  }
		  if (Point.distanceSq(x, y, bottomLeft.x, bottomLeft.y) < minDist){
			  minDist = Point.distanceSq(x, y, bottomLeft.x, bottomLeft.y);
			  revisedPt.x = bottomLeft.x;
			  revisedPt.y = bottomLeft.y;
		  }
		  if (Point.distance(x,y,bottomRight.x,bottomRight.y) < minDist){
			  revisedPt.x =bottomRight.x;
			  revisedPt.y = bottomRight.y;
		  }
	  }
	  //revisedPt.x = revisedPt.x - AnchorShape.ANCHOR_WIDTH/2 + objLineWidth;
	  //revisedPt.y = revisedPt.y - AnchorShape.ANCHOR_WIDTH/2 + objLineWidth;
	  return revisedPt;
  }
  
  protected int getDefaultWidth() {
    return DEFAULT_WIDTH;
  }
  
  protected Point2D.Double getRealObjectIntersectionPoint(Point2D.Double pt) {
    if (connectedShape == null) return null;
    
//    Canvas canvas = getParentCanvas();
//    if (canvas == null)
//      return connectedShape.getIntersectionPoint(pt);
//    else {
//      Point newPt = connectedShape.getParentCanvas().convertPoint(canvas, pt);
//      return canvas.convertPoint(connectedShape.getParentCanvas(), connectedShape.getIntersectionPoint(newPt));
//    }
    Point2D.Double tailPt = PointUtilities.convertPoint(this, pt, connectedShape);
    Point2D.Double intersection = connectedShape.getIntersectionPoint(tailPt);
    return PointUtilities.convertPoint(connectedShape, intersection, this);
    
  }
  
  public Point2D.Double getObjectIntersectionPoint(Point2D.Double pt) {
    if (connection == null || connection.getAnchor().size()==0) {
      return getRealObjectIntersectionPoint(pt);
    } else {
      AnchorType firstAnchor = (AnchorType) connection.getAnchor().get(0);
      return new Point2D.Double(firstAnchor.getX(), firstAnchor.getY());
    }
  }
  
  public Point getRealObjectCenterPoint() {
    if (connectedShape == null) return null;
//    Canvas canvas = getParentCanvas();
//    if (canvas == null)
//      return connectedShape.getCenterPoint();
//    else
//      return getParentCanvas().convertPoint(connectedShape.getParentCanvas(), connectedShape.getCenterPoint());
    return SwingUtilities.convertPoint(connectedShape, connectedShape.getCenterPoint(), this);
  }
  
  //test
  public Point getRealObjectCenterPointAccordingToAngle() {
	    if (connectedShape == null) return null;
//	    Canvas canvas = getParentCanvas();
//	    if (canvas == null)
//	      return connectedShape.getCenterPoint();
//	    else
//	      return getParentCanvas().convertPoint(connectedShape.getParentCanvas(), connectedShape.getCenterPoint());
	    return SwingUtilities.convertPoint(connectedShape, connectedShape.getConnectionPointAccordingToAngle(this.getAngle()), this);
	  }  
  //end test
  
  private Point getRealObjectConnectionPointAccordingToAngle() {
	  if (connectedShape == null) return null;
	  return SwingUtilities.convertPoint(connectedShape, connectedShape.getConnectionPointAccordingToAngle(getAngle()), this);	  
  }
  
  public Point getObjectCenterPoint() {
    if(anchors.size() == 0)
    	return getRealObjectCenterPoint();
    else {
      AnchorShape lastAnchor = (AnchorShape) anchors.get(anchors.size() - 1);
      return new Point(lastAnchor.getPoint());
    }
  }
  
  public Point getObjectConnectionPointAccordingToAngle() {
	  if(anchors.size() == 0){
		  return getRealObjectConnectionPointAccordingToAngle();
	  }
	  else {
		  AnchorShape lastAnchor = (AnchorShape) anchors.get(anchors.size() - 1);
	      return new Point(lastAnchor.getPoint());
	  }
  }
  public boolean contains(int x, int y) {
    return contains(new Point(x, y));
  }
  
  public boolean contains(Point pt) {
    Point anchorPt;
    for (Iterator i = anchors.iterator(); i.hasNext();) {
      AnchorShape anchor = (AnchorShape) i.next();
      anchorPt = SwingUtilities.convertPoint(this, pt, anchor);
      if (anchor.contains(anchorPt)) return true;
    }
    
    Point connectorShapePt = SwingUtilities.convertPoint(this, pt, connectorShape);
    if (connectorShape.masksConnectionsAt(connectorShapePt)) return false;
    
    // method returns true if pt is within CLICKRANGE pixels from any segment
    // of the connection's path but is not masked by the connector.
    boolean result = false;
    
    List anchorElems = connection.getAnchor();
    for (int i = 0; i < anchorElems.size() + 1; i++) {
      Point2D.Double segStart, segEnd;
      if (i == 0) {
        segStart = getConnectionPoint();
        segStart = PointUtilities.convertPoint(connectorShape, segStart, this);
      } else {
        AnchorType anchor = (AnchorType) anchorElems.get(i-1);
        segStart = new Point2D.Double(anchor.getX(), anchor.getY());
      }
      if (i == anchorElems.size()) {
        segEnd = PointUtilities.hiResPoint(connectedShape.getCenterPoint());
        segEnd = PointUtilities.convertPoint(connectedShape, segEnd, this);
      } else {
        AnchorType anchor = (AnchorType) anchorElems.get(i);
        segEnd = new Point2D.Double(anchor.getX(), anchor.getY());
      }
      if (Line2D.ptSegDist(segStart.x, segStart.y, segEnd.x, segEnd.y, pt.x, pt.y) <= CLICKRANGE) {
        result = true;
        break;
      }
    }
    return result;
  }
  
  public void setSelected(Rectangle r, boolean inclusive) {
    // intentional: do nothing
  } // method setSelected(Rectangle, boolean)
  
  public void setSelected(boolean selectedState) {
    // show or hide anchors according to selectedState
    for (Iterator i = anchors.iterator(); i.hasNext();) {
      AnchorShape anchor = (AnchorShape) i.next();
      anchor.setVisible(selectedState);
    }
    //handleShape.setVisible(selectedState);// test: set the handle visible
  }
  
  protected void setPosition(Point pt) {
    // do nothing
  }
  
  protected void setPosition(int x, int y) {
    // do nothing here
  }
  
  public Point2D.Double getConnectionPoint() {
    Point2D.Double pt = connectorShape.getConnectionPoint();
    if (pt == null) return new Point2D.Double(0, 0);
    pt = PointUtilities.convertPoint(connectorShape, pt, this);
    return pt;
  }
  
  public Rectangle getBoundingBox() {
    double min_x, min_y, max_x, max_y;

    Point2D.Double origin = getConnectionPoint();
    min_x = max_x = origin.x;
    min_y = max_y = origin.y;

    Point pt;
    List anchorElems = connection.getAnchor();
    for (Iterator it = anchorElems.iterator(); it.hasNext();) {
      AnchorType anchor = (AnchorType) it.next();
      min_x = Math.min(min_x, anchor.getX());
      max_x = Math.max(max_x, anchor.getX());
      min_y = Math.min(min_y, anchor.getY());
      max_y = Math.max(max_y, anchor.getY());
    }
    pt = SwingUtilities.convertPoint(connectedShape, connectedShape.getCenterPoint(), this);
    min_x = Math.min(min_x, pt.x);
    max_x = Math.max(max_x, pt.x);
    min_y = Math.min(min_y, pt.y);
    max_y = Math.max(max_y, pt.y);

    return (new Rectangle((int)min_x, (int)min_y,
                          (int)(max_x - min_x + 0.5),
                          (int)(max_y - min_y + 0.5)));
  }

    public void removeAnchor(AnchorShape anchor) {
        if (anchor != null) {
            anchor.delete();
            anchors.remove(anchor);
            updateConfiguration();
        }
    }
    
    public void updateConfiguration() {
        connectorShape.centerPointMoved();
        connectorShape.invalidate();
    }

    

} /* class ConnectionShape */
