/*
 * ConnectorShape.java
 *
 * Created on July 5, 2003, 7:46 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.PositionedElement;
import java.awt.Container;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.SwingUtilities;

/**
 *
 * @author  Alan
 */
public abstract class ConnectorShape extends PlanarShape {
    // NOTE: Replicates functionality from BaseConnector.java (use as reference)

    protected List connections = new ArrayList();
    protected Point centerPoint = null;
    protected Point oldCenterPoint = null;
    protected boolean floatingCenter = true;
    private ComponentAdapter canvasSizeListener;

    /** Creates a new instance of ConnectorShape */
    public ConnectorShape(Container c, OSMXElement elem) {
        super(c, elem);
        setOpaque(false);
        setLayout(null);

        //initLocation();
        initSize();
        initCenterPoint();
        initConnections();
        storeInitialLocation();
    }

    public void anchorCenterPoint() {
        if (element instanceof PositionedElement && centerPoint != null) {
            PositionedElement pe = (PositionedElement) element;
            pe.setX(centerPoint.x);
            pe.setY(centerPoint.y);
            floatingCenter = false;
        }
    }

    public void unanchorCenterPoint() {
        if (element instanceof PositionedElement) {
            PositionedElement pe = (PositionedElement) element;
            pe.unsetX();
            pe.unsetY();
            floatingCenter = true;
            centerPoint = null;
            getCenterPoint();
            centerPointMoved();
            invalidate();
        }        
    }

    protected void updateSelectionState() { /* do nothing */ }

    protected void initLocation() {
        setLocation(0, 0);
    }

    protected void initBorder() {
        // override super to do nothing since we don't want a border
    }

    protected void initSize() {
        canvasSizeListener = new ComponentAdapter() {
            public void componentResized(ComponentEvent evt) {
                setSize(((Container) getParentCanvas()).getSize());
            }
      
            public void componentMoved(ComponentEvent evt) {
                setLocation(0, 0);
            }
        };
        ((Container) getParentCanvas()).addComponentListener(canvasSizeListener);
        setSize(((Container)getParentCanvas()).getSize());
    }

    protected void initCenterPoint() {
        if (element == null) return;
        PositionedElement posElem = (PositionedElement) element;
        if (!posElem.isSetPosition()) return;
        centerPoint = new Point(posElem.getX(), posElem.getY());
        oldCenterPoint = new Point(centerPoint);
        floatingCenter = false;
    }

    public void setLocation(int x, int y) {
        super.setLocation(0, 0);
    }

    public void setLocation(Point pt) {
        setLocation(0, 0);
    }

    protected void setLocationFromPosition() {
        // do nothing
    }

    public Point getPosition() {
        return getCenterPoint();
    }

    protected void locationChanged(Point newLoc) {
        // do nothing
    }

    protected void locationChanged(int x, int y) {
        // do nothing
    }

    public void storeInitialLocation() {
        if (centerPoint != null) initialLocation = new Point(centerPoint);
    }

    protected abstract void initConnections();

    protected void centerPointMoved() { /* do nothing */ }

    protected void recalculateCenterPoint() {
        ConnectionShape conn1 = getHeadConnection();
        //int angle1 = conn1.getAngle();
        ConnectionShape conn2 = getTailConnection();
        //int angle2 = conn2.getAngle();
        if (conn1 != null && conn2 != null) {
            Point2D.Double pt1, pt2;
            //pt1 = PointUtilities.convertPoint(conn2, PointUtilities.hiResPoint(conn2.getObjectCenterPoint()), conn1);
            pt1 = PointUtilities.convertPoint(conn2, PointUtilities.hiResPoint(conn2.getObjectConnectionPointAccordingToAngle()), conn1);//Zonghui's modification
            //pt1 = conn1.getObjectIntersectionPoint(pt1);//Zonghui's modification
            //pt2 = PointUtilities.convertPoint(conn1, pt1, conn2); Zonghui's modification
            pt2 = PointUtilities.convertPoint(conn1, PointUtilities.hiResPoint(conn1.getObjectConnectionPointAccordingToAngle()),conn2);//Zonghui's modification
            //pt2 = conn2.getObjectIntersectionPoint(pt2);//Zonghui's modification
            pt1 = PointUtilities.convertPoint(conn1, pt1, this);
            pt2 = PointUtilities.convertPoint(conn2, pt2, this);
            centerPoint = new Point();
            centerPoint.x = (int)Math.round((pt1.x + pt2.x) / 2);
            centerPoint.y = (int)Math.round((pt1.y + pt2.y) / 2);
            //System.out.println("Center Point: " + centerPoint);
        }
    }

    public Point getCenterPoint() {
        if (!floatingCenter && centerPoint != null) {
            return centerPoint;
        }
        recalculateCenterPoint();

        return centerPoint;
    }

    public abstract ConnectionShape getHeadConnection();

    public abstract ConnectionShape getTailConnection();

    public void delete() {
        getParent().removeComponentListener(canvasSizeListener);
        super.delete();
    }

    protected void setPosition(int x, int y) {
        super.setPosition(x, y);
        centerPointMoved();
    }

    protected void setPosition(Point pt) {
        super.setPosition(pt);
        centerPointMoved();
    }

    public void setSelected(Rectangle r, boolean inclusive) {
        Rectangle bounds = getBounds();
        boolean select = r.contains(bounds);
        setSelected(select);
    } // method setSelected(Rectangle, boolean)
  
    public boolean contains(int x, int y) {
        return contains(new Point(x, y));
    }
  
    public boolean contains(Point pt) {
        boolean result = false;
        for (Iterator i = connections.iterator(); i.hasNext();) {
            ConnectionShape conn = (ConnectionShape) i.next();
            if (conn.contains(SwingUtilities.convertPoint(this, pt, conn))) {
                result = true;
                break;
            }
        }
        return result;
    }

    public abstract boolean masksConnectionsAt(Point pt);

    public Rectangle getBoundingBox() {
        Rectangle bounds = null;

        for (Iterator i = connections.iterator(); i.hasNext(); ) {
            ConnectionShape conn = (ConnectionShape) i.next();
            if (bounds == null) {
                bounds = conn.getBoundingBox();
            } else {
                bounds = bounds.union(conn.getBoundingBox());
            }
        }

        return (bounds);
    }

} /* class ConnectorShape */
