/*
 * GenSpecShape.java
 *
 * Created on September 24, 2003, 8:23 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.epsgraphics.EpsGraphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.beans.PropertyChangeEvent;
import java.util.*;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.GenSpecType;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;

/**
 *
 * @author  Alan
 */
public class GenSpecShape extends ConnectorShape {

    // Constants
    protected static final float    TRIANGLE_WIDTH = 30;
    protected static final float    MITER_BUFFER = 16;
    protected static final float    PLUS_WIDTH = 6.0f,
                                    U_WIDTH = 9.0f;

    // Attributes
    protected GenSpecType           genSpec;
    protected List                  genConnections;
    protected List                  specConnections;
    protected TrianglePanel         triangleShape;
  
    /** Creates a new instance of GenSpecShape */
    public GenSpecShape(Container c, OSMXElement elem) {
        super(c, elem);
        initGenSpec();
        triangleShape = new TrianglePanel();
        add(triangleShape, 0);
        repositionTriangle();
    }
  
    public void addGenConnection(OSMXGeneralizationConnection conn) {
        String objSetID = conn.getObjectSet();
        OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
        OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
        ConnectableShape connectedShape;
        if (objSet ==  null) {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
        } else {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
        }
        if (connectedShape != null) {
            if (!genSpec.getGenConnection().contains(conn)) {
                genSpec.getGenConnection().add(conn);
            }
            ConnectionShape genConn = new GenConnectionShape(this, (ObjConnectableShape)connectedShape, this, conn);
            genConnections.add(genConn);
        }
    }
  
    public void addSpecConnection(OSMXSpecializationConnection conn) {
        String objSetID = conn.getObjectSet();
        OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
        OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
        ConnectableShape connectedShape;
        if (objSet ==  null) {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
        } else {
            connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
        }
        if (connectedShape != null) {
            if (!genSpec.getSpecConnection().contains(conn)) {
                genSpec.getSpecConnection().add(conn);
            }
            ConnectionShape specConn = new SpecConnectionShape(this, (ObjConnectableShape) connectedShape, this, conn);
            specConnections.add(specConn);
        }
    }
  
    protected void centerPointMoved() {
        super.centerPointMoved();
        repositionTriangle();
    }
  
    public boolean contains(int x, int y) {
        return contains(new Point(x, y));
    }
  
    public boolean contains(Point localPoint) {
        boolean result = false;
        for (Iterator i = genConnections.iterator(); i.hasNext();) {
            ConnectionShape conn = (ConnectionShape) i.next();
            if (conn.contains(SwingUtilities.convertPoint(this, localPoint, conn))) {
                result = true;
                break;
            }
        }
        if (result) return result;
            for (Iterator i = specConnections.iterator(); i.hasNext();) {
            ConnectionShape conn = (ConnectionShape) i.next();
            if (conn.contains(SwingUtilities.convertPoint(this, localPoint, conn))) {
                result = true;
                break;
            }
        }
        if (result) return result;
        return masksConnectionsAt(localPoint);
    }
  
    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        if (OSMXGenSpecType.CONSTRAINT_PROPERTY.equals(evt.getPropertyName())) {
            repaint();
        }
    }
  
    public Rectangle getBoundingBox() {
        Rectangle bounds = triangleShape.getBounds();

        for (Iterator it = genConnections.iterator(); it.hasNext(); ) {
            ConnectionShape conn = (ConnectionShape) it.next();
            if (bounds == null) {
                bounds = conn.getBoundingBox();
            } else {
                bounds = bounds.union(conn.getBoundingBox());
            }
        }
        for (Iterator it = specConnections.iterator(); it.hasNext(); ) {
            ConnectionShape conn = (ConnectionShape) it.next();
            if (bounds == null) {
                bounds = conn.getBoundingBox();
            } else {
                bounds = bounds.union(conn.getBoundingBox());
            }
        }

        return (bounds);
    }

    public Point2D.Double getConnectionPoint() {
        return getSpecConnectionPoint();
    }
  
    public Point2D.Double getGenConnectionPoint() {
        Point centerPt = getCenterPoint();
        double theta = getShapeOrientation() + Math.PI / 2;
        double dx = TRIANGLE_WIDTH * Math.cos(theta) / Math.sqrt(3);
        double dy = TRIANGLE_WIDTH * Math.sin(theta) / Math.sqrt(3);
        return (new Point2D.Double(centerPt.x - dx, centerPt.y - dy));
    }

    public ConnectionShape getHeadConnection() {
        if (genConnections.size() < 1) return null;
        return (ConnectionShape) genConnections.get(0);
    }

    private double getShapeOrientation() {
        ConnectionShape head = getHeadConnection();
        ConnectionShape tail = getTailConnection();
        if (head != null && tail != null) {
            Point2D.Double ctrPt = PointUtilities.hiResPoint(getCenterPoint());
            if (ctrPt == null) return 0.0;
            Point2D.Double tmp = PointUtilities.convertPoint(this, ctrPt, head);
            Point2D.Double headPt = head.getObjectIntersectionPoint(tmp);
            double w = headPt.x - ctrPt.x;
            double h = headPt.y - ctrPt.y;
            return Math.atan2(h, w) + Math.PI / 2;
        }
        return 0.0;
    }
    
    public Point2D.Double getSpecConnectionPoint() {
        double theta = getShapeOrientation() + Math.PI / 2;
        Point2D.Double pt = PointUtilities.hiResPoint(getCenterPoint());
        pt.setLocation(pt.x + 4 * Math.cos(theta), pt.y + 4 * Math.sin(theta));
        return pt;
    }

    public ConnectionShape getTailConnection() {
        if (specConnections.size() < 1) return null;
        return (ConnectionShape) specConnections.get(0);
    }

    protected void initConnections() {
        initGenSpec();
        if (genSpec == null) return;

        List genConnList = genSpec.getGenConnection();
        for (Iterator i = genConnList.iterator(); i.hasNext();) {
            OSMXGeneralizationConnection connection = (OSMXGeneralizationConnection) i.next();
            addGenConnection(connection);
        }
        List specConnList = genSpec.getSpecConnection();
        for (Iterator i = specConnList.iterator(); i.hasNext();) {
            OSMXSpecializationConnection connection = (OSMXSpecializationConnection) i.next();
            addSpecConnection(connection);
        }
    }
  
    protected void initGenSpec() {
        if (genSpec == null) {
            genSpec = (GenSpecType)element;
        }
        if (genConnections == null) {
            genConnections = new ArrayList();
        }
        if (specConnections == null) {
            specConnections = new ArrayList();
        }
    }

    public boolean masksConnectionsAt(Point pt) {
        if (genConnections.size() < 1 || specConnections.size() < 1) return false;
        return triangleShape.contains(SwingUtilities.convertPoint(this, pt, triangleShape));
    }
  
    protected void repositionTriangle() {
        Point ctrPt = getCenterPoint();
        if (triangleShape == null) triangleShape = new TrianglePanel();
        if (ctrPt != null) {
            int x = (int)(ctrPt.x - triangleShape.getWidth() / 2.0 + 0.5);
            int y = (int)(ctrPt.y - triangleShape.getHeight() / 2.0 + 0.5);
            triangleShape.setLocation(x, y);
        }
    }
  
    protected void updateSelectionState() {
        super.updateSelectionState();
        triangleShape.updateSelectionState(selected);
    }
    
    private class TrianglePanel extends JPanel {
        private GeneralPath             triangle;
        private GeneralPath             typeShape;
        private float                   triangleWidth = TRIANGLE_WIDTH;
        private boolean                 triangleSelected = false;
        private final float             PANEL_WIDTH = triangleWidth + MITER_BUFFER;
        private final AffineTransform   TRANS_TO_CENTER;

        public TrianglePanel() {
          TRANS_TO_CENTER = AffineTransform.getTranslateInstance(
                                PANEL_WIDTH / 2, PANEL_WIDTH / 2);
          setOpaque(false);
          setLayout(null);
          setSize((int)(PANEL_WIDTH + 0.5), (int)(PANEL_WIDTH + 0.5));
          initTriangle();
        }

        private void initTriangle() {
          triangle = new GeneralPath();
          typeShape = new GeneralPath();
          float offset = triangleWidth / (float)Math.sqrt(3);
          triangle.moveTo(0, -offset);
          triangle.lineTo(triangleWidth / 2, offset / 2);
          triangle.lineTo(-triangleWidth / 2, offset / 2);
          triangle.closePath();
          triangle.transform(TRANS_TO_CENTER);
        }

        private void updateTypeShape() {
          String type = genSpec.getGenSpecConstraint().toUpperCase();
          typeShape.reset();

          if (type.equals(OSMXGenSpecType.UNION_TYPE) || type.equals(OSMXGenSpecType.PARTITION_TYPE))
          {
              // Draw a "U"
              // Left upright
              typeShape.moveTo(-U_WIDTH / 2 - 0.5f, -U_WIDTH / 2);
              typeShape.lineTo(-U_WIDTH / 2 - 0.5f, 1);

              // Bottom curve
              typeShape.curveTo(-U_WIDTH / 2 + 0.5f, U_WIDTH / 2 + 2,
                                U_WIDTH / 2 - 0.5f, U_WIDTH / 2 + 2,
                                U_WIDTH / 2 + 0.5f, 1);
              // Right upright
              typeShape.lineTo(U_WIDTH / 2 + 0.5f, -U_WIDTH / 2);
          }
          if (type.equals(OSMXGenSpecType.PARTITION_TYPE) || type.equals(OSMXGenSpecType.MUTEX_TYPE))
          {
              // Draw a plus sign in the middle of the triangle

              // Vertical cross bar
              typeShape.moveTo(0, -PLUS_WIDTH / 2);
              typeShape.lineTo(0, PLUS_WIDTH / 2);
              // Horizontal cross bar
              typeShape.moveTo(-PLUS_WIDTH / 2, 0);
              typeShape.lineTo(PLUS_WIDTH / 2, 0);
          }
          if (type.equals(OSMXGenSpecType.INTERSECTION_TYPE))
          {
              // Draw an inverted "U"
              // Left descender
              typeShape.moveTo(-U_WIDTH / 2 - 0.5f, U_WIDTH / 2);
              typeShape.lineTo(-U_WIDTH / 2 - 0.5f, -1);

              // Top curve
              typeShape.curveTo(-U_WIDTH / 2 + 0.5f, -U_WIDTH / 2 - 2,
                                U_WIDTH / 2 - 0.5f, -U_WIDTH / 2 - 2,
                                U_WIDTH / 2 + 0.5f, -1);
              // Right descender
              typeShape.lineTo(U_WIDTH / 2 + 0.5f, U_WIDTH / 2);
          }
          typeShape.transform(TRANS_TO_CENTER);
        }

        private AffineTransform getRotationTransform() {
          return AffineTransform.getRotateInstance(getShapeOrientation(), PANEL_WIDTH/2, PANEL_WIDTH/2);
        }

        private void paintTriangle(Graphics2D g) {
          try {
            AffineTransform trans = getRotationTransform();
            AffineTransform inverseTrans = trans.createInverse();
            triangle.transform(trans);
            g.setStroke(getStroke());
            g.setColor(getFillColor());
            g.fill(triangle);
            g.setColor(getLineColor());
            g.draw(triangle);
            if (triangleSelected) {
                // draw selection handles at triangle vertices
                g.setColor(Color.lightGray);
                PathIterator    it = triangle.getPathIterator(new AffineTransform());
                double[]        coord = new double[6];
                double          handleWidth = 6;
                int             seg_type;

                while (!it.isDone()) {
                    seg_type = it.currentSegment(coord);
                    if (seg_type == PathIterator.SEG_MOVETO ||
                                            seg_type == PathIterator.SEG_LINETO) {
                        g.fill(new Rectangle2D.Double(coord[0] - handleWidth / 2,
                                                      coord[1] - handleWidth / 2,
                                                      handleWidth, handleWidth));
                    }
                    it.next();
                }
            }
            triangle.transform(inverseTrans);
          } catch (Exception e) {
            // do nothing
          }
        }

        private void paintTypeShape(Graphics2D g) {
          try {
            AffineTransform trans = getRotationTransform();
            AffineTransform inverseTrans = trans.createInverse();
            updateTypeShape();
            typeShape.transform(trans);
            g.setStroke(getStroke());
            g.setColor(((OSMXStyleType) getStyle()).getLineColor());
            g.draw(typeShape);
            typeShape.transform(inverseTrans);
          } catch (Exception e) {
            // do nothing
          }
        }

        public void paintComponent(Graphics g) {
            if (g instanceof EpsGraphics2D) {
                ((EpsGraphics2D)g).addComment("");
                ((EpsGraphics2D)g).addComment("GenSpec<" + genSpec.getGenSpecConstraint() + ">");
            }

            Graphics2D g2d = (Graphics2D) g.create();
            paintTriangle(g2d);
            paintTypeShape(g2d);
        }

        public boolean contains(int x, int y) {
            Point pt = new Point(x,y);
            return contains(pt);
        }

        public boolean contains(Point point) {
            boolean result = false;
            try {
                AffineTransform trans = getRotationTransform();
                AffineTransform inverseTrans = trans.createInverse();
                triangle.transform(trans);
                result = triangle.contains(point);
                triangle.transform(inverseTrans);
            } catch (Exception e) {
                // do nothing
            }
            return result;
        }

        public void processMouseEvent(MouseEvent evt) {
            evt = SwingUtilities.convertMouseEvent(this, evt, GenSpecShape.this);
            super.processMouseEvent(evt);
        }

        protected void updateSelectionState(boolean selected) {
            triangleSelected = selected;
        }

  } /* nested class TrianglePanel */
  
} /* class GenSpecShape */
