/*
 * ObjectSetShape.java
 *
 * Created on June 23, 2003, 9:15 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.*;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;
import edu.byu.deg.OntologyEditor.epsgraphics.EpsGraphics2D;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.beans.PropertyChangeEvent;
import javax.swing.JPanel;

/**
 *
 * @author  Alan
 */
public class ObjectSetShape extends ConnectableContainerShape
implements TextChangeListener {

    protected EditableText      cardinalityConstraint = null;
    protected PrimaryMarker     primaryMarker = null;
    protected String            objSetName = null;
    protected OSMXObjectSetType objSet = null;
    protected JPanel            constraintPanel = null;
    protected GridBagLayout     gridbag = null;

    protected static final int  PRIMARY_WIDTH = 20;
    protected static final int  PRIMARY_HEIGHT = 5;

    /** Creates a new instance of ObjectSetShape */
    public ObjectSetShape(Container c, OSMXElement elem) {
        super(c, elem);
        getObjectSet();
        objSet.addPositionListener(this);
        ((ObjBorder) shapeBorder).setLexical(objSet.isLexical());
        ((ObjBorder) shapeBorder).setReadOnly(objSet.isReadOnly());
          gridbag = new GridBagLayout();
        constraintPanel = new JPanel(gridbag);
        constraintPanel.setOpaque(false);
        add(ConnectableContainerLayout.TOP_RIGHT, constraintPanel);
        initCardinalityConstraint();
        initPrimaryMarker();
        updateConstraints();
       
    }

    private void updateConstraints() {
        GridBagConstraints  c = new GridBagConstraints();

        c.weightx = 1.0;
        c.weighty = 0.0;
        c.gridheight = 1;
        c.gridwidth = 1;
        c.gridy = 0;

        constraintPanel.removeAll();
        if (objSet.isSetCardinalityConstraint()) {
            c.fill = GridBagConstraints.HORIZONTAL;
            c.gridx = 0;
            c.anchor = GridBagConstraints.SOUTHWEST;
            gridbag.setConstraints(cardinalityConstraint, c);
            constraintPanel.add(cardinalityConstraint);
        }
        if (objSet.isPrimary()) {
            c.fill = GridBagConstraints.BOTH;
            c.gridx = 1;
            c.anchor = GridBagConstraints.NORTHEAST;
            gridbag.setConstraints(primaryMarker, c);
            constraintPanel.add(primaryMarker);
        }
        constraintPanel.validate();
    }

    public ObjectSetType getObjectSet() {
        if (objSet != null) return objSet;
        objSet = (OSMXObjectSetType) element;
        return objSet;
    } // method getObjectSet

    public OSMType getOSM() {
        return getObjectSet().getOSM();
    }

    public String getShapeName() {
        if (objSetName == null && getObjectSet() != null) objSetName = objSet.getPrimaryName();
        return objSetName;
    }

    public void textChanged(String newText) {
        suppressDocEvents = true;
        nameText.setText(newText);
        suppressDocEvents = false;
    } // method textChanged

    protected void setShapeName(String s) {
        objSetName = s;
        if (objSet != null) objSet.setName(objSetName);
    }

    protected void initPrimaryMarker() {
        if (primaryMarker == null) {
            primaryMarker = new PrimaryMarker();
            primaryMarker.setColor(getLineColor());
            primaryMarker.setVisible(objSet.isPrimary());
        }
    }

    protected void initCardinalityConstraint() {
        if (cardinalityConstraint == null) {
            cardinalityConstraint = new EditableText();
            if (objSet.isSetCardinalityConstraint()) {
                cardinalityConstraint.setText(objSet.getCardinalityConstraint());
            } else {
                cardinalityConstraint.setText("0:*");
            }
            updateCardinalityConstraintStyle();
            cardinalityConstraint.setVisible(objSet.isSetCardinalityConstraint());
        }
    }

    protected void updateCardinalityConstraintStyle() {
        cardinalityConstraint.setFont(new Font(getFontFamily(),
                                      OSMXFontType.styleToInt(getFontStyle()),
                                      (int)getFontSize() - 2));
        cardinalityConstraint.setForeground(getFontColor());
        cardinalityConstraint.setBackground(getFillColor());
    }

    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        if (OSMXObjectSetType.LEXICAL_PROPERTY.equals(evt.getPropertyName())) {
            ((ObjBorder) shapeBorder).setLexical(objSet.isLexical());
            repaint();
        } else if (OSMXObjectSetType.READ_ONLY_PROPERTY.equals(evt.getPropertyName())) {
            ((ObjBorder) shapeBorder).setReadOnly(objSet.isReadOnly());
            repaint();
        } else if (OSMXObjectSetType.CARDINALITY_CONSTRAINT_PROPERTY.equals(evt.getPropertyName())) {
            cardinalityConstraint.setVisible(objSet.isSetCardinalityConstraint());
            updateConstraints();
            setSize(getPreferredSize());
            validate();
        } else if (OSMXObjectSetType.PRIMARY_OBJECT_SET_PROPERTY.equals(evt.getPropertyName())) {
            primaryMarker.setVisible(objSet.isPrimary());
            updateConstraints();
            setSize(getPreferredSize());
            validate();
        } else if (OSMXStyleType.STYLE_PROPERTY.equals(evt.getPropertyName())) {
            primaryMarker.setColor(getLineColor());
            // TODO: refresh other aspects of style too
            primaryMarker.setBackground(this.getBackground());
            primaryMarker.setBorder(this.getBorder());
            primaryMarker.setFont(this.getFont());
            primaryMarker.setForeground(this.getForeground());
            updateCardinalityConstraintStyle();

        } else if (evt.getPropertyName().equalsIgnoreCase("Sequence")) {
            ((ObjBorder) shapeBorder).setLexical(objSet.isLexical());
            repaint();
        }
    }

    public void paintComponent(Graphics g) {
        if (g instanceof EpsGraphics2D) {
            ((EpsGraphics2D)g).addComment("");
            ((EpsGraphics2D)g).addComment("ObjectSet<" + nameText.getText() + ">");
        }
        super.paintComponent(g);
    }

} /* class ObjectSetShape */
