/*
 * PlanarShape.java
 *
 * Created on June 14, 2003, 11:08 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.CustomCoordinatePlane;
import edu.byu.deg.OntologyEditor.ObjBorder;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.PositionListener;
import edu.byu.deg.osmx.PositionedElement;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;


/**
 *
 * @author  Alan
 */
public abstract class PlanarShape extends DrawShape
implements PositionListener {
  
  protected boolean positionChanging = false;
  protected PositionedElement posElem;
  
  /** Creates a new instance of PlanarShape */
  public PlanarShape(Container c, OSMXElement elem) {
    super(c, elem);
    initBorder();
  } // constructor
  
  protected void initElement() {
    posElem = (PositionedElement) element;
    posElem.addPositionListener(this);
  }
  
  protected void initBorder() {
    ObjBorder bord = (ObjBorder) shapeBorder;
    int selOptions = 0;
    selOptions |= (ObjBorder.BOTTOM_CENTER | ObjBorder.BOTTOM_LEFT | ObjBorder.BOTTOM_RIGHT);
    selOptions |= (ObjBorder.LEFT_CENTER | ObjBorder.RIGHT_CENTER);
    selOptions |= (ObjBorder.TOP_CENTER | ObjBorder.TOP_LEFT | ObjBorder.TOP_RIGHT);
    bord.setSelOptions(selOptions);
  }
  
  protected void initLocation() {
    setLocationFromPosition();
  }
  
  protected void setLocationFromPosition() {
    positionChanging = true;
    // read position from posElem
    // calculate location from position coords
    int x = posElem.getX();
    int y = posElem.getY();
    Point pt = new Point(getOrigin());
    pt.x = -pt.x;
    pt.y = -pt.y;
    pt.translate(x, y);
    Point parentOrigin = ((CustomCoordinatePlane) getParent()).getOrigin();
    if (parentOrigin != null) 
      pt.translate(parentOrigin.x, parentOrigin.y);
    // call setLocation
    setLocation(pt);
    positionChanging = false;
  }
  
  protected void updatePosition() {
	  //System.out.println("Up:"+element);
    positionChanging = true;
    if (!posElem.isSetPosition()) {
      this.setPosition(null);
    } else {
      int x = posElem.getX();
      int y = posElem.getY();
      this.setPosition(x, y);
    }
    positionChanging = false;
  } // method updatePosition
  
  protected void locationChanged() {
    if (positionChanging) return;
    Point pt = new Point(getOrigin());
    pt.translate(posElem.getX(), posElem.getY());
    setPosition(pt);
    //System.out.println("Lc:"+element);
  }
  
  protected void setPosition(Point pt) {
    if (pt != null) {
      setPosition(pt.x, pt.y);
    } else if (!positionChanging && element instanceof PositionedElement) {
      PositionedElement pe = (PositionedElement) element;
      pe.unsetX();
      pe.unsetY();
    }
  }
  
  protected void setPosition(int x, int y) {
    //System.out.println("SetPos:"+element);
    if (!positionChanging && element instanceof PositionedElement) {
      PositionedElement pe = (PositionedElement) element;
      pe.setX(x);
      pe.setY(y);
    }
  }
  
  public void updateOrder(OSMXElement src, int newOrder) {
    // TODO
  } // method updateOrder
  
  public void updateX(OSMXElement src, int newX) {
	setLocationFromPosition();
    updatePosition();
    this.repaint();
  } // method updateX
  
  public void updateY(OSMXElement src, int newY) {
    setLocationFromPosition();
    updatePosition();
    this.repaint();
  } // method updateY
  
  public void setBorderSize(Dimension d) {
	  this.setSize(d);
	  //System.out.println("setBorderSize()");
	  OSMXObjectSetType obst = (OSMXObjectSetType)element;
	 // OSMXStateType obst = (OSMXStateType)element;
	  obst.setWidth(d.width);
	  obst.setHeight(d.height);
  }
  
}
