/*
 * RelSetConnectionShape.java
 *
 * Created on October 7, 2003, 8:28 AM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.epsgraphics.EpsGraphics2D;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.geom.*;
import java.beans.PropertyChangeEvent;
import java.util.List;
import javax.swing.SwingUtilities;


/**
 *
 * @author  Alan Wessman
 */
public class RelSetConnectionShape extends ConnectionShape {
  
    protected RelSetConnectionType          relSetConn;
    protected ParticipationConstraintShape  partConst;

    protected static final int              ARROWHEAD_LENGTH = 10;
    protected static final int              OPTIONAL_RADIUS = 3;

    /** Creates a new instance of RelSetConnectionShape */
    public RelSetConnectionShape(Container c,
                                 ObjConnectableShape connectable,
                                 RelationshipSetShape connector,
                                 RelSetConnectionType conn) {
        super(c, connectable, connector, conn);
        relSetConn = conn;
        initParticipationConstraint();
    }
    /*public RelSetConnectionShape(Container c,
            ObjConnectableShape connectable,
            AggregationShape connector,
            RelSetConnectionType conn) {
    	super(c, connectable, connector, conn);
    	relSetConn = conn;
    	initParticipationConstraint();
    }*/

    public boolean contains(Point pt) {
        boolean result = super.contains(pt);
        if (result) return result;
        if (partConst == null) return result;
        Point pcPt = SwingUtilities.convertPoint(this, pt, partConst);
        if (partConst.isVisible() && partConst.contains(pcPt)) 
            return true;
        else return false;
    }
  
    protected void initParticipationConstraint() {
        boolean functional = relSetConn.isFunctional();
        boolean optional = relSetConn.isOptional();
        ParticipationConstraintType pc = relSetConn.getParticipationConstraint();
  
        //discard this part of program, coz no participation constraint is allowed
        /*if (pc == null) {
            pc = new OSMXParticipationConstraintType();
            pc.setContent("0:*");
            relSetConn.setParticipationConstraint(pc);
        }
        
        // create the view shape based on the model
        partConst = new ParticipationConstraintShape(this, (OSMXElement) pc);
        partConst.setSize(partConst.getPreferredSize());
        this.add(partConst, 0);
        if (relSetConn.isFunctional() || relSetConn.isOptional()) {
            partConst.setVisible(false);
        }*/
        
        if (pc != null) {
        	//create the view shape based on the model
            partConst = new ParticipationConstraintShape(this, (OSMXElement) pc, false);
            partConst.setSize(partConst.getPreferredSize());
            this.add(partConst, 0);
            if (relSetConn.isFunctional() || relSetConn.isOptional()) { //why ????
                partConst.setVisible(false);
            }
            repositionParticipationConstraint();
        }
        //repositionParticipationConstraint();
        relSetConn.setFunctional(functional);
        relSetConn.setOptional(optional);
    }
  
    public void repositionParticipationConstraint() {
        OSMXPositionedText pc = (OSMXPositionedText)
                                 relSetConn.getParticipationConstraint();
        Point ctrPt = getRealObjectCenterPoint();
        if (ctrPt == null) return;
        if (!pc.isSetPosition()) {
            partConst.setPosition(-connectedShape.getWidth(),
                                  -connectedShape.getHeight());
        } else {
            partConst.setLocationFromPosition();
        }
    }
    
    public void setParticipationConstraint(ParticipationConstraintShape pcs) {
    	partConst = pcs;
    	this.add(pcs, 0);
    }
    
    public void removeParticipationconstraint() {
    	partConst = null;
    }
    
    public void elementPropertyChange(PropertyChangeEvent evt) {
        super.elementPropertyChange(evt);
        if (OSMXRelSetConnectionType.PARTICIPATION_CONSTRAINT_PROPERTY.
                                            equals(evt.getPropertyName())) {
        	ParticipationConstraintType pct = (ParticipationConstraintType)evt.getNewValue();
        	if (pct != null){
	        	partConst = new ParticipationConstraintShape(this, (OSMXElement) pct, false);
	        	partConst.setSize(partConst.getPreferredSize());
	        	this.repositionParticipationConstraint();
        	}
            if (partConst != null) {
                partConst.setVisible(evt.getNewValue() != null);
            }
            
        }
    }
  
    protected void notifyConnectorShape(OSMXElement elem) {
        if (isHeadConnectionPoint(elem) && partConst != null) {
            partConst.setLocationFromPosition();
            partConst.repaint();
        }
        super.notifyConnectorShape(elem);
    }
  
    public void paintComponent(Graphics g) {
        if (g instanceof EpsGraphics2D) {
            ((EpsGraphics2D)g).addComment("");
            ((EpsGraphics2D)g).addComment("RelSetConnection");
        }
        super.paintComponent(g);
        if (relSetConn == null) return;

        OSMXStyleType style = (OSMXStyleType) getStyle();
        Graphics2D g2d = (Graphics2D) g.create();
        Stroke originalStroke = g2d.getStroke();
        Color originalColor = g2d.getColor();
        g2d.setStroke(getStroke());
        Point2D.Double center, end, connCtr;
        connCtr = PointUtilities.hiResPoint(connectorShape.getCenterPoint());
        if (connCtr == null) return;
        List anchorList = relSetConn.getAnchor();
        if (anchorList.size() > 0) {
            AnchorType anchor = (AnchorType) anchorList.get(anchorList.size() - 1);
            center = new Point2D.Double(anchor.getX(), anchor.getY());
        } else {
            center = PointUtilities.convertPoint(connectorShape, connCtr, this);
        }
        end = getRealObjectIntersectionPoint(center);

        // tan (theta) = OppositeSide / AdjacentSide
        double theta = Math.atan2(end.y - center.y, end.x - center.x);
        double line_width = getLineWidth();
        double radius = OPTIONAL_RADIUS + line_width / 2 - 1;
        double length = ARROWHEAD_LENGTH + line_width - 1;
        double x = end.x - getX();
        double y = end.y - getY();

        if (relSetConn.isFunctional()) {
            // draw the functional arrow
            double xoff = -radius * Math.sin(theta);
            double deltax = x - length * Math.cos(theta);
            double yoff = radius * Math.cos(theta);
            double deltay = y - length * Math.sin(theta);

            GeneralPath tri = new GeneralPath();
            tri.moveTo((float)x, (float)y);
            tri.lineTo((float)(deltax - xoff), (float)(deltay - yoff));
            tri.lineTo((float)(deltax + xoff), (float)(deltay + yoff));
            tri.closePath();

            g2d.setColor(style.getLineColor());
            g2d.fill(tri);
            g2d.draw(tri);
        }
        if (relSetConn.isOptional()) {
            // draw the optional circle
            double offset = radius;
            if (relSetConn.isFunctional()) {
                offset += length;
            }
            double deltax = x - offset * Math.cos(theta) - radius;
            double deltay = y - offset * Math.sin(theta) - radius;
            Ellipse2D.Double circle = new Ellipse2D.Double(
                                    deltax, deltay, 2 * radius, 2 * radius);
            g2d.setColor(Color.white);
            g2d.fill(circle);
            g2d.setColor(style.getLineColor());
            g2d.draw(circle);
        }
        g2d.setStroke(originalStroke);
        g2d.setColor(originalColor);
        //g2d.setColor(Color.BLUE);

    } /* paintComponent */
  
    public Rectangle getBoundingBox() {
        Rectangle bounds = super.getBoundingBox();
        if (partConst != null && partConst.isVisible()) {
            bounds = bounds.union(partConst.getBoundingBox());
        }
        return (bounds);
    }
  
} /* class RelSetConnectionShape */
