/*
 * RelationshipSetShape.java
 *
 * Created on August 9, 2003, 4:57 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.*;
import edu.byu.deg.OntologyEditor.epsgraphics.EpsGraphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import java.awt.geom.Ellipse2D.Float;
import java.util.Iterator;
import java.util.List;
import javax.swing.JPanel;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.OSMXOSMType;
import edu.byu.deg.osmx.OSMXObjectSetType;
import edu.byu.deg.osmx.OSMXObjectType;
import edu.byu.deg.osmx.OSMXPositionedText;
import edu.byu.deg.osmx.OSMXRelSetConnectionType;
import edu.byu.deg.osmx.OSMXRelationshipSetType;
import edu.byu.deg.osmx.TextChangeListener;
import edu.byu.deg.osmx.OSMXElementList.ListObserver;
import edu.byu.deg.osmx.binding.RelationshipSetType;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;

/**
 *
 * @author  Alan
 */
public class RelationshipSetShape extends ConnectorShape implements TextChangeListener, ListObserver {
  
  public final static int           MIN_DIAMOND_WIDTH = 15;
  public final static double        HIGH_LEVEL_SCALE_FACTOR = 0.85;

  protected OSMXRelationshipSetType relSet = null;
  private Arrow                     nameArrow;
  private TextShape                 relSetNameText;
  protected boolean                 suppressDocEvents = false;
  protected JTextArea               nameText = null;
  protected DiamondPanel            diamondPanel;
  protected Ellipse2D               centerRegion;
   
  /** Creates a new instance of RelationshipSetShape */
  public RelationshipSetShape(Container c, OSMXElement elem) {
    super(c, elem);
    //System.out.println("RelationshipSet Constrauctor");
    initRelSet();
    initDiamond();
    initNameArrow();
    initName();
  }
  
  protected void initConnections() {
    initRelSet();
    if (relSet == null) return;
    
    List modelConnections = relSet.getRelSetConnection();
    for (Iterator i = modelConnections.iterator(); i.hasNext();) {
      OSMXRelSetConnectionType connection = (OSMXRelSetConnectionType) i.next();
      addConnection(connection);
    }
  }
  
  public void addConnection(OSMXRelSetConnectionType conn) {
    String objSetID = conn.getObjectSet();
    OSMXOSMType osm = (OSMXOSMType) parentCanvas.getOSM();
    OSMXObjectSetType objSet = (OSMXObjectSetType) osm.findObjectSet(objSetID);
    ConnectableShape connectedShape = null;
    if (objSet == null) {
        connectedShape = (ConnectableShape) parentCanvas.findShapeForElement((OSMXObjectType)osm.findObject(objSetID));
    } else {
        connectedShape = (ConnectableShape) parentCanvas.findShapeForElement(objSet);
    }
    if (connectedShape != null) {
      if (!relSet.getRelSetConnection().contains(conn)) {
        relSet.getRelSetConnection().add(conn);
      }
      RelSetConnectionShape connShape = new RelSetConnectionShape(this, (ObjConnectableShape) connectedShape, this, conn);
      connections.add(connShape);
    } else {
        System.out.println("WARNING: null connection");
    }
  }
  
  protected void initRelSet() {
    if (relSet == null) {
      relSet = (OSMXRelationshipSetType) element;
    }
  }
  
  protected void initDiamond() {
    diamondPanel = new DiamondPanel();
    if (relSet == null) return;
    int diamondWidth = relSet.getWidth();
    if (diamondWidth < MIN_DIAMOND_WIDTH) {
      diamondWidth = MIN_DIAMOND_WIDTH;
    }
    diamondPanel.setDiamondWidth(diamondWidth);
    this.add(diamondPanel, 0);
    repositionDiamond();
  }
  
  protected void repositionDiamond() {
    Point ctrPt = getCenterPoint();
    if (ctrPt == null || diamondPanel == null) return;
    int x = ctrPt.x - diamondPanel.getWidth() / 2;
    int y = ctrPt.y - diamondPanel.getHeight() / 2;
    diamondPanel.setLocation(x, y);
  }
  
  protected void repositionName() {
    OSMXPositionedText textElem = (OSMXPositionedText) relSet.getName();
    Point ctrPt = getCenterPoint();
    if (ctrPt == null || textElem == null || relSetNameText == null) return;
    if (!textElem.isSetPosition()) {
      relSetNameText.setPosition(0, -relSetNameText.getPreferredSize().height);
    } else {
      relSetNameText.setLocationFromPosition();
    }
  }
  
  protected void centerPointMoved() {
    super.centerPointMoved();
    repositionDiamond();
    repositionName();
  }
  
  protected void initNameArrow() {
    if (relSet == null) return;
    nameArrow = new Arrow();
    boolean showLeftArrow = relSet.isSetLeftArrow() && relSet.isLeftArrow();
    boolean showRightArrow = relSet.isSetRightArrow() && relSet.isRightArrow();
    nameArrow.setLeftArrowVisible(showLeftArrow);
    nameArrow.setRightArrowVisible(showRightArrow);
    nameArrow.recompute();
  }
  
  protected void initName() {
    OSMXPositionedText textElem = (OSMXPositionedText) relSet.getName();
    if (textElem == null) {
      textElem = new OSMXPositionedText();
      textElem.setContent("has");
      relSet.setName(textElem);
    }
    relSetNameText = new TextShape(this, textElem);
    relSetNameText.setSize(relSetNameText.getPreferredSize());
    this.add(relSetNameText, 0);
    repositionName();
  }
  
  public Point getOrigin() {
    return getCenterPoint();
  }
  
  public void elementAdded(OSMXElement elem) { /* do nothing */ }
  
  public void elementRemoved(OSMXElement elem) { /* do nothing */ }
  
  public void textChanged(String newText) {
    suppressDocEvents = true;
    nameText.setText(newText);
    suppressDocEvents = false;
    repositionName();
  }
  
  public void updateArrow() {
    initNameArrow();
  }
  
  protected RelationshipSetType getRelationshipSet() {
    if (relSet != null) return relSet;
    relSet = (OSMXRelationshipSetType) element;
    return relSet;
  } // method getRelationshipSet
    
  public boolean contains(int x, int y) {
    Point pt = new Point(x,y);
    return contains(pt);
  }
  
  public boolean contains(Point point) {
    if (relSet == null) return false;
    if (super.contains(point)) return true;
    return masksConnectionsAt(point);
  }
  
    public Rectangle getBoundingBox() {
        Rectangle bounds = super.getBoundingBox();
        // nameArrow
        // relSetNameText
        // diamondPanel
        if (relSetNameText != null) {
            Rectangle rsnt = relSetNameText.getBoundingBox();
            bounds = bounds.union(rsnt);
        }
        if (diamondPanel != null) {
            Rectangle dp = diamondPanel.getBounds();
            bounds = bounds.union(dp);
        }
        return (bounds);
    }

  public Stroke getStroke() {
    BasicStroke stroke = (BasicStroke) super.getStroke();
    BasicStroke result = new BasicStroke(getLineWidth(), stroke.getEndCap(), BasicStroke.JOIN_MITER, stroke.getMiterLimit(), stroke.getDashArray(), stroke.getDashPhase());
    return result;
  }
  
  public void paintComponent(Graphics g) {
    if (g instanceof EpsGraphics2D) {
        OSMXPositionedText textElem = (OSMXPositionedText) relSet.getName();
        String name = "";

        if (textElem != null) {
            name = textElem.getContent();
        }

        ((EpsGraphics2D)g).addComment("");
        ((EpsGraphics2D)g).addComment("RelationshipSet<" + name + ">");
    }

    //super.paintComponent(g);
    Graphics2D g2d = (Graphics2D) g.create();
    
    g2d.setStroke(DEFAULT_STROKE);
    diamondPanel.setVisible(showDiamond());
    if (!showDiamond() && isSelected()) {
      Ellipse2D ctrRegion = getCenterRegion();
      g2d.setColor(Color.gray);
      if (ctrRegion != null) {
        g2d.fill(ctrRegion);
      }
    }
    paintArrow(g);
  } // method paintComponent
  
  protected Ellipse2D getCenterRegion() {
    if (centerRegion == null) {
      centerRegion = new Float();
    }
    Point ctrPt = getCenterPoint();
    if (ctrPt == null) return null;
    centerRegion.setFrameFromCenter(ctrPt, new Point(ctrPt.x + 6, ctrPt.y + 6));
    return centerRegion;
  }
  
  protected void paintArrow(Graphics g) {
    if (relSet != null) {
      if (relSet.getRelSetConnection().size() != 2) return;
    }
    // calculate offset from connector line
    int offset;
    
    if (relSet.isShowDiamond()) {
      offset = (int)(MIN_DIAMOND_WIDTH + 0.8 * Arrow.ARROW_SEGMENT);
    }
    else
      offset = (int)(0.8 * Arrow.ARROW_SEGMENT);
    
    // draw directional arrow and text
    // rotate
    ConnectionShape tailConn = getTailConnection();
    Point targetPoint = getTailConnection().getObjectCenterPoint();
    targetPoint = SwingUtilities.convertPoint(tailConn, targetPoint, this);
    Point ctrPt = getCenterPoint();
    
    // tan (theta) = OppositeSide / AdjacentSide
    double theta = Math.atan2(targetPoint.y - ctrPt.y, targetPoint.x - ctrPt.x);

    boolean negate = false;
    int dY = targetPoint.y - ctrPt.y;
    boolean yNeg = (dY<0);
    Point textPoint = relSetNameText.getCenterPoint();
    textPoint = SwingUtilities.convertPoint(relSetNameText, textPoint, this);
    if (ctrPt.x == targetPoint.x) {
      negate = ((textPoint.x > ctrPt.x && !yNeg) || (textPoint.x < ctrPt.x && yNeg));
    }
    else if (ctrPt.y == targetPoint.y) {
      yNeg = (targetPoint.x - ctrPt.x > 0);
      negate = ((textPoint.y > ctrPt.y && !yNeg) || (textPoint.y < ctrPt.y && yNeg));
    }
    else {
      double slope = (double)(dY) / (double)(targetPoint.x - ctrPt.x);
      int y = (int)(slope * (textPoint.x - ctrPt.x) + ctrPt.y);
      
      negate = ((y >= textPoint.y && !yNeg) || (y < textPoint.y && yNeg));
      
      if (slope < 0)
        negate=!negate;
    }
    
    Graphics2D g2d = (Graphics2D) g.create();
    Color originalColor = g2d.getColor();
    g2d.setColor(getLineColor());
    g2d.translate(ctrPt.x, ctrPt.y);
    g2d.rotate(theta);
    if (negate)
      g2d.translate(0, -offset);
    else
      g2d.translate(0, offset);
    
    g2d.draw(nameArrow);
    g2d.fill(nameArrow);
    g2d.setColor(originalColor);
  } // method paintArrow
  
  protected boolean showDiamond() {
    if (relSet == null) return false;
    Point ctrPt = getCenterPoint();
    if (ctrPt == null) return false;
    return (relSet.isShowDiamond() /* || connections.size() > 2 */);
  }
  
  public ConnectionShape getHeadConnection() {
    if (connections.size() == 0) return null;
    return (ConnectionShape) connections.get(0);
  }
  
  public ConnectionShape getTailConnection() {
    if (connections.size() < 2) return null;
    return (ConnectionShape) connections.get(1);
  }
  
  public boolean masksConnectionsAt(Point pt) {
    boolean result = false;
    Point childPoint;
    if (diamondPanel != null && showDiamond()) {
      childPoint = SwingUtilities.convertPoint(this, pt, diamondPanel);
      result = result || diamondPanel.contains(childPoint);
    } else {
      Ellipse2D ctrRegion = getCenterRegion();
      if (ctrRegion != null) {
        result = result || ctrRegion.contains(pt.x, pt.y);
      }
    }
    if (relSetNameText != null) {
      childPoint = SwingUtilities.convertPoint(this, pt,  relSetNameText);
      result = result || relSetNameText.contains(childPoint);
    }
    return result;
  }
  
    protected void updateSelectionState() {
        super.updateSelectionState();
        diamondPanel.updateSelectionState(selected);
    }
    
    private class DiamondPanel extends JPanel {
        private Polygon diamond;
        private int diamondWidth = MIN_DIAMOND_WIDTH;

        public DiamondPanel() {
            ObjBorder b = new ObjBorder();
            b.setRectDraw(false);
            b.setHandleWidth(5);
            b.setSelOptions(ObjBorder.BOTTOM_CENTER | ObjBorder.LEFT_CENTER |
                            ObjBorder.TOP_CENTER | ObjBorder.RIGHT_CENTER);
            setBorder(b);
            
            setOpaque(false);
            setLayout(null);
            setDiamondWidth(MIN_DIAMOND_WIDTH);
            initDiamond();
        }

        private void initDiamond() {
            diamond = new Polygon();
            diamond.addPoint(0, diamondWidth);
            diamond.addPoint(diamondWidth, 0);
            diamond.addPoint(0, -(diamondWidth));
            diamond.addPoint(-(diamondWidth), 0);
            diamond.translate(diamondWidth + 3, diamondWidth + 3);
        }

        public void setSize(int width, int height) {
            super.setSize(width + 6, width + 6);
            diamondWidth = width / 2;
            invalidate();
        }

        public void setDiamondWidth(int width) {
            setSize(width * 2, width * 2);
        }

        public void paintComponent(Graphics g) {
            Graphics2D g2d = (Graphics2D) g.create();
            Color fillColor = getFillColor();
            if (relSet.isHighLevel()) {
                // We use our own scale factor instead of the java.awt.Color.darker()
                // method because darker() goes too dark.
                fillColor = new Color((int)(fillColor.getRed() * HIGH_LEVEL_SCALE_FACTOR),
                                      (int)(fillColor.getGreen() * HIGH_LEVEL_SCALE_FACTOR),
                                      (int)(fillColor.getBlue() * HIGH_LEVEL_SCALE_FACTOR));
            }
            g2d.setColor(fillColor);
            g2d.fillPolygon(diamond);
            g2d.setColor(getLineColor());
            g2d.setStroke(RelationshipSetShape.this.getStroke());
            g2d.drawPolygon(diamond);
        }

        public boolean contains(int x, int y) {
            Point pt = new Point(x,y);
            return contains(pt);
        }

        public boolean contains(Point point) {
            return diamond.contains(point);
        }

        protected void processMouseEvent(MouseEvent evt) {
            evt = SwingUtilities.convertMouseEvent(this, evt, RelationshipSetShape.this);
            super.processMouseEvent(evt);
        }
    
        protected void updateSelectionState(boolean selected) {
            ((ObjBorder) getBorder()).setSelect(selected);
        }

    } /* nested class DiamondPanel */
  
} /* class RelationshipSetShape */
