/*
 * TextShape.java
 *
 * Created on June 14, 2003, 11:22 PM
 */

package edu.byu.deg.OntologyEditor.shapes;

import edu.byu.deg.OntologyEditor.EditableText;
import edu.byu.deg.OntologyEditor.ObjBorder;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.OSMXFontType;
import edu.byu.deg.osmx.OSMXPositionedText;
import edu.byu.deg.osmx.OSMXStyleType;
import edu.byu.deg.osmx.TextChangeListener;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.beans.PropertyChangeEvent;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 *
 * @author  Alan
 */
public class TextShape extends PlanarShape
implements DocumentListener, TextChangeListener {

  protected JTextArea text;
  protected static final Color BACKGROUND_COLOR = Color.WHITE;
  protected OSMXPositionedText textElement;
  protected boolean suppressDocEvents = false;

  /** Creates a new instance of TextShape */
  public TextShape(Container c, OSMXElement elem) {
    super(c, elem);
    String s = textElement.getContent();
    
    textElement.addTextChangeListener(this);
    text = new EditableText(s);
    text.getDocument().addDocumentListener(this);
    text.addMouseListener(this.getRootCanvas());
    text.addMouseMotionListener(this.getRootCanvas());
    setBackground(BACKGROUND_COLOR);

    FlowLayout fl = new FlowLayout();

    fl.setHgap(0);
    fl.setVgap(0);
    setLayout(fl);

    /* TODO: get font stuff working
    Font font = model.getFont();
    if (font != null) {
      m_text.setFont(font);
      m_text.setForeground(model.getFontColor());
    }
     */
    text.setFont(this.getFont());
    text.setForeground(Color.BLACK);
    text.setBackground(Color.WHITE);
    refreshFromStyle();
    // text.setOpaque(false);
    add(text);
    text.getDocument().addDocumentListener(this);
  } // constructor

  protected void initElement() {
    super.initElement();
    textElement = (OSMXPositionedText) element;
  }

  protected void updateSelectionState() {
    ((ObjBorder) shapeBorder).setSelect(selected);
    if (!selected) {
      // de-select the text in the nested text component
      text.select(0, 0);
    }
  } // method updateSelectionState

  public void setBackground(Color c) {
    if (text != null) {
       // System.out.println("text fill color = " + c.getRed() + " " + c.getGreen() + " " + c.getBlue());
       text.setBackground(c);
    }
  }

  /** Gives notification that an attribute or set of attributes changed.
   *
   * @param e the document event
   *
   */
  public void changedUpdate(DocumentEvent e) {
    insertUpdate(e);
  } // method changedUpdate

  /** Gives notification that there was an insert into the document.  The
   * range given by the DocumentEvent bounds the freshly inserted region.
   *
   * @param e the document event
   *
   */
  public void insertUpdate(DocumentEvent e) {
    suppressDocEvents = true;
    setSize(getPreferredSize());
    invalidate();
    textElement.setContent(text.getText());
    suppressDocEvents = false;
  } // method insertUpdate

  /** Gives notification that a portion of the document has been
   * removed.  The range is given in terms of what the view last
   * saw (that is, before updating sticky positions).
   *
   * @param e the document event
   *
   */
  public void removeUpdate(DocumentEvent e) {
    insertUpdate(e);
  } // method removeUpdate

  public void textChanged(String newText) {
    if (!suppressDocEvents) {
      text.setText(newText);
    }
  } // method textChanged

  protected void elementProperyChange(PropertyChangeEvent evt) {
    super.elementPropertyChange(evt);
    if (OSMXStyleType.STYLE_PROPERTY.equals(evt.getPropertyName())) {
      refreshFromStyle();
    }
  }

  protected void refreshFromStyle() {
    setBackground(getFillColor());
    if (text != null) {
      text.setForeground(getFontColor());
      Font font = new Font(getFontFamily(), OSMXFontType.styleToInt(getFontStyle()), (int) getFontSize());
      text.setFont(font);
    }
  }

  protected void paintComponent(Graphics g) {
      if (text.getText().trim().length() > 0) {
          super.paintComponent(g);
      }
  }

  public void paint(Graphics g) {
      if (text.getText().trim().length() > 0) {
          super.paint(g);
      }
  }

}

