package edu.byu.deg.OntologyEditor.text_tools;


/**
 *
 * Copyright (c)1999 - 2002 Dr. William C. Jones, Jr.
 * http://www.javabook.org
 * Professor of Computer Science
 * Central Connecticut State University
 * Office: Maria Sanford 225. Office phone: (860)832-2716
 * email: jonesw@ccsu.edu or author@javabook.org
 *
 * @author  William C. Jones
 */
public class LongestCommonSubsequence
{
	public final static int MAX = 1000;
	private static       int[][]  answer  = new int[MAX][MAX];
	private static       int[][]  arrow  = new int[MAX][MAX];
	private static       String   one;
	private static       String   two;


	private static final int N = 0;
	private static final int W = 1;
	private static final int NW = 2;
	private static final int NONE = 3;

	/**
	 * Return the length of the longest common subsequence of
	 *  the two given Strings. Return 0 if either is null or
	 *  has MAX or more characters.
	 */

	public static int longest(String first, String second)
	{
		if (first == null || second == null
			|| first.length() >= MAX || second.length() >= MAX)
			return 0;
		for (int row = 0; row <= first.length(); row++)
		{
			for (int col = 0; col <= second.length(); col++)
			{
				answer[row][col] = row * col == 0 ? 0 : -1;
				arrow[row][col] = NONE;
			}
		}
		one = first;
		two = second;
		return longest(one.length(), two.length());
	} //================================================


	public static int numCharsAdded()
	{
		int row = one.length();
		int col = two.length();

		int charsAdded = 0;

		while (row > 0 && col > 0 && one.charAt(row-1) != two.charAt(col-1))
		{
			if (arrow[row][col] == N)
				row--;
			else if (arrow[row][col] == W)
				col--;
			else
				break;
		}

		while(arrow[row][col] != NONE && answer[row][col] > 0)
		{
			if (arrow[row][col] == N)
			{
				charsAdded++;
				row--;
			}
			else if (arrow[row][col] == W)
			{
				charsAdded++;
				col--;
			}
			else
			{
				row--;
				col--;
			}
		}
		return charsAdded;
	}

	private static String getDirName(int dir)
	{
		switch (dir)
		{
			case N: return "N";
			case NW: return "NW";
			case W: return "W";
			default: return "";
		}
	}

	public static void print()
	{

		System.out.print("\t\t");
		for (int col = 0; col < two.length(); col++)
			System.out.print(two.charAt(col)+"\t");
		System.out.print("\n");
		for (int row = 0; row <= one.length(); row++)
		{
			if (row > 0)
				System.out.print(one.charAt(row-1));
			System.out.print("\t");

			for (int col = 0; col <= two.length(); col++)
			{
				System.out.print("("+getDirName(arrow[row][col])+")"+answer[row][col]+"\t");
			}
			System.out.print("\n");
		}
	}

	private static int longest(int len1, int len2)
	{
		if (answer[len1][len2] >= 0)
			return answer[len1][len2];
		if (one.charAt(len1 - 1) == two.charAt(len2 - 1))
		{
			answer[len1][len2] = longest(len1 - 1, len2 - 1) + 1;
			arrow[len1][len2] = NW;
		}
		else
		{
			int up = longest(len1 - 1, len2);
			int left = longest(len1, len2 - 1);

			if (up > left)
			{
				answer[len1][len2] = up;
				arrow[len1][len2] = N;
			}
			else
			{
				answer[len1][len2] = left;
				arrow[len1][len2] = W;
			}
		}
		return answer[len1][len2];
	} //================================================



	/**
	 * Return the length of the LCS of the first len1 characters
	 *  of the first String and the first len2 characters of the
	 *  second String.  Throw an Exception for bad indexes.
	 */

	public static int lcs(int len1, int len2)
	{
		return answer[len1][len2];
	} //================================================


/*
	private static int longest_0(int len1, int len2)
	{
		for (int row = 1; row <= len1; row++)
			for (int col = 1; col <= len2; col++)
			{
				if (one.charAt(row - 1) == two.charAt(col - 1))
					answer[row][col] = answer[row - 1][col - 1] + 1;
				else if (answer[row - 1][col] > answer[row][col - 1])
					answer[row][col] = answer[row - 1][col];
				else
					answer[row][col] = answer[row][col - 1];
			}
		return answer[len1][len2];
	} //================================================
*/
}
