package edu.byu.deg.askOntos.extractor;


import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.UIManager;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.AbstractDocument;

import edu.byu.deg.framework.DocumentRetriever;
import edu.byu.deg.framework.retriever.DOMRetriever;
import edu.byu.deg.osmx.OSMXDocument;
import edu.byu.deg.util.Timer;

/**
 * A GUI to run the Ontos extraction engine
 * @author mvickers
 *
 */
public class OntosExtractorGUI extends JFrame implements ActionListener, HyperlinkListener
{		
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final String ONTOLOGY = "Choose Ontology";
	private final String HTMLDIR = "Choose HTML Dir";
	private final String OUTPUTDIR = "Choose Output Dir";
	
	private final String defaultDir = "C:\\AskOntosData";	//determines where JFileChooser opens
	
	private final int M_WIDTH = 1200;
	private final int M_HEIGHT= 900;	
	private final int TEXTAREA_WIDTH = M_WIDTH;
	private final int TEXTAREA_HEIGHT = M_HEIGHT;
	
	private String ontologyFile, htmlDir, outputDir;
	private String[] htmlFiles;
	
	private ChooseFilePanel ontologyChooser, htmlChooser, outputChooser;
	private JButton b_run;
	private JEditorPane ep_output;
	private JScrollPane queryScrollPane;
	
	
	public OntosExtractorGUI()
	{
		super("Ontos Extractor");
					
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);//.EXIT_ON_CLOSE);
		setResizable(true);
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));		
		mainPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		mainPanel.setPreferredSize(new Dimension(M_WIDTH, M_HEIGHT));
		mainPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
		
		ontologyFile = "";
		htmlDir = defaultDir + "\\html";    //"C:\\AskOntosData\\html";		
		//outputDir = System.getProperty("user.dir");
		outputDir = defaultDir;
		htmlFiles = null;
		
		ontologyChooser = new ChooseFilePanel(ONTOLOGY, ontologyFile, "Ontology File");
		htmlChooser = new ChooseFilePanel(HTMLDIR, htmlDir, "HTML Directory");
		outputChooser = new ChooseFilePanel(OUTPUTDIR, outputDir, "Output Directory");
		
			
		b_run = new JButton("RUN");
		b_run.setActionCommand("run");
		b_run.addActionListener(this);
		
//		Create a text area.		
		ep_output = new JEditorPane();
		ep_output.setPreferredSize(new Dimension(TEXTAREA_WIDTH, TEXTAREA_HEIGHT));
		ep_output.setAlignmentX(JTextArea.LEFT_ALIGNMENT);
		ep_output.setFont(new Font("Arial", Font.BOLD, 14));		
		ep_output.setEditable(false);
		ep_output.addHyperlinkListener(this);
				
		queryScrollPane = new JScrollPane(ep_output);
		queryScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		queryScrollPane.setPreferredSize(new Dimension(TEXTAREA_WIDTH, TEXTAREA_HEIGHT));
		queryScrollPane.setAlignmentX(JScrollPane.LEFT_ALIGNMENT);
		queryScrollPane.setBorder(BorderFactory.createCompoundBorder(
							BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
													BorderFactory.createEmptyBorder(10, 10, 10, 10)),
													queryScrollPane.getBorder()));
		
		JPanel filterPanel = new JPanel();
		filterPanel.setLayout(new BoxLayout(filterPanel, BoxLayout.X_AXIS));
		filterPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		filterPanel.setPreferredSize(new Dimension(400, 40));
		filterPanel.setMinimumSize(new Dimension(M_WIDTH, 40));
		filterPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
				
		JPanel runPanel = new JPanel();
		SpringLayout layout = new SpringLayout();		
		layout.putConstraint(SpringLayout.EAST, b_run, -30, SpringLayout.EAST, runPanel);
		runPanel.setLayout(layout);
		runPanel.setMinimumSize(new Dimension(50, 80));
		runPanel.add(b_run);
			
		
		mainPanel.add(ontologyChooser);
		mainPanel.add(htmlChooser);
		mainPanel.add(outputChooser);
		mainPanel.add(filterPanel);
		mainPanel.add(runPanel);
		mainPanel.add(queryScrollPane);
			
		getContentPane().add(mainPanel);
		pack();
		
	}//end of constructor
	

	public void actionPerformed(ActionEvent e) 
	{
		
		if("run".equals(e.getActionCommand()) )
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					
			ontologyFile = ontologyChooser.getFileString();
			htmlDir = htmlChooser.getFileString();
			outputDir = outputChooser.getFileString();
			
			htmlFiles = getHTMLFiles(htmlDir);
			
			try{
			      if(Timer.time) Timer.timer.startTask("Extracting");
			      OSMXDocument ontology = new OSMXDocument();

			      if(Timer.time) Timer.timer.startTask("Loading ontology");
			      ontology = ontology.openDocument(ontologyFile);
			      if(Timer.time) Timer.timer.endTask("Loading ontology");

			      if(Timer.time) Timer.timer.startTask("Initiating Document Retriever");
			      DocumentRetriever retriever = new DOMRetriever();
			      for(int i = 0; i < htmlFiles.length; i++)
			    	  retriever.addDocumentLocation(htmlFiles[i]);
			      if(Timer.time) Timer.timer.endTask("Initiating Document Retriever");
			      
			      System.out.println("****" + htmlFiles.length + " HTML Files found ****");
			      
			      edu.byu.deg.extractionmodule.MetaExtractor extractor = new edu.byu.deg.extractionmodule.MetaExtractor(ontology);
			      extractor.startExtraction(retriever, null);
			      
			      if(Timer.time) Timer.timer.startTask("Saving ontology");
			      ontology.saveAsDocument("resultingOSMX.xml");
			      if(Timer.time) Timer.timer.endTask("Saving ontology");

			      if(Timer.time) Timer.timer.startTask("Saving result page");
			      DataPrinter printer = new DataPrinter();
			      //printer.printToFile("result.html", ontology);
			      printer.printToFile(outputDir+"\\OntosResult.html", ontology);
			      if(Timer.time) Timer.timer.endTask("Saving result page");

			      if(Timer.time) Timer.timer.endTask("Extracting");
			      
			      setCursor(null); //turn off the wait cursor
			      
			      //To force a document reload it is necessary to clear the stream description property of the document
				AbstractDocument doc = (AbstractDocument) ep_output.getDocument();
				doc.putProperty(AbstractDocument.StreamDescriptionProperty, null);
				
				ep_output.setPage("file:///" + outputDir+"\\OntosResult.html");
				  
				
							      
		    }catch(Exception exc) {exc.printStackTrace();return;}
		  }
	}
	

	private String[] getHTMLFiles(String retPath)
	{
		File dir = new File(retPath);
	    
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	        	if(name.endsWith(".htm") || name.endsWith(".html"))
	        		return true;
	        	return false;	            
	        }
	    };
	    File[] files =  dir.listFiles(filter);
	    String[] fullPaths = new String[files.length];
	    for(int i = 0; i < files.length; i++)
	    	fullPaths[i] = files[i].getAbsolutePath();
	    
	    return fullPaths;
	}
	
	
	
	/*    
	 * Implement the HyperlinkListener Interface
	 */
	public void hyperlinkUpdate(HyperlinkEvent e) 
	{
		if(e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
		{
			String url = e.getURL().toString();
			try {
				Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url); //reference:  http://www.centerkey.com/java/browser/
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}		
	}	
	
	
	
	

	///////////////////////////////////////////////////////////////////////////////////
	//A Dialog Class for choosing the configuration files
	//////////////////////////////////////////////////////////////////////////////////
	class ChooseFilePanel extends JPanel implements ActionListener
	{
		
		private static final long serialVersionUID = -4865999267909511418L;
		private final int W = 800;
		private final int H = 200;
		private final int TEXT_FIELD_LEN = 60;
		
		JTextField tf_file;
		JButton b_fileChooser; 
		JPanel configPanel;
		String title;
		String fileString;
		String btnName;
				
		/** Creates the reusable dialog. */
		public ChooseFilePanel(String bName, String defaultFile, String title) 
		{	
			
			this.title = title;
			fileString = defaultFile;
			btnName = bName;
									
			this.setSize(new Dimension(W, H));
			
			//this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
			b_fileChooser = new JButton(btnName);		
			b_fileChooser.addActionListener(this);		
			b_fileChooser.setActionCommand("browse");
									
			tf_file = new JTextField(fileString, TEXT_FIELD_LEN);
				
			this.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(title), 
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			
			add(tf_file);
			add(b_fileChooser);						
			
		}//end of constructor
		
		
		/*
		 * Implement the ActionListener Interface
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) 
		{
			
			if("browse".equals(e.getActionCommand()) )
			{
				String file = launchFileChooser(btnName);
				if(!file.equals(""))
				{
					tf_file.setText(file);
					fileString = file;
				}
			}						
		}
					
		public String getFileString() {return fileString; }
	}//end of ConfigFileDialog
	
	
	
	/**
	 * Lauches JFileChooser with the custom filter
	 * @return File
	 */
	public String launchFileChooser(String title)
	{
		JFileChooser chooser = new JFileChooser(defaultDir);
		chooser.setDialogTitle(title);
		chooser.setApproveButtonText("OK");
		if(title.equals(HTMLDIR) || title.equals(OUTPUTDIR))
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		   
		int returnVal = chooser.showOpenDialog(this);		
		if(returnVal == JFileChooser.APPROVE_OPTION) 
		{
			File file = chooser.getSelectedFile();    	
			return file.getAbsolutePath();
		}
		return "";
		
	}//end of LaunchFileChooser()

	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
		try 
		{			
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			//UIManager.setLookAndFeel(UIManager.setLookAndFeel(new LookAndFeel()));
	        
		} catch (Exception e) 
		{
			System.out.println("Problem setting the look and feel:" + e.toString());
		}
		OntosExtractorGUI gui = new OntosExtractorGUI();
		gui.pack();
		gui.setVisible(true);

	}

}//end fo ExtractorGUI
