/*
 * BasicListPanel.java
 *
 * Created on July 21, 2003, 11:11 PM
 */

package edu.byu.deg.dataframe;

import java.awt.BorderLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.List;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.TitledBorder;

/**
 *
 * @author  Alan
 */
public class BasicListPanel extends JPanel {
  
  private JScrollPane scrTableScroller = new JScrollPane();
  private BorderLayout mainLayout = new BorderLayout();
  private JTable tblItems = new JTable();
  private JPanel pnlRightButtonSet = new JPanel();
  private JPanel pnlOrderButtons = new JPanel();
  private JButton btnMoveUp = new JButton();
  private JButton btnMoveDown = new JButton();
  private BoxLayout buttonLayout = new BoxLayout(pnlOrderButtons, BoxLayout.Y_AXIS);
  private JButton btnAdd = new JButton();
  private JButton btnDelete = new JButton();
  private TitledBorder mainBorder = new TitledBorder("");
  private TableListModel panelModel;
  
  /** Creates a new instance of BasicListPanel */
  public BasicListPanel() {
    try {
      jbInit();
    } catch (Exception e) {
      e.printStackTrace();
    }
  } // constructor()
  
  public BasicListPanel(TableListModel tlm) {
    this();
    setListModel(tlm);
  } // constructor(TableListModel)
  
  public void setListModel(TableListModel tlm) {
    panelModel = tlm;
    if (getTableModel() != null) getTableModel().setListModel(tlm);
  } // method setListModel
  
  public void setTitle(String title) {
    mainBorder.setTitle(title);
  } // method setTitle
  
  protected ListPanelTableModel getTableModel() {
    return (ListPanelTableModel) tblItems.getModel();
  } // method getTableModel
  
  public void setColumnNames(String[] names) {
    if (getTableModel() != null) {
      getTableModel().setColumnNames(names);
    }
  } // method setColumnNames
  
  private void jbInit() throws Exception {
    this.setBorder(mainBorder);
    this.setLayout(mainLayout);
    Class thisClass = this.getClass();
    URL moveUpIconURL = thisClass.getResource("images/moveup.gif");
    URL moveDownIconURL = thisClass.getResource("images/movedown.gif");
    URL addIconURL = thisClass.getResource("images/add.gif");
    URL removeIconURL = thisClass.getResource("images/remove.gif");
    btnMoveUp.setText(null);
    btnMoveDown.setText(null);
    btnAdd.setText(null);
    btnDelete.setText(null);
    btnMoveUp.setIcon(new ImageIcon(moveUpIconURL));
    btnMoveDown.setIcon(new ImageIcon(moveDownIconURL));
    btnAdd.setIcon(new ImageIcon(addIconURL));
    btnDelete.setIcon(new ImageIcon(removeIconURL));
    Dimension minButtonSize = new Dimension(20, 20);
    Dimension maxButtonSize = new Dimension(24, 24);
    btnMoveUp.setMinimumSize(minButtonSize);
    btnMoveDown.setMinimumSize(minButtonSize);
    btnAdd.setMinimumSize(minButtonSize);
    btnDelete.setMinimumSize(minButtonSize);
    btnMoveUp.setMaximumSize(maxButtonSize);
    btnMoveDown.setMaximumSize(maxButtonSize);
    btnAdd.setMaximumSize(maxButtonSize);
    btnDelete.setMaximumSize(maxButtonSize);
    btnMoveUp.setToolTipText("Move up");
    btnMoveDown.setToolTipText("Move down");
    btnAdd.setToolTipText("Add new entry");
    btnDelete.setToolTipText("Delete selected entry");
    scrTableScroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
    scrTableScroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrTableScroller.getViewport().add(tblItems, null);
    this.add(scrTableScroller, BorderLayout.CENTER);
    pnlOrderButtons.setLayout(buttonLayout);
    pnlOrderButtons.add(Box.createVerticalGlue(), null);
    pnlOrderButtons.add(btnMoveUp, null);
    pnlOrderButtons.add(Box.createRigidArea(new Dimension(1, 4)));
    pnlOrderButtons.add(btnMoveDown, null);
    pnlOrderButtons.add(Box.createRigidArea(new Dimension(1, 40)));
    pnlOrderButtons.add(btnAdd, null);
    pnlOrderButtons.add(Box.createRigidArea(new Dimension(1, 4)));
    pnlOrderButtons.add(btnDelete, null);
    pnlOrderButtons.add(Box.createVerticalGlue(), null);
    pnlRightButtonSet.add(Box.createHorizontalGlue(), null);
    pnlRightButtonSet.add(pnlOrderButtons, null);
    pnlRightButtonSet.add(Box.createRigidArea(new Dimension(4, 1)), null);
    this.add(pnlRightButtonSet, BorderLayout.EAST);
    tblItems.setModel(new ListPanelTableModel(panelModel));
    btnAdd.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (panelModel == null) return;
          panelModel.addItem();
          List modelList = panelModel.getList();
          int listSize = (modelList == null ? 0 : modelList.size());
          ((AbstractTableModel) tblItems.getModel()).fireTableRowsInserted(listSize, listSize);
        }
      }
    );
    btnDelete.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (panelModel == null) return;
          int[] selIndexes = tblItems.getSelectedRows();
          panelModel.deleteItems(selIndexes);
          AbstractTableModel mod = (AbstractTableModel) tblItems.getModel();
          mod.fireTableDataChanged();
          
          // set selection to row before deleted item (if any exists)
          if (selIndexes.length == 1 && selIndexes[0] > 0) {
            tblItems.addRowSelectionInterval(selIndexes[0] - 1, selIndexes[0] - 1);
          }
        }
      }
    );
    btnMoveUp.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (panelModel == null) return;
          int[] selIndexes = tblItems.getSelectedRows();
          panelModel.moveItemsUp(selIndexes);
          AbstractTableModel mod = (AbstractTableModel) tblItems.getModel();
          mod.fireTableDataChanged();
          
          // restore selection
          for (int i = selIndexes.length - 1; i >= 0; i--) {
            tblItems.addRowSelectionInterval(selIndexes[i] - 1, selIndexes[i] - 1);
          }
        }
      }
    );
    btnMoveDown.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (panelModel == null) return;
          int[] selIndexes = tblItems.getSelectedRows();
          panelModel.moveItemsDown(selIndexes);
          AbstractTableModel mod = (AbstractTableModel) tblItems.getModel();
          mod.fireTableDataChanged();
          
          // restore selection
          for (int i = 0; i < selIndexes.length; i++) {
            tblItems.addRowSelectionInterval(selIndexes[i] + 1, selIndexes[i] + 1);
          }
        }
      }
    );
    tblItems.getSelectionModel().addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          if (e.getValueIsAdjusting()) return;
          ListSelectionModel lsm = (ListSelectionModel) e.getSource();
          if (panelModel == null || lsm.isSelectionEmpty()) {
            btnDelete.setEnabled(false);
            btnMoveUp.setEnabled(false);
            btnMoveDown.setEnabled(false);
          } else {
            btnDelete.setEnabled(true);
            if (lsm.getMinSelectionIndex() <= 0) btnMoveUp.setEnabled(false);
            else btnMoveUp.setEnabled(true);
            
            List modelList = panelModel.getList();
            int listSize = (modelList == null ? 0 : modelList.size());
            
            if (lsm.getMaxSelectionIndex() >= listSize - 1) btnMoveDown.setEnabled(false);
            else btnMoveDown.setEnabled(true);
          }
        }
      }
    );
  } // method jbInit
  
  protected class ListPanelTableModel extends AbstractTableModel {
    
    private String[] columnNames;
    private TableListModel listModel;
    
    public ListPanelTableModel() {
      super();
    } // constructor()
    
    public ListPanelTableModel(TableListModel tlm) {
      setListModel(tlm);
    } // constructor(TableListModel)
    
    public void setListModel(TableListModel tlm) {
      listModel = tlm;
      if (listModel != null) columnNames = listModel.getColumnNames();
      this.fireTableStructureChanged();
      this.fireTableDataChanged();
    } // method setListModel
    
    public void setColumnNames(String[] names) {
      columnNames = names;
    } // method setColumnNames
    
    public String getColumnName(int column) {
      if (column >= columnNames.length - 1) return columnNames[columnNames.length - 1];
      else if (column <= 0) return columnNames[0];
      else return columnNames[column];
    } // method getColumnName
    
    public int getColumnCount() {
      return (columnNames == null ? 0 : columnNames.length);
    } // method getColumnCount
    
    public int getRowCount() {
      if (listModel != null) {
        List theList = listModel.getList();
        return (theList == null? 0 : theList.size());
      }
      return 0;
    } // method getRowCount
    
    public Object getValueAt(int rowIndex, int columnIndex) {
      if (listModel == null) return null;
      List theList = listModel.getList();
      if (theList == null || theList.size() == 0) return null;
      if (rowIndex < 0 || rowIndex >= theList.size()) return null;
      return listModel.getColumnValue(theList.get(rowIndex), columnIndex);
    } // method getValueAt
  }
}
