/*
 * DataFrameDetailPanel.java
 *
 * Created on September 12, 2003, 3:09 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.KeywordListOwner;
import edu.byu.deg.osmx.OSMXDataFrameType;
import edu.byu.deg.osmx.OSMXElement;
import edu.byu.deg.osmx.OSMXObjectSetType;
import edu.byu.deg.osmx.binding.*;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.border.TitledBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 *
 * @author  aew2
 */
public class DataFrameDetailPanel extends JPanel {
  
  private OSMXObjectSetType objectSet;
  private JCheckBox cbHasDataFrame = new JCheckBox();
  private JPanel pnlCards = new JPanel();
  private BorderLayout mainBorderLayout = new BorderLayout();
  private CardLayout cardLayout = new CardLayout();
  private JPanel pnlGeneral = new JPanel();
  private BoxLayout generalLayout = new BoxLayout(pnlGeneral, BoxLayout.Y_AXIS);
  private JPanel pnlInternalRep = new JPanel();
  private JPanel pnlCanonicalization = new JPanel();
  private JPanel pnlValuePhrases = new BasicListPanel();
  private JPanel pnlMethods = new BasicListPanel();
  private JPanel pnlCanonMethodName = new JPanel();
  private JPanel pnlKeywordPhrases = new BasicListPanel();
  private JPanel pnlKeywordPhrase = new KeywordPhrasePanel();
  private JPanel pnlValuePhrase = new ValuePhrasePanel();
  private JPanel pnlOperationPhrases = new BasicListPanel();
  private JPanel pnlParameter = new ParameterPanel();
  private JPanel pnlMethod = new MethodPanel();
  private JPanel pnlBlank = new JPanel();
  private JPanel pnlDataFrameType = new TypeSpecificationPanel();
  private JPanel pnlParameters = new BasicListPanel();
  private BorderLayout canonBorderLayout = new BorderLayout();
  private BorderLayout internalRepBorderLayout = new BorderLayout();
  private BoxLayout canonMethodNameBoxLayout = new BoxLayout(pnlCanonMethodName, BoxLayout.X_AXIS);
  private JTextField txtCanonMethodName = new JTextField();
  private JLabel lblCanonMethodName = new JLabel();
  
  protected Object[] currentPath;
  protected JTree navTree;
  
  /** Creates a new instance of DataFrameDetailPanel */
  public DataFrameDetailPanel() {    
    currentPath = null;
    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
  
  public DataFrameDetailPanel(OSMXObjectSetType initialObjSet, JTree controllerTree) {
    this();
    setNavTree(controllerTree);
    setObjectSet(initialObjSet);
  } // constructor (OSMXObjectSetType)
  
  public void setObjectSet(ObjectSetType objSet) {
    objectSet = (OSMXObjectSetType) objSet;
    updateFromModel();
  } // method setObjectSet
  
  public void setNavTree(JTree controllerTree) {
    navTree = controllerTree;
    if (navTree == null) return;
    
    navTree.addTreeSelectionListener(
      new TreeSelectionListener() {
        public void valueChanged(TreeSelectionEvent e) {
          setCurrentPath(e.getPath());
        } // method valueChanged
      } // anonymous inner class TreeSelectionListener
    );
  } // method setNavTree
  
  public OSMXObjectSetType getObjectSet() {
    return objectSet;
  } // method getObjectSet
  
  protected DefaultTreeModel getTreeModel() {
    if (navTree == null) return null;    
    return (DefaultTreeModel) navTree.getModel();
  } // method getTreeModel
  
  public void setCurrentPath(TreePath path) {
    if (path == null || path.getPathCount() <= 1) {
      setObjectSet(null);
      return;
    }
    Object[] pathNodes = path.getPath();
    currentPath = pathNodes;
    int i = pathNodes.length - 1;
    while(!(pathNodes[i] instanceof ObjectSetNode) && i >= 0) i--;
    ObjectSetNode node = (i > 0 ? (ObjectSetNode) path.getPathComponent(i) : null);
    if (node != null) setObjectSet(node.getObjectSet());
    
    showCard((DefaultMutableTreeNode) pathNodes[pathNodes.length - 1]);
    updateFromModel();
  } // method setCurrentPath
  
  protected void showCard(DefaultMutableTreeNode node) {
    if (node == null) {
      cardLayout.show(pnlCards, "pnlBlank");
    } else if (!objectSet.isSetDataFrame()) {
      cardLayout.show(pnlCards, "pnlGeneral");
    } else if (node instanceof ObjectSetNode) {
      cardLayout.show(pnlCards, "pnlGeneral");
    } else if (node instanceof KeywordPhraseListNode) {
      cardLayout.show(pnlCards, "pnlKeywordPhrases");
      KeywordListOwner klo = null;
      DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) currentPath[currentPath.length - 1];
      Object[] pathObjects = currentNode.getUserObjectPath();
      for (int i = pathObjects.length - 1; i >= 0; i--) {
        if (pathObjects[i] instanceof KeywordListOwner) {
          klo = (KeywordListOwner) pathObjects[i];
          break;
        } else if (pathObjects[i] instanceof ObjectSetType) {
          if (pathObjects[i] != null) {
            DataFrameType df = ((ObjectSetType) pathObjects[i]).getDataFrame();
            klo = (OSMXDataFrameType) df;
            break;
          }
        }
      }
      if (klo != null) {
        TableListModel mod = new KeywordPhraseListModel(klo);
        mod.addTableListModelListener(
          new TableListModelListener() {
            public void itemAdded(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof KeywordPhraseListNode) {
                final KeywordPhraseListNode kpln = (KeywordPhraseListNode) currentPath[currentPath.length - 1];
                final KeywordPhraseNode kpn = new KeywordPhraseNode((KeywordPhraseType) elem, getTreeModel());
                addNodeChild(kpln, kpn);
              }
            }
            
            public void itemRemoved(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof KeywordPhraseListNode) {
                KeywordPhraseListNode kpln = (KeywordPhraseListNode) currentPath[currentPath.length - 1];
                removeNodeChild(kpln, elem);
              }
            }
            
            public void itemsReordered() {
              if (currentPath[currentPath.length - 1] instanceof KeywordPhraseListNode) {
                KeywordPhraseListNode kpln = (KeywordPhraseListNode) currentPath[currentPath.length - 1];
                if (kpln == null) return;
                List phraseList = kpln.getKeywordPhraseList();
                if (phraseList == null) return;
                reorderNodeChildren(kpln, phraseList);
              }
            }
          }
        );
        ((BasicListPanel) pnlKeywordPhrases).setListModel(mod);
      }
    }
    else if (node instanceof ValuePhraseListNode) {
      cardLayout.show(pnlCards, "pnlValuePhrases");
      DataFrameType dft = objectSet.getDataFrame();
      if (dft != null) {
        TableListModel mod = new ValuePhraseListModel(dft);
        mod.addTableListModelListener(
          new TableListModelListener() {
            public void itemAdded(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof ValuePhraseListNode) {
                final ValuePhraseListNode vpln = (ValuePhraseListNode) currentPath[currentPath.length - 1];
                final ValuePhraseNode vpn = new ValuePhraseNode((ValuePhraseType) elem, getTreeModel());
                addNodeChild(vpln, vpn);
              }
            }
            
            public void itemRemoved(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof ValuePhraseListNode) {
                ValuePhraseListNode vpln = (ValuePhraseListNode) currentPath[currentPath.length - 1];
                removeNodeChild(vpln, elem);
              }
            }
            
            public void itemsReordered() {
              if (currentPath[currentPath.length - 1] instanceof ValuePhraseListNode) {
                ValuePhraseListNode vpln = (ValuePhraseListNode) currentPath[currentPath.length - 1];
                if (vpln == null) return;
                List phraseList = vpln.getValuePhraseList();
                if (phraseList == null) return;
                reorderNodeChildren(vpln, phraseList);
              }
            }
          }
        );
        ((BasicListPanel) pnlValuePhrases).setListModel(mod);
      }
    } else if (node instanceof MethodListNode) {
      cardLayout.show(pnlCards, "pnlMethods");
      DataFrameType dft = objectSet.getDataFrame();
      if (dft != null) {
        TableListModel mod = new MethodListModel(dft);
        mod.addTableListModelListener(
          new TableListModelListener() {
            public void itemAdded(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof MethodListNode) {
                final MethodListNode mln = (MethodListNode) currentPath[currentPath.length - 1];
                final MethodNode mn = new MethodNode((MethodType) elem, getTreeModel());
                addNodeChild(mln, mn);
              }
            }
            
            public void itemRemoved(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof MethodListNode) {
                MethodListNode mln = (MethodListNode) currentPath[currentPath.length - 1];
                removeNodeChild(mln, elem);
              }
            }
            
            public void itemsReordered() {
              if (currentPath[currentPath.length - 1] instanceof MethodListNode) {
                MethodListNode mln = (MethodListNode) currentPath[currentPath.length - 1];
                if (mln == null) return;
                List methodList = mln.getMethodList();
                if (methodList == null) return;
                reorderNodeChildren(mln, methodList);
              }
            }
          }
        );
        ((BasicListPanel) pnlMethods).setListModel(mod);
      }
    } else if (node instanceof KeywordPhraseNode) {
      cardLayout.show(pnlCards, "pnlKeywordPhrase");
      KeywordPhraseNode phraseNode = (KeywordPhraseNode) node;
      KeywordPhraseType phrase = phraseNode.getKeywordPhrase();
      if (phrase == null) return;
      ((KeywordPhrasePanel) pnlKeywordPhrase).setKeywordPhrase(phrase);
    }else if (node instanceof ValuePhraseNode) {
      cardLayout.show(pnlCards, "pnlValuePhrase");
      ValuePhraseNode phraseNode = (ValuePhraseNode) node;
      ValuePhraseType phrase = phraseNode.getValuePhrase();
      if (phrase == null) return;
      ((ValuePhrasePanel) pnlValuePhrase).setValuePhrase(phrase);
    } else if (node instanceof MethodNode) {
      cardLayout.show(pnlCards, "pnlMethod");
      MethodNode methodNode = (MethodNode) node;
      MethodType method = methodNode.getMethod();
      if (method == null) return;
      ((MethodPanel) pnlMethod).setMethod(method);
    } else if (node instanceof ParameterListNode) {
      cardLayout.show(pnlCards, "pnlParameters");
      MethodType mt = null;
      DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) currentPath[currentPath.length - 1];
      Object[] pathObjects = currentNode.getUserObjectPath();
      for (int i = 0; i < pathObjects.length; i++) {
        if (pathObjects[i] instanceof MethodType) {
          mt = (MethodType) pathObjects[i];
          break;
        }
      }
      if (mt != null) {
        TableListModel mod = new ParameterListModel(mt);
        mod.addTableListModelListener(
          new TableListModelListener() {
            public void itemAdded(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof ParameterListNode) {
                final ParameterListNode pln = (ParameterListNode) currentPath[currentPath.length - 1];
                final ParameterNode pn = new ParameterNode((ParameterType) elem, getTreeModel());
                addNodeChild(pln, pn);
              }
            }
            
            public void itemRemoved(OSMXElement elem) {
              if (currentPath[currentPath.length - 1] instanceof ParameterListNode) {
                ParameterListNode pln = (ParameterListNode) currentPath[currentPath.length - 1];
                removeNodeChild(pln, elem);
              }
            }
            
            public void itemsReordered() {
              if (currentPath[currentPath.length - 1] instanceof ParameterListNode) {
                ParameterListNode pln = (ParameterListNode) currentPath[currentPath.length - 1];
                if (pln == null) return;
                List paramList = pln.getParameterList();
                if (paramList == null) return;
                reorderNodeChildren(pln, paramList);
              }
            }
          }
        );
        ((BasicListPanel) pnlParameters).setListModel(mod);
      }
    } else if (node instanceof ParameterNode) {
      cardLayout.show(pnlCards, "pnlParameter");
      ParameterNode paramNode = (ParameterNode) node;
      ParameterType param = paramNode.getParameter();
      if (param == null) return;
      ((ParameterPanel) pnlParameter).setParameter(param);
    } else {
      cardLayout.show(pnlCards, "pnlGeneral");
    }
  } // method showCard
  
  protected void reorderNodeChildren(DefaultMutableTreeNode node, List userObjects) {
    Enumeration en = node.children();
    Map objectsToNodes = new HashMap();
    Object[] nodeItems = null;
    if (userObjects != null) {
      nodeItems = userObjects.toArray();
    }
    while (en.hasMoreElements()) {
      DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) en.nextElement();
      objectsToNodes.put(childNode.getUserObject(), childNode);
    }
    node.removeAllChildren();
    if (nodeItems != null) {
      for (int i = 0; i < nodeItems.length; i++) {
        DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) objectsToNodes.get(nodeItems[i]);
        if (childNode != null) node.add(childNode);
      }
    }
    DefaultTreeModel treeModel = getTreeModel();
    treeModel.nodeStructureChanged(node);
    TreePath newPath;
    if (node.getChildAt(0) == null) newPath = new TreePath(node.getPath());
    else newPath = new TreePath(((DefaultMutableTreeNode) node.getChildAt(0)).getPath());
    navTree.scrollPathToVisible(newPath);
  } // method reorderNodeChildren
  
  protected void removeNodeChild(DefaultMutableTreeNode node, OSMXElement elem) {
    Enumeration en = node.children();
    DefaultMutableTreeNode toRemove = null;
    while (en.hasMoreElements()) {
      DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) en.nextElement();
      if (childNode.getUserObject() == elem) {
        toRemove = childNode;
        break;
      }
    }
    if (toRemove != null) {
      node.remove(toRemove);
    }
    TreePath newPath = new TreePath(node.getPath());
    navTree.scrollPathToVisible(newPath);
  } // method removeNodeChild
  
  protected void addNodeChild(DefaultMutableTreeNode parent, DefaultMutableTreeNode child) {
    parent.add(child);
    TreePath newPath = new TreePath(child.getPath());
    navTree.scrollPathToVisible(newPath);
  } // method addNodeChild
  
  protected void updateFromModel() {
    if (objectSet == null) {
      cbHasDataFrame.setEnabled(false);
      pnlDataFrameType.setEnabled(false);
      ((TypeSpecificationPanel) pnlDataFrameType).setTypeSpecification(null);
      showCard(null);
    } else {
      cbHasDataFrame.setEnabled(true);
      cbHasDataFrame.setSelected(objectSet.isSetDataFrame());
      pnlDataFrameType.setEnabled(objectSet.isSetDataFrame());
      if (objectSet.isSetDataFrame()) {
        TypeSpecification ts = objectSet.getDataFrame().getInternalRepresentation();
        ((TypeSpecificationPanel) pnlDataFrameType).setTypeSpecification(ts);
      } else {
        ((TypeSpecificationPanel) pnlDataFrameType).setTypeSpecification(null);
      }
    }
    repaint();
  }
  
  
  private void jbInit() throws Exception {
    this.setLayout(mainBorderLayout);
    this.setSize(new Dimension(645, 532));
    cbHasDataFrame.setText("Object set has data frame");
    pnlCards.setLayout(cardLayout);
    pnlGeneral.setLayout(generalLayout);
    pnlInternalRep.setBorder(new TitledBorder("Internal representation"));
    pnlInternalRep.setLayout(internalRepBorderLayout);
    pnlCanonicalization.setBorder(new TitledBorder("Default canonicalization method"));
    pnlCanonicalization.setLayout(canonBorderLayout);
    pnlValuePhrase.setBorder(new TitledBorder("Value Phrase"));
    pnlKeywordPhrase.setBorder(new TitledBorder("Keyword Phrase"));
    pnlMethod.setBorder(new TitledBorder("Method"));
    pnlParameter.setBorder(new TitledBorder("Parameter"));
    ((BasicListPanel) pnlKeywordPhrases).setTitle("Keyword Phrases");
    ((BasicListPanel) pnlKeywordPhrases).setColumnNames(KeywordPhraseListModel.columnNames);
    ((BasicListPanel) pnlOperationPhrases).setTitle("Operation Phrases");
    ((BasicListPanel) pnlValuePhrases).setTitle("Value Phrases");
    ((BasicListPanel) pnlValuePhrases).setColumnNames(ValuePhraseListModel.columnNames);
    ((BasicListPanel) pnlMethods).setTitle("Methods");
    ((BasicListPanel) pnlMethods).setColumnNames(MethodListModel.columnNames);
    ((BasicListPanel) pnlParameters).setTitle("Parameters");
    ((BasicListPanel) pnlParameters).setColumnNames(ParameterListModel.columnNames);
    pnlCanonMethodName.setLayout(canonMethodNameBoxLayout);
    cbHasDataFrame.setAlignmentX(Component.LEFT_ALIGNMENT);
    pnlInternalRep.setAlignmentX(Component.LEFT_ALIGNMENT);
    pnlCanonicalization.setAlignmentX(Component.LEFT_ALIGNMENT);
    pnlGeneral.add(cbHasDataFrame);
    pnlGeneral.add(pnlInternalRep);
    pnlGeneral.add(pnlCanonicalization);
    pnlGeneral.add(Box.createVerticalGlue());
    pnlInternalRep.add(pnlDataFrameType, BorderLayout.CENTER);
    lblCanonMethodName.setText("Canonicalization method name:");
    pnlCanonMethodName.add(lblCanonMethodName, null);
    pnlCanonMethodName.add(Box.createRigidArea(new Dimension(4,1)));
    pnlCanonMethodName.add(txtCanonMethodName, null);
    pnlCanonicalization.add(pnlCanonMethodName, BorderLayout.NORTH);
   // this.add(cbHasDataFrame, BorderLayout.NORTH);
    pnlCards.add(pnlGeneral, "pnlGeneral");
    pnlCards.add(pnlValuePhrases, "pnlValuePhrases");
    pnlCards.add(pnlKeywordPhrases, "pnlKeywordPhrases");
    pnlCards.add(pnlOperationPhrases, "pnlOperationPhrases");
    pnlCards.add(pnlMethods, "pnlMethods");
    pnlCards.add(pnlValuePhrase, "pnlValuePhrase");
    pnlCards.add(pnlKeywordPhrase, "pnlKeywordPhrase");
    pnlCards.add(pnlMethod, "pnlMethod");
    pnlCards.add(pnlParameters, "pnlParameters");
    pnlCards.add(pnlParameter, "pnlParameter");
    pnlCards.add(pnlBlank, "pnlBlank");
    this.add(pnlCards, BorderLayout.CENTER);
    
    cbHasDataFrame.addActionListener(
    new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (objectSet != null) {
          DefaultMutableTreeNode n = null;
          if (currentPath.length > 0) {
            n = (DefaultMutableTreeNode) currentPath[currentPath.length - 1];
          }
          if (cbHasDataFrame.isSelected()) {
            objectSet.restoreDataFrame();
            showCard(n);
            if (!objectSet.isSetDataFrame()) {
              // create a new DataFrame element and set it in the object set
              OSMXDataFrameType dft = new OSMXDataFrameType();
              objectSet.setDataFrame(dft);
            }
            navTree.expandPath(new TreePath(n.getPath()));
          } else {
            objectSet.suppressDataFrame();
            showCard(n);
          }
          navTree.scrollPathToVisible(new TreePath(n.getPath()));
        }
        updateFromModel();
      } // method actionPerformed
    }
    );
  }
  
}
