/*
 * DataFrameEditorPanel.java
 *
 * Created on September 12, 2003, 2:59 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXDocument;
import edu.byu.deg.osmx.OSMXOSMType;
import edu.byu.deg.osmx.OSMXObjectSetType;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.text.Collator;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

/**
 *
 * @author  aew2
 */
public class DataFrameEditorPanel extends JPanel {
  
  private OSMXDocument ontologyDoc;
  private JTree navTree;
  private DefaultTreeModel navTreeModel;
  
  /** Creates a new instance of DataFrameEditorPanel */
  public DataFrameEditorPanel() {
    this.setLayout(new BorderLayout());
    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true);
    this.add(split, BorderLayout.CENTER);
    JScrollPane treeScroller = new JScrollPane();
    treeScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    treeScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    DefaultMutableTreeNode dmtNode = new DefaultMutableTreeNode("Data Frames");
    navTreeModel = new DefaultTreeModel(dmtNode);

    navTree = new JTree(navTreeModel);
    navTree.setBackground(Color.WHITE);
    navTree.setRootVisible(false);
    navTree.setShowsRootHandles(true);
    DefaultTreeCellRenderer r = (DefaultTreeCellRenderer) navTree.getCellRenderer();
    r.setClosedIcon(null);
    r.setIcon(null);
    r.setOpenIcon(null);
    r.setLeafIcon(null);
    navTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

    final DataFrameDetailPanel rightPanel = new DataFrameDetailPanel();
    rightPanel.setNavTree(navTree);
    rightPanel.setObjectSet(null);

    treeScroller.setMinimumSize(new Dimension(80, 300));
    treeScroller.setPreferredSize(new Dimension(120, 400));
    treeScroller.setViewportView(navTree);

    split.setLeftComponent(treeScroller);
    split.setRightComponent(rightPanel); 
    split.setDividerLocation(0.20);
  }
  
  public DataFrameEditorPanel(OSMXDocument doc) {
    this();
    ontologyDoc = doc;
    OSMXOSMType osm = (OSMXOSMType) doc.getModelRoot();
    if (osm != null) {
      populateNavTreeNode((DefaultMutableTreeNode) navTreeModel.getRoot(), osm);
      navTreeModel.reload();
    }
  }
    
  private void populateNavTreeNode(DefaultMutableTreeNode node, OSMXObjectSetType objSet) {
    ObjectSetNode toAdd = new ObjectSetNode(null, navTreeModel);
    node.add(toAdd);
    if (objSet.isHighLevel() && objSet.isSetOSM()) {
      populateNavTreeNode(toAdd, (OSMXOSMType) objSet.getOSM());
    }
    toAdd.setUserObject(objSet);
  } // method populateNavTreeNode(OSMXObjectSetType)

  private void populateNavTreeNode(DefaultMutableTreeNode node, OSMXOSMType osm) {
    final Collator coll = Collator.getInstance();
    coll.setStrength(Collator.PRIMARY);
    List childObjSets = osm.getObjectSets(
      new Comparator() {
        public int compare(Object o1, Object o2) {
          OSMXObjectSetType objSet1 = (OSMXObjectSetType) o1;
          OSMXObjectSetType objSet2 = (OSMXObjectSetType) o2;
          return coll.compare(objSet1.getPrimaryName(), objSet2.getPrimaryName());
        }

        public boolean equals(Object obj) {
          return false;
        }
      }
    );
    for (Iterator i = childObjSets.iterator(); i.hasNext();) {
      OSMXObjectSetType objSet = (OSMXObjectSetType) i.next();
      populateNavTreeNode(node, objSet);
    }
  } // method populateNavTreeNode(OSMXOSMType)
  
  public OSMXDocument getOntologyDocument() {
    return ontologyDoc;
  }
  
}
