package edu.byu.deg.dataframe;

import edu.byu.deg.common.ColorButton;
import edu.byu.deg.common.ColorListener;
import edu.byu.deg.osmx.OSMXDataFrameExpression;
import edu.byu.deg.osmx.binding.DataFrameExpression;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


public class DataFrameExpressionPanel extends JPanel  {
  public static final String REQUEST_NEW_EXPRESSION_ACTION = "requestNewExpression";
  public static final String REQUEST_REMOVE_EXPRESSION_ACTION = "requestRemoveExpression";
  
  private GridBagLayout gridBagLayout1 = new GridBagLayout();
  private JTextField txtExpressionText = new JTextField();
  private JButton btnChooseColor = new ColorButton();
  private TitledBorder border = new TitledBorder("Data frame expression");
  private OSMXDataFrameExpression expression;
  private PropertyChangeListener expressionListener;
  
  private List actionListeners = new ArrayList();
  
  private boolean textChanging = false;

  public DataFrameExpressionPanel() {
    expressionListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        Object obj = evt.getSource();
        if (obj instanceof DataFrameExpression) {
          if (OSMXDataFrameExpression.COLOR_PROPERTY.equals(evt.getPropertyName())) {
            Color newColorVal = (Color) evt.getNewValue();
            setColor(newColorVal);
          }
          else if (OSMXDataFrameExpression.EXPRESSION_TEXT_PROPERTY.equals(evt.getPropertyName())) {
            if (!textChanging) {
              txtExpressionText.setText((String) evt.getNewValue());
            }
          }
        }
      }
    };
    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }
  } // constructor()
  
  public DataFrameExpressionPanel(String title, DataFrameExpression expr) {
    this();
    setTitle(title);
    setExpression(expr);
  } // constructor(String, DataFrameExpression)
  
  public void setTitle(String title) {
    border.setTitle(title);
  } // method setTitle
  
  protected void setColor(Color c) {
    ((ColorButton) btnChooseColor).setColor(c);
    txtExpressionText.setBackground(c);
  } // method setColor(Color)
  
//  protected void setColor(String s) {
//    if (s == null) {
//      setColor(Color.WHITE);
//      return;
//    }
//    Color newColor = null;
//    try {
//      newColor = Color.decode(s);
//    } catch (NumberFormatException e) {
//      newColor = Color.getColor(s);
//    }
//    if (newColor == null) {
//      setColor(Color.WHITE);
//      return;
//    }
//    setColor(newColor);
//  } // method setColor(String)
  
  public void setExpression(DataFrameExpression expr) {
    if (expression != null) {
      // unhook the old expression
      expression.removePropertyChangeListener(expressionListener);
    }
    expression = (OSMXDataFrameExpression) expr;
    if (expression != null) {
      expression.addPropertyChangeListener(expressionListener);
      if (expression.isSetColor()) setColor(expression.getColor());
      else expression.setColor(((ColorButton) btnChooseColor).getColor());
      if (!textChanging) txtExpressionText.setText(expression.getExpressionText());
    } else {
      if (!textChanging) txtExpressionText.setText(null);
    }
  } // method setExpression
  
  public void addActionListener(ActionListener listener) {
    if (!actionListeners.contains(listener)) actionListeners.add(listener);
  }
  
  public void removeActionListener(ActionListener listener) {
    actionListeners.remove(listener);
  }
  
  public void notifyActionListeners(ActionEvent evt) {
    for (Iterator i = actionListeners.iterator(); i.hasNext();) {
      ((ActionListener) i.next()).actionPerformed(evt);
    }
  }
  
  public void setEnabled(boolean enabled) {
    super.setEnabled(enabled);
    btnChooseColor.setEnabled(enabled);
    txtExpressionText.setEnabled(enabled);
  } // method setEnabled

  private void jbInit() throws Exception {
    this.setBorder(border);
    this.setLayout(gridBagLayout1);
    btnChooseColor.setToolTipText("Expression highlight color");
    txtExpressionText.setToolTipText("Expression text");
    setColor(Color.WHITE);
    ((ColorButton) btnChooseColor).addColorListener(
      new ColorListener() {
        public void colorChanged(Color newColor) {
          if (expression != null && newColor != null) {
            expression.setColor(newColor);
          }
        }
      }
    );
    txtExpressionText.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (expression == null) {
            notifyActionListeners(
              new ActionEvent(DataFrameExpressionPanel.this, ActionEvent.ACTION_PERFORMED, REQUEST_NEW_EXPRESSION_ACTION)
            );
          }
          if (expression != null) expression.setExpressionText(txtExpressionText.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          textChanging = true;
          String exprText = null;
          if (expression != null) {
              expression.setExpressionText(txtExpressionText.getText());
              exprText = expression.getExpressionText();
          }
          if (exprText == null || exprText.length() == 0) {
            notifyActionListeners(
              new ActionEvent(DataFrameExpressionPanel.this, ActionEvent.ACTION_PERFORMED, REQUEST_REMOVE_EXPRESSION_ACTION)
            );
          }
          textChanging = false;
        }
        
        public void changedUpdate(DocumentEvent evt) { /* not implemented */ }
      }
    );
    this.add(txtExpressionText, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    this.add(btnChooseColor, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  }
}