/*
 * KeywordPhraseListModel.java
 *
 * Created on July 22, 2003, 12:27 AM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.KeywordListOwner;
import edu.byu.deg.osmx.OSMXKeywordPhraseType;
import edu.byu.deg.osmx.binding.KeywordPhraseType;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 *
 * @author  Alan
 */
public class KeywordPhraseListModel implements TableListModel {
  
  private java.util.List keywordPhraseList;
  private KeywordListOwner owner;
  public static final String[] columnNames = {"Name", "Keyword expression"};
  private List tableModelListeners;
  
  /** Creates a new instance of KeywordPhraseListModel */
  public KeywordPhraseListModel(KeywordListOwner klo) {
    tableModelListeners = new ArrayList();
    owner = klo;
    if (owner != null) {
      keywordPhraseList = owner.getKeywordPhrase();
    }
  }
  
  public void addItem() {
    if (owner == null) return;
/*    if (keywordPhraseList == null) {
      keywordPhraseList = new OSMXKeywordPhraseListType();
      owner.setKeywordPhraseList(keywordPhraseList);
    }*/
    OSMXKeywordPhraseType newPhrase = new OSMXKeywordPhraseType();
    int currListSize = keywordPhraseList.size();
    newPhrase.setHint("New phrase " + (currListSize + 1));
    keywordPhraseList.add(newPhrase);
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemAdded(newPhrase);
    }
  } // method addItem
  
  public void deleteItems(int[] selections) {
    if (keywordPhraseList == null || selections == null) return;
    List keywordPhrases = keywordPhraseList;
    for (int i = selections.length - 1; i >= 0; i--) {
      OSMXKeywordPhraseType kp = (OSMXKeywordPhraseType) keywordPhrases.get(selections[i]);
      keywordPhrases.remove(selections[i]);
      for (Iterator iter = tableModelListeners.iterator(); iter.hasNext();) {
        TableListModelListener tlml = (TableListModelListener) iter.next();
        tlml.itemRemoved(kp);
      }
    }
  } // method deleteItems
  
  public String[] getColumnNames() {
    return columnNames;
  } // method getColumnNames
  
  public Object getColumnValue(Object item, int columnIndex) {
    if (keywordPhraseList == null) return null;
    KeywordPhraseType phrase = (KeywordPhraseType) item;
    if (columnIndex <= 0) {
      return phrase.getHint();
    } else if (columnIndex >= 1) {
      if (!phrase.isSetKeywordExpression()) return null;
      return phrase.getKeywordExpression().getExpressionText();
    }
    return null;
  } // method getColumnValue
  
  public List getList() {
    return keywordPhraseList;
  } // method getList
  
  public void moveItemsDown(int[] selections) {
    if (keywordPhraseList == null || selections == null) return;
    List keywordPhrases = keywordPhraseList;
    int listSize = keywordPhrases.size();
    
    // if last row is selected, can't move selection down
    if (selections[selections.length - 1] == listSize) return;
    
    for (int i = selections.length - 1; i >= 0; i--) {
      Object o = keywordPhrases.remove(selections[i]);
      // add at one position later
      keywordPhrases.add(selections[i] + 1, o);
    }
    notifyReordered();
  } // method moveItemsDown
  
  public void moveItemsUp(int[] selections) {
    if (keywordPhraseList == null || selections == null) return;
    
    // uppermost row can't go higher
    if (selections[0] == 0) return; 
    
    List keywordPhrases = keywordPhraseList;
    for (int i = 0; i < selections.length; i++) {
      Object o = keywordPhrases.remove(selections[i]);
      // add at one position earlier
      keywordPhrases.add(selections[i] - 1, o);
    }
    notifyReordered();
  }
  
  protected void notifyReordered() {
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemsReordered();
    }
  }
  
  public void addTableListModelListener(TableListModelListener tlml) {
    if (tableModelListeners.contains(tlml)) return;
    tableModelListeners.add(tlml);
  }
  
  public void removeTableListModelListener(TableListModelListener tlml) {
    tableModelListeners.remove(tlml);
  }
  
 // method moveItemsUp
  
}
