/*
 * KeywordPhraseNode.java
 *
 * Created on July 19, 2003, 5:17 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXDataFrameExpression;
import edu.byu.deg.osmx.OSMXKeywordPhraseType;
import edu.byu.deg.osmx.binding.DataFrameExpression;
import edu.byu.deg.osmx.binding.KeywordPhraseType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;


/**
 *
 * @author  Alan
 */
public class KeywordPhraseNode extends AbstractDataFrameTreeNode {
  
  protected OSMXKeywordPhraseType keywordPhrase;
  protected PropertyChangeListener contentListener;
  
  /** Creates a new instance of KeywordPhraseNode */
  public KeywordPhraseNode(KeywordPhraseType kp, DefaultTreeModel model) {
    super(kp, model);
    contentListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        String propName = evt.getPropertyName();
        Object source = evt.getSource();
        if (source instanceof DataFrameExpression) {
          if (OSMXDataFrameExpression.EXPRESSION_TEXT_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(KeywordPhraseNode.this);
            }
          }
        } else if (source instanceof KeywordPhraseType) {
          if (OSMXKeywordPhraseType.HINT_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(KeywordPhraseNode.this);
            }
          }
        }
      }
    };
    setUserObject(kp);
  } // constructor
  
  public String toString() {
    String result = "<html><i>Null</i></html>";
    if (keywordPhrase.isSetHint() && keywordPhrase.getHint().length() > 0) {
      result = keywordPhrase.getHint();
      return result;
    }
    if (!keywordPhrase.isSetKeywordExpression()) {
      return result;
    }
    DataFrameExpression expr = keywordPhrase.getKeywordExpression();
    if (expr.getExpressionText() == null || expr.getExpressionText().length() == 0) {
      return result;
    }
    return expr.getExpressionText();
  } // method toString
  
  public KeywordPhraseType getKeywordPhrase() {
    return keywordPhrase;
  } // method getKeywordPhrase
  
  public void setUserObject(Object userObj) {
    if (keywordPhrase != null) {
      keywordPhrase.removePropertyChangeListener(contentListener);
      OSMXDataFrameExpression expr = (OSMXDataFrameExpression) keywordPhrase.getKeywordExpression();
      if (expr != null) {
        expr.removePropertyChangeListener(contentListener);
      }
    }
    if (userObj instanceof OSMXKeywordPhraseType) {
      super.setUserObject(userObj);
      keywordPhrase = (OSMXKeywordPhraseType) userObj;
      if (keywordPhrase != null) {
        keywordPhrase.addPropertyChangeListener(contentListener);
        OSMXDataFrameExpression expr = (OSMXDataFrameExpression) keywordPhrase.getKeywordExpression();
        if (expr != null) {
          expr.addPropertyChangeListener(contentListener);
        }
      }
    }
  } // method setUserObject
  
  public void nodeAdded(TreeNode child) { /* not implemented */ }
  
  public void nodeRemoved(TreeNode child) { /* not implemented */ }
  
}
