/*
 * KeywordPhrasePanel.java
 *
 * Created on July 29, 2003, 8:00 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXKeywordPhraseType;
import edu.byu.deg.osmx.binding.KeywordPhraseType;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
 *
 * @author  Alan
 */
public class KeywordPhrasePanel extends JPanel implements PropertyChangeListener {
  
  private DataFrameExpressionPanel pnlExpression = new DataFrameExpressionPanel();
  private BoxLayout mainLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
  private JPanel pnlHint = new JPanel();
  private BoxLayout hintLayout = new BoxLayout(pnlHint, BoxLayout.X_AXIS);
  private JPanel pnlConfidence = new JPanel();
  private BoxLayout confidenceLayout = new BoxLayout(pnlConfidence, BoxLayout.X_AXIS);
  private JCheckBox cbCaseSensitive = new JCheckBox();
  private JLabel lblHint = new JLabel();
  private JLabel lblConfidence = new JLabel();
  private JTextField txtHint = new JTextField();
  private JTextField txtConfidence = new JTextField();
  
  protected OSMXKeywordPhraseType keywordPhrase = null;
  private boolean textChanging = false;
  
  /** Creates a new instance of KeywordPhrasePanel */
  public KeywordPhrasePanel() {
    try {
      initGUI();
    } catch (Exception e) {
      e.printStackTrace();
    }
  } // constructor
  
  public KeywordPhrasePanel(KeywordPhraseType kp) {
    this();
    setKeywordPhrase(kp);
  } // constructor(KeywordPhraseType)
  
  public void setKeywordPhrase(KeywordPhraseType kp) {
    if (keywordPhrase != null) {
      keywordPhrase.removePropertyChangeListener(this);
    }
    keywordPhrase = (OSMXKeywordPhraseType) kp;
    boolean hasPhrase = (keywordPhrase != null);
    if (hasPhrase) {
      keywordPhrase.addPropertyChangeListener(this);
      pnlExpression.setExpression(keywordPhrase.getKeywordExpression());
      txtHint.setText(keywordPhrase.getHint());
      txtConfidence.setText(keywordPhrase.getConfidenceTag());
      cbCaseSensitive.setSelected(keywordPhrase.isCaseSensitive());
    }
    pnlExpression.setEnabled(hasPhrase);
    lblConfidence.setEnabled(hasPhrase);
    lblHint.setEnabled(hasPhrase);
    txtConfidence.setEnabled(hasPhrase);
    txtHint.setEnabled(hasPhrase);
    cbCaseSensitive.setEnabled(hasPhrase);
  } // method setKeywordPhrase
  
  protected void initGUI() throws Exception {
    this.setLayout(mainLayout);
    
    pnlHint.setLayout(hintLayout);
    pnlHint.add(lblHint);
    pnlHint.add(Box.createRigidArea(new Dimension(4, 1)));
    pnlHint.add(txtHint);
    lblHint.setEnabled(false);
    txtHint.setEnabled(false);
    lblHint.setText("Hint:");
    lblHint.setToolTipText("This gives your expression a name.");
    pnlHint.setMaximumSize(new Dimension(4000, 80));
    pnlHint.setAlignmentX(Component.LEFT_ALIGNMENT);
    txtHint.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          keywordPhrase.setHint(txtHint.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* not implemented */ }
      }
    );
    this.add(pnlHint);
    
    cbCaseSensitive.setEnabled(false);
    cbCaseSensitive.setText("Case sensitive");
    cbCaseSensitive.setAlignmentX(Component.LEFT_ALIGNMENT);
    cbCaseSensitive.setHorizontalTextPosition(AbstractButton.LEADING);
    cbCaseSensitive.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          if (keywordPhrase == null) return;
          keywordPhrase.setCaseSensitive(cbCaseSensitive.isSelected());
        }
      }
    );
    this.add(cbCaseSensitive);
    
    pnlConfidence.setLayout(confidenceLayout);
    pnlConfidence.add(lblConfidence);
    pnlConfidence.add(Box.createRigidArea(new Dimension(4, 1)));
    pnlConfidence.add(txtConfidence);
    pnlConfidence.add(Box.createHorizontalGlue());
    lblConfidence.setEnabled(false);
    txtConfidence.setEnabled(false);
    txtConfidence.setMinimumSize(new Dimension(40, 8));
    lblConfidence.setText("Confidence tag:");
    lblConfidence.setToolTipText("This lets you assign a numeric confidence value to the expression.");
    pnlConfidence.setMaximumSize(new Dimension(200, 80));
    pnlConfidence.setAlignmentX(Component.LEFT_ALIGNMENT);
    txtConfidence.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          keywordPhrase.setConfidenceTag(txtConfidence.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* not implemented */ }
      }
    );
    this.add(pnlConfidence);
    
    this.add(Box.createRigidArea(new Dimension(0, 15)));
    
    pnlExpression.setTitle("Keyword phrase expression");
    pnlExpression.setExpression(null);
    pnlExpression.setEnabled(false);
    pnlExpression.setMaximumSize(new Dimension(4000, 100));
    pnlExpression.setAlignmentX(Component.LEFT_ALIGNMENT);
    this.add(pnlExpression);
    
    this.add(Box.createVerticalGlue());
  } // method initGUI
  
  public void propertyChange(PropertyChangeEvent evt) {
    if (!(evt.getSource() instanceof KeywordPhraseType)) return;
    String propertyName = evt.getPropertyName();
    if (OSMXKeywordPhraseType.HINT_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtHint.setText(keywordPhrase.getHint());
      }
    } else if (OSMXKeywordPhraseType.CONFIDENCE_TAG_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtConfidence.setText(keywordPhrase.getConfidenceTag());
      }
    } else if (OSMXKeywordPhraseType.CASE_SENSITIVE_PROPERTY.equals(propertyName)) {
      cbCaseSensitive.setSelected(((Boolean) evt.getNewValue()).booleanValue());
    }
  } // method propertyChange
  
}
