/*
 * MethodListModel.java
 *
 * Created on July 23, 2003, 10:54 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXMethodType;
import edu.byu.deg.osmx.binding.DataFrameType;
import edu.byu.deg.osmx.binding.MethodType;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 *
 * @author  Alan
 */
public class MethodListModel implements TableListModel {
  
  private List methodList;
  private DataFrameType dataFrame;
  public static final String[] columnNames = {"Method name", "Language", "Signature"};
  private List tableModelListeners;
  
  /** Creates a new instance of MethodListModel */
  public MethodListModel(DataFrameType df) {
    tableModelListeners = new ArrayList();
    dataFrame = df;
    if (df != null) {
      methodList = df.getMethod();
    }
  }
  
  public void addItem() {
    if (dataFrame == null) return;
    OSMXMethodType newMethod = new OSMXMethodType();
    int currListSize = methodList.size();
    newMethod.setHint("New method " + (currListSize + 1));
    newMethod.setReturnType(null);
    methodList.add(newMethod);
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemAdded(newMethod);
    }
  }
  
  public void deleteItems(int[] selections) {
    if (methodList == null || selections == null) return;
    for (int i = selections.length - 1; i >= 0; i--) {
      OSMXMethodType m = (OSMXMethodType) methodList.get(selections[i]);
      methodList.remove(selections[i]);
      for (Iterator iter = tableModelListeners.iterator(); iter.hasNext();) {
        TableListModelListener tlml = (TableListModelListener) iter.next();
        tlml.itemRemoved(m);
      }
    }
  }
  
  public String[] getColumnNames() {
    return columnNames;
  }
  
  public Object getColumnValue(Object item, int columnIndex) {
    if (methodList == null) return null;
    MethodType method = (MethodType) item;
    if (columnIndex <= 0) {
      if (method != null) return method.getHint();
    } else if (columnIndex == 1) {
      String lang = method.getLanguage();
      if (lang == null || lang.length() == 0) {
        lang = "<html><i>Unspecified</i></html>";
      }
      return lang;
    } else if (columnIndex >= 2) {
      if (method != null) return ((OSMXMethodType) method).getSignature();
    }
    return null;
  }
  
  public List getList() {
    return methodList;
  }
  
  public void moveItemsDown(int[] selections) {
    if (methodList == null || selections == null) return;
    int listSize = methodList.size();
    
    // if last row is selected, can't move selection down
    if (selections[selections.length - 1] == listSize) return;
    
    for (int i = selections.length - 1; i >= 0; i--) {
      Object o = methodList.remove(selections[i]);
      // add at one position later
      methodList.add(selections[i] + 1, o);
    }
    notifyReordered();
  }
  
  public void moveItemsUp(int[] selections) {
    if (methodList == null || selections == null) return;
    
    // uppermost row can't go higher
    if (selections[0] == 0) return; 
    
    for (int i = 0; i < selections.length; i++) {
      Object o = methodList.remove(selections[i]);
      // add at one position earlier
      methodList.add(selections[i] - 1, o);
    }
    notifyReordered();
  }
  
  protected void notifyReordered() {
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemsReordered();
    }
  }
  
  public void addTableListModelListener(TableListModelListener tlml) {
    if (tableModelListeners.contains(tlml)) return;
    tableModelListeners.add(tlml);
  }
  
  public void removeTableListModelListener(TableListModelListener tlml) {
    tableModelListeners.remove(tlml);
  }
  
}
