/*
 * MethodNode.java
 *
 * Created on July 19, 2003, 7:00 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXMethodType;
import edu.byu.deg.osmx.binding.MethodType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;


/**
 *
 * @author  Alan
 */
public class MethodNode extends AbstractDataFrameTreeNode {
  
  protected OSMXMethodType method;
  protected PropertyChangeListener contentListener;
  
  /** Creates a new instance of MethodNode */
  public MethodNode(MethodType m, DefaultTreeModel model) {
    super(m, model);
    contentListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        String propName = evt.getPropertyName();
        Object source = evt.getSource();
        if (source instanceof MethodType) {
          if (OSMXMethodType.HINT_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(MethodNode.this);
            }
          } else if (OSMXMethodType.NAME_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(MethodNode.this);
            }
          }
        }
      }
    };
    method = (OSMXMethodType) m;
    if (method != null) {
      this.add(new ParameterListNode(method.getParameter(), treeModel));
      this.add(new KeywordPhraseListNode(method.getKeywordPhrase(), treeModel));
    }
    setUserObject(m);
  } // constructor
  
  public String toString() {
    String result = "<html><i>Null</i></html>";
    if (method == null) return result;
    if (method.getHint() != null && method.getHint().length() > 0) result = method.getHint();
    else result = method.getSignature();
    return result;
  } // method toString
  
  public MethodType getMethod() {
    return method;
  } // method getMethod
  
  public void setUserObject(Object o) {
    if (method != null) {
      method.removePropertyChangeListener(contentListener);
    }
    if (userObject instanceof OSMXMethodType) {
      super.setUserObject(userObject);
      method = (OSMXMethodType) userObject;
      if (method != null) {
        method.addPropertyChangeListener(contentListener);
      }
    }
  }
  
  public void nodeAdded(TreeNode child) { /* do nothing */ }
  
  public void nodeRemoved(TreeNode child) { /* do nothing */ }
  
}
