/*
 * MethodPanel.java
 *
 * Created on July 31, 2003, 11:19 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXMethodType;
import edu.byu.deg.osmx.OSMXTypeSpecification;
import edu.byu.deg.osmx.binding.MethodType;
import edu.byu.deg.osmx.binding.TypeSpecification;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
 *
 * @author  Alan
 */
public class MethodPanel extends JPanel implements PropertyChangeListener {
  private JPanel pnlHint = new JPanel();
  private JPanel pnlName = new JPanel();
  private JPanel pnlLanguage = new JPanel();
  private JPanel pnlHasReturnType = new JPanel();
  private JPanel pnlReturnType = new TypeSpecificationPanel();
  
  private JLabel lblHint = new JLabel();
  private JLabel lblName = new JLabel();
  private JLabel lblLanguage = new JLabel();
  
  private JTextField txtHint = new JTextField();
  private JTextField txtName = new JTextField();
  private JTextField txtLanguage = new JTextField();
  
  private JCheckBox cbReturnType = new JCheckBox();
  
  private BoxLayout mainLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
  private BoxLayout hintLayout = new BoxLayout(pnlHint, BoxLayout.X_AXIS);
  private BoxLayout nameLayout = new BoxLayout(pnlName, BoxLayout.X_AXIS);
  private BoxLayout langLayout = new BoxLayout(pnlLanguage, BoxLayout.X_AXIS);
  private BoxLayout hasReturnTypeLayout = new BoxLayout(pnlHasReturnType, BoxLayout.Y_AXIS);
  
  private TitledBorder hasReturnTypeBorder = new TitledBorder("");
  
  protected OSMXMethodType method = null;
  protected boolean textChanging = false;
  
  /** Creates a new instance of MethodPanel */
  public MethodPanel() {
    try {
      initGUI();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  public MethodPanel(MethodType mt) {
    this();
    setMethod(mt);
  }
  
  public void setMethod(MethodType mt) {
    if (method != null) {
      method.removePropertyChangeListener(this);
    }
    method = (OSMXMethodType) mt;
    boolean hasMethod = (method != null);
    if (hasMethod) {
      method.addPropertyChangeListener(this);
      
      ((TypeSpecificationPanel) pnlReturnType).setTypeSpecification(method.getReturnType());
      
      txtHint.setText(method.getHint());
      txtName.setText(method.getName());
      txtLanguage.setText(method.getLanguage());
      cbReturnType.setSelected(method.isSetReturnType());
    }
    lblName.setEnabled(hasMethod);
    lblHint.setEnabled(hasMethod);
    lblLanguage.setEnabled(hasMethod);
    txtName.setEnabled(hasMethod);
    txtHint.setEnabled(hasMethod);
    txtLanguage.setEnabled(hasMethod);
    cbReturnType.setEnabled(hasMethod);
  }
  
  protected void initGUI() throws Exception {
    pnlHint.setLayout(hintLayout);
    pnlHint.setMaximumSize(new Dimension(4000, 80));
    pnlHint.setAlignmentX(Component.LEFT_ALIGNMENT);
    lblHint.setText("Hint:");
    pnlHint.add(lblHint);
    pnlHint.add(Box.createRigidArea(new Dimension(4, 0)));
    txtHint.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (method != null) method.setHint(txtHint.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    pnlHint.add(txtHint);
    
    pnlName.setLayout(nameLayout);
    pnlName.setMaximumSize(new Dimension(4000, 80));
    pnlName.setAlignmentX(Component.LEFT_ALIGNMENT);
    lblName.setText("Method name:");
    pnlName.add(lblName);
    pnlName.add(Box.createRigidArea(new Dimension(4, 0)));
    txtName.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (method != null) method.setName(txtName.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    pnlName.add(txtName);
    
    pnlLanguage.setLayout(langLayout);
    pnlLanguage.setMaximumSize(new Dimension(4000, 80));
    pnlLanguage.setAlignmentX(Component.LEFT_ALIGNMENT);
    lblLanguage.setText("Language:");
    pnlLanguage.add(lblLanguage);
    pnlLanguage.add(Box.createRigidArea(new Dimension(4, 0)));
    txtLanguage.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (method != null) method.setLanguage(txtLanguage.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    pnlLanguage.add(txtLanguage);
    
    pnlHasReturnType.setLayout(hasReturnTypeLayout);
    pnlHasReturnType.setAlignmentX(Component.LEFT_ALIGNMENT);
    cbReturnType.setText("Method has return type");
    cbReturnType.setSelected(false);
    cbReturnType.setAlignmentX(Component.LEFT_ALIGNMENT);
    cbReturnType.setHorizontalTextPosition(AbstractButton.LEADING);
    cbReturnType.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          if (method == null) return;
          if (cbReturnType.isSelected()) {
            method.restoreReturnType();
            if (!method.isSetReturnType()) {
              TypeSpecification ts = new OSMXTypeSpecification();
              method.setReturnType(ts);
              ((TypeSpecificationPanel) pnlReturnType).setTypeSpecification(ts);
            }
          } else {
            method.suppressReturnType();
          }
        }
      }
    );
    pnlHasReturnType.add(cbReturnType);
    hasReturnTypeBorder.setTitle("Return type");
    pnlReturnType.setBorder(hasReturnTypeBorder);
    ((TypeSpecificationPanel) pnlReturnType).setTypeSpecification(null);
    pnlReturnType.setAlignmentX(Component.LEFT_ALIGNMENT);
    pnlHasReturnType.add(pnlReturnType);
    
    this.setLayout(mainLayout);
    this.add(pnlHint);
    this.add(pnlName);
    this.add(pnlLanguage);
    this.add(Box.createRigidArea(new Dimension(0, 15)));
    this.add(pnlHasReturnType);
    this.add(Box.createVerticalGlue());
    this.add(Box.createRigidArea(new Dimension(1, 1)));
  }
  
  public void propertyChange(PropertyChangeEvent evt) {
    if (!(evt.getSource() instanceof MethodType)) return;
    String propertyName = evt.getPropertyName();
    if (OSMXMethodType.HINT_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtHint.setText(method.getHint());
      }
    } else if (OSMXMethodType.NAME_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtName.setText(method.getName());
      }
    } else if (OSMXMethodType.LANGUAGE_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtLanguage.setText(method.getLanguage());
      }
    }
  }
  
}
