/*
 * ObjectSetNode.java
 *
 * Created on July 15, 2003, 2:59 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXObjectSetType;
import edu.byu.deg.osmx.binding.DataFrameType;
import edu.byu.deg.osmx.binding.ObjectSetType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;


/**
 *
 * @author  aew2
 */
public class ObjectSetNode extends AbstractDataFrameTreeNode
implements PropertyChangeListener {
  
  protected OSMXObjectSetType objectSet;
  protected ValuePhraseListNode valuePhrasesNode = null;
  protected KeywordPhraseListNode keywordPhrasesNode = null;
  protected MethodListNode methodsNode = null;
  
  /** Creates a new instance of ObjectSetNode */
  public ObjectSetNode(ObjectSetType objSet, DefaultTreeModel model) {
    super(objSet, model);
    setUserObject(objSet);
  }
  
  public String toString() {
    if (objectSet == null) return "<html><i>Null object set</i></html>";
    String result = objectSet.getPrimaryName();
    return result;
  } // method toString
  
  public ObjectSetType getObjectSet() {
    return objectSet;
  } // method getObjectSet
  
  public void setUserObject(Object userObject) {
    if (objectSet != null) {
      objectSet.removePropertyChangeListener(this);
    }
    if (userObject instanceof OSMXObjectSetType) {
      super.setUserObject(userObject);
      objectSet = (OSMXObjectSetType) userObject;
      if (objectSet != null) {
        objectSet.addPropertyChangeListener(this);
        if (objectSet.isSetDataFrame()) {
          DataFrameType dataFrame = objectSet.getDataFrame();
          valuePhrasesNode = new ValuePhraseListNode(dataFrame.getValuePhrase(), getModel());
          keywordPhrasesNode = new KeywordPhraseListNode(dataFrame.getKeywordPhrase(), getModel());          
          methodsNode = new MethodListNode(dataFrame.getMethod(), getModel());
          this.add(valuePhrasesNode);
          this.add(keywordPhrasesNode);          
          this.add(methodsNode);
          getModel().nodeStructureChanged(this);
        }
      }
    }
  }
  
  public void nodeAdded(TreeNode child) { /* do nothing */ }
  
  public void nodeRemoved(TreeNode child) { /* do nothing */ }
  
  public void propertyChange(PropertyChangeEvent evt) {
    Object source = evt.getSource();
    if (source instanceof OSMXObjectSetType) {
      OSMXObjectSetType objSet = (OSMXObjectSetType) source;
      if (OSMXObjectSetType.DATA_FRAME_PROPERTY.equals(evt.getPropertyName())) {
        if (objSet.isSetDataFrame()) {
          DataFrameType dataFrame = objSet.getDataFrame();
          valuePhrasesNode = new ValuePhraseListNode(dataFrame.getValuePhrase(), getModel());
          keywordPhrasesNode = new KeywordPhraseListNode(dataFrame.getKeywordPhrase(), getModel());
          methodsNode = new MethodListNode(dataFrame.getMethod(), getModel());
          this.add(valuePhrasesNode);
          this.add(keywordPhrasesNode);
          this.add(methodsNode);
        } else {
          this.remove(valuePhrasesNode);
          this.remove(keywordPhrasesNode);
          this.remove(methodsNode);
          valuePhrasesNode = null;
          keywordPhrasesNode = null;
          methodsNode = null;
        }
        getModel().nodeStructureChanged(this);
      }
    }
  }
  
}
