/*
 * ParameterListModel.java
 *
 * Created on July 31, 2003, 9:53 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXParameterType;
import edu.byu.deg.osmx.binding.MethodType;
import edu.byu.deg.osmx.binding.ParameterType;
import edu.byu.deg.osmx.binding.TypeSpecification;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author  Alan
 */
public class ParameterListModel implements TableListModel {
  
  private List parameterList;
  private MethodType method;
  public static final String[] columnNames = {"Decorator", "Name", "Type"};
  private List tableModelListeners;
  
  /** Creates a new instance of MethodListModel */
  public ParameterListModel(MethodType m) {
    tableModelListeners = new ArrayList();
    method = m;
    if (m != null) {
      parameterList = m.getParameter();
    }
  }
  
  public void addItem() {
    if (method == null) return;
    OSMXParameterType newParam = new OSMXParameterType();
    int currListSize = parameterList.size();
    newParam.setName("param" + currListSize);
    parameterList.add(newParam);
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemAdded(newParam);
    }
  }
  
  public void deleteItems(int[] selections) {
    if (parameterList == null || selections == null) return;
    for (int i = selections.length - 1; i >= 0; i--) {
      OSMXParameterType p = (OSMXParameterType) parameterList.get(selections[i]);
      parameterList.remove(selections[i]);
      for (Iterator iter = tableModelListeners.iterator(); iter.hasNext();) {
        TableListModelListener tlml = (TableListModelListener) iter.next();
        tlml.itemRemoved(p);
      }
    }
  }
  
  public String[] getColumnNames() {
    return columnNames;
  }
  
  public Object getColumnValue(Object item, int columnIndex) {
    if (parameterList == null) return null;
    ParameterType param = (ParameterType) item;
    if (columnIndex <= 0) {
      return param.getDecorator();
    } else if (columnIndex == 1) {
      String lang = param.getName();
      if (lang == null || lang.length() == 0) {
        lang = "<html><i>Unspecified</i></html>";
      }
      return lang;
    } else if (columnIndex >= 2) {
      // create the method signature and return it
      TypeSpecification ts = param.getType();
      if (ts == null) return "<html><i>Unspecified</i></html>";
      else return ts.toString();
    }
    return null;
  }
  
  public List getList() {
    return parameterList;
  }
  
  public void moveItemsDown(int[] selections) {
    if (parameterList == null || selections == null) return;
    int listSize = parameterList.size();
    
    // if last row is selected, can't move selection down
    if (selections[selections.length - 1] == listSize) return;
    
    for (int i = selections.length - 1; i >= 0; i--) {
      Object o = parameterList.remove(selections[i]);
      // add at one position later
      parameterList.add(selections[i] + 1, o);
    }
    notifyReordered();
  }
  
  public void moveItemsUp(int[] selections) {
    if (parameterList == null || selections == null) return;
    
    // uppermost row can't go higher
    if (selections[0] == 0) return; 
    
    for (int i = 0; i < selections.length; i++) {
      Object o = parameterList.remove(selections[i]);
      // add at one position earlier
      parameterList.add(selections[i] - 1, o);
    }
    notifyReordered();
  }
  
  protected void notifyReordered() {
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemsReordered();
    }
  }
  
  public void addTableListModelListener(TableListModelListener tlml) {
    if (tableModelListeners.contains(tlml)) return;
    tableModelListeners.add(tlml);
  }
  
  public void removeTableListModelListener(TableListModelListener tlml) {
    tableModelListeners.remove(tlml);
  }
  
}
