/*
 * ParameterPanel.java
 *
 * Created on July 31, 2003, 11:20 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXParameterType;
import edu.byu.deg.osmx.binding.ParameterType;
import java.awt.Component;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
 *
 * @author  Alan
 */
public class ParameterPanel extends JPanel implements PropertyChangeListener {
  
  private JPanel pnlName = new JPanel();
  private JPanel pnlDecorator = new JPanel();
  private JPanel pnlType = new TypeSpecificationPanel();
  
  private JLabel lblName = new JLabel();
  private JLabel lblDecorator = new JLabel();
  
  private JTextField txtName = new JTextField();
  private JTextField txtDecorator = new JTextField();
  
  private BoxLayout mainLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
  private BoxLayout nameLayout = new BoxLayout(pnlName, BoxLayout.X_AXIS);
  private BoxLayout decoratorLayout = new BoxLayout(pnlDecorator, BoxLayout.X_AXIS);
  
  private TitledBorder typeBorder = new TitledBorder("");
  
  protected OSMXParameterType parameter;
  protected boolean textChanging = false;
  
  /** Creates a new instance of ParameterPanel */
  public ParameterPanel() {
    try {
      initGUI();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  public ParameterPanel(ParameterType param) {
    this();
    setParameter(param);
  }
  
  public void setParameter(ParameterType param) {
    if (parameter != null) {
      parameter.removePropertyChangeListener(this);
    }
    parameter = (OSMXParameterType) param;
    boolean hasParam = (parameter != null);
    if (hasParam) {
      parameter.addPropertyChangeListener(this);
      
      ((TypeSpecificationPanel) pnlType).setTypeSpecification(parameter.getType());
      
      txtName.setText(parameter.getName());
      txtDecorator.setText(parameter.getDecorator());
    } else {
      ((TypeSpecificationPanel) pnlType).setTypeSpecification(null);
      txtName.setText("");
      txtDecorator.setText("");
    }
    lblName.setEnabled(hasParam);
    lblDecorator.setEnabled(hasParam);
    txtName.setEnabled(hasParam);
    txtDecorator.setEnabled(hasParam);
  }
  
  protected void initGUI() throws Exception {
    pnlName.setLayout(nameLayout);
    pnlName.setMaximumSize(new Dimension(4000, 80));
    pnlName.setAlignmentX(Component.LEFT_ALIGNMENT);
    lblName.setText("Parameter name:");
    pnlName.add(lblName);
    pnlName.add(Box.createRigidArea(new Dimension(4, 0)));
    txtName.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (parameter != null) parameter.setName(txtName.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    pnlName.add(txtName);
    
    pnlDecorator.setLayout(decoratorLayout);
    pnlDecorator.setMaximumSize(new Dimension(4000, 80));
    pnlDecorator.setAlignmentX(Component.LEFT_ALIGNMENT);
    lblDecorator.setText("Decorator name:");
    pnlDecorator.add(lblDecorator);
    pnlDecorator.add(Box.createRigidArea(new Dimension(4, 0)));
    txtDecorator.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          if (parameter != null) parameter.setDecorator(txtDecorator.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    pnlDecorator.add(txtDecorator);
    
    typeBorder.setTitle("Parameter type");
    pnlType.setBorder(typeBorder);
    ((TypeSpecificationPanel) pnlType).setTypeSpecification(null);
    pnlType.setAlignmentX(Component.LEFT_ALIGNMENT);
    
    this.setLayout(mainLayout);
    this.add(pnlName);
    this.add(pnlDecorator);
    this.add(Box.createRigidArea(new Dimension(0, 15)));
    this.add(pnlType);
    this.add(Box.createVerticalGlue());
  }
  
  public void propertyChange(PropertyChangeEvent evt) {
    if (!(evt.getSource() instanceof ParameterType)) return;
    String propertyName = evt.getPropertyName();
    if (OSMXParameterType.NAME_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtName.setText(parameter.getName());
      }
    } else if (OSMXParameterType.DECORATOR_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtDecorator.setText(parameter.getDecorator());
      }
    }
  }
  
}
