package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXDataTypeType;
import edu.byu.deg.osmx.OSMXDocument;
import edu.byu.deg.osmx.OSMXOSMType;
import edu.byu.deg.osmx.OSMXObjectSetReferenceType;
import edu.byu.deg.osmx.OSMXTypeSpecification;
import edu.byu.deg.osmx.binding.DataTypeType;
import edu.byu.deg.osmx.binding.ObjectSetReferenceType;
import edu.byu.deg.osmx.binding.ObjectSetType;
import edu.byu.deg.osmx.binding.TypeSpecification;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


public class TypeSpecificationPanel extends JPanel
implements PropertyChangeListener {
  private BorderLayout outerBorder = new BorderLayout();
  private JPanel pnlInner = new JPanel();
  private GridBagLayout gblMain = new GridBagLayout();
  private JRadioButton rdoObjectSetRef = new JRadioButton();
  private JPanel pnlObjectSetName = new JPanel();
  private JLabel jLabel1 = new JLabel();
  private JComboBox cboObjectSetRef = new JComboBox();
  private JRadioButton rdoDataType = new JRadioButton();
  private JPanel pnlDataType = new JPanel();
  private GridBagLayout gblDataType = new GridBagLayout();
  private JLabel jLabel3 = new JLabel();
  private JTextField txtTypeName = new JTextField();
  private JLabel jLabel4 = new JLabel();
  private JTextField txtUnitOfMeasure = new JTextField();
  private GridBagLayout gridBagLayout1 = new GridBagLayout();
  
  protected OSMXTypeSpecification typeSpec;
  private boolean processingUIEvent = false;
  private boolean listenForSelectionEvents = true;
  
  public TypeSpecificationPanel() {
    try {
      initGUI();
    } catch(Exception e) {
      e.printStackTrace();
    }
  } // constructor()
  
  public TypeSpecificationPanel(TypeSpecification ts) {
    this();
    setTypeSpecification(ts);
  } // constructor(TypeSpecification)
  
  public void setTypeSpecification(TypeSpecification ts) {
    if (typeSpec == ts) return;
    if (typeSpec != null) {
      typeSpec.removePropertyChangeListener(this);
    }
    typeSpec = (OSMXTypeSpecification) ts;
    if (typeSpec != null) {
      typeSpec.addPropertyChangeListener(this);
    }
    updateFromModel();
  } // method setTypeSpecification
  
  private void initGUI() throws Exception {
    this.setLayout(outerBorder);
    pnlInner.setLayout(gblMain);
    rdoObjectSetRef.setText("Object set");
    pnlObjectSetName.setLayout(gridBagLayout1);
    jLabel1.setText("Object set name:");
    rdoDataType.setText("Data type");
    pnlDataType.setLayout(gblDataType);
    jLabel3.setText("Type name:");
    jLabel4.setText("Unit of measure:");
    cboObjectSetRef.setRenderer(new ObjectSetListCellRenderer());
    pnlInner.add(rdoObjectSetRef, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    pnlInner.add(pnlObjectSetName, new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 0, 4), 0, 0));
    pnlInner.add(rdoDataType, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    pnlDataType.add(jLabel3, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    pnlDataType.add(txtTypeName, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 0, 4), 0, 0));
    pnlDataType.add(jLabel4, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    pnlDataType.add(txtUnitOfMeasure, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 0, 4), 0, 0));
    pnlInner.add(pnlDataType, new GridBagConstraints(0, 3, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 20, 0, 0), 0, 0));
    pnlObjectSetName.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    pnlObjectSetName.add(cboObjectSetRef, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 0, 0), -1000, 0));
    this.add(pnlInner, BorderLayout.NORTH);
    
    cboObjectSetRef.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          processingUIEvent = true;
          ObjectSetType objSet = (ObjectSetType) cboObjectSetRef.getSelectedItem();
          if (objSet != null && listenForSelectionEvents) {
            typeSpec.setObjectSetRefId(objSet.getId());
          }
          processingUIEvent = false;
        }
      }
    );
    txtTypeName.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          processingUIEvent = true;
          if (typeSpec != null) typeSpec.setDataTypeName(txtTypeName.getText());
          processingUIEvent = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    txtUnitOfMeasure.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          processingUIEvent = true;
          if (typeSpec != null) typeSpec.setDataTypeUnits(txtUnitOfMeasure.getText());
          processingUIEvent = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    
    ButtonGroup radioGroup = new ButtonGroup();
    radioGroup.add(rdoDataType);
    radioGroup.add(rdoObjectSetRef);
    
    updateTextFieldsEnabled();
    
    rdoDataType.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (rdoDataType.isSelected()) {
            if (typeSpec != null) {
              typeSpec.switchType(OSMXTypeSpecification.DATA_TYPE);
            }
            updateTextFieldsEnabled();
          }
        }
      }
    );
    rdoObjectSetRef.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (rdoObjectSetRef.isSelected()) {
            if (typeSpec != null) {
              typeSpec.switchType(OSMXTypeSpecification.OBJECT_SET_REF);
            }
            updateTextFieldsEnabled();
          }
        }
      }
    );
  } // method jbInit
  
  public void setEnabled(boolean enabled) {
    super.setEnabled(enabled);
    rdoDataType.setEnabled(enabled);
    rdoObjectSetRef.setEnabled(enabled);
    updateTextFieldsEnabled();
  } // method setEnabled
  
  public void updateTextFieldsEnabled() {
    cboObjectSetRef.setEnabled(false);
    txtTypeName.setEnabled(false);
    txtTypeName.setEditable(false);
    txtUnitOfMeasure.setEnabled(false);
    txtUnitOfMeasure.setEditable(false);
    jLabel1.setEnabled(false);
    jLabel3.setEnabled(false);
    jLabel4.setEnabled(false);
    if (typeSpec != null && this.isEnabled()) {
      if (typeSpec.isSetDataType()) {
        txtTypeName.setEnabled(true);
        txtTypeName.setEditable(true);
        txtUnitOfMeasure.setEnabled(true);
        txtUnitOfMeasure.setEditable(true);
        jLabel3.setEnabled(true);
        jLabel4.setEnabled(true);
      }
      if (typeSpec.isSetObjectSetReference()) {
        cboObjectSetRef.setEnabled(true);
        jLabel1.setEnabled(true);
      }
    }
  } // method updateTextFieldsEnabled
  
  public void updateFromModel() {
    cboObjectSetRef.removeAllItems();
    if (typeSpec != null) {
      OSMXDocument parentDoc = typeSpec.getParentDocument();
      OSMXOSMType osm = (OSMXOSMType) parentDoc.getModelRoot();
      List objSets = osm.getObjectSets();
      listenForSelectionEvents = false;
      for (Iterator i = objSets.iterator(); i.hasNext();) {
        cboObjectSetRef.addItem(i.next());
      }
      listenForSelectionEvents = true;
      boolean wasDataType = typeSpec.isSetDataType();
      typeSpec.switchType(OSMXTypeSpecification.DATA_TYPE);
      if (typeSpec.isSetDataType()) {
        txtTypeName.setText(typeSpec.getDataType().getTypeName());
        txtUnitOfMeasure.setText(typeSpec.getDataType().getUnitOfMeasure());
      }
      typeSpec.switchType(OSMXTypeSpecification.OBJECT_SET_REF);
      if (typeSpec.isSetObjectSetReference()) {
        Object objSet = osm.findObjectSet(typeSpec.getObjectSetReference().getIdref());
        cboObjectSetRef.setSelectedItem(objSet);
      }
      if (wasDataType) typeSpec.switchType(OSMXTypeSpecification.DATA_TYPE);
    } else {
      txtTypeName.setText(null);
      txtUnitOfMeasure.setText(null);
    }
    updateTextFieldsEnabled();
  } // method updateFromModel
  
  public void propertyChange(PropertyChangeEvent evt) {
    String propertyName = evt.getPropertyName();
    Object source = evt.getSource();
    if (source instanceof ObjectSetReferenceType) {
      if (OSMXObjectSetReferenceType.REF_PROPERTY.equals(propertyName)) {
        if (!processingUIEvent) {
          //txtObjectSetRef.setText(typeSpec.getObjectSetRefId());
          OSMXOSMType osm = (OSMXOSMType) typeSpec.getParentDocument().getModelRoot();
          cboObjectSetRef.setSelectedItem(osm.findObjectSet(typeSpec.getObjectSetRefId()));
        }
      }
    } else if (source instanceof DataTypeType) {
      if (OSMXDataTypeType.NAME_PROPERTY.equals(propertyName)) {
        if (!processingUIEvent) {
          txtTypeName.setText(typeSpec.getDataTypeName());
        }
      } else if (OSMXDataTypeType.UNIT_PROPERTY.equals(propertyName)) {
        if (!processingUIEvent) {
          txtUnitOfMeasure.setText(typeSpec.getDataTypeUnits());
        }
      }
    } else if (source instanceof TypeSpecification && source == typeSpec) {
      if (OSMXTypeSpecification.TYPE_CHANGE_PROPERTY.equals(propertyName)) {
        if (typeSpec.isSetDataType()) rdoDataType.setSelected(true);
        else if (typeSpec.isSetObjectSetReference()) rdoObjectSetRef.setSelected(true);
      }
    }
  }
  
  
}