/*
 * ValuePhraseListModel.java
 *
 * Created on July 23, 2003, 6:25 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXValuePhraseType;
import edu.byu.deg.osmx.binding.DataFrameType;
import edu.byu.deg.osmx.binding.ValuePhraseType;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 *
 * @author  Alan
 */
public class ValuePhraseListModel implements TableListModel {
  
  private List valuePhraseList;
  private DataFrameType dataFrame;
  public static final String[] columnNames = {"Name", "Value expression"};
  private List tableModelListeners;
  
  /** Creates a new instance of ValuePhraseListModel */
  public ValuePhraseListModel(DataFrameType df) {
    tableModelListeners = new ArrayList();
    dataFrame = df;
    if (df != null) {
      valuePhraseList = df.getValuePhrase();
    }
  }
  
  public void addItem() {
    if (dataFrame == null) return;
    OSMXValuePhraseType newPhrase = new OSMXValuePhraseType();
    int currListSize = valuePhraseList.size();
    newPhrase.setHint("New phrase " + (currListSize + 1));
    valuePhraseList.add(newPhrase);
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemAdded(newPhrase);
    }
  }
  
  public void deleteItems(int[] selections) {
    if (valuePhraseList == null || selections == null) return;
    for (int i = selections.length - 1; i >= 0; i--) {
      OSMXValuePhraseType vp = (OSMXValuePhraseType) valuePhraseList.get(selections[i]);
      valuePhraseList.remove(selections[i]);
      for (Iterator iter = tableModelListeners.iterator(); iter.hasNext();) {
        TableListModelListener tlml = (TableListModelListener) iter.next();
        tlml.itemRemoved(vp);
      }
    }
  }
  
  public String[] getColumnNames() {
    return columnNames;
  }
  
  public Object getColumnValue(Object item, int columnIndex) {
    if (valuePhraseList == null) return null;
    ValuePhraseType phrase = (ValuePhraseType) item;
    if (columnIndex <= 0) {
      return phrase.getHint();
    } else if (columnIndex >= 1) {
      if (!phrase.isSetValueExpression()) return null;
      return phrase.getValueExpression().getExpressionText();
    }
    return null;
  }
  
  public List getList() {
    if (valuePhraseList == null) return null;
    return valuePhraseList;
  }
  
  public void moveItemsDown(int[] selections) {
    if (valuePhraseList == null || selections == null) return;
    int listSize = valuePhraseList.size();
    
    // if last row is selected, can't move selection down
    if (selections[selections.length - 1] == listSize) return;
    
    for (int i = selections.length - 1; i >= 0; i--) {
      Object o = valuePhraseList.remove(selections[i]);
      // add at one position later
      valuePhraseList.add(selections[i] + 1, o);
    }
    notifyReordered();
  }
  
  public void moveItemsUp(int[] selections) {
    if (valuePhraseList == null || selections == null) return;
    
    // uppermost row can't go higher
    if (selections[0] == 0) return; 
    
    for (int i = 0; i < selections.length; i++) {
      Object o = valuePhraseList.remove(selections[i]);
      // add at one position earlier
      valuePhraseList.add(selections[i] - 1, o);
    }
    notifyReordered();
  }
  
  protected void notifyReordered() {
    for (Iterator i = tableModelListeners.iterator(); i.hasNext();) {
      TableListModelListener tlml = (TableListModelListener) i.next();
      tlml.itemsReordered();
    }
  }
  
  public void addTableListModelListener(TableListModelListener tlml) {
    if (tableModelListeners.contains(tlml)) return;
    tableModelListeners.add(tlml);
  }
  
  public void removeTableListModelListener(TableListModelListener tlml) {
    tableModelListeners.remove(tlml);
  }
  
}
