/*
 * ValuePhraseNode.java
 *
 * Created on July 19, 2003, 6:47 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXDataFrameExpression;
import edu.byu.deg.osmx.OSMXValuePhraseType;
import edu.byu.deg.osmx.binding.DataFrameExpression;
import edu.byu.deg.osmx.binding.ValuePhraseType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;


/**
 *
 * @author  Alan
 */
public class ValuePhraseNode extends AbstractDataFrameTreeNode {
  
  protected OSMXValuePhraseType valuePhrase;
  protected PropertyChangeListener contentListener;
  
  /** Creates a new instance of ValuePhraseNode */
  public ValuePhraseNode(ValuePhraseType vp, DefaultTreeModel model) {
    super(vp, model);
    contentListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        String propName = evt.getPropertyName();
        Object source = evt.getSource();
        if (source instanceof DataFrameExpression) {
          if (OSMXDataFrameExpression.EXPRESSION_TEXT_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(ValuePhraseNode.this);
            }
          }
        } else if (source instanceof ValuePhraseType) {
          if (OSMXValuePhraseType.HINT_PROPERTY.equals(propName)) {
            if (treeModel != null) {
              treeModel.nodeChanged(ValuePhraseNode.this);
            }
          }
        }
      }
    };
    valuePhrase = (OSMXValuePhraseType) vp;
    if (valuePhrase != null) {
      this.add(new KeywordPhraseListNode(valuePhrase.getKeywordPhrase(), treeModel));
    }
    setUserObject(vp);
  } // constructor
  
  public String toString() {
    String result = "<html><i>Null</i></html>";
    if (valuePhrase.isSetHint() && valuePhrase.getHint().length() > 0) {
      result = valuePhrase.getHint();
      return result;
    }
    if (!valuePhrase.isSetValueExpression()) {
      return result;
    }
    DataFrameExpression expr = valuePhrase.getValueExpression();
    if (expr.getExpressionText() == null || expr.getExpressionText().length() == 0) {
      return result;
    }
    return expr.getExpressionText();
  } // method toString
  
  public ValuePhraseType getValuePhrase() {
    return valuePhrase;
  } // method getValuePhrase
  
  public void setUserObject(Object userObject) {
    if (valuePhrase != null) {
      valuePhrase.removePropertyChangeListener(contentListener);
      OSMXDataFrameExpression expr = (OSMXDataFrameExpression) valuePhrase.getValueExpression();
      if (expr != null) {
        expr.removePropertyChangeListener(contentListener);
      }
    }
    if (userObject instanceof OSMXValuePhraseType) {
      super.setUserObject(userObject);
      valuePhrase = (OSMXValuePhraseType) userObject;
      if (valuePhrase != null) {
        valuePhrase.addPropertyChangeListener(contentListener);
        OSMXDataFrameExpression expr = (OSMXDataFrameExpression) valuePhrase.getValueExpression();
        if (expr != null) {
          expr.addPropertyChangeListener(contentListener);
        }
      }
    }
  }
  
  public void nodeAdded(TreeNode child) { /* do nothing */ }
  
  public void nodeRemoved(TreeNode child) { /* do nothing */ }
  
 // method setUserObject
  
}
