/*
 * ValuePhrasePanel.java
 *
 * Created on July 31, 2003, 11:18 PM
 */

package edu.byu.deg.dataframe;

import edu.byu.deg.osmx.OSMXDataFrameExpression;
import edu.byu.deg.osmx.OSMXValuePhraseType;
import edu.byu.deg.osmx.binding.DataFrameExpression;
import edu.byu.deg.osmx.binding.ValuePhraseType;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
 *
 * @author  Alan
 */
public class ValuePhrasePanel extends JPanel implements ActionListener, PropertyChangeListener {
  
  private DataFrameExpressionPanel pnlValExpr = new DataFrameExpressionPanel();
  private DataFrameExpressionPanel pnlExceptExpr = new DataFrameExpressionPanel();
  private DataFrameExpressionPanel pnlLeftContextExpr = new DataFrameExpressionPanel();
  private DataFrameExpressionPanel pnlRightContextExpr = new DataFrameExpressionPanel();
  private BoxLayout mainLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
  private JPanel pnlHint = new JPanel();
  private BoxLayout hintLayout = new BoxLayout(pnlHint, BoxLayout.X_AXIS);
  private JPanel pnlConfidence = new JPanel();
  private BoxLayout confidenceLayout = new BoxLayout(pnlConfidence, BoxLayout.X_AXIS);
  private JCheckBox cbCaseSensitive = new JCheckBox();
  private JLabel lblHint = new JLabel();
  private JLabel lblConfidence = new JLabel();
  private JTextField txtHint = new JTextField();
  private JTextField txtConfidence = new JTextField();
  private JTextField txtCanonicalization = new JTextField();
  
  protected OSMXValuePhraseType valuePhrase = null;
  private boolean textChanging = false;
  
  /** Creates a new instance of ValuePhrasePanel */
  public ValuePhrasePanel() {
    try {
      initGUI();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  public ValuePhrasePanel(ValuePhraseType vp) {
    this();
    setValuePhrase(vp);
  } // constructor(ValuePhraseType)
  
  public void setValuePhrase(ValuePhraseType vp) {
    if (valuePhrase != null) {
      valuePhrase.removePropertyChangeListener(this);
    }
    valuePhrase = (OSMXValuePhraseType) vp;
    boolean hasPhrase = (valuePhrase != null);
    if (hasPhrase) {
      valuePhrase.addPropertyChangeListener(this);
      
      pnlValExpr.setExpression(valuePhrase.getValueExpression());
      pnlExceptExpr.setExpression(valuePhrase.getExceptionExpression());
      pnlLeftContextExpr.setExpression(valuePhrase.getLeftContextExpression());
      pnlRightContextExpr.setExpression(valuePhrase.getRightContextExpression());
      
      txtHint.setText(valuePhrase.getHint());
      txtConfidence.setText(valuePhrase.getConfidenceTag());
      cbCaseSensitive.setSelected(valuePhrase.isCaseSensitive());
      txtCanonicalization.setText(valuePhrase.getCanonicalizationMethod());
    }
    pnlValExpr.setEnabled(hasPhrase);
    pnlExceptExpr.setEnabled(hasPhrase);
    pnlLeftContextExpr.setEnabled(hasPhrase);
    pnlRightContextExpr.setEnabled(hasPhrase);
    lblConfidence.setEnabled(hasPhrase);
    lblHint.setEnabled(hasPhrase);
    txtConfidence.setEnabled(hasPhrase);
    txtHint.setEnabled(hasPhrase);
    cbCaseSensitive.setEnabled(hasPhrase);
  } // method setValuePhrase
  
  protected void initGUI() throws Exception {
    this.setLayout(mainLayout);
    
    pnlHint.setLayout(hintLayout);
    pnlHint.add(lblHint);
    pnlHint.add(Box.createRigidArea(new Dimension(4, 1)));
    pnlHint.add(txtHint);
    lblHint.setEnabled(false);
    txtHint.setEnabled(false);
    lblHint.setText("Hint:");
    lblHint.setToolTipText("This gives your value phrase a name.");
    pnlHint.setMaximumSize(new Dimension(4000, 80));
    pnlHint.setAlignmentX(Component.LEFT_ALIGNMENT);
    txtHint.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          valuePhrase.setHint(txtHint.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    this.add(pnlHint);
    
    cbCaseSensitive.setEnabled(false);
    cbCaseSensitive.setText("Case sensitive");
    cbCaseSensitive.setAlignmentX(Component.LEFT_ALIGNMENT);
    cbCaseSensitive.setHorizontalTextPosition(AbstractButton.LEADING);
    cbCaseSensitive.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          if (valuePhrase == null) return;
          valuePhrase.setCaseSensitive(cbCaseSensitive.isSelected());
        }
      }
    );
    this.add(cbCaseSensitive);
    
    pnlConfidence.setLayout(confidenceLayout);
    pnlConfidence.add(lblConfidence);
    pnlConfidence.add(Box.createRigidArea(new Dimension(4, 1)));
    pnlConfidence.add(txtConfidence);
    pnlConfidence.add(Box.createHorizontalGlue());
    lblConfidence.setEnabled(false);
    txtConfidence.setEnabled(false);
    txtConfidence.setMinimumSize(new Dimension(40, 8));
    lblConfidence.setText("Confidence tag:");
    lblConfidence.setToolTipText("This lets you assign a numeric confidence value to the expression.");
    pnlConfidence.setMaximumSize(new Dimension(200, 80));
    pnlConfidence.setAlignmentX(Component.LEFT_ALIGNMENT);
    txtConfidence.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent evt) {
          textChanging = true;
          valuePhrase.setConfidenceTag(txtConfidence.getText());
          textChanging = false;
        }
        
        public void removeUpdate(DocumentEvent evt) {
          insertUpdate(evt);
        }
        
        public void changedUpdate(DocumentEvent evt) { /* do nothing */ }
      }
    );
    this.add(pnlConfidence);
    
    this.add(Box.createRigidArea(new Dimension(0, 15)));
    
    addExpressionPanel(pnlValExpr, "Value expression");
    
    this.add(Box.createRigidArea(new Dimension(0, 15)));
    
    addExpressionPanel(pnlExceptExpr, "Exception expression");
    pnlExceptExpr.addActionListener(this);
    addExpressionPanel(pnlLeftContextExpr, "Left context expression");
    pnlLeftContextExpr.addActionListener(this);
    addExpressionPanel(pnlRightContextExpr, "Right context expression");
    pnlRightContextExpr.addActionListener(this);
    
    this.add(Box.createVerticalGlue());
  } // method initGUI
  
  protected void addExpressionPanel(DataFrameExpressionPanel panel, String title) {
    if (panel == null) return;
    panel.setTitle(title);
    panel.setExpression(null);
    panel.setEnabled(false);
    panel.setMaximumSize(new Dimension(4000, 100));
    panel.setAlignmentX(Component.LEFT_ALIGNMENT);
    this.add(panel);
  }
  
  public void propertyChange(PropertyChangeEvent evt) {
    if (!(evt.getSource() instanceof ValuePhraseType)) return;
    String propertyName = evt.getPropertyName();
    if (OSMXValuePhraseType.HINT_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtHint.setText(valuePhrase.getHint());
      }
    } else if (OSMXValuePhraseType.CONFIDENCE_TAG_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtConfidence.setText(valuePhrase.getConfidenceTag());
      }
    } else if (OSMXValuePhraseType.CASE_SENSITIVE_PROPERTY.equals(propertyName)) {
      cbCaseSensitive.setSelected(((Boolean) evt.getNewValue()).booleanValue());
    } else if (OSMXValuePhraseType.CANONICALIZATION_PROPERTY.equals(propertyName)) {
      if (!textChanging) {
        txtCanonicalization.setText(valuePhrase.getCanonicalizationMethod());
      }
    }
  }
  
  public void actionPerformed(ActionEvent e) {
    Object obj = e.getSource();
    if (!(obj instanceof DataFrameExpressionPanel)) return;
    if (obj == null) return;
    if (valuePhrase == null) return;
    DataFrameExpressionPanel panel = (DataFrameExpressionPanel) obj;
    if (DataFrameExpressionPanel.REQUEST_NEW_EXPRESSION_ACTION.equals(e.getActionCommand())) {
      DataFrameExpression exp = new OSMXDataFrameExpression();
      panel.setExpression(exp);
      if (panel == pnlExceptExpr) {
        valuePhrase.setExceptionExpression(exp);
      } else if (panel == pnlLeftContextExpr) {
        valuePhrase.setLeftContextExpression(exp);
      } else if (panel == pnlRightContextExpr) {
        valuePhrase.setRightContextExpression(exp);
      }
    } else if (DataFrameExpressionPanel.REQUEST_REMOVE_EXPRESSION_ACTION.equals(e.getActionCommand())) {
      panel.setExpression(null);
      if (panel == pnlExceptExpr) {
        valuePhrase.setExceptionExpression(null);
      } else if (panel == pnlLeftContextExpr) {
        valuePhrase.setLeftContextExpression(null);
      } else if (panel == pnlRightContextExpr) {
        valuePhrase.setRightContextExpression(null);
      }
    }
  }
  
}
