/*
 * MetaExtractor.java
 *
 * Created on March 17, 2004, 3:51 PM
 */

package edu.byu.deg.extractionmodule;

import edu.byu.deg.framework.*;
import edu.byu.deg.osmx.OSMXDocument;
import edu.byu.deg.ontos.*;
import edu.byu.deg.query.OSMQuery;
import java.util.*;

import edu.byu.deg.util.Timer;
/**
 *
 * @author  troywalk
 */
public class MetaExtractor {
  public static final String MODULES = "ExtractionModules";
  private OSMXDocument ontology;
  private DataFrameMatcher dfm;
  private boolean halt;
  private ResourceBundle resources;
  private ExtractionModule[] modules;
  
  /** Creates a new instance of MetaExtractor */
  public MetaExtractor(OSMXDocument ont) {
    ontology = ont;
    dfm = new DataFrameMatcher(ontology);
    halt = false;
    initModules();
  }
  
  private void initModules(){
    try {
      resources = ResourceBundle.getBundle("edu.byu.deg.extractionmodule.Options");
      String[] classes = resources.getString(MODULES).split(" ");
      ArrayList mods = new ArrayList();
      for(int i=0;i<classes.length;i++){
        try{
          ExtractionModule mod = (ExtractionModule)Class.forName(classes[i]).newInstance();
          mod.init(ontology);
          mods.add(mod);
        }catch(Exception e){
          System.err.println("Error instantiating ExtractionModule " + classes[i]);
        }
      }
      modules = new ExtractionModule[mods.size()];
      for(int i=0;i<modules.length;i++)
        modules[i] = (ExtractionModule)mods.get(i);
      
    }
    catch (MissingResourceException mre) {
      mre.printStackTrace(System.err);
      System.err.println("TIQS.Options not found");
    }
  }

  
  public void halt(){
    halt = true;
  }
  
  public void startExtraction(DocumentRetriever retriever, OSMQuery query){
    System.out.println("started");
    if(ontology==null)
      return;
    halt = false;
    try{
      java.util.Iterator it = retriever.retrieveDocuments();
      while(!halt&&it.hasNext()){
        if(Timer.time) Timer.timer.startTask("Retrieving");
        TextDocument doc = (TextDocument)it.next();
        if(Timer.time) Timer.timer.endTask("Retrieving");
        if(Timer.time) Timer.timer.startTask("Matching");
        dfm.doMatches(doc);
        OverlapFilter.filter(ontology);
        
        ExtractionModule extractor = getBestExtractorFor(doc);
        extractor.extract(doc, query, retriever);
        System.out.println(doc.toString());
        dfm.clearMatches();

        if(Timer.time) Timer.timer.endTask("Matching");
        Thread.yield();
      }
    } catch (RetrieverConfigurationException e) { return; }
  }

  private ExtractionModule getBestExtractorFor(Document doc){
    ExtractionModule extractor = null;
    double best = -1;
    for(int i=0;i<modules.length;i++){
      double conf = modules[i].getConfidence(doc);
      if(conf>best){
        best = conf;
        extractor = modules[i];
      }
    }
    return extractor;
  }
}

