/*
 * ContentFilter.java
 *
 * Created on January 5, 2004, 11:34 PM
 */

package edu.byu.deg.framework;

/** The <CODE>ContentFilter</CODE> interface represents a preprocessor for the
 * contents of a {@link Document}.  Content preprocessing may include such
 * activities as:
 * <UL>
 * <LI>Stripping out markup</LI>
 * <LI>Normalizing whitespace</LI>
 * <LI>Including contents of other {@link Document}s inline</LI>
 * <LI>etc.</LI>
 * </UL>
 * @author Alan Wessman
 */
public interface ContentFilter {
  
  /** Performs preprocessing on the specified {@link Document}, returning the filtered
   * content within a {@link Document} object.
   * @param doc The {@link Document} with the content to filter.
   * @return A {@link Document} whose content has been preprocessed.  It is up to the
   * implementing classes whether to allow/require the returned {@link Document}
   * to be the same object as was passed in as input to this method.
   */  
  public Document filterDocument(Document doc);
  
}
