/*
 * VSMManager.java
 *
 * Created on August 20, 2003, 12:02 PM
 */

package edu.byu.deg.framework.DSP.VSM;

import edu.byu.deg.osmx.binding.*;
import edu.byu.deg.osmx.*;
import org.w3c.dom.Node;
import java.util.*;
import edu.byu.deg.util.DOMString;

/**
 *
 * @author  troywalk
 */
public class VSMManager {
    private static final double DEFAULT_AVERAGE = 1.0;
    private static final double STAR_VALUE = 100.0;
    private static final double SPLIT_MAGNITUDE_THRESHHOLD = 1.8;
    private static final double SPLIT_DOCUMENT_THRESHHOLD = 8.5;
    private static final double COS_THRESHHOLD = 0.60;
    private static final double IGNORE_LOWER_THRESHHOLD = 0.05;
    //private static final double IGNORE_UPPER_THRESHHOLD = 2.0;
    VSMMeasure OV;
    VSMMeasure maxOV;
    VSMMeasure normalizedOV;
    Vector axis;
    OSMXDocument ontologyDoc;
    int dimension;
    NodeMatchCounter counter;
    HashMap objectsets;
    boolean[] ignore;
    boolean[] emptyignore;
    
    /** Creates a new instance of VSMManager */
    public VSMManager(OSMXDocument ontology) {
        ontologyDoc = ontology;
        
        // populate the vectors based on the ontology
        makeOntologyVectors(ontology.getModelRoot());
        
        // initialize array of ones
        double[] ones = new double[dimension];
        for(int i=0;i<dimension;i++)
            ones[i] = 1;
        
        // use the ones array to construct a new VSMMeasure
        normalizedOV = new VSMMeasure(ones);
    }
    
    private double getAverageFromParticipationConstraint(ParticipationConstraintType pc){
        double avg = DEFAULT_AVERAGE;
        if(pc!=null){
            String constraint = pc.getContent();
            StringTokenizer toker = new StringTokenizer(constraint, ":");
            int count = toker.countTokens();
            try {
                switch(count){
                    case 1: avg = Double.parseDouble(toker.nextToken());        //in the form 1
                    break;
                    case 2: double min = Double.parseDouble(toker.nextToken()); //in the form 0:*
                    String maxStr = toker.nextToken();
                    double max;
                    if(maxStr.equals("*"))
                        max = STAR_VALUE;
                    else
                        max = Double.parseDouble(maxStr);
                    avg = (max+min)/2;
                    break;
                    case 3: toker.nextToken();                                  //in the form 0:3.62:*
                    avg = Double.parseDouble(toker.nextToken());
                    break;
                    default:
                }
            } catch(NumberFormatException e){
                System.err.println("Invalid number in constraint '" + constraint + "'");
            }
        }
        return avg;
    }
    
    private double getMaxFromParticipationConstraint(ParticipationConstraintType pc){
        double max = DEFAULT_AVERAGE;
        if(pc!=null){
            String constraint = pc.getContent();
            StringTokenizer toker = new StringTokenizer(constraint, ":");
            String maxStr = "";
            if(!toker.hasMoreTokens())
                return max;
            while(toker.hasMoreTokens())
                maxStr = toker.nextToken();
            try {
                if(maxStr.equals("*"))
                    max = STAR_VALUE;
                else
                    max = Double.parseDouble(maxStr);
            } catch(NumberFormatException e){
                System.err.println("Invalid number in constraint '" + constraint + "'");
            }
        }
        return max;
    }
    
    
    private void getAverageFromObjectSets(OSMXObjectSetType objset, String parentID, double avg, double max, Vector avgs, Vector maxs, OSMXOSMType ontology){
        DataFrameType df = objset.getDataFrame();
        String myID = objset.getId();
        if(df!=null){
            List keywords = df.getKeywordPhrase();
            if(keywords!=null&&keywords.size()>0){
                axis.add(myID + "Keyword");
                avgs.add(new Double(avg));
                maxs.add(new Double(max));
                return;
            }
            if(objset.isLexical()){
                //System.out.println(objset.getName());
                axis.add(myID + "Match");
                avgs.add(new Double(avg));
                maxs.add(new Double(max));
                return;
            }
            
        }
        Iterator i;
        List l = ontology.getAllModelElements();
        for(i=l.iterator();i.hasNext();){
            Object o = i.next();
            if(o instanceof OSMXRelationshipSetType){
                OSMXRelationshipSetType r = (OSMXRelationshipSetType)o;
                OSMXRelSetConnectionType thisside = r.getConnectionFor(objset);
                if(thisside!=null){
                    double newavg = avg * getAverageFromParticipationConstraint(thisside.getParticipationConstraint());
                    double newmax = max * getMaxFromParticipationConstraint(thisside.getParticipationConstraint());
                    for(Iterator j=r.getRelSetConnection().iterator(); j.hasNext();){
                        OSMXRelSetConnectionType thatside = (OSMXRelSetConnectionType)j.next();
                        if(thatside==thisside)
                            continue;
                        String id = thatside.getObjectSet();
                        if(id.equals(parentID))
                            continue;
                        //OSMXObjectSetType nextobjset = (OSMXObjectSetType)objset.elementForID(id);
                        OSMXObjectSetType nextobjset = (OSMXObjectSetType)objectsets.get(id);
                        getAverageFromObjectSets(nextobjset, myID, newavg, newmax, avgs, maxs, ontology);
                    }
                }
            }
        }
    }
    
    private void makeOntologyVectors(OSM ontology){
        OSMXObjectSetType primary = null;
        Vector avgs = new Vector();
        Vector maxs = new Vector();
        axis = new Vector();
        List l = ontology.getAllModelElements();
        objectsets = new HashMap();
        
        // find the primary object set and populate the objectsets map
        for(Iterator i=l.iterator();i.hasNext();){
            Object o = i.next();
            if(o instanceof OSMXObjectSetType){
                OSMXObjectSetType objset = (OSMXObjectSetType)o;
                objectsets.put(objset.getId(), objset);
                if(objset.isPrimary())
                    primary = objset;
            }
        }
        if(primary==null){
            System.err.println("VSMManager: Ontology has no primary object set");
            return;
        }
        getAverageFromObjectSets(primary, "", 1.0, 1.0, avgs, maxs, (OSMXOSMType) ontology);
        dimension = avgs.size();
        double[] result = new double[dimension];
        for(int i=0;i<dimension;i++){
            result[i] = ((Double)avgs.get(i)).doubleValue();
        }
        OV = new VSMMeasure(result);
        System.out.println("Ontology Vector: " + OV.toString());
        for(int i=0;i<dimension;i++){
            result[i] = ((Double)maxs.get(i)).doubleValue();
        }
        maxOV = new VSMMeasure(result);
    }
    
    public int getIndexOf(String id){
        for(int i=0;i<axis.size();i++){
            if(axis.get(i).equals(id))
                return i;
        }
        return -1;
    }
    
    private boolean[] createIgnoreArray(VSMMeasure vsm){
        //get the average
        boolean[] ignore = new boolean[dimension];
        //get rid of the dimensions that are too low
        double total=0;
        for(int i=0;i<dimension;i++)
            total += vsm.get(i)/maxOV.get(i);
        double average = total / dimension;
        double min = average*IGNORE_LOWER_THRESHHOLD;
        for(int i=0;i<dimension;i++)
            ignore[i] = (vsm.get(i)/maxOV.get(i) < min);
    /*
    //get rid of the dimensions that are too high
    total = 0;
    int count = 0;
    for(int i=0;i<dimension;i++)
      if(!ignore[i]){
        count++;
        total += vsm.get(i);
      }
    average = total / count;
    for(int i=0;i<dimension;i++)
      if(!ignore[i])
        ignore[i] = (vsm.get(i) > average*IGNORE_UPPER_THRESHHOLD*OV.get(i));
     */
        return ignore;
    }
    
    public void calculateVSM(DOMString str, Node node){
        counter = new NodeMatchCounter(ontologyDoc, str, this);
        VSMMeasure rootMeasure = counter.getCount(node);
        ignore = createIgnoreArray(rootMeasure);
        emptyignore = new boolean[dimension];
        for(int i=0;i<dimension;i++)
            emptyignore[i] = false;
        System.out.println(rootMeasure.toString());
        //for(int i=0;i<dimension;i++)
        //  System.out.println(ignore[i]);
    }
    
    public void calculateVSM(Node node){
        calculateVSM(new DOMString(node), node);
        edu.byu.deg.util.DOMPrinter.print(node, System.out, "", false);
    }
    
    public boolean containsRecord(Node node){
        //check cos measure
        VSMMeasure dv = counter.getCount(node);
        double cos = VSMMeasure.cos(dv.normalize(OV), normalizedOV, ignore);
        System.out.println("evaluating cos score of " + node.getNodeName() + dv.toString() + "  cos is " + cos);
        if(cos<COS_THRESHHOLD)
            return false;
        else
            return true;
    }
    
    public double getCos(Node node){
        VSMMeasure dv = counter.getCount(node);
        return VSMMeasure.cos(dv.normalize(OV), normalizedOV, ignore);
    }
    
    public double getMag(Node node){
        if(node.getChildNodes().getLength()==0)
            return 0.0;
        //check magnitude
        VSMMeasure dv = counter.getCount(node);
        return dv.normalize(OV).getLength(ignore)/normalizedOV.getLength(ignore);
    }
    
    public boolean isSplitable(Node node){
        if(node.getChildNodes().getLength()==0)
            return false;
        //check magnitude
        VSMMeasure dv = counter.getCount(node);
        double magnitude = dv.normalize(OV).getLength(ignore)/normalizedOV.getLength(ignore);
        System.out.println("evaluating magnitude of " + node.getNodeName() + dv.toString() + "  mag is " + magnitude);
        if(magnitude<SPLIT_MAGNITUDE_THRESHHOLD)
            return false;
        else
            return true;
    }
    
    public boolean isDocumentSplitable(Node node){
        if(node.getChildNodes().getLength()==0)
            return false;
        //check magnitude
        VSMMeasure dv = counter.getCount(node);
        double magnitude = dv.normalize(OV).getLength(ignore)/normalizedOV.getLength(ignore);
        System.out.println("evaluating magnitude of " + node.getNodeName() + dv.toString() + "  mag is " + magnitude);
        if(magnitude<SPLIT_DOCUMENT_THRESHHOLD)
            return false;
        else
            return true;
    }
    
    public boolean canMerge(Node node1, Node node2){
        VSMMeasure count1 = new VSMMeasure(counter.getCount(node1));
        VSMMeasure count2 = new VSMMeasure(counter.getCount(node2));
        VSMMeasure combined1 = VSMMeasure.add(count1, count2);
        combined1.applyOneMaxNormalization(maxOV);
        count1.applyOneMaxNormalization(maxOV);
        count2.applyOneMaxNormalization(maxOV);
        VSMMeasure combined2 = VSMMeasure.add(count1, count2);
        if(count1.getLength(emptyignore)==0||count2.getLength(emptyignore)==0)
            return false;
        //System.out.println("testing:");
        //System.out.println("evaluating length of " + node1.getNodeName() + count1.toString() + "  length is " + count1.getLength(emptyignore));
        //System.out.println("evaluating length of " + node2.getNodeName() + count2.toString() + "  length is " + count2.getLength(emptyignore));
        //System.out.println("length 3: " + combined1.getLength(emptyignore));
        //System.out.println("length 4: " + combined2.getLength(emptyignore));
        
        if(combined2.getLength(emptyignore)==combined1.getLength(emptyignore))
            return true;
        return false;
    }
}
