/*
 * VSMMeasure.java
 *
 * Created on August 18, 2003, 2:00 PM
 */

package edu.byu.deg.framework.DSP.VSM;

/**
 *
 * @author  troywalk
 */
public class VSMMeasure {
  double[] vector;
  boolean[] lastIgnoreArray = null;
  double length;
  boolean islengthok;
  int size;
  
  /** Creates a new instance of VSMMeasure */
  public VSMMeasure(double[] vect) {
    size = vect.length;
    vector = new double[size];
    System.arraycopy(vect, 0, vector, 0, size);
    islengthok = false;
  }

  /** Creates a copy of a VSMMeasure */
  public VSMMeasure(VSMMeasure m) {
    size = m.size;
    vector = new double[size];
    System.arraycopy(m.vector, 0, vector, 0, size);
    islengthok = false;
  }

  public void set(int i, double val){
    vector[i] =  val;
    islengthok = false;
  }
  
  public double get(int i){
    return vector[i];
  }
  
  public double getLength(boolean[] ignoreArray){
    if(ignoreArray!=lastIgnoreArray)
      islengthok = false;
    if(!islengthok)
      setLength(ignoreArray);
    return length; 
  }
  
  private void setLength(boolean[] ignoreArray){
    double total=0;
    lastIgnoreArray = ignoreArray;
    for(int i=0;i<size;i++)
      if(!ignoreArray[i])
        total += vector[i]*vector[i];
    length = Math.sqrt(total);
    islengthok = true;
  }
  
  public VSMMeasure normalize(VSMMeasure o){
    double[] newvector = new double[size];
    for(int i=0;i<size;i++)
      if(o.vector[i]!=0)
        newvector[i] = vector[i] / o.vector[i];
      else
        newvector[i] = 0;
    return new VSMMeasure(newvector);
  }
  
  public void applyOneMaxNormalization(VSMMeasure max){
    for(int i=0;i<size;i++){
      if(max.vector[i]==1){
        vector[i] = vector[i]>0?1:0;
      }
      else {
        vector[i] = vector[i]/max.vector[i];
      }
    }
    islengthok = false;
  }
  
  public void add(VSMMeasure v){
    for(int i=0;i<size;i++)
      vector[i] += v.vector[i];
  }

  public static VSMMeasure add(VSMMeasure v1, VSMMeasure v2){
    VSMMeasure res = new VSMMeasure(v1.vector);
    res.add(v2);
    return res;
  }

  public static double cos(VSMMeasure v1, VSMMeasure v2, boolean[] ignoreArray){
    double total=0;
    if(v1.getLength(ignoreArray)==0 || v2.getLength(ignoreArray)==0)
      return 0;
    for(int i=0;i<v1.size;i++)
      if(!ignoreArray[i])
        total += v1.vector[i] * v2.vector[i];
    total /= v1.getLength(ignoreArray);
    total /= v2.getLength(ignoreArray);
    return total;
  }
  
  public String toString(){
    StringBuffer b = new StringBuffer();
    b.append("{ ");
    for(int i=0;i<size-1;i++){
      b.append(vector[i]);
      b.append( ", ");
    }
    b.append(vector[size-1]);
    b.append("};");
    return b.toString();
    
  }
}
