/*
 * VSMRecordSeparator.java
 *
 * Created on January 20, 2004, 12:39 PM
 */

package edu.byu.deg.framework.DSP.VSM;

import edu.byu.deg.framework.DocumentStructureParser;
import edu.byu.deg.framework.Ontology;
import edu.byu.deg.framework.OntologySubscriber;
import edu.byu.deg.framework.UnrecognizedFormatException;
import edu.byu.deg.framework.DSP.heuristic.CombinedHeuristic;
import edu.byu.deg.framework.document.DOMDocument;
import edu.byu.deg.osmx.OSMXDocument;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.tidy.Tidy;

/**
 *
 * @author  troywalk
 */
public class VSMRecordSeparator implements DocumentStructureParser, OntologySubscriber {
    public static final String SEPARATOR_TAG = "record";
    public static final double CHILD_RECORD_THRESHOLD = 0.66;
    public static final double MIN_CHILDREN_TO_KEEP_ALL = 4;
    static Tidy tidy = null;
    private VSMManager manager;
    private OSMXDocument ontDoc;
    public VSMRecordSeparator(OSMXDocument ont){
        this();
        setOntology(ont);
    }
    
    public VSMRecordSeparator() {
        if(tidy==null){
            tidy = new Tidy();
            tidy.setShowWarnings(false);
            tidy.setQuiet(true);
        }
    }
    
    public edu.byu.deg.framework.Document parse(edu.byu.deg.framework.Document d) throws UnrecognizedFormatException {
        if(!(d instanceof DOMDocument))
            throw new UnrecognizedFormatException("RecordSeparator cannot handle this document");
        DOMDocument doc = (DOMDocument)d;
        manager.calculateVSM(doc.getDOMString(), doc.getRoot());
        //manager.calculateVSM(doc.getRoot());
        Node root = doc.getRoot();
        doc.setConfidence(manager.getCos(root));
        if(manager.isDocumentSplitable(root))
            return parse(doc.getRoot(), doc);
        else {
            doc.addSubDocument(root);
            return doc;
        }
    }
    
    private Set recombine(Node node, ArrayList splitnodes, DOMDocument d){
        Set recombined = new HashSet();
        if(splitnodes.size()>0){
            Node previous = (Node)splitnodes.get(0);
            for(int i=1;i<splitnodes.size();i++){
                Node next = (Node)splitnodes.get(i);
                if(!manager.isSplitable(previous)&&!manager.isSplitable(next)&&manager.canMerge(previous, next)){
                    Node record = d.getParentDocument().createElement(SEPARATOR_TAG);
                    recombined.add(record);
                    try{            
                        node.insertBefore(record,  previous);
                        node.removeChild(previous);
                        node.removeChild(next);
                        
                        record.appendChild(previous);
                        record.appendChild(next);
                    } catch(Exception e){ e.printStackTrace(); }
                    splitnodes.set(i-1, record);
                    splitnodes.remove(i--);
                    previous = record;
                }
                else
                    previous = next;
            }
        }
        return recombined;
    }  
    
    private boolean getRecords(ArrayList nodes, ArrayList records, ArrayList fragments){
        int count=0;
        for(int i=0;i<nodes.size();i++){
            Node next = (Node)nodes.get(i);
            if(manager.containsRecord(next)){
                if(!manager.isSplitable(next))
                    count++;
                records.add(next);
            }
            else
                fragments.add(next);
        }
        //System.out.println(count + " of " + nodes.size() + " = " + ((double)count/nodes.size()));
        //if a certain portion of the nodes are records, they all probably are
        if(nodes.size()>=MIN_CHILDREN_TO_KEEP_ALL&&count>nodes.size()*CHILD_RECORD_THRESHOLD){
            System.out.println("using all");
            records.clear();
            records.addAll(nodes);
            //just trim off the possible header
            while(!manager.containsRecord((Node)records.get(0)))
                records.remove(0);
            //and the possible footer
            while(!manager.containsRecord((Node)records.get(records.size()-1)))
                records.remove(records.size()-1);
            return true;
        }
        return false;
    }
    
    private edu.byu.deg.framework.Document parse(Node node, DOMDocument d){
        if(manager.isSplitable(node)){
            Node child = node.getFirstChild();
            
            System.out.println("d =  \n-----------------\n" + d.toString() + "\n----------------\n");
            System.out.println("node =  \n-----------------\n" + node.toString() + "\n----------------\n");
            System.out.println("child =  \n-----------------\n" + child.toString() + "\n----------------\n");
            
            //split
            ArrayList nodes = split(node, d.getParentDocument());
            
            //avoid infinite recursion
            if(nodes.contains(node.getNextSibling())){
                d.addSubDocument(node);
                return d;
            }
            
            //recombine adjacent nodes as possible
            Set recombined = recombine(node, nodes, d);
            
            //sort results into fragments and records
            ArrayList fragments = new ArrayList();
            ArrayList records = new ArrayList();
            boolean issimple = getRecords(nodes, records, fragments);
            
            int before = d.getSubDocumentCount();
            //recurse on records
            for(int i=0;i<records.size();i++){
                child = (Node)records.get(i);
                if(!issimple&&!recombined.contains(child))
                    parse(child, d);
                else
                    d.addSubDocument(child); //if we try to split a record we just recombined, we could have infinite recursion
            }
            int diff = d.getSubDocumentCount() - before;
            //if we didn't get any records in the children use this
            if(diff==0){
                d.addSubDocument(node);
            }
            
            //if we only got one record in the children use this because we might have lost a stray match
            if(diff==1){
                d.setSubDocument(d.getSubDocumentCount()-1, node);
            }/**/
        }
        else {
            d.addSubDocument(node);
        }
        return d;
    }
    
    private static ArrayList split(Node tree, Document parentDoc){
        CombinedHeuristic h = new CombinedHeuristic(tree);
        String separator = h.getSeparator();
        if(!separator.equals(""))
            return split(tree, separator, parentDoc);
        else {
            ArrayList result = new ArrayList();
            Node next = tree.getFirstChild();
            while(next!=null){
                result.add(next);
                next = next.getNextSibling();
            }
            return result;
        }
    }
    
    private static ArrayList split(Node tree, String tag, Document parentDoc){
        System.out.println("Splitting on <" + tag + ">");
        ArrayList trees = new ArrayList();
        Node curtree = parentDoc.createElement(SEPARATOR_TAG);
        boolean haveadded = false;
        Node node = tree.getFirstChild();
        while(node!=null){
            //System.out.println("doing "+ tag + "\t" + node.getNodeName());
            //edu.byu.deg.util.DOMPrinter.print(node);
            if(haveadded&&node.getNodeName().toLowerCase().equals(tag)){
                trees.add(curtree);
                curtree = parentDoc.createElement(SEPARATOR_TAG);
                haveadded = false;
            }
            Node oldnode = node;
            node = node.getNextSibling();
            tree.removeChild(oldnode);
            curtree.appendChild(oldnode);
            haveadded = true;
        }
        if(haveadded)
            trees.add(curtree);
        for(int i=0;i<trees.size();i++) {
            try {
                tree.appendChild((Node)trees.get(i));
            }
            catch(Exception e){ /* do nothing */ }
        }
        return trees;
    }
    
    public Ontology getOntology() {
        return ontDoc;
    }
    
    public void setOntology(Ontology ontology) {
        ontDoc = (OSMXDocument) ontology;
        manager = new VSMManager(ontDoc);
    }
    
}
