/*
 * FanoutRecordSeparator.java
 *
 * Created on August 18, 2003, 10:54 AM
 */

package edu.byu.deg.framework.DSP.fanout;
import edu.byu.deg.framework.*;
import edu.byu.deg.framework.Document;
import org.w3c.dom.*;
import org.w3c.tidy.Tidy;
import java.util.ArrayList;
import edu.byu.deg.framework.document.DOMDocument;
import edu.byu.deg.framework.DSP.heuristic.CombinedHeuristic;
/**
 *
 * @author  troywalk
 */
public class FanoutRecordSeparator implements DocumentStructureParser{
  public static final String SEPARATOR_TAG = "record";
  static Tidy tidy = null;
  public FanoutRecordSeparator(){
    if(tidy==null){
      tidy = new Tidy();
      tidy.setShowWarnings(false);
      tidy.setQuiet(true);
    }
  }
  
  public Document parse(Document d) throws UnrecognizedFormatException {
    if(!(d instanceof DOMDocument))
      throw new UnrecognizedFormatException("RecordSeparator cannot handle this document");
    DOMDocument doc = (DOMDocument)d;
    Node tree = doc.getRoot();
    Node temp = getHighestFanout(tree);
    //System.out.println("Highest Fanout is " + temp.getNodeName());
    CombinedHeuristic h = new CombinedHeuristic(temp);
    //System.out.println("Separator is " + h.getSeparator());
    ArrayList trees = split(temp, h.getSeparator());
    doc.addSubDocuments(trees);
    return doc;
  }
  
  private static ArrayList split(Node tree, String tag){
    ArrayList trees = new ArrayList();
    org.w3c.dom.Document parentDoc = tree.getOwnerDocument();
    Node curtree = parentDoc.createElement(SEPARATOR_TAG);
    boolean haveadded = false;
    Node node = tree.getFirstChild();
    while(node!=null){
      // System.out.println("doing "+ tag + "\t" + node.getNodeName());
      // edu.byu.deg.util.DOMPrinter.print(node);
      if(haveadded&&node.getNodeName().toLowerCase().equals(tag)){
        trees.add(curtree);
         System.out.println("***RECORD***");
         edu.byu.deg.util.DOMPrinter.print(curtree);
         System.out.println("***RECORD***");
        curtree = parentDoc.createElement(SEPARATOR_TAG);
      }
      curtree.appendChild(node.cloneNode(true));
      node = node.getNextSibling();
      haveadded = true;
    }
    if (haveadded && curtree != null) {
        trees.add(curtree);
         System.out.println("***RECORD***");
         edu.byu.deg.util.DOMPrinter.print(curtree);
         System.out.println("***RECORD***");
    }
    return trees;
  }
  
  private static int getFanout(Node node){
    return node.getChildNodes().getLength();
  }
  
  private static Node getHighestFanout(Node tree){
    int best = getFanout(tree);
    Node bestNode = tree;
    Node node = tree.getFirstChild();
    while(node!=null){
      Node temp = getHighestFanout(node);
      int tempfanout = getFanout(temp);
      if(tempfanout>best){
        best = tempfanout;
        bestNode = temp;
      }
      node = node.getNextSibling();
    }
    return bestNode; 
  }
  
  
  //test code
  public static void main(String[] args) {
    try{
      DOMDocument d = new DOMDocument("c:/wwwroot/demo/default_data/defaulthtmlsrc/obit.html");
      FanoutRecordSeparator s = new FanoutRecordSeparator();
      Document pd = s.parse(d);
      java.util.Iterator i = pd.getSubDocuments();
      while(i.hasNext()){
        DOMDocument doc = (DOMDocument)i.next();
        edu.byu.deg.util.DOMPrinter.print(doc.getRoot());
      }
    }catch (Exception e) {e.printStackTrace();}
    System.out.println("done");
  }
}
