/*
 * CombinedHeuristic.java
 *
 * Created on December 17, 2003, 12:46 PM
 */

package edu.byu.deg.framework.DSP.heuristic;

/**
 *
 * @author  troywalk
 */
import org.w3c.dom.Node;
import java.util.ArrayList;
public class CombinedHeuristic implements SeparationHeuristic {
  ArrayList ranking;
  
  /** Creates a new instance of CombinedHeuristic */
  public CombinedHeuristic(Node tree) {
    TagCounter tagcounter = new TagCounter(tree);
    SeparationHeuristic[] heuristics = new SeparationHeuristic[4];
    //build the heuristics
    heuristics[0] = new HighestCountTagHeuristic(tagcounter);
    heuristics[1] = new IdentifiableTagHeuristic(tree);
    heuristics[2] = new RepeatingPatternHeuristic(tree, tagcounter);
    heuristics[3] = new StandardDeviationHeuristic(tree, tagcounter);
    //The old demo does not use the OM heuristic so I left it out for now,
    //but it would be good to add.  We would have to pass in the ontology though
    ranking = new ArrayList();
    ArrayList tags = tagcounter.getTagList(2);
    for(int i=0;i<tags.size();i++){
      String tag = (String)tags.get(i);
      double certainty = 0;
      //System.out.println("tag is " + tag);
      for(int j=0;j<heuristics.length;j++){
        //System.out.println("rank is " + heuristics[j].getRank(tag));
        certainty = stanfordCertaintyMeasure(certainty, heuristics[j].getCertainty(heuristics[j].getRank(tag)));
      }
      int pos = 0;
      //System.out.println("Combined certainty for " + tag + " is " + certainty);
      while(pos<ranking.size()&&certainty<((StringDoubleWrapper)ranking.get(pos)).num)
        pos++;
      //System.out.println("pos is " + pos);
      ranking.add(pos, new StringDoubleWrapper(tag, certainty));
    }
  }
  
  //calculates p(a|b) from p(a) and p(b)
  private double stanfordCertaintyMeasure(double a, double b){
     return a + b - (a * b);
  }
  
  public double getCertainty(int i) {
    if(i<0||i>=ranking.size())
      return 0;
    return ((StringDoubleWrapper)ranking.get(i)).num;
  }
  
  public int getRank(String tag) {
    for(int i=0;i<ranking.size();i++){
      if(((StringDoubleWrapper)ranking.get(i)).str.equals(tag))
        return i;
    }
    return -1;
  }
  
  public String getSeparator() {
    if(ranking.size()==0)
      return "";
    return ((StringDoubleWrapper)ranking.get(0)).str;
  }
  
}
