/*
 * RepeatingPatternHeuristic.java
 *
 * Created on December 11, 2003, 2:06 PM
 */

package edu.byu.deg.framework.DSP.heuristic;

/**
 *
 * @author  troywalk
 */
import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import java.util.ArrayList;
public class RepeatingPatternHeuristic implements SeparationHeuristic {
  protected static double[] certainty = {0.775, 0.125, 0.09, 0.01};
  private ArrayList ranking;
  /** Creates a new instance of RepeatingPatternHeuristic */
  public RepeatingPatternHeuristic(Node tree, TagCounter tc) {
    ArrayList tags = tc.getTagList(3);
    ArrayList patterns = countTagPatterns(tree);
    ranking = new ArrayList();
    for(int i=0;i<tags.size();i++){
      //get the smallest difference for this tag
      String tag = (String)tags.get(i);
      StringIntegerWrapper tagdiff = new StringIntegerWrapper(tag, Integer.MAX_VALUE);
      for(int j=0;j<patterns.size();j++){
        TagPattern pattern = (TagPattern)patterns.get(j);
        if(pattern.first.equals(tag)||pattern.second.equals(tag)){
          int diff = Math.abs(tc.getCount(tag) - pattern.count);
          if(diff<tagdiff.num){
            tagdiff.num = diff;
          }
        }
      }
      //insert the tag into the list in ascending order
      if(tagdiff.num!=Integer.MAX_VALUE){
        int pos = 0;
        while(pos<ranking.size()&&((StringIntegerWrapper)ranking.get(pos)).num<tagdiff.num){
          pos++;
        }
        ranking.add(pos, tagdiff);
      }
    }
    
  }
  
  boolean containsText(Node node){
    if(node.getNodeType()==Node.TEXT_NODE){
      try {
        String str = node.getNodeValue().trim();
        return (str.length()>0);
      }
      catch (DOMException e) { System.err.print("This is impossible");}
      return false;
    }
    Node next = node.getFirstChild();
    while(next!=null){
      if(containsText(next))
        return true;
      next = next.getNextSibling();
    }
    return false;
  }
  private TagPattern getPattern(String tag1, String tag2, ArrayList patterns){
    for(int i=0;i<patterns.size();i++){
      TagPattern p = (TagPattern)patterns.get(i);
      if(p.first.equals(tag1)&&p.second.equals(tag2))
        return p;
    }
    return null;
  }
  
  private ArrayList countTagPatterns(Node tree){
    ArrayList patterns = new ArrayList();
    Node node1 = tree.getFirstChild();
    if(node1==null)
      return patterns;
    Node node2 = node1.getNextSibling();
    while(node2!=null){
      if(!containsText(node1)&&node2.getNodeType()!=Node.TEXT_NODE){
        String tag1 = node1.getNodeName().toLowerCase();
        String tag2 = node2.getNodeName().toLowerCase();
        TagPattern p = getPattern(tag1, tag2, patterns);
        if(p==null){
          patterns.add(new TagPattern(tag1, tag2, 1));
        } else {
          p.count++;
        }
      }
      node1 = node2;
      node2 = node2.getNextSibling();
      
    }
    return patterns;
  }
  
  public double getCertainty(int i) {
    if(i<0||i>=certainty.length)
      return 0;
    return certainty[i];
  }
  
  public int getRank(String tag) {
    for(int i=0;i<ranking.size();i++){
      if(((StringIntegerWrapper)ranking.get(i)).str.equals(tag))
        return i;
    }
    return -1;
  }
  
}

class TagPattern {
   String first = new String();
   String second = new String();
   int count;

   TagPattern(String f, String b, int cnt) {
	  first = f;
	  second = b;
	  count = cnt;
   }
}
