/*
 * StandardDeviationHeuristic.java
 *
 * Created on December 3, 2003, 2:19 PM
 */

package edu.byu.deg.framework.DSP.heuristic;

/**
 *
 * @author  troywalk
 */

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import java.util.ArrayList;
public class StandardDeviationHeuristic implements SeparationHeuristic {
  protected static double[] certainty = {0.655, 0.225, 0.12};
  private ArrayList ranking;
  private Node tree;
  /** Creates a new instance of StandardDeviationHeuristic */
  public StandardDeviationHeuristic(Node tree, TagCounter tc) {
    this.tree = tree;
    ArrayList tags = tc.getTagList(2);
    ranking = new ArrayList();
    for(int i=0;i<tags.size();i++){
      String tag = (String)tags.get(i);
      int variance = getVariance(tag);
      int j=0;
      while(j<ranking.size()&&(variance<((StringIntegerWrapper)ranking.get(j)).num)){
        j++;
      }
      ranking.add(j, new StringIntegerWrapper(tag, variance));
    }
  }
  
  public int getRank(String tag) {
    for(int i=0;i<ranking.size();i++){
      if(((StringIntegerWrapper)ranking.get(i)).str.equals(tag))
        return i;
    }
    return -1;
  }
  
  public double getCertainty(int i){
    if(i<0||i>=certainty.length)
      return 0;
    return certainty[i];
  }
  
  int sizeOf(Node node){
    int size = 0;
    if(node.getNodeType()==Node.TEXT_NODE){
      try { size = node.getNodeValue().length(); }
      catch (DOMException e) { System.err.print("This is impossible");}
      return size;
    }
    Node next = node.getFirstChild();
    while(next!=null){
      size += sizeOf(next);
      next = next.getNextSibling();
    }
    return size;
  }
  
  private int[] getLengths(String tag){
    ArrayList lengths = new ArrayList();
    int curlen = 0;
    Node node = tree.getFirstChild();
    while(node!=null){
       if(curlen!=0&&node.getNodeName().toLowerCase().equals(tag)){
         lengths.add(new Integer(curlen));
         curlen = 0;
       }
       curlen += sizeOf(node);
       node = node.getNextSibling();
    }
    int[] out = new int[lengths.size()];
    for(int i=0;i<lengths.size();i++){
      out[i] = ((Integer)lengths.get(i)).intValue();
    }
    return out;
  }
  
  private int getSum(int[] lengths){
    int sum=0;
    for(int i=0;i<lengths.length;i++) 
      sum += lengths[i];
    return sum;
  }
  
  private int removeOutlier(int[] lengths, int avg){
    int worst = -1;
    int outlier = 0;
    for(int i=0;i<lengths.length;i++){
      int diff = Math.abs(avg-lengths[i]);
      if(diff>worst){
        worst = diff;
        outlier = i;
      }
    }
    for(int i=outlier;i<lengths.length-1;i++){
      lengths[i] = lengths[i+1];
    }
    return worst;
  }
  
  private int getVariance(String tag){
    int[] lengths = getLengths(tag);
    int size = lengths.length;
    int sum = getSum(lengths);
    if(size==0)
      return Integer.MAX_VALUE;
    int avg = sum/size;
    //take out the outlier and update the average
    if(size>2){
      sum -= removeOutlier(lengths, avg);
      size--;
    }
    avg = sum/size;
    //calculate variance
    int total=0;
    for(int i=0;i<size;i++){
      int diff = Math.abs(lengths[i]-avg);
      total += diff*diff;
    }
    int variance = total / size;
    return variance;
  }
}
