/*
 * Document.java
 *
 * Created on February 13, 2003, 8:05 PM
 */

package edu.byu.deg.framework;

import java.util.Iterator;
import java.net.URI;
import java.io.InputStream;
import java.io.IOException;

/** This interface represents documents, which for the purposes of data extraction
 * are units of unstructured or semi-structured data.  Documents may also contain
 * markup information, metadata, etc. but these are generally ignored by a data
 * extraction engine.
 * <br>
 * Implementations may represent linear text strings, binary-encoded graphics,
 * tabular data, hierarchies of content, and so forth.
 * @author Alan Wessman
 */
public interface Document {
    
    /** Returns an <CODE>Iterator</CODE> of <CODE>Document</CODE>s that are aggregated
     * by this <CODE>Document</CODE>.
     * @return The sub-documents belonging to this document, accessed via the returned
     * <CODE>Iterator</CODE>.  Implementing classes will determine whether the iterator
     * accesses the sub-documents in depth-first order, breadth-first order, or some
     * other sequence.
     */    
    public Iterator getSubDocuments();
    
    /** Returns an identifier for the document; may be relative (particularly if this is
     * a subdocument of another document) or absolute.
     * @return The URI of this document.
     */    
    public URI getURI();
    
    /**
     * Returns the contents of the document as a stream in order to avoid size issues.
     * @throws IOException Thrown if there is any problem with creating the stream.
     * @return The stream that yields the contents of the document.
     */    
    public InputStream getContentStream() throws IOException;
    
}
