/*
 * DocumentRetriever.java
 *
 * Created on February 8, 2003, 11:36 AM
 */

package edu.byu.deg.framework;

import java.net.URISyntaxException;

/** Defines an interface for retrieving documents from a specified location.
 * @author Alan
 */
public interface DocumentRetriever
{
  /** Sets the parameter that instructs the <CODE>DocumentRetriever</CODE> where to
   * look for documents to retrieve.
   * @param uri Indicates the location of the source for documents, or the starting point of a
   * crawler.
   * @throws URISyntaxException Thrown when the input string is not a valid URI.
   */  
  public void setRetrievalLocation(String uri) throws URISyntaxException;
  
  /** Stores a URI for a document that should be retrieved.
   * @param uri Location of the actual document to be retrieved.
   * @throws URISyntaxException Thrown when the input string is not a valid URI.
   */  
  public void addDocumentLocation(String uri) throws URISyntaxException;
  
  /** Lists the current set of document URIs.
   * @return <CODE>Iterator</CODE> containing the URIs as <CODE>String</CODE>s.
   */  
  public java.util.Iterator getDocumentLocations();
  
  /**
   * Retrieves documents from the stored locations and returns a collection of
   * <CODE>Document</CODE> objects.
   * @return Iterator containing <CODE>Document</CODE> objects representing documents that
   * have been retrieved.
   * @throws RetrieverConfigurationException Thrown if no retrieval locations have been specified via
   * {@link #setRetrievalLocation}.
   */  
  public java.util.Iterator retrieveDocuments() throws RetrieverConfigurationException;
  
  /**
   * Removes a single location from the list of retrieval locations.
   * @param uri The URI of the document to remove from the list.
   */  
  public void removeDocumentLocation(String uri);
  
  /** Resets the list of document URIs. */  
  public void clearDocumentLocations();
  
  /** Performs any necessary setup work before documents may be retrieved. */  
  public void initialize();
}

/*==============================================================================
 * $Author: wessmana $
 * $Revision: 1.8 $
 * $Date: 2005/04/20 05:33:48 $
 *
 * $Log: DocumentRetriever.java,v $
 * Revision 1.8  2005/04/20 05:33:48  wessmana
 * Code cleanup to get rid of Eclipse warnings.
 *
 * Revision 1.7  2005/04/09 21:01:32  wessmana
 * Merged from osmx-refactor branch.
 *
 * Revision 1.6.2.2  2005/04/01 05:15:26  wessmana
 * Import cleanup.
 *
 * Revision 1.6  2004/07/13 04:44:33  wessmana
 * Documentation.
 *
 * Revision 1.5  2004/02/17 06:57:31  wessmana
 * Added removeDocumentLocation().
 *
 * Revision 1.4  2004/02/13 08:17:01  wessmana
 * Added initialize method; retrieveDocuments() throws RetrieverConfigurationException.
 *
 * Revision 1.3  2003/02/14 05:01:28  wessmana
 * Created additional components of the framework.
 *
 * Revision 1.2  2003/02/12 04:44:50  wessmana
 * Added comments to bottom of file.
 *
 *
 *------------------------------------------------------------------------------
 * Copyright 2003, Brigham Young University
 */