/*
 * DocumentStructureParser.java
 *
 * Created on February 13, 2003, 8:43 PM
 */

package edu.byu.deg.framework;

/** Represents modules that are responsible for breaking down a {@link Document}
 * into more manageable chunks for extraction purposes.  Implementations determine
 * how complex the resulting document hierarchy will be, as well as whether the
 * output <CODE>Document</CODE> is the same (modified) object that was originally
 * parsed, or if a new <CODE>Document</CODE> is output instead.
 * @author Alan
 */
public interface DocumentStructureParser
{
  
    /** Transforms a "flat" document into a document hierarchy based on the input
     * document's structure.
     * @param d The "flat" document to be parsed.
     * @throws UnrecognizedFormatException Thrown if the document cannot be parsed because its structure is unrecognized by
     * the parser.
     * @return The document hierarchy; the root <CODE>Document</CODE> may or may not be the
     * same object that was input to the method.
     */    
  public Document parse(Document d) throws UnrecognizedFormatException;
  
}
