/*
 * DocumentStructureRecognizer.java
 *
 * Created on January 5, 2004, 11:00 PM
 */

package edu.byu.deg.framework;

/**
 * This class bears the responsibility of choosing a {@link DocumentStructureParser}
 * that can transform a single {@link Document} into a document tree suitable for
 * extraction.  As different <CODE>Document</CODE>s may have different
 * structures, this class differentiates between them to reduce the risk of
 * choosing the wrong parser and winding up with bad record boundaries.
 * @author Alan
 */
public interface DocumentStructureRecognizer {
  
    /**
     * Determines the <CODE>DocumentStructureParser</CODE> that is most suited for the
     * structure of the given <CODE>Document</CODE>.
     * @param doc The Document on whose structure the choice of parser is based.
     * @return The most appropriate parser for the structure of the input document.
     */    
  public DocumentStructureParser getDocumentParser(Document doc);
  
  /**
   * Returns a list of the available <CODE>DocumentStructureParser</CODE>s, ranked in
   * order from best (lowest index in list) to worst (highest index) with respect to
   * the input document.
   * @param doc The document on whose structure the ranking is based.
   * @return An array of parsers in order of their suitability for parsing the input document; best is earliest in the array.
   */  
  public DocumentStructureParser[] getLikelyDocumentParsers(Document doc);
  
}
