/*
 * ExtractionPlan.java
 *
 * Created on January 5, 2004, 10:33 PM
 */

package edu.byu.deg.framework;

/**
 * This abstract class represents an algorithm that performs the actual extraction
 * process (as opposed to initialization and finalization steps).  The actual
 * extraction algorithm is left to the subclass to define; it may be
 * hard-coded into the class itself, read from an external file, or be dynamically
 * constructed and executed by the subclass.
 * @author Alan
 */
public abstract class ExtractionPlan {
  
    /**
     * Reference back to the <CODE>DataExtractionEngine</CODE> that created and invoked
     * this extraction plan.  Useful for obtaining references to initialization
     * parameters.
     */    
  protected DataExtractionEngine engine;
  
  /**
   * Creates a new instance of ExtractionPlan
   * @param e The <CODE>DataExtractionEngine</CODE> that invoked the constructor.
   */
  public ExtractionPlan(DataExtractionEngine e) {
    engine = e;
  }
  
  /**
   * Executes the extraction step.
   * @throws PlanExecutionException Thrown when some failure during the extraction process prevents the process from
   * continuing.
   */  
  public abstract void execute() throws PlanExecutionException;
  
}
