/*
 * LexiconAccessor.java
 *
 * Created on August 1, 2003, 11:20 AM
 */

package edu.byu.deg.framework;

import java.net.URL;
import java.io.IOException;

/**
 * Defines a standard API for obtaining information from a lexicon.
 * @author troywalk
 */
public interface LexiconAccessor {

  /** Adds a word to the lexicon
   * @param word the word to be added
   */  
  public void addWord(String word);
 
  /** Looks for a string in the lexicon.
   * @param input the string to look up.  Must be lower case if not case sensitive
   * @param offset position at which to start
   * @return length of the longest match
   * @param inputToken first token in input.  This is the minimum match.
   */  
  public int lookup(String inputToken, String input, int offset);
  
  /**
   * Counts the number of entries in the lexicon.
   * @return The number of entries in the lexicon.
   */  
  public int getSize();
  
  /**
   * Provides access to the list of entries contained in the lexicon.
   * @return The list of this lexicon's entries.  It is assumed that the order of the list is
   * significant; commonly, it will be the order in which the entries appeared in the
   * lexicon file, but it might also be in dictionary order.
   */  
  public java.util.List getEntries();
  
  /**
   * Causes the contents of the lexicon to be loaded from the lexicon file.
   * @param lexURL Location of the lexicon file.
   * @throws IOException Thrown when an error occurs in opening or reading from the lexicon file.
   */  
  public void load(URL lexURL) throws IOException;
  
  /**
   * Identifies those entries in the lexicon that appear in the specified string.
   * @param input The string to search for occurrences of lexicon entries.
   * @param caseSensitive Specifies whether to consider case when determining occurrences of entries.
   * @return The set of lexicon entries (<CODE>String</CODE>s) that appear in the input.
   */  
  public java.util.Set findContainedEntries(String input, boolean caseSensitive);
}
