/*
 * OntologyWriter.java
 *
 * Created on April 23, 2004, 10:46 AM
 */

package edu.byu.deg.framework;

import java.io.Writer;

/**
 * Abstraction for components that are responsible for translating an ontology's
 * structure or instance data into some other format.  This generally should not
 * cause changes in the input ontology; the intended mechanism is more like a
 * snapshot
 * @author Alan Wessman
 */
public abstract class OntologyWriter {
    
    /**
     * Reference to the ontology to be output.
     */    
    protected Ontology ontology;
    
    /** Creates a new instance of OntologyWriter */
    public OntologyWriter() { /* default constructor */ }
    
    /**
     * Creates the <CODE>OntologyWriter</CODE> with an initial reference to an ontology.
     * @param ont The ontology to use for generating output.
     */    
    public OntologyWriter(Ontology ont) {
        setOntology(ont);
    }
    
    /**
     * Stores a reference to the ontology in the object.
     * @param ont The ontology to use for generating output.
     */    
    public void setOntology(Ontology ont) {
        ontology = ont;
    }
    
    /**
     * Outputs the structure of the ontology to the specified <CODE>Writer</CODE>.
     * @param w The destination for the model output.
     * @throws java.io.IOException Thrown if an error occurs on the I/O channel.
     */    
    public abstract void writeModelInstance(Writer w) throws java.io.IOException;
    
    /**
     * Outputs the instance data connected to the ontology to the specified
     * <CODE>Writer</CODE>.
     * @param w The destination for the data output.
     * @throws java.io.IOException Thrown if an error occurs on the I/O channel.
     */    
    public abstract void writeDataInstance(Writer w) throws java.io.IOException;
    
}
