/*
 * ValueMapper.java
 *
 * Created on January 5, 2004, 11:46 PM
 */

package edu.byu.deg.framework;

/**
 * Represents the module that links content matches to elements of the ontology,
 * infers nonlexical objects, infers relationships, and binds them all together in
 * a data instance.
 * @author Alan
 */
public interface ValueMapper extends OntologySubscriber {
      
    /**
     * Performs the value mappings on the default ontology and without regard for which
     * document the mapped values belong to.
     * @throws ValueMapperConfigurationException Thrown if the module is improperly configured when the mapping process is run,
     * particularly if no default ontology exists.
     */    
  public void generateValueMappings() throws ValueMapperConfigurationException;
  
  /**
   * Performs the value mappings on the specified ontology and without regard for which
   * document the mapped values belong to.
   * @param ont The ontology to use in the mapping process.
   */  
  public void generateValueMappings(Ontology ont);
  
  /**
   * Performs the value mappings on the default ontology, but only for those matches
   * that belong to the specified document or its children.
   * @param doc The root document of the document tree whose values are to be mapped.
   * @throws ValueMapperConfigurationException Thrown if the module is improperly configured when the mapping process is run,
   * particularly if no default ontology exists.
   */  
  public void generateValueMappings(Document doc) throws ValueMapperConfigurationException;
  
  /**
   * Performs the value mappings on the given ontology, but only for those matches
   * that belong to the specified document or its children.
   * @param ont The ontology to use for the mapping process.
   * @param doc The root document of the document tree whose values are to be mapped.
   */  
  public void generateValueMappings(Ontology ont, Document doc);
  
}
