/*
 * ValueRecognizer.java
 *
 * Created on February 4, 2004, 6:25 PM
 */

package edu.byu.deg.framework;

import java.net.URI;

/**
 * Represents the module that identifies within a document values that may be of
 * relevance to an ontology.
 * @author Alan Wessman
 */
public interface ValueRecognizer extends OntologySubscriber {
    
    /**
     * Locates values within the given document for the default ontology.
     * @param doc The <CODE>Document</CODE> to search for values.
     * @throws ValueRecognizerConfigurationException Thrown if the <CODE>ValueRecognizer</CODE> is improperly configured when the
     * value recognition process is run, particularly if no default ontology has been
     * set.
     */    
    public void findValues(Document doc) throws ValueRecognizerConfigurationException;
    
    /**
     * Locates values within the given document for the specified ontology.
     * @param ontology The ontology to use during the value recognition process.  It provides the
     * <CODE>ValueRecognizer</CODE> with information about which values in the document
     * are likely to be relevant.
     * @param doc The <CODE>Document</CODE> to search for values.
     */    
    public void findValues(Ontology ontology, Document doc);
    
    /**
     * Locates values within the given string for the default ontology.
     * @param contents The string to search for values.
     * @param sourceURI The URI that identifies the source of the content string.  This is needed to
     * distinguish matches that originate from different content strings.
     */    
    public void findValues(String contents, URI sourceURI);
    
    /**
     * Locates values within the given string for the specified ontology.
     * @param ontology The ontology to use during the value recognition process.  It provides the
     * <CODE>ValueRecognizer</CODE> with information about which values in the document
     * are likely to be relevant.
     * @param contents The string to search for values.
     * @param sourceURI The URI that identifies the source of the content string.  This is needed to
     * distinguish matches that originate from different content strings.
     */    
    public void findValues(Ontology ontology, String contents, URI sourceURI);
    
}
