/*
 * DOMDocument.java
 *
 * Created on August 18, 2003, 11:59 AM
 */

package edu.byu.deg.framework.document;
import org.w3c.dom.Node;
/**
 *
 * @author  troywalk
 */
import org.w3c.tidy.Tidy;
import java.io.*;
import java.util.ArrayList;
import edu.byu.deg.util.DOMFilter;
import edu.byu.deg.util.DOMString;
import java.net.URI;
public class DOMDocument implements edu.byu.deg.framework.TextDocument {
  private Node root;
  private ArrayList subDocuments = new ArrayList();
  static Tidy tidy = null;
  private DOMString domstr = null;
  private URI uri;
  private BufferedInputStream in = null;
  private org.w3c.dom.Document parentDoc;
  private double confidence;
  private static URI getURI(String f){
    f = f.replace('\\','/');
    if(f.indexOf("//")==-1){
      if(f.charAt(0)!='/')
        f = "/" + f;
      f = "file://" + f;
    }
    try{
      return new URI(f);
    } catch(java.net.URISyntaxException e){
      System.err.println("Not a valid URI: " + f);
      try{
        return new URI("");
      } catch(Exception impossible){return null;}
    }
  }
  
  /** Creates a new instance of DOMDocument */
  public DOMDocument(String filename) {
      this(getURI(filename));
  }
  
  public DOMDocument(URI u) {
    uri = u;
    if(tidy==null){
      tidy = new Tidy();
      tidy.setShowWarnings(false);
      tidy.setQuiet(true);
    }
    try {
      in = new BufferedInputStream(uri.toURL().openStream());
      parentDoc = tidy.parseDOM(in, null);
    } catch(Exception e) {
      parentDoc = Tidy.createEmptyDocument();
      e.printStackTrace();
    }
    root = parentDoc;
    DOMFilter.filter(root);
  }
  
  public DOMDocument(Node node, org.w3c.dom.Document parent){
    root = node;
    domstr = new DOMString(root);
    try {
      uri = new URI("doc" + node.hashCode());
    } catch(Exception e) {
      e.printStackTrace();
    }
    parentDoc = parent;
  }
  
  public Node getRoot(){
    return root;
  }
  
  public DOMString getDOMString(){
    if(domstr==null)
      domstr = new DOMString(root);
    return domstr;
  }
  
  public java.util.Iterator getSubDocuments() {
    return subDocuments.iterator();
  }
  
  public DOMDocument getSubDocument(int index) {
      return (DOMDocument) subDocuments.get(index);
  }
  
  public void addSubDocuments(ArrayList l){
    java.util.Iterator i=l.iterator();
    while(i.hasNext()) {
        addSubDocument((Node) i.next());
    }
  }
  
  public void addSubDocument(Node node){
    DOMDocument child = new DOMDocument(node, parentDoc);
    subDocuments.add(child);
    child.setURI(getURI().toString() + "/" + (getSubDocumentCount() + 1));
    //System.out.println("------------------------------------");
    //System.out.print(child);
    //System.out.println("------------------------------------");
    
  }
  
  public void setSubDocument(int i, Node node){
    DOMDocument child = new DOMDocument(node, parentDoc);
    subDocuments.set(i, child);
    child.setURI(getURI().toString() + "/" + i);
    //System.out.println("------------------------------------");
    //System.out.print(child);
    //System.out.println("------------------------------------");
    
  }

  
  public java.net.URI getURI() {
    return uri;
  }
  
  public void setURI(java.net.URI u) {
      uri = u;
  }
  
  public void setURI(String s) {
      try {
          java.net.URI u = new java.net.URI(s);
          uri = u;
      } catch (Exception e) {
          System.err.println("Not a valid URI: " + s);
      }
  }
  
  public org.w3c.dom.Document getParentDocument(){
    return parentDoc; 
  }
  
  public int getSubDocumentCount(){
    return subDocuments.size();
  }
  
  public String toString(){
    if(subDocuments.size()==0){
      return getDOMString().toString();
    }
    StringBuffer b = new StringBuffer();
    b.append("------------\n");
    for(int i=0;i<subDocuments.size();i++){
      b.append(subDocuments.get(i).toString());
      b.append("\n------------\n");
    }
    b.append("size: " + subDocuments.size());
    return b.toString();
  }
  
  public InputStream getContentStream() {
      return in;
  }
  
  public String getContentString() {
    return getDOMString().toString();
  }
  public void setConfidence(double conf){
    confidence = conf;
  }
  
  public double getConfidence(){
    return confidence;
  }
}
