/*
 * PlaintextDocument.java
 *
 * Created on February 20, 2004, 1:30 AM
 */

package edu.byu.deg.framework.document;

import edu.byu.deg.framework.Document;
import edu.byu.deg.framework.TextDocument;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.net.URI;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


/**
 *
 * @author  Alan Wessman
 */
public class PlaintextDocument implements TextDocument {
    
    private List subDocuments;
    private StringBuffer contents;
    private URI uri;
    
    /** Creates a new instance of PlaintextDocument */
    public PlaintextDocument() {
        subDocuments = new LinkedList();
    }
    
    public String getContentString() {
        return contents.toString();
    }
    
    public InputStream getContentStream() {
        return new InputStream() {
            private final Reader contentReader = new BufferedReader(new StringReader(getContentString()));
            public int read() throws IOException {
                return contentReader.read();
            }
        };
    }
    
    public Iterator getSubDocuments() {
        return subDocuments.iterator();
    }
    
    public URI getURI() {
        return uri;
    }
    
    public void setURI(URI newURI) {
        uri = newURI;
    }
    
    public void addSubDocument(Document doc) {
        subDocuments.add(doc);
    }
    
    public void setContents(String s) {
        contents = new StringBuffer(s);
    }
    
    public void setContents(StringBuffer sb) {
        contents = sb;
    }
    
    public void setContents(Reader r) throws IOException {
        contents = new StringBuffer();
        BufferedReader reader = new BufferedReader(r);
        int readVal;
        do {
            readVal = reader.read();
            if (readVal >= 0) contents.append((char) readVal);
        } while (readVal != -1);
    }
    
    public void append(String s) {
        if (contents == null) contents = new StringBuffer();
        contents.append(s);
    }
    
    public void append(StringBuffer sb) {
        if (contents == null) contents = sb;
        else contents.append(sb);
    }
    
    public void append(Reader r) throws IOException {
        if (contents == null) setContents(r);
        else {
            BufferedReader reader = new BufferedReader(r);
            int readVal;
            do {
                readVal = reader.read();
                contents.append(readVal);
            } while (readVal != -1);
        }
    }
    
}
