/*
 * FillData.java
 *
 * Created on February 5, 2004, 1:42 PM
 */

/**
 *
 * @author  troywalk
 */
package edu.byu.deg.geneTIQS;

import java.util.*;
import javax.xml.bind.JAXBException;

import edu.byu.deg.framework.document.DOMDocument;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;

import edu.byu.deg.osmx.binding.MatchedTextType;
import edu.byu.deg.util.DOMString;
import edu.byu.deg.query.OSMQuery;
public class FillData {
  private static final int NO_LOCATION = -1;
  DOMDocument documents;
  OSMXDocument ontology;
  HashMap boundaries;
  HashMap used;
  /** Creates a new instance of FillData */
  public FillData() {
  }
  
  private boolean canUse(DOMDocument doc, MatchedTextType match){
    if(used.get(match)!=null)
      return false;
    StartEnd se = (StartEnd)boundaries.get(doc);
    if(match.getStartPos()<se.start)
      return false;
    if(match.getEndPos()>se.end)
      return false;
 
    Object o = edu.byu.deg.ontos.OverlapFilter.conflicts.get(match);
    if(o!=null){
      ArrayList conflicts = (ArrayList)o;
      for(int j=0;j<conflicts.size();j++){
        o = conflicts.get(j);
        if(match==o)
          continue;
        o = used.get(o);
        if(o!=null) return false;
      }
    }
    return true;
  }
  
  private ObjectType makeObject(MatchedTextType match, ObjectSetType obj, boolean mark) throws JAXBException{
    ObjectType object = ontology.getObjectFactory().createOSMObject();
    object.setObjectSet(obj.getId());
    object.setLocation("");
    PositionedText name = ontology.getObjectFactory().createPositionedText();
    name.setContent(obj.getName());
    object.setName(name);
    if(match!=null){
      if(mark)
        used.put(match, match);
      object.setValue(match.getValue());
    } else {
      object.setValue(obj.getId());
    }
    return object;
  }
  
  private ObjectBindingType makeObjectBinding(String conID, String objID) throws JAXBException{
    ObjectBindingType binding = ontology.getObjectFactory().createObjectBindingType();
    binding.setConnectionRef(conID);
    binding.setObjectRef(objID);
    return binding;
  }
  
  private ObjectType buildObject(OSMXObjectSetType obj, int location, DOMDocument doc, String parentID, SourceDocument source) throws JAXBException{
    ObjectType object = null;
    DataFrameType df = obj.getDataFrame();
    if(df!=null){
      List keywords = df.getKeywordPhrase();
      if(keywords!=null){ //create an object if we find an appropriate keyword match.  Update the location
        Iterator i = keywords.iterator();
        while(i.hasNext()&&object==null){
          KeywordPhraseType keyword = (KeywordPhraseType)i.next();
          Iterator j = keyword.getKeywordExpression().getMatchedText().iterator();
          while(j.hasNext()&&object==null){
            MatchedTextType match = (MatchedTextType)j.next();
            if(canUse(doc, match)){
              object = makeObject(match, obj, true);
              location = (match.getStartPos()+match.getEndPos())/2;
            }
          }
        }
        if(keywords.size()>0&&object==null)
          return null;
      }
      List vps = df.getValuePhrase();
      if(vps!=null){  //get the value closest to location
        Iterator i = vps.iterator();
        MatchedTextType bestMatch = null;
        int bestDistance = Integer.MAX_VALUE;
        while(i.hasNext()){
          ValuePhraseType vp = (ValuePhraseType)i.next();
          Iterator j = vp.getValueExpression().getMatchedText().iterator();
          while(j.hasNext()){
            MatchedTextType match = (MatchedTextType)j.next();
            if(canUse(doc, match)){
              int distance = Math.abs(location - (match.getStartPos() + match.getEndPos())/2);
              if(distance<bestDistance){
                 bestMatch = match;
                 bestDistance = distance;
              }
            }
          }
        }
        if(bestMatch!=null){
          if(object==null)
            object = makeObject(bestMatch, obj, true);
            object.setValue(bestMatch.getValue());
            int newlocation = (bestMatch.getStartPos() + bestMatch.getEndPos())/2;
            object.setOrder(Math.abs(location - newlocation));
            location = newlocation;
            used.put(bestMatch, object);
            used.put(object, bestMatch);
        }
      }
    }
    if(object==null&&obj.isLexical())
      return null;
    Iterator i = ontology.getModelRoot().getAllModelElements().iterator();
    boolean haveadded = false;
    while(i.hasNext()) {
      Object o = i.next();
      if(o instanceof DataInstance)
        break;
      if(o instanceof OSMXRelationshipSetType){
        OSMXRelationshipSetType relset = (OSMXRelationshipSetType)o;
        OSMXRelSetConnectionType c = relset.getConnectionFor(obj);
        if(c!=null){
          boolean nomore = false;
          int count = 0;
          int max = c.getMax();
          while(count<max && !nomore){
            ArrayList objs = new ArrayList();
            ArrayList cons = new ArrayList();
            for(Iterator j = relset.getRelSetConnection().iterator(); j.hasNext(); ) {
              o = j.next();
              if((c!=o) && (o instanceof OSMXRelSetConnectionType)){
                OSMXRelSetConnectionType con = (OSMXRelSetConnectionType)o;
                String nextID = con.getObjectSet();
                if(nextID.equals(parentID))
                  nomore=true;
                cons.add(con);
              }
            }
            if(nomore==true)
              continue;
            for(int j=0;j<cons.size();j++){
              OSMXRelSetConnectionType con = (OSMXRelSetConnectionType)cons.get(j);
              objs.add(this.buildObject((OSMXObjectSetType)ontology.getElement(con.getObjectSet()), location, doc, obj.getId(), source));
            }
            if(objs.size()==0)
              nomore = true;
            for(int a=0;a<objs.size();a++)
              if(objs.get(a)==null)
                nomore = true;
            if(!nomore){
              Relationship relationship = ontology.getObjectFactory().createRelationship();
              relationship.setRelationshipSet(relset.getId());
              List l = relationship.getObjectBinding();
              if(object==null)
                object = makeObject(null, obj, true);
              l.add(makeObjectBinding(c.getId(), object.getId()));
              for(int a=0;a<objs.size();a++){
                RelSetConnectionType con = (RelSetConnectionType)cons.get(a);
                ObjectType nextObj = (ObjectType)objs.get(a);
                l.add(makeObjectBinding(con.getId(), nextObj.getId()));
              }
              PositionedText name = ontology.getObjectFactory().createPositionedText();
              name.setContent(relset.getName().getContent());
              relationship.setName(name);
              source.getDataInstanceElements().add(relationship);
              count++;
              haveadded = true;
            }
          }
        }
      }
    }
    if(object==null||!(obj.isLexical()||haveadded))
      return null;
    source.getDataInstanceElements().add(object);
    return object;
  }
  
  private void setBoundaries(){
    DOMString str = documents.getDOMString();
    Iterator i = documents.getSubDocuments();
    while(i.hasNext()){
      DOMDocument d = (DOMDocument)i.next();
      DOMString substr = d.getDOMString();
      int start = str.getStartOf(substr.getFirstText());
      int end = start + substr.length();
      boundaries.put(d, new StartEnd(start, end));
    }
  }

  public void fillData(DOMDocument d, OSMXDocument ont){
    fillData(d, ont, null);
  }
  
  public void fillData(DOMDocument d, OSMXDocument ont, OSMQuery query){
    documents = d;
    boundaries = new HashMap();
    ontology = ont;
    used = new HashMap();
    OSMXObjectSetType primary = ont.getPrimaryObjectSet();
    if(primary==null)
      return;
    setBoundaries();
    Iterator i = documents.getSubDocuments();
    try{
      DataInstance instance = ontology.getObjectFactory().createDataInstance();
      instance.setId(d.getURI().toString());
      while(i.hasNext()){
        SourceDocument source = ontology.getObjectFactory().createSourceDocument();
        source.setUri(documents.getURI().toString());
        if(buildObject(primary, NO_LOCATION, (DOMDocument)i.next(), null, source)!=null){
          if(query==null||query.satisfiesQuery(source))
            instance.getDataInstanceElements().add(source);
        }
      }
      if(instance.getDataInstanceElements().size()>0){
        instance.setConfidenceTag(Double.toString(d.getConfidence()));
        ontology.getModelRoot().getAllModelElements().add(instance);
      }
      
    }catch(Exception e){ e.printStackTrace(); }
  }
  
  class StartEnd{
    int start;
    int end;
    public StartEnd(int s, int e){
      start = s;
      end = e;
    }
  }
}
