/*
 * ExpressionMatcher.java
 *
 * Created on August 6, 2003, 6:42 PM
 */

package edu.byu.deg.ontos;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.regex.*;
import edu.byu.deg.ontos.lexicon.LexiconMatcher;

/**
 *
 * @author  troywalk
 */
public class ExpressionMatcher {
  Vector matchers;
  LexiconMatcher lexiconMatcher;
  
  /** Creates a new instance of ExpressionMatcher */
  public ExpressionMatcher(String expression, LexiconMatcher lexiconMatcher, MacroMatcher macroMatcher, int flags) {
    matchers = new Vector();
    StringTokenizer st = new StringTokenizer(expression, "{}", true);
    boolean isLabel = false;
    String tok;
    String pattern;
    this.lexiconMatcher = lexiconMatcher;
    try{
      StringBuffer b = new StringBuffer();
      while(st.hasMoreTokens()){
        tok = st.nextToken();
        if(tok.equals("{")){
          if(isLabel)
            System.err.println("Error parsing expression '" + expression + "':  Nesting not allowed");
          isLabel = true;
        }
        else if(tok.equals("}")){
          isLabel = false;
        }
        else if(isLabel){
          if(tok.charAt(0)>='0'&&tok.charAt(0)<='9'){
            //token is a count expression in the regular expression
            b.append('{');
            b.append(tok);
            b.append('}');
          }
          else{
            //first, compile and clear the previous expression
            if(b.length()>0){
              matchers.add(Pattern.compile(b.toString(), flags).matcher(null));
            }
            b.setLength(0);
            //token is within {}, look up the macro or lexicon
            pattern = macroMatcher.getPattern(tok);
            if(pattern==null){
              if(lexiconMatcher.exists(tok)){
                matchers.add(tok);
              }
              else {
                System.err.println("Error:  Macro/Lexicon {" + tok + "} not found");
              }
            }
            else {
              b.append('(');
              b.append(pattern);
              b.append(')');
            }
          }
        }
        else {
          //token is a regular expression
          b.append(tok);
        }
      }
      if(b.length()!=0)
        matchers.add(Pattern.compile(b.toString(), flags).matcher(null));
      
    } catch (PatternSyntaxException e){
      System.err.println(e.getMessage());
      matchers.setSize(0);
    }
  }
  /* OBSOLETE? AW
  public int getMatchAt(String str, int offset){
    Object o;
    Matcher m;
//    String lexLabel;
    LexiconMatch lexMatch;
    int matchLength = 0;
    String sub;
    for(Iterator i=matchers.iterator(); i.hasNext(); ){
      o = i.next();
      if(o instanceof Matcher){
        sub = str.substring(offset + matchLength);
        m = ((Matcher)o);
        m.reset(sub);
        if(!m.lookingAt())
          return -1;
        matchLength += m.end();
      }
      else {
        lexMatch = lexiconMatcher.getMatchAt((String)o, str, offset+matchLength);
        if(lexMatch==null)
          return -1;
        matchLength += lexMatch.getLength();
      }
    }
    return matchLength;
  }
  
  public int start = -1;
  public int getMatchAfter(String str, int offset){
    Iterator i=matchers.iterator();
    Object first = i.next();
    Object o;
    Matcher m;
    LexiconMatch lexMatch;
    String sub;
    boolean isMatch;
    if(first==null)
      return -1;
    start = offset;
    int matchLength;
    while(start<str.length()){
      if(first instanceof Matcher){
        m = ((Matcher)first);
        m.reset(str);
        if(!m.find(start)){
          start = -1;
          return -1;
        }
        start = m.start();
        matchLength = m.end() - start;
      }
      else{
        lexMatch = lexiconMatcher.getMatchAfter((String)first, str, start);
        if(lexMatch==null)
          return -1;
        start = lexMatch.getStartPosition();
        matchLength = lexMatch.getLength();
      }
      isMatch = true;
      i = matchers.iterator();
      i.next();  //skip first element (already matched)
      while(i.hasNext()){
        o = i.next();
        if(o instanceof Matcher){
          sub = str.substring(start + matchLength);
          m = ((Matcher)o);
          m.reset(sub);
          if(!m.lookingAt()){
            isMatch = false;
            break;
          }
          matchLength += m.end();
        }
        else {
          lexMatch = lexiconMatcher.getMatchAt((String)o, str, start+matchLength);
          if(lexMatch==null){
            isMatch = false;
            break;
          }
          matchLength += lexMatch.getLength();
        }
      }
      if(isMatch){
        return matchLength;
      }
      start = start + matchLength;
    }
    return -1;
  }
   */
  
  String doReplace(String str, String to){
    Object o;
    if(matchers.size()!=1||(o = matchers.get(0)) instanceof String){
      System.err.println("Unsupported regular expression in SubFromExpression, no substitiution made.");
      return str;
    }
    Matcher m = (Matcher)o;
    m.reset(str);
    return m.replaceAll(to);
  }
}
