/*
 * HTMLFilter.java
 *
 * Created on February 19, 2004, 10:15 PM
 */

package edu.byu.deg.ontos;

import edu.byu.deg.framework.ContentFilter;
import edu.byu.deg.framework.Document;
import edu.byu.deg.framework.document.DOMDocument;
import edu.byu.deg.framework.document.PlaintextDocument;
import edu.byu.deg.util.HTMLStripper;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;

/**
 * A <CODE>ContentFilter</CODE> implementation that strips all HTML tags from the
 * input.
 * @author Alan Wessman
 */
public class HTMLFilter implements ContentFilter {
    
    private HTMLStripper htmlCleaner;
    
    /** Creates a new instance of HTMLFilter */
    public HTMLFilter() {
        htmlCleaner = new HTMLStripper();
    }
    
    /**
     * Strips HTML tags from the contents of the input <CODE>Document</CODE>, returning
     * a new {@link edu.byu.deg.framework.document.PlaintextDocument}.
     * @param doc The input document whose contents we wish to clean of HTML markup.
     * @return A new <CODE>PlaintextDocument</CODE> containing no HTML tags.
     */    
    public Document filterDocument(Document doc) {
        PlaintextDocument retDoc = new PlaintextDocument();
        retDoc.setURI(doc.getURI());
        try {
            if (doc instanceof DOMDocument) {
                DOMDocument domDoc = (DOMDocument) doc;
                retDoc.setContents(domDoc.getDOMString().toString());
            } else {
                retDoc.setContents(htmlCleaner.stripHTML(stringFromStream(doc.getContentStream())));
            }
            // normalize whitespace
            retDoc.setContents(retDoc.getContentString().replaceAll("(\\s)+", " "));
        } catch (IOException ioe) {
            /* REVISIT: handle better? */
        }
        
        for (Iterator i = doc.getSubDocuments(); i.hasNext();) {
            Document subDoc = filterDocument((Document) i.next());
            retDoc.addSubDocument(subDoc);
        }
        return retDoc;
    }
    
    protected static String stringFromStream(InputStream in) throws IOException {
        StringBuffer contents = new StringBuffer();
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        int readVal;
        do {
            readVal = reader.read();
            if (readVal >= 0) contents.append((char) readVal);
        } while (readVal != -1);
        return contents.toString();
    }
    
}
