/*
 * OverlapFilter.java
 *
 * Created on August 18, 2003, 4:12 PM
 */

package edu.byu.deg.ontos;

import java.util.Iterator;
import edu.byu.deg.osmx.binding.*;
import edu.byu.deg.osmx.binding.MatchedTextType;
import edu.byu.deg.osmx.OSMXDocument;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;
/**
 *
 * @author  troywalk
 */
public class OverlapFilter {
  public static HashMap conflicts;
  private static int distanceToKeyword(MatchedTextType match, List keywordlist){
    int best = Integer.MAX_VALUE;
    for(Iterator i=keywordlist.iterator(); i.hasNext();){
      KeywordPhraseType keyword = (KeywordPhraseType)i.next();
      List l = keyword.getKeywordExpression().getMatchedText();
      if(l==null)
        break;
      for(Iterator j=l.iterator();j.hasNext();){
        MatchedTextType keywordMatch = (MatchedTextType)j.next();
        int dist1 = Math.abs(match.getEndPos() - keywordMatch.getStartPos());
        int dist2 = Math.abs(keywordMatch.getEndPos() - match.getStartPos());
        if(dist2<dist1)
          dist1 = dist2;
        if(dist1<best)
          best = dist1;
      }
    }
    return best;
  }
  
  private static int distanceToKeyword(MatchedTextType match, OSMXDocument ontology){
    Object o;
    DataFrameType df;
    ValuePhraseType valuePhrase;
    List keywords;
    for(Iterator i=ontology.getModelRoot().getAllModelElements().iterator(); i.hasNext(); ){
      o = i.next();
      if(o instanceof ObjectSet){
        boolean thisistheobject = false;
        df = ((ObjectSet)o).getDataFrame();
        if(df==null)
          continue;
          for(Iterator j=df.getValuePhrase().iterator(); j.hasNext(); ){
            valuePhrase = (ValuePhraseType)j.next();
            if(valuePhrase.getValueExpression().getMatchedText().contains(match)){
              keywords = valuePhrase.getKeywordPhrase();
              if(keywords!=null)
                return distanceToKeyword(match, keywords);
              keywords = df.getKeywordPhrase();
              if(keywords!=null)
                return distanceToKeyword(match, keywords);
              thisistheobject = true;
            }
        }
        List kp = df.getKeywordPhrase();
        if(thisistheobject&&kp!=null){
          return distanceToKeyword(match, kp);
        }
        return Integer.MAX_VALUE;
      }
    }
    return Integer.MAX_VALUE;
  }
  
  private static boolean lessthan(MatchedTextType r1, MatchedTextType r2, OSMXDocument ontology){
    if(r1==null)
      return false;
    if(r2==null)
      return true;
    int start1 = r1.getStartPos();
    int start2 = r2.getStartPos();
    int end1 = r1.getEndPos();
    int end2 = r2.getEndPos();
    if(start1<start2)
      return true;
    if(start1>start2)
      return false;
    //if the matches start at the same place, we want the longer one to be first
    if(end1>end2)
      return true;
    if(end1<end2)
      return false;
    //if they have the same length, find the one with the closest keyword
    if(distanceToKeyword(r1, ontology)<distanceToKeyword(r2, ontology))
      return true;
    else
      return false;
  }
  
  private static MatchedTextType[] mergeSort(ArrayList groups, int start, int end, OSMXDocument ontology){
    MatchedTextType[] result;
    MatchedTextType r1, r2;
    int s1,s2,i1=0,i2=0,index=0;
    if(end-start<=1){
      List g1;
      List g2;
      g1 = (List)groups.get(start);
      if(end==start){
        result = new MatchedTextType[g1.size()];
        for(int i=0;i<result.length;i++)
          result[i] = (MatchedTextType)g1.get(i);
        return result;
      }
      g2 = (List)groups.get(end);
      s1 = g1.size();
      s2 = g2.size();
      result = new MatchedTextType[s1+s2];
      r1 = (MatchedTextType)g1.get(i1);
      r2 = (MatchedTextType)g2.get(i2);
      while(i1!=s1||i2!=s2){
        if(lessthan(r1, r2, ontology)){
          result[index++] = r1;
          i1++;
          if(i1<s1)
            r1 = (MatchedTextType)g1.get(i1);
          else
            r1 = null;
        } else {
          result[index++] = r2;
          i2++;
          if(i2<s2)
            r2 = (MatchedTextType)g2.get(i2);
          else
            r2 = null;
        }
      }
      System.out.print("");
    }
    else{
      MatchedTextType[] a1 = mergeSort(groups, start, (end+start)/2, ontology);
      MatchedTextType[] a2 = mergeSort(groups, (end+start)/2+1, end, ontology);
      s1 = a1.length;
      s2 = a2.length;
      result = new MatchedTextType[s1+s2];
      r1 = a1[0];
      r2 = a2[0];
      while(i1!=s1||i2!=s2){
        if(lessthan(r1, r2, ontology)){
          result[index++] = r1;
          i1++;
          if(i1<s1)
            r1 = a1[i1];
          else
            r1 = null;
        } else {
          result[index++] = r2;
          i2++;
          if(i2<s2)
            r2 = a2[i2];
          else
            r2 = null;
        }
      }
      if(i1<s1)
        System.arraycopy(a1, i1, result, index, s1-i1);
      else if (i2<s2)
        System.arraycopy(a2, i2, result, index, s2-i2);
    }
    return result;
  }
  
  public static void filter(OSMXDocument ontology){
    conflicts = new HashMap();
    ArrayList groups = new ArrayList();
    HashMap container = new HashMap();
    List group;
    Object o;
    DataFrameType df;
    ValuePhraseType valuePhrase;
    //collect all matches in a sorted array
    for(Iterator i=ontology.getModelRoot().getAllModelElements().iterator(); i.hasNext(); ){
      o = i.next();
      if(o instanceof ObjectSet){
        df = ((ObjectSet)o).getDataFrame();
        if(df==null)
          continue;
          for(Iterator j=df.getValuePhrase().iterator(); j.hasNext(); ){
            valuePhrase = (ValuePhraseType)j.next();
            group = valuePhrase.getValueExpression().getMatchedText();
            for(int k = 0;k<group.size();k++){
              container.put(group.get(k), group);//hash the list containing each match
            }
            if(group.size()>0)
              groups.add(group);
          }
      }
    }
    if(groups.size()==0)
      return;
    MatchedTextType[] matches = mergeSort(groups, 0, groups.size()-1, ontology);
    MatchedTextType m1, m2;
    
    //detect and delete overlaps
    for(int i=0;i<matches.length;i++){
      m1 = matches[i];
      if(m1==null)
        continue;
      for(int j=i+1;j<matches.length;j++){
        m2 = matches[j];
        if(m2==null)
          continue;
        if(m2.getStartPos()>m1.getEndPos())
          break;
        if(m1.getStartPos()==m2.getStartPos()&&m1.getEndPos()==m2.getEndPos()){
          ArrayList conflictlist = (ArrayList)conflicts.get(m1);
          if(conflictlist==null)
            conflictlist = new ArrayList();
          conflictlist.add(m1);
          conflictlist.add(m2);
          conflicts.put(m1, conflictlist);
          conflicts.put(m2, conflictlist);
        }
        else
          ((List)container.get(m2)).remove(m2);
        matches[j] = null;
      }
    }
  }
}
