/*
 * LexiconMatcher.java
 *
 * Created on August 1, 2003, 4:09 PM
 */

package edu.byu.deg.ontos.lexicon;

/** Class for storing a match.
 * @author troywalk
 */
public class LexiconMatch {
  
  private String sourceString;
  private int	startPosition;
  private int	length;
  private float	confidence;

  /** Creats a LexiconMatch instance
   * @param source string that the match was found in
   * @param start beginning offset of the match
   * @param len length of the match
   * @param conf confidence of the match
   */  
  public LexiconMatch(String source, int start, int len, float conf) {
    sourceString = source;
    startPosition = start;
    length = len;
    confidence = conf;
  }

  /** Creats a LexiconMatch instance with a default confidence of 1.0
   * @param source string that the match was found in
   * @param start beginning offset of the match
   * @param len length of the match
   */  
  public LexiconMatch(String source, int start, int len) {
          this(source, start, len, 1);
  }

  /** Gets the start of the match
   * @return starting offset of the match
   */  
  public int getStartPosition(){
    return startPosition;
  }

  /** Gets the length of the match
   * @return length of the match
   */  
  public int getLength(){
    return length;
  }
  
  /** gets the end position of the match
   * @return ending offset of the match
   */  
  public int getEndPosition(){
    return startPosition + length;
  }

  /** Creates a string containing the string that was matched.
   * @return string that was matched
   */  
  public String getMatchString() {
    return sourceString.substring(startPosition, startPosition+length);
  }

  /** Returns the confidence of the match.
   * @return the confidence of the match.
   */  
  public float getConfidence(){
    return confidence;
  }
}