%{
static char const rcsid[] =
        "$Id: harmony.lex,v 1.1.1.2 2005/07/01 22:44:05 mja47 Exp $";

/*========================================================================
 * $RCSfile: harmony.lex,v $
 *
 * $Author: mja47 $
 * $Date: 2005/07/01 22:44:05 $
 * $Revision: 1.1.1.2 $
 *
 * $Description: 
 * Harmony lexer source, borrowed from Melody lexer. $
 * 
 * $Log: harmony.lex,v $
 * Revision 1.1.1.2  2005/07/01 22:44:05  mja47
 * no message
 *
 * Revision 1.1.1.1  2003/08/09 20:48:42  pwinward
 * no message
 *
 * Revision 1.1.1.1  1999/05/11 15:16:49  swliddle
 * Ontos.rev2 stuff
 *
 * Revision 1.5  1998/02/05  22:20:14  liddl
 * Checkpoint of reasonably decent version.
 *
 * Revision 1.4  1998/02/05 18:24:02  liddl
 * Working version for VLDB Ontology paper.
 *
 * Revision 1.3  1996/04/24  13:43:54  liddl
 * About to hand off to Ting.
 *
 * Revision 1.2  1995/10/20  23:03:57  liddl
 * Major integration of Melody/OSM-SW.
 *
 * Revision 1.1  1995/10/16  21:43:58  liddl
 * Initial revision
 *
 */

/*------------------------------------------------------------------------
 *                      INCLUDES
 */
#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <ctype.h>

#include "ontology.H"
#include "semantic.H"
#include "lex_sym.h"

/*------------------------------------------------------------------------
 *                      CONSTANTS
 */
#define INFORMAL_CHUNK	1000

/*------------------------------------------------------------------------
 *                      MACROS
 */
#define DEBUG 1
#ifdef DEBUG
#  if DEBUG
#    define P(x)  Prev_col = Col_num; Col_num += strlen(YYText()); \
		  if (Debug_lex) \
                    fprintf(stderr, "line %ld: %-17s %s\n", Line_num, \
			    x, YYText())
#  else
#    define P(x)  Prev_col = Col_num; Col_num += strlen(YYText());
#  endif
#else
#  define P(x)   Prev_col = Col_num; Col_num += strlen(YYText());
#endif

#define DUP_STR	yylval.p_char = Dup_String(YYText(), YYLeng())

#define KEY(x) if (Case_Is_Key()) { \
                 P(#x); return (x); \
               } else { \
		 REJECT; \
               }

/*------------------------------------------------------------------------
 *                      STATIC GLOBAL VARIABLES
 */
static long    		Comment_level = 0;

/*------------------------------------------------------------------------
 *                      GLOBAL VARIABLES
 */
int			Debug_lex = 0;
long           		Col_num = 1,
               		Line_num = 1,
               		Prev_col = 1,
               		Prev_line = 1;
char           		*Parse_str = NULL;

/*------------------------------------------------------------------------
 *                      GLOBAL VARIABLES
 */
extern FlexLexer	*Harmony_lexer;

%}

%option		reject
%e              2000
%a              4000
%p              5000
%o              6000
%x              COMMENT INFORML

digit           ([0-9])
nonnegint       ({digit}+)
float           ({nonnegint}"."{nonnegint})

upper           ([A-Z])
lower           ([a-z])
special1        ("%"|"("|")"|"*"|"+"|","|"-"|"."|"/"|"\\"|";")
special2        ("<"|"="|">"|"@@"|"["|"]"|"^"|"{"|"|"|"}"|":")
special3        ("!"|"$"|"&"|"`"|"#"|"?")
special         ({special1}|{special2}|{special3}|"_"|"~")
symbol          ({special3}|"\\"{special})
upperletter     ({upper}|"\\"{upper}|{symbol}|"\\"{digit})
lowerletter     ({lower}|"\\"{lower})
any_letter      ({upperletter}|{lowerletter}|{digit}|"\\")
quote           (["])
tic             (['])
space           ([ ])
character       ({upper}|{lower}|{digit}|{special}|{space})
letter_nt       ({character}|{quote})
letter_nq       ({character}|{tic})

upperword       (({upperletter}{any_letter}*)|("~"+{any_letter}+))
lowerword       (({lowerletter}{any_letter}*)|("_"+{any_letter}+))

qstring         (({tic}{letter_nt}*{tic})|({quote}{letter_nq}*{quote}))

white           ([ \t\f\r\n]*)
iswsa           ([iI][sS]{white}[aA])

case		([cC][aA][sS][eE]{white})
sensitive	([sS][eE][nN][sS][iI][tT][iI][vV][eE])

caseinsens	({case}[iI][nN]{sensitive})
casesens	({case}{sensitive})
context         ([cC][oO][nN][tT][eE][xX][tT])
constant        ([cC][oO][nN][sS][tT][aA][nN][tT])
end             ([eE][nN][dD])
except          ([eE][xX][cC][eE][pP][tT])
extract         ([eE][xX][tT][rR][aA][cC][tT])
filter          ([fF][iI][lL][tT][eE][rR])
filename        ([fF][iI][lL][eE][nN][aA][mM][eE])
header          ([hH][eE][aA][dD][eE][rR])
isa             ({iswsa})
isam            ({isa}{white}"["{white}mutex{white}"]")
isap            ({isa}{white}"["{white}partition{white}"]")
isau            ({isa}{white}"["{white}union{white}"]")
keyword         ([kK][eE][yY][wW][oO][rR][dD])
lexicon		([lL][eE][xX][iI][cC][oO][nN])
matches		([mM][aA][tT][cC][hH][eE][sS])
object          ([oO][bB][jJ][eE][cC][tT])
separator	([sS][eE][pP][aA][rR][aA][tT][oO][rR])
substitute	([sS][uU][bB][sS][tT][iI][tT][uU][tT][eE])
usequote	([uU][sS][eE]{white}[qQ][uU][oO][tT][eE])

%%
{caseinsens}    { KEY(CASEINSENS); }
{casesens}      { KEY(CASESENS); }
{context}       { KEY(CONTEXT); }
{constant}      { KEY(CONSTANT); }
{end}           { KEY(ENDKEY); }
{except}        { KEY(EXCEPT); }
{extract}       { KEY(EXTRACT); }
{filter}        { KEY(FILTER); }
{filename}      { KEY(FILENAME); }
{header}        { KEY(HEADER); }
{isa}           { KEY(ISA); }
{isam}          { KEY(ISAM); }
{isap}          { KEY(ISAP); }
{isau}          { KEY(ISAU); }
{keyword}       { KEY(KEYWORD); }
{lexicon}       { KEY(LEXICON); }
{matches}       { KEY(MATCHES); }
{object}        { KEY(OBJECTKEY); }
{separator}     { KEY(SEPARATOR); }
{substitute}    { KEY(SUBSTITUTE); }
{usequote}      { KEY(USEQUOTE); }

"|"             { P("BAR");          return (BAR); }
":"             { P("COLON");        return (COLON); }
","             { P("COMMA");        return (COMMA); }
"->"            { P("DETERMINES");   return (DETERMINES); }
"."             { P("DOT");          return (DOT); }
"{"             { P("LBRACE");       return (LBRACE); }
"["             { P("LBRACKET");     return (LBRACKET); }
"("             { P("LPAREN");       return (LPAREN); }
"}"             { P("RBRACE");       return (RBRACE); }
"]"             { P("RBRACKET");     return (RBRACKET); }
")"             { P("RPAREN");       return (RPAREN); }
";"             { P("SEMICOLON");    return (SEMICOLON); }
"*"             { P("STAR");         return (STAR); }

{float}         { yylval.fval = (float)atof(YYText());
		  P("FLOAT");        return (FLOAT); }
{nonnegint}     { yylval.num = atoi(YYText());
		  P("NONNEGINT");    return (NONNEGINT); }

{qstring}       { DUP_STR; P("STRING");       return (STRING); }
{lowerword}     { DUP_STR; P("LOWERWORD");    return (LOWERWORD); }
{upperword}     { DUP_STR; P("UPPERWORD"); return (UPPERWORD); }

"--".*$         { P("NOTE"); }

"/-"            { BEGIN(COMMENT);
                  ++Comment_level;
                  Prev_col = Col_num;
                  Prev_line = Line_num;
                  Col_num += 2;
                }
<COMMENT>"-/"   { Col_num += 2;
                  if (--Comment_level <= 0) {
                    BEGIN(INITIAL);
                    P("NOTE");
                  }
                }
<COMMENT>"/-"   { ++Comment_level; Col_num += 2; }
<COMMENT>\r*    { Col_num = 1; }
<COMMENT>\n     { Col_num = 1; ++Line_num; }
<COMMENT>.      { ++Col_num; }

[\t]            { Col_num += 8 - (Col_num % 8); Prev_col = Col_num; }
[ \f]*          { Col_num += strlen(YYText()); Prev_col = Col_num; }
\r*             { Prev_col = Col_num; Col_num = 1; }
\n              { Prev_col = Col_num = 1; Prev_line = ++Line_num; }

.               {
                  Dump_Text(stderr, Parse_str, Col_num, Line_num);
                  fprintf(stderr,
                          "ontology: Line %ld Col %ld: Invalid input (%s)\n",
                          Line_num, Col_num, YYText());
                  exit (-1);
                }

%%
/*=========================================================================
 *			FUNCTIONS
 */

/*========================================================================
 *                      END OF FILE $RCSfile: harmony.lex,v $
 */
