%{
static char const rcsid[] =
        "$Id: harmony.yacc,v 1.1.1.2 2005/07/01 22:44:05 mja47 Exp $";

/*========================================================================
 * $RCSfile: harmony.yacc,v $
 *
 * $Author: mja47 $
 * $Date: 2005/07/01 22:44:05 $
 * $Revision: 1.1.1.2 $
 *
 * $Description: 
 * Harmony Language Grammar. $
 * 
 * $Log: harmony.yacc,v $
 * Revision 1.1.1.2  2005/07/01 22:44:05  mja47
 * no message
 *
 * Revision 1.1.1.1  2003/08/09 20:48:42  pwinward
 * no message
 *
 * Revision 1.1.1.1  2002/04/25 23:00:01  tim
 * no message
 *
 * Revision 1.1.1.1  1999/05/11 15:16:49  swliddle
 * Ontos.rev2 stuff
 *
 * Revision 1.5  1998/02/05  22:20:14  liddl
 * Checkpoint of reasonably decent version.
 *
 * Revision 1.4  1998/02/05 18:24:02  liddl
 * Working version for VLDB Ontology paper.
 *
 * Revision 1.3  1996/04/24  13:43:54  liddl
 * About to hand off to Ting.
 *
 * Revision 1.2  1996/01/17  21:37:57  liddl
 * Lots of updates for the Harmony language.  Now we need to
 * spend some serious time debugging and improving it.
 *
 * Revision 1.1  1995/11/04  01:00:46  liddl
 * Initial revision
 *
 */
/*------------------------------------------------------------------------
 *			INCLUDES
 */
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <fstream.h>
#include <iostream.h>

#include "ontology.H"
#include <FlexLexer.h>
#include "semantic.H"

#define yylex Harmonylex

/*------------------------------------------------------------------------
 *          GLOBALS
 */
extern ofstream* OSMX;
extern string fileName;
string ENDL = "\n";

string nameList = "";
string objectPrimary = "";
bool inObjectSet = false;
string caseSensitive = "";
float confidenceValue = -1.0;
string contextExpression = "";
string valueExpression = "";
string subFromExpression = "";
string subToExpression = "";
string exceptionExpression = "";
string connections = "";
string partic = "";
string binaryName = "";
string ternaryName = "";
bool isBinary;
bool isTernary;
char buffer[20];
bool inGenSpec = false;
string specializations = "";
string url = "";
string label = "";
string delimiters = "";
string caseSens = "";
string dataFrame = "";
string frameComponent = "";
string constantComponent = "";
string keywordComponent = "";
int ID = 1;
int tempID = 1;
string name = "";
string extract = "";
string context = "";

struct Lexicon {
	Lexicon();
	string label;
	string url;
	string delimiters;
	string caseSens;
	struct Lexicon * next;
};

Lexicon::Lexicon() {
	label = "";
	url = "";
	delimiters = "";
	caseSens = "N";
	next = NULL;
}
typedef struct Lexicon * LexPtr;

LexPtr lexStart = NULL;
LexPtr lexEnd = NULL;

string intToString(int i) {
	char buffer[20];
	sprintf(buffer, "%d", i);

	return buffer;
}

string floatToString(float f) {
	char buffer[20];
	gcvt(f, 5, buffer);
	
	return buffer;
}

string LeftAndRightContext(string extract, string context) {
	// extract and context must not include quotes
	string left = "";
	string right = "";

	int leftEnd = context.find(extract);
	if (leftEnd == -1 || leftEnd == int(string::npos)) {		// extract not a subset
		left =	"<LeftContextExpression>" + ENDL +
				"<ExpressionText>***MANUALLY INPUT***</ExpressionText>" + ENDL +
				"</LeftContextExpression>" + ENDL +
				"<RightContextExpression>" + ENDL +
				"<ExpressionText>***MANUALLY INPUT***</ExpressionText>" + ENDL +
				"</RightContextExpression>" + ENDL;
	}
	else {
		if (leftEnd != 0) {							// if there's a left context
			left =	"<LeftContextExpression>" + ENDL +
					"<ExpressionText>" + context.substr(0, leftEnd) + "</ExpressionText>" + ENDL +
					"</LeftContextExpression>" + ENDL;
		}

		int rightStart = leftEnd + extract.length();

		// if there's a right context
		if (rightStart < int(context.length())) {
			right = "<RightContextExpression>" + ENDL +
					"<ExpressionText>" + 
					context.substr(rightStart, context.length() - rightStart) +
					"</ExpressionText>" + ENDL +
					"</RightContextExpression>" + ENDL;
		}
	}

	return (left + right);
}


void initVars() {
	nameList = "";
	objectPrimary = "";
	inObjectSet = false;
	caseSensitive = "";
	confidenceValue = -1.0;
	contextExpression = "";
	valueExpression = "";
	subFromExpression = "";
	subToExpression = "";
	exceptionExpression = "";
	connections = "";
	partic = "";
	binaryName = "";
	ternaryName = "";
	isBinary = false;
	isTernary = false;
	inGenSpec = false;
	specializations = "";
	url = "";
	label = "";
	delimiters = "";
	caseSens = "";
	dataFrame = "";
	frameComponent = "";
	constantComponent = "";
	keywordComponent = "";
	ID = 1;
	tempID = 1;
	name = "";
	lexStart = NULL;
	lexEnd = NULL;
	extract = "";
	context = "";
}



%}
/*------------------------------------------------------------------------
 *                      TOKENS
 */

%union {
	char		*p_char;
	NODE		*p_node;
	EDGE		*p_edge;
	PCONSTR		*p_cc;
	FRAMEC		*p_frame;
	LEXICON_ST	*p_lex;
	int		num;
	float		fval;
       }

/*
 * Operator Tokens
 */
%token		BAR COLON COMMA DETERMINES DOT LBRACE LBRACKET
%token		LPAREN RBRACE RBRACKET RPAREN SEMICOLON
%token <fval>	FLOAT
%token <num>	NONNEGINT
%token <p_char>	LOWERWORD STRING UPPERWORD

%left           STAR

%type <p_char>	ObjectSetName LowerWordSeq LowerWordSeqOpt UpperWordSeq
%type <p_char>	PatternList PatternComponent

%type <p_node>	DataFrameBodyOpt(<p_char>, <p_cc>)
%type <p_node>	DataFrameBody(<p_char>, <p_cc>)
%type <p_node>	MatchesKey(<p_char>, <p_cc>)

%type <p_frame>	LBraceKey(<p_node>)

%type <p_lex>	LexLBraceKey

%type		DefCaseSensOpt(<p_node>) MaxLengthOpt(<p_node>)

%type		FrameComponent(<p_node>) FrameComponentList(<p_node>)
%type		ConstantComponent(<p_node>) LexiconComponent(<p_node>)
%type		LexiconBodyList(<p_node>) LexiconBody(<p_node>)
%type		KeywordComponent(<p_node>) ConstantBodyList(<p_node>)
%type		ConstantBody(<p_node>) KeywordBodyList(<p_node>)

%type		ConstantTagList(<p_node>, <p_frame>)
%type		ConstantTag(<p_node>, <p_frame>)
%type		LexiconTag(<p_node>, <p_lex>)
%type		LexiconTagList(<p_node>, <p_lex>)

%type		ConfidenceValue(<p_frame>)
%type		ExpectedParticipation(<p_cc>)

%type		RS_DeclTermList(<p_edge>) RelSetDeclTerm(<p_edge>)

%type <p_edge>	FirstRelSetDeclTerm

%type <p_cc>	ParticipationConstraint CardConstraintBody
%type <p_cc>	CardConstrBodyOpt ConstraintTerm ConstraintTerm1
%type <p_cc>	CardinalityConstrOpt CardinalityConstraint

%type		ReplacementTail(<p_cc>)

%type <num>	ConstraintMin ConstraintMax ConstraintMaxOpt(<num>)
%type <num>	UseQuoteOpt

%type <num>	CaseSensitiveOpt(<p_node>)

%type <fval>	RealLiteral KeywordConfidence

/*
 * Key-Word Tokens
 */
%token		CASEINSENS CASESENS CONTEXT CONSTANT ENDKEY EXCEPT
%token		EXTRACT FILTER FILENAME HEADER ISA ISAM ISAP ISAU
%token		KEYWORD LEXICON MATCHES OBJECTKEY SEPARATOR SUBSTITUTE
%token		USEQUOTE


%start          OSM_Module

%%
/*========================================================================
 *                      RULE DECLARATIONS
 */
/*------------------------------------------------------------------------
 *                      START RULES
 */
OSM_Module		:	{ 	initVars();
						
						*OSMX	<< "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl
								<< "<OSM ontologyName=\"" << fileName << "\"" << endl
								<< "xmlns=\"http://www.deg.byu.edu/xml/osmx.xsd\"" << endl
								//<< "xmlns:jaxb=\"http://java.sun.com/xml/ns/jaxb\"" << endl
								//<< "xmlns:xjc=\"http://java.sun.com/xml/ns/jaxb/xjc\"" << endl
								<< "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"" << endl
								<< "xsi:schemaLocation=\"http://www.deg.byu.edu/xml/osmx.xsd http://www.deg.byu.edu/xml/osmx.xsd\">" << endl;
					}
					ORM_DeclList
					{		
						// print out lexicons
						LexPtr temp = lexStart;
						
						while (temp != NULL) {
							*OSMX	<< "<Lexicon label=\"" << temp->label
									<< "\" uri=\"file///" << temp->url
									<< "\" delimiters=\"" << temp->delimiters
									<< "\" caseSensitive=\"" << temp->caseSens 
									<< "\"/>" << endl;
									

							temp = temp->next;
						}
						
					
						*OSMX << "</OSM>" << endl;
						//Write_Expressions(); 
					}
			;

ORM_DeclList		: ORM_Declaration SEMICOLON
			| ORM_DeclList ORM_Declaration SEMICOLON
			;

ORM_Declaration		: ObjectSetDecl
			| RelSetDecl
			| GenSpecDecl
			| LBRACKET STRING RBRACKET
			  { fprintf(stderr, "General Constraint [%s]\n",
				    Extract_String($2));
			  }
			;

/*------------------------------------------------------------------------
 *                      OBJECT-SET RULES
 */
ObjectSetDecl	: 	{	
						inObjectSet = true;
						objectPrimary = " primary = \"N\"";
						*OSMX << "<ObjectSet id=\""; 	
					}
					ObjectSetName 
					{	
						name = $2;
						*OSMX << $2 << "\"";
						nameList =	"<NameList>" + ENDL +
									"<Name content=\"" + $2 + "\"/>" + ENDL;
					}
					AliasesOpt 
					{	
						nameList += "</NameList>" + ENDL; 
					}
					CardinalityConstrOpt
					DataFrameBodyOpt($2, $6)
					{
						
						*OSMX	<< objectPrimary;
												
						if ($7 == NULL) {
							NODE *node = Add_Node($2);

							if ($6 != NULL && $6->rp_object)
								node->rp_object = TRUE;

							*OSMX << " lexical=\"N\">" << endl;
						}
						else {
							*OSMX << " lexical=\"Y\">" << endl;
						}

						*OSMX << nameList << dataFrame << "</ObjectSet>" << endl;
						inObjectSet = false;
					}
			;

AliasesOpt		: ObjectSetAliases
			| /* empty */
			;


DataFrameBodyOpt($nm, $cc)
			: DataFrameBody($nm, $cc)
			  { $$ = $1; }
			| /* empty */
			  { $$ = NULL; }
			;

DataFrameBody($nm, $cc)	: MatchesKey($nm, $cc) 
						{
							// at this point, either a DataFrame or an error
							dataFrame =		"<DataFrame>" + ENDL +
											"<InternalRepresentation>" + ENDL +
											"<DataType typeName=\"java:java.lang.String\"/>" + ENDL +
											"</InternalRepresentation>" + ENDL;
						}
						MaxLengthOpt($1)
						DefCaseSensOpt($1) 
						FrameComponentList($1)
						ENDKEY
						{
							dataFrame += "</DataFrame>" + ENDL;
						}
			  { $$ = $1; }
			;

MatchesKey($nm, $cc)	: MATCHES
			  {
			    $$ = Add_Node($nm);
			    if ($cc != NULL && $cc->rp_object)
			      $$->rp_object = TRUE;
			  }
			;

MaxLengthOpt($os)	: LBRACKET NONNEGINT RBRACKET
			  { $os->max_len = $2; }
			| /* empty */
			;

DefCaseSensOpt($os)	: CASESENS
			  {	$os->case_sens = TRUE; 
				caseSensitive = " caseSensitive=\"Y\"";}
			| CASEINSENS
			  { $os->case_sens = FALSE; }
			| /* empty */
			  { $os->case_sens = FALSE; }
			;

FrameComponentList($os)	: FrameComponent($os)
			| FrameComponent($os) FrameComponentList($os)
			;

FrameComponent($os)	: 	
			
			LexiconComponent($os)
		
		|	ConstantComponent($os) 
			{
				dataFrame += constantComponent;
			}
		
		|	KeywordComponent($os)
			{	
				dataFrame += keywordComponent;
			}
			;

LexiconComponent($os)	: LEXICON LexiconBodyList($os) SEMICOLON
			;

LexiconBodyList($os)	: 
					{ /* remove this line and watch it segfault ! */ }
					LexiconBody($os)
			
			|		LexiconBody($os) 
					COMMA 
					LexiconBodyList($os)
			;
			
LexiconBody($os)	: LexLBraceKey 
					{
						// create a new lexicon and append to list
						if (lexStart == NULL) {
							lexStart = new struct Lexicon;
							lexEnd = lexStart;
						}
						else {
							LexPtr temp = new struct Lexicon;
							lexEnd->next = temp;
							lexEnd = lexEnd->next;
						}
						
					}
					LexiconTagList($os, $1) 
					RBRACE
			
				|	LexLBraceKey 
					LexiconTagList($os, $1) 
					RBRACE
					COMMA 
					LexiconBody($os)
				;

LexLBraceKey		: LBRACE
			  { $$ = Use_Lexicon(); }
			;

LexiconTagList($os, $lx)
			: LexiconTag($os, $lx)
			| LexiconTag($os, $lx) LexiconTagList($os, $lx)

LexiconTag($os, $lx)	: ObjectSetName CaseSensitiveOpt($os) SEMICOLON
			  {
			    $lx->name = $1;
			    $lx->case_sens = $2;
				lexEnd->label = $1;
				if ($2)
					lexEnd->caseSens = "Y";

			  }
			| FILENAME STRING SEMICOLON
			  { 
				$lx->filename = Extract_String($2); 
				lexEnd->url = Extract_String($2);
			  }
			| SEPARATOR STRING UseQuoteOpt SEMICOLON
			  {
			    $lx->separators = Extract_String($2);
			    if ($3)
			      strcat($lx->separators, "\"");
				lexEnd->delimiters = Extract_String($2);
			  }
			;

UseQuoteOpt		: USEQUOTE
			  { $$ = TRUE; }
			| /* empty */
			  { $$ = FALSE; }
			;

ConstantComponent($os)	: CONSTANT 
			{
				constantComponent = "<ValuePhraseList>" + ENDL;			
				tempID = 1;
			}
			ConstantBodyList($os) 
			{
				constantComponent += "</ValuePhraseList>" + ENDL;
			}
			SEMICOLON
			;

ConstantBodyList($os)	: ConstantBody($os)
			| ConstantBodyList($os) COMMA ConstantBody($os)
			;

ConstantBody($os)	: LBraceKey($os) 
			{
				constantComponent += "<ValuePhrase";
				confidenceValue = -1.0;
				valueExpression = "";
				exceptionExpression = "";
				contextExpression = "";
				subFromExpression = "";
				subToExpression = "";
				caseSensitive = " caseSensitive=\"N\"";
				extract = "";
				context = "";
			}
			ConstantTagList($os, $1) RBRACE	ConfidenceValue($1)
			{
				
				constantComponent += " hint=\"" + name + "_" + intToString(tempID++) + "\"";
				if (confidenceValue != -1.0)
					constantComponent += " confidenceTag=\"" + floatToString(confidenceValue) + "\"";
				constantComponent += caseSensitive + ">" + ENDL;
				
				if (extract != "" && context != "")
					contextExpression = LeftAndRightContext(extract, context);

				string valuePhrase = valueExpression + exceptionExpression + contextExpression + 
									 subFromExpression + subToExpression;
				constantComponent += valuePhrase + "</ValuePhrase>" + ENDL;
			}
			;

ConfidenceValue($df)	: RealLiteral  
				{	$df->confidence = $1; 
					confidenceValue = $1;
				}
			|	/* empty */
				{	$df->confidence = -1;
					confidenceValue = -1.0;
				}
			;

LBraceKey($os)		: LBRACE
			  { $$ = Use_Framec($os); }
			;

ConstantTagList($os, $df)
			: ConstantTag($os, $df) SEMICOLON
			| ConstantTag($os, $df) SEMICOLON
			  ConstantTagList($os, $df)
			;

ConstantTag($os, $df)	: CONTEXT PatternList CaseSensitiveOpt($os)
			  {		
				// output determined by LeftAndRightContext function, called in ConstantBody
				context = $2;
			  	  
				$df->context = $2;
			    $df->cs_context = $3;
			  }
			| EXTRACT PatternList CaseSensitiveOpt($os)
			  { 
				valueExpression =	"<ValueExpression>" + ENDL + 
									"<ExpressionText>" + $2 + "</ExpressionText>" + ENDL +
									"</ValueExpression>" + ENDL;
				extract = $2;
				$df->extract = $2;
			    $df->cs_extract = $3;
			  }
			| EXCEPT STRING CaseSensitiveOpt($os)
			  { exceptionExpression =	"<ExceptionExpression>" + ENDL + 
										"<ExpressionText>" + Extract_String($2) + "</ExpressionText>" + ENDL +
										"</ExceptionExpression>" + ENDL;
			  	$df->except = $2;
			    $df->cs_except = $3;
			  }
			| FILTER STRING UseQuoteOpt
			  { $df->filter = $2;
			    $df->use_quote = $3;
			  }
			| SUBSTITUTE STRING DETERMINES STRING
			  { subFromExpression =	"<SubFromExpression>" + ENDL + 
									"<ExpressionText>" + Extract_String($2) + "</ExpressionText>" + ENDL +
									"</SubFromExpression>" + ENDL;
				subToExpression =	"<SubToExpression>" + ENDL + 
									"<ExpressionText>" + Extract_String($4) + "</ExpressionText>" + ENDL +
									"</SubToExpression>" + ENDL;

				
				$df->subst1 = $2;
			    $df->subst2 = $4;
			  }
			;

PatternList		: PatternComponent
			  { $$ = $1; }
			| PatternComponent COMMA PatternList
			  { $$ = Append_String($1, "", $3); }
			;

PatternComponent	: STRING
			  { $$ = Extract_String($1); }
			| ObjectSetName
			  { $$ = Append_String("{", $1, "}"); }
			;

CaseSensitiveOpt($os)	: CASESENS
			  {	caseSensitive = " caseSensitive=\"Y\"";
				$$ = TRUE; }
			| CASEINSENS
			  { $$ = FALSE; }
			| /* empty */
			  { $$ = $os->case_sens; }
			;

KeywordComponent($os)	: KEYWORD 
				{	
					keywordComponent = "<KeywordPhraseList>" + ENDL; 
					tempID = 1;
				}
				KeywordBodyList($os) SEMICOLON
				{	keywordComponent += "</KeywordPhraseList>" + ENDL; }
			;

KeywordBodyList($os)	: STRING KeywordConfidence
			  { 
				Add_Keyword($os, $1, $2); 
				keywordComponent += "<KeywordPhrase hint=\"" + name + "_keyword_" + intToString(tempID++) + "\"";
				if ($2 != -1.0)
					 keywordComponent +=  " confidenceTag=\"" + floatToString($2) + string("\"");
				
				keywordComponent += ">" + ENDL +
					 "<KeywordExpression>" + ENDL +
					 "<ExpressionText>" + Extract_String($1) + "</ExpressionText>" + ENDL + 
					 "</KeywordExpression>" + ENDL +
					 "</KeywordPhrase>" + ENDL;
			  }

			| STRING KeywordConfidence
			  COMMA KeywordBodyList($os)
			  { 							  
				Add_Keyword($os, $1, $2); 
				keywordComponent += "<KeywordPhrase hint=\"" + name + "_keyword_" + intToString(tempID++) + "\"";
				if ($2 != -1.0)
					keywordComponent += " confidenceTag=\"" + floatToString($2) + "\"";
				
				keywordComponent += ">" + ENDL +
					 "<KeywordExpression>" + ENDL +
					 "<ExpressionText>" + Extract_String($1) + "</ExpressionText>" + ENDL + 
					 "</KeywordExpression>" + ENDL + 
					 "</KeywordPhrase>" + ENDL;
			  }
			;

KeywordConfidence	: RealLiteral
			  { $$ = $1; }
			| /* empty */
			  { $$ = (float)-1.0; }
			;

ObjectSetAliases	: ObjSetAliasList
			;


ObjSetAliasList		: BAR ObjectSetName 
					{ 
						nameList += "<Name content=\"" + string($2) + "\"/>" + ENDL; 
					}
			|		ObjSetAliasList BAR ObjectSetName 
					{ 
						nameList += "<Name content=\"" + string($3) + "\"/>" + ENDL; 
					}
			;

/*------------------------------------------------------------------------
 *			RELATIONSHIP-SET RULES
 */
RelSetDecl	: 
			{	isBinary = false;
				isTernary = false;
				binaryName = "";
				ternaryName = "";
			}
			RelSetDeclName RS_AliasesOpt CoOccurrenceListOpt
			{
				*OSMX	<< "<RelationshipSet id=\"" << ID++ << "\">" << endl
						<< nameList
						<< connections
						<< "</RelationshipSet>" << endl;
			}
			;


RS_AliasesOpt		: RelSetAliases
			| /* empty */
			;

CoOccurrenceListOpt	: CoOccurrenceList
			| /* empty */
			;

CoOccurrenceList	: CoOccurrenceDecl
			| CoOccurrenceList COMMA CoOccurrenceDecl
			;

CoOccurrenceDecl	: LBRACKET ObjectSetNameList DETERMINES
			  CardinalityConstrOpt ObjectSetNameList
			  RBRACKET
			  { Num_spec_names = 0; }
			;

RelSetDeclName	: FirstRelSetDeclTerm 
				{
					nameList =	"<NameList>" + ENDL +
								"<Name content=\"";
				}
				RS_DeclTermList($1)
				{
					if (isTernary) 
						nameList += ternaryName + "\"/>" + ENDL;
					else if (isBinary) 
						nameList += binaryName + "\"/>" + ENDL;
									
					nameList += "</NameList>" + ENDL;
				}
				;


RS_DeclTermList($i1)	: RelSetDeclTerm($i1)
			| RS_DeclTermList($i1) RelSetDeclTerm($i1)
			;

RelSetDeclTerm($i1)	: LowerWordSeqOpt
			  ObjectSetName
			  {
				if (binaryName != "")
					binaryName += " " + string($1);
				else
					binaryName += $1;

				if (string($1) != "")
					ternaryName += " " + string($1) + " " + string($2);
				else
					ternaryName += " " + string($2);
								
				if (isBinary)
					isTernary = true;
				isBinary = true;

				connections +=	"<Connection objectSet=\"" + string($2) + "\">" + ENDL +
								//string(" isDomainSide=\"N\">") + ENDL;
								"<ParticipationConstraint>" + ENDL +
								+ "<Text content=\"";
				partic = "";

			  }
			  ParticipationConstraint
			  {
			    if (strlen($1) > 0)
			      Add_Edge_Name($1, $i1);
			    Add_Edge_Name($2, $i1);
			    Add_Connection($1, Add_Node($2),
					   $4->pc_optional,
					   $4->pc_multiple,
					   $4->pc_mean,
					   $4->pc_stddev,
					   $4->replace,
					   $4->rp_optional,
					   $4->rp_multiple,
					   $i1);
				
				connections +=	partic + "\"/>" + ENDL +
								"</ParticipationConstraint>" + ENDL +
								"</Connection>" + ENDL;
			  }
			;

FirstRelSetDeclTerm	: LowerWordSeqOpt
			  ObjectSetName
			  {
				binaryName += $1;
				if (string($1) != "")
					ternaryName += $1 + string(" ") + $2;
				else
					ternaryName += $2;

				connections =	"<Connection objectSet=\"" + string($2) + "\">" + ENDL +
								//string(" isDomainSide=\"Y\">") + ENDL
								"<ParticipationConstraint>" + ENDL
								+ "<Text content=\"";
				partic = "";

			  }
			  ParticipationConstraint
			  { EDGE	*e;
			    if (strlen($1) > 0) {
			      e = Add_Edge($1);
			      Add_Edge_Name($2, e);
			    } else {
			      e = Add_Edge($2);
			    }
			    Add_Connection($1, Add_Node($2),
					   $4->pc_optional,
					   $4->pc_multiple,
					   $4->pc_mean,
					   $4->pc_stddev,
					   $4->replace,
					   $4->rp_optional,
					   $4->rp_multiple,
					   e);
			    $$ = e;

				connections +=	partic + "\"/>" + ENDL +
								"</ParticipationConstraint>" + ENDL +
								"</Connection>" + ENDL;
			  }
			;

LowerWordSeqOpt		: LowerWordSeq
			  { $$ = $1; }
			| /* empty */
			  { $$ = ""; }
			;


LowerWordSeq		: LOWERWORD
			  { $$ = $1; }
			| LowerWordSeq LOWERWORD
			  { $$ = Append_String($1, " ", $2); }
			;

RelSetAliases		: BAR RelSetName
			| RelSetAliases BAR RelSetName
			;

/*------------------------------------------------------------------------
 *                      CARDINALITY-CONSTRAINT RULES
 */
CardinalityConstrOpt	: CardinalityConstraint
			  { $$ = $1; }
			| /* empty */
			  { $$ = NULL; }
			;

CardinalityConstraint	: ParticipationConstraint
			  { $$ = $1; }
			;

ParticipationConstraint	: LBRACKET CardConstrBodyOpt ReplacementTail($2)
			  RBRACKET
			  { $$ = $2; }
			;

CardConstrBodyOpt  	: CardConstraintBody
			  { $$ = $1; }
			| /* empty */
			  {
			    PCONSTR *cc = new PCONSTR;

			    cc->replace = FALSE;
			    cc->rp_object = FALSE;
			    cc->rp_header = FALSE;
			    $$ = cc;
			  }
			;

CardConstraintBody  	: ConstraintTerm1 ConstraintTail
			  { $$ = $1; }
			;

ReplacementTail($cc)	: DETERMINES CardConstraintBody
			  {
			    $cc->replace = TRUE;
			    $cc->rp_optional = $2->pc_optional;
			    $cc->rp_multiple = $2->pc_multiple;
			  }
			| DETERMINES OBJECTKEY
			{	
				if (inObjectSet) {
					objectPrimary = " primary=\"Y\""; }
		
				$cc->rp_object = TRUE; 
			}
			| DETERMINES HEADER
			{ 
				$cc->rp_header = TRUE; 
			}
			| /* empty */ 
		;

ConstraintTail		: ConstrTermList
			| /* empty */
			;

ConstrTermList		: COMMA ConstraintTerm1
			| ConstrTermList COMMA ConstraintTerm1
			;

ConstraintTerm1		: ConstraintTerm ExpectedParticipation($1)
			;

ConstraintTerm		: ConstraintMin ConstraintMaxOpt($1)
			  { $$ = new PCONSTR;
			    $$->pc_optional = ($1 < 1);
			    $$->pc_multiple = ($2 != 1);
			    $$->replace = FALSE;
			    $$->rp_object = FALSE;
			    $$->rp_header = FALSE;
			    $$->pc_mean = -1.0;
			    $$->pc_stddev = -1.0;
				
				sprintf(buffer, "%d", $1);
				partic += buffer + string(":");
				if ($2 == PC_STAR)
					partic += "*";
				else {
					sprintf(buffer, "%d", $2);
					partic += buffer;
				}
			  }
			;


ConstraintMaxOpt($i1)	: COLON ConstraintMax
			  { $$ = $2; }
			| DOT DOT ConstraintMax
			  { $$ = $3; }
			| /* empty */
			  { $$ = $i1; }
			;

ConstraintMin		: NONNEGINT
			  { $$ = $1; }
			;

ConstraintMax		: NONNEGINT
			  { $$ = $1; }
			| STAR
			  { $$ = PC_STAR; }
			;

RealLiteral		: NONNEGINT
			  { $$ = (float)$1; }
			| FLOAT
			  { $$ = $1; }
			;

ExpectedParticipation($cc)
			: LPAREN RealLiteral COMMA RealLiteral RPAREN
			  { $cc->pc_mean = $2;
			    $cc->pc_stddev = $4;
			  }
			| /* empty */
			;

/*------------------------------------------------------------------------
 *		GENERALIZATION-SPECIALIZATION RULES
 */
GenSpecDecl		: 
				{ 
					inGenSpec = true;
					*OSMX << "<GenSpec>" << endl;
					specializations = "";
				}
				ObjectSetNameList 
				{
					inGenSpec = false;
				}
				GenSpecKey ObjectSetName
				{	NODE	*obj_set;
			  
					for (int i = 0; i < Num_spec_names; i++) {
						obj_set = Add_Node(Spec_name[i]);
						obj_set->parent = Add_Node($5);
					}
					Num_spec_names = 0;
					
					*OSMX	<< "<GenConnection objectSet=\"" << $5 << "\"/>" << endl
							<< specializations
							<< "</GenSpec>" << endl;
					inGenSpec = false;
				}
			;

GenSpecKey		: COLON
			| ISA
			| ISAM
			| ISAP
			| ISAU
			;

/*------------------------------------------------------------------------
 *                      NAME RULES
 */
ObjectSetNameList	: ObjectSetName
			  { strcpy(Spec_name[Num_spec_names++], $1); }
			| ObjectSetNameList COMMA ObjectSetName
			  { strcpy(Spec_name[Num_spec_names++], $3); }
			;

ObjectSetName		: UpperWordSeq
					{ 
						if (inGenSpec)
							specializations += "<SpecConnection objectSet=\"" + string($1) + "\"/>" + ENDL;

						$$ = $1; 
					}
			;

RelSetName		: UpperWordSeq RelSetNameTail
			| RelSetName RelSetNameTail
			;

RelSetNameTail		: LowerWordSeq UpperWordSeq
			;

UpperWordSeq		: UPPERWORD
			  { $$ = $1; }
			| UpperWordSeq UPPERWORD
			  { $$ = Append_String($1, "", $2); }
			;

/*========================================================================
 *                      END OF RULE DECLARATIONS
 */
%%

/*========================================================================
 *                      END OF FILE $RCSfile: harmony.yacc,v $
 */
