static char const rcsid[] = "$Id: semantic.C,v 1.1.1.2 2005/07/01 22:44:05 mja47 Exp $";

/*========================================================================
* $RCSfile: semantic.C,v $
*
* $Author: mja47 $
* $Date: 2005/07/01 22:44:05 $
* $Revision: 1.1.1.2 $
*
* $Description: 
* Parser support routines. $
* 
* $Log: semantic.C,v $
* Revision 1.1.1.2  2005/07/01 22:44:05  mja47
* no message
*
* Revision 1.1.1.1  2003/08/09 20:48:43  pwinward
* no message
*
* Revision 1.3  2003/02/26 21:41:16  tim
* Added misc. bug fixes and minor features to demo,ontology editor, and ontos
*
* Revision 1.2  2003/01/14 03:31:44  tim
* fixed bug in emmision of create table statements for single-field tables
*
* Revision 1.1.1.1  2002/04/25 23:00:01  tim
* no message
*
* Revision 1.1.1.1  1999/05/11 15:16:49  swliddle
* Ontos.rev2 stuff
*
* Revision 1.3  1998/02/05  22:20:14  liddl
* Checkpoint of reasonably decent version.
*
* Revision 1.2  1998/02/05 18:24:02  liddl
* Working version for VLDB Ontology paper.
*
* Revision 1.1  1996/04/24  13:46:10  liddl
* Initial revision
*
*/

/*------------------------------------------------------------------------
*                      INCLUDES
*/
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>
//#include <unistd.h>
#include <ctype.h>

#include "ontology.H"
#include "semantic.H"

#include "FlexLexer.h"

#if WIN32
#define strcasecmp strcmpi
#endif

/*------------------------------------------------------------------------
*			CONSTANTS
*/
#define PC_EXPECTED	true
#define PC_ONLY		false
#define WINDOW_SIZE	5

/*------------------------------------------------------------------------
*			STATIC GLOBAL VARIABLES
*/
static const char	*Lex_type = "varchar",
*Nonlex_type = "integer";

/*------------------------------------------------------------------------
*			GLOBAL VARIABLES
*/
CASE_TYPE		Case_switch = LOWERCASE;
LEXICON_ST		Lexicon[MAX_LEXICONS];
NODE			Obj_set[MAX_NODES];
EDGE			Rel_set[MAX_EDGES];
FRAMEC			Framec[MAX_FRAMECS];
int			Num_framecs = 0,
Num_lexicons = 0,
Num_obj_sets = 0,
Num_rel_sets = 0,
Num_spec_names = 0;
char			Spec_name[MAX_SPEC_NAMES][MAX_NAME_LEN];

/*------------------------------------------------------------------------
*			EXTERNAL GLOBAL VARIABLES
*/
extern long		Col_num,
Line_num,
Prev_col,
Prev_line;
extern char		*Parse_str;
//extern HarmonyFlexLexer	*Harmony_lexer;
//extern yyFlexLexer	*Harmony_lexer;
extern FlexLexer	*Harmony_lexer;


/*========================================================================
*			PUBLIC FUNCTIONS
*/

/*------------------------------------------------------------------------
* FUNCTION Add_Connection -- add a connection to an edge
*/
void
Add_Connection(char *rs_name_frag, NODE *obj_set, bool pc_optional,
			   bool pc_multiple, float pc_mean, float pc_stddev,
			   bool replace, bool rp_optional,
			   bool rp_multiple, EDGE *edge)
{
	if (edge != NULL && edge->num_conns < MAX_REL_SET_ARITY) {
		edge->conn[edge->num_conns].rs_name_frag = rs_name_frag;
		edge->conn[edge->num_conns].obj_set = obj_set;
		edge->conn[edge->num_conns].pc_optional = pc_optional;
		edge->conn[edge->num_conns].pc_multiple = pc_multiple;
		edge->conn[edge->num_conns].pc_mean = pc_mean;
		edge->conn[edge->num_conns].pc_stddev = pc_stddev;
		edge->conn[edge->num_conns].replace = replace;
		edge->conn[edge->num_conns].rp_optional = rp_optional;
		edge->conn[edge->num_conns].rp_multiple = rp_multiple;
		++edge->num_conns;
	}

} /* Add_Connection */

/*------------------------------------------------------------------------
* FUNCTION Add_Edge -- add a relationship-set edge to the graph
*/
EDGE *
Add_Edge(char *name)
{
	if (Num_rel_sets < MAX_EDGES) {
		Rel_set[Num_rel_sets].name = name;
		Rel_set[Num_rel_sets].marked = FALSE;
		Rel_set[Num_rel_sets].num_conns = 0;
		++Num_rel_sets;
	}
	return (&Rel_set[Num_rel_sets - 1]);

} /* Add_Edge */

/*------------------------------------------------------------------------
* FUNCTION Add_Edge_Name -- append a name onto this relationship-set name
*/
void
Add_Edge_Name(const char *name, EDGE *edge)
{
	if (edge != NULL) {
		char *new_name = new char[strlen(name) + strlen(edge->name) + 2];

		strcpy(new_name, edge->name);
		strcat(new_name, " ");
		strcat(new_name, name);
		edge->name = new_name;
	}

} /* Add_Edge_Name */

/*------------------------------------------------------------------------
* FUNCTION Add_Frame_Constant -- add a frame constant to an object set
*/
void
Add_Frame_Constant(NODE *obj_set, FRAMEC *frame)
{
	if (obj_set->num_framec >= obj_set->alloc_framec) {
		if (obj_set->alloc_framec) {
			FRAMEC  **tmpf = new FRAMEC *[obj_set->alloc_framec + ALLOC_CHUNK];

			for (int i = 0; i < obj_set->num_framec; i++)
				tmpf[i] = obj_set->framec[i];
			delete [] obj_set->framec;
			obj_set->framec = tmpf;
		} else {
			obj_set->framec = new FRAMEC *[ALLOC_CHUNK];
		}
		obj_set->alloc_framec += ALLOC_CHUNK;
	}
	obj_set->framec[obj_set->num_framec] = frame;
	++obj_set->num_framec;

} /* Add_Frame_Constant */

/*------------------------------------------------------------------------
* FUNCTION Add_Keyword -- add a keyword pattern for this object set
*/
void
Add_Keyword(NODE *obj_set, char *keyword, float conf)
{
	if (obj_set->num_keywd >= obj_set->alloc_keywd) {
		if (obj_set->alloc_keywd) {
			char	**tmpk = new char *[obj_set->alloc_keywd + ALLOC_CHUNK];
			float	*tmpc = new float[obj_set->alloc_keywd + ALLOC_CHUNK];

			for (int i = 0; i < obj_set->num_keywd; i++) {
				tmpk[i] = obj_set->keyword[i];
				tmpc[i] = obj_set->keyconf[i];
			}
			delete [] obj_set->keyword;
			delete [] obj_set->keyconf;
			obj_set->keyword = tmpk;
			obj_set->keyconf = tmpc;
		} else {
			obj_set->keyword = new char *[ALLOC_CHUNK];
			obj_set->keyconf = new float[ALLOC_CHUNK];
		}
		obj_set->alloc_keywd += ALLOC_CHUNK;
	}
	obj_set->keyword[obj_set->num_keywd] = keyword;
	obj_set->keyconf[obj_set->num_keywd] = conf;
	++obj_set->num_keywd;

} /* Add_Keyword */

/*------------------------------------------------------------------------
* FUNCTION Add_Node -- add an object-set node to the graph
*/
NODE *
Add_Node(char *name)
{
	if (Num_obj_sets < MAX_NODES) {
		/* Don't allow duplicate names. */
		for (int i = 0; i < Num_obj_sets; i++)
			if (!strcmp(Obj_set[i].name, name)) {
				return (&Obj_set[i]);
			}

			Obj_set[Num_obj_sets].name = name;
			Obj_set[Num_obj_sets].max_len = PC_STAR;
			Obj_set[Num_obj_sets].marked = FALSE;
			Obj_set[Num_obj_sets].case_sens = FALSE;
			Obj_set[Num_obj_sets].rp_object = FALSE;
			++Num_obj_sets;
	}
	return (&Obj_set[Num_obj_sets - 1]);

} /* Add_Node */

/*------------------------------------------------------------------------
* FUNCTION Append_String -- append and duplicate the given strings
*/
char *
Append_String(const char *str1, const char *str2, const char *str3)
{
	char	*new_str = new char[strlen(str1) + strlen(str2) + strlen(str3) + 1];

	strcpy(new_str, str1);
	strcat(new_str, str2);
	strcat(new_str, str3);

	return (new_str);

} /* Append_String */


/*------------------------------------------------------------------------
* FUNCTION Case_Is_Key -- see if key word matches case-sensitivity switch
*/
int
Case_Is_Key()
{
	switch (Case_switch) {
	case ANYCASE:
		return (TRUE);
	case EITHERCASE:
		return (Is_All_Lower(Harmony_lexer->YYText()) ||
			Is_All_Upper(Harmony_lexer->YYText()));
	case LOWERCASE:
		return (Is_All_Lower(Harmony_lexer->YYText()));
	case UPPERCASE:
		return (Is_All_Upper(Harmony_lexer->YYText()));
	}
	return (FALSE);

} /* Case_Is_Key */

/*------------------------------------------------------------------------
* FUNCTION Clear_Marks -- clear all marks in the ORM graph
*/
void
Clear_Marks()
{
	int	i;

	for (i = 0; i < Num_obj_sets; i++) {
		Obj_set[i].marked = FALSE;
		Obj_set[i].has_spec = FALSE;
	}

	/* Set generalization flags. */
	for (i = 0; i < Num_obj_sets; i++)
		if (Obj_set[i].parent != NULL)
			Obj_set[i].parent->has_spec = TRUE;

	for (i = 0; i < Num_rel_sets; i++) {
		Rel_set[i].marked = FALSE;
	}

} /* Clear_Marks */

/*------------------------------------------------------------------------
* FUNCTION Compute_Max_Ones -- find relationship sets we can assume in
*				the instance ontology
*/
void
Compute_Max_Ones()
{
	int	i;
	bool	changed = TRUE;

	/* Mark object sets.  Seed point is defined by rp_object flag. */
	for (i = 0; i < Num_obj_sets; i++)
		Obj_set[i].max_one = Obj_set[i].rp_object;

	/* Clear relationship set max_one marks. */
	for (i = 0; i < Num_rel_sets; i++)
		Rel_set[i].max_one = FALSE;

	/* Propagate max_one marks. */
	while (changed) {
		changed = FALSE;
		for (i = 0; i < Num_rel_sets; i++) {
			if (Rel_set[i].num_conns == 2) {
				if (!Rel_set[i].max_one &&
					Rel_set[i].conn[0].obj_set->max_one &&
					!Rel_set[i].conn[0].pc_multiple) {
						Rel_set[i].conn[1].obj_set->max_one = TRUE;
						Rel_set[i].max_one = TRUE;
						Rel_set[i].obj_conn = &Rel_set[i].conn[0];
						Rel_set[i].gen_conn = &Rel_set[i].conn[1];
						changed = TRUE;
					}
					if (!Rel_set[i].max_one &&
						Rel_set[i].conn[1].obj_set->max_one &&
						!Rel_set[i].conn[1].pc_multiple) {
							Rel_set[i].conn[0].obj_set->max_one = TRUE;
							Rel_set[i].max_one = TRUE;
							Rel_set[i].obj_conn = &Rel_set[i].conn[1];
							Rel_set[i].gen_conn = &Rel_set[i].conn[0];
							changed = TRUE;
						}
			}
		}
	}

} /* Compute_Max_Ones */

/*------------------------------------------------------------------------
* FUNCTION Compare_Connections -- do alpha comparison on two RS conns
*/
int
Compare_Connections(const void *p1, const void *p2)
{
	CONNECTION	*c1 = (CONNECTION *)p1,
		*c2 = (CONNECTION *)p2;

	return (strcmp(c1->obj_set->name, c2->obj_set->name));

} /* Compare_Connections */

/*------------------------------------------------------------------------
* FUNCTION Dump_Graph -- display the current ORM graph
*/
void
Dump_Graph(FILE *out_fp)
{
	int	i,
		j;

	fprintf(out_fp, "%d object sets, %d relationship sets\n", Num_obj_sets,
		Num_rel_sets);
	for (i = 0; i < Num_obj_sets; i++) {
		fprintf(out_fp, "%s%c (%slexical, max %d, is %scase sensitive)",
			Obj_set[i].name,
			Obj_set[i].marked ? '*' : ' ',
			Is_Lexical(&Obj_set[i]) ? "" : "non",
			Max_Length(&Obj_set[i]),
			Obj_set[i].case_sens ? "" : "not ");
		if (Obj_set[i].parent != NULL)
			fprintf(out_fp, " : %s", Obj_set[i].parent->name);
		fprintf(out_fp, "\n");
		if (Obj_set[i].num_keywd) {
			fprintf(out_fp, "\t(%d keywords: ", Obj_set[i].num_keywd);
			for (j = 0; j < Obj_set[i].num_keywd; j++)
				fprintf(out_fp, "%s%s %g", j > 0 ? ", " : "",
				Obj_set[i].keyword[j], Obj_set[i].keyconf[j]);
			fprintf(out_fp, ")\n");
		}
		if (Obj_set[i].num_framec) {
			fprintf(out_fp, "\t%d frames:\n", Obj_set[i].num_framec);
			for (j = 0; j < Obj_set[i].num_framec; j++) {
				fprintf(out_fp, "\t    {\n");
				if (Obj_set[i].framec[j]->context != NULL)
					fprintf(out_fp, "\t\tcontext(%d): %s\n",
					Obj_set[i].framec[j]->cs_context,
					Obj_set[i].framec[j]->context);
				if (Obj_set[i].framec[j]->extract != NULL)
					fprintf(out_fp, "\t\textract(%d): %s\n",
					Obj_set[i].framec[j]->cs_extract,
					Obj_set[i].framec[j]->extract);
				if (Obj_set[i].framec[j]->except != NULL)
					fprintf(out_fp, "\t\texcept(%d): %s\n",
					Obj_set[i].framec[j]->cs_except,
					Obj_set[i].framec[j]->except);
				if (Obj_set[i].framec[j]->filter != NULL)
					fprintf(out_fp, "\t\tfilter(%d): %s\n",
					Obj_set[i].framec[j]->use_quote,
					Obj_set[i].framec[j]->filter);
				if (Obj_set[i].framec[j]->subst1 != NULL &&
					Obj_set[i].framec[j]->subst2 != NULL)
					fprintf(out_fp, "\t\tsubstitute: %s -> %s\n",
					Obj_set[i].framec[j]->subst1,
					Obj_set[i].framec[j]->subst2);
				fprintf(out_fp, "\t    }\n");
			}
		}
	}
	for (i = 0; i < Num_rel_sets; i++) {
		fprintf(out_fp, "%s%c (%d connections)\n", Rel_set[i].name,
			Rel_set[i].marked ? '*' : ' ', Rel_set[i].num_conns);
		for (j = 0; j < Rel_set[i].num_conns; j++) {
			fprintf(out_fp, "    %s %s [%soptional, %smultiple (%g,%g)]\n",
				Rel_set[i].conn[j].rs_name_frag,
				Rel_set[i].conn[j].obj_set->name,
				Rel_set[i].conn[j].pc_optional ? "" : "not ",
				Rel_set[i].conn[j].pc_multiple ? "" : "not ",
				Rel_set[i].conn[j].pc_mean,
				Rel_set[i].conn[j].pc_stddev);
		}
	}

} /* Dump_Graph */

/*------------------------------------------------------------------------
* FUNCTION Dump_Text -- write the current parse buffer to an output file
*/
void
Dump_Text(FILE *out_fp, char *str, int col, int row)
{
	int	bol = 1,
		in_window = FALSE,
		line = 0,
		printed = 0;

	fprintf(out_fp, "Input Text:\n===========\n");
	if (line >= Line_num - WINDOW_SIZE && line <= Line_num + WINDOW_SIZE)
		in_window = TRUE;
	else
		in_window = FALSE;
	for (line = 0; *str; str++) {
		if (bol) {
			++line;
			if (line >= Line_num - WINDOW_SIZE &&
				line <= Line_num + WINDOW_SIZE)
				in_window = TRUE;
			else
				in_window = FALSE;
			if (line == row + 1) {
				for (printed = 0; printed < col + 9; printed++)
					putc('-', out_fp);
				fprintf(out_fp, "^\n");
				printed = 1;
			}
			if (in_window)
				fprintf(out_fp, "%8d: ", line);
			bol = 0;
		}
		if (*str == '\n')
			bol = 1;
		if (in_window)
			putc(*str, out_fp);
	}
	if (!bol)
		putc('\n', out_fp);
	if (!printed) {
		for (printed = 0; printed < col + 9; printed++)
			putc('-', out_fp);
		fprintf(out_fp, "^\n");
	}

} /* Dump_Text */

/*------------------------------------------------------------------------
* FUNCTION Dup_String -- duplicate the given string
*/
char *
Dup_String(const char *str, int len)
{
	char	*new_str = new char[len + 1];

	memcpy(new_str, str, len);
	new_str[len] = '\0';

	return (new_str);

} /* Dup_String */


/*------------------------------------------------------------------------
* Temporary marks are used by Emit_Table_SQL
*/
void Clear_Tmp_Marks()
{
	for (int i = 0; i < Num_obj_sets; i++)
		Obj_set[i].tmp_marked = false;
}


/*------------------------------------------------------------------------
* FUNCTION Emit_SQL -- generate SQL DDL for the ORM graph
*/
void
Emit_SQL(FILE *sql_fp, FILE *orc_fp, FILE *ecf_fp)
{
	int		i,
		j;
	bool		first,
		first_rs,
		found_rs;
	CONNECTION	*c1,
		*c2;

	Clear_Marks();
	Compute_Max_Ones();

	/* First list the objects. */
	first = TRUE;
	for (i = 0; i < Num_obj_sets; i++) {
		if (Obj_set[i].rp_object) {
			if (first) {
				fprintf(orc_fp, "Object: ");
				fprintf(ecf_fp, "Object: ");
				first = FALSE;
			} else {
				fprintf(orc_fp, ", ");
				fprintf(ecf_fp, ", ");
			}
			fprintf(orc_fp, "%s", Obj_set[i].name);
			fprintf(ecf_fp, "%s", Obj_set[i].name);
		}
	}
	if (!first) {
		fprintf(orc_fp, ";\n");
		fprintf(ecf_fp, ";\n");
	}

	/* Then list the nonlexical object sets. */
	fprintf(orc_fp, "Nonlexical: ");
	fprintf(ecf_fp, "Nonlexical: ");
	first = TRUE;
	for (i = 0; i < Num_obj_sets; i++) {
		if (!Is_Lexical(&Obj_set[i]) && !Obj_set[i].rp_object) {
			if (!first) {
				fprintf(orc_fp, ", ");
				fprintf(ecf_fp, ", ");
			}
			fprintf(orc_fp, "%s", Obj_set[i].name);
			fprintf(ecf_fp, "%s", Obj_set[i].name);
			first = FALSE;
			found_rs = FALSE;
			first_rs = TRUE;
			for (j = 0; j < Num_rel_sets; j++) {
				c1 = &Rel_set[j].conn[0];
				c2 = &Rel_set[j].conn[1];
				if ( Rel_set[j].num_conns == 2 &&
					(c1->obj_set == &Obj_set[i] ||
					c2->obj_set == &Obj_set[i]) ) {
						if (c2->obj_set == &Obj_set[i]) {
							CONNECTION	*temp = c1;

							c1 = c2;
							c2 = temp;
						}
						if (!c1->pc_multiple && Is_Lexical(c2->obj_set)) {
							if (first_rs) {
								first_rs = FALSE;
								fprintf(orc_fp, " {");
								fprintf(ecf_fp, " {");
							} else {
								fprintf(orc_fp, ", ");
								fprintf(ecf_fp, ", ");
							}
							fprintf(orc_fp, "%s", c2->obj_set->name);
							fprintf(ecf_fp, "%s", c2->obj_set->name);
							found_rs = TRUE;
						}
					}
			}
			if (found_rs) {
				fprintf(orc_fp, "}");
				fprintf(ecf_fp, "}");
			}
		}
	}
	fprintf(orc_fp, ";\n");
	fprintf(ecf_fp, ";\n");

	/* Then list the lexical object sets. */
	fprintf(orc_fp, "Lexical: ");
	fprintf(ecf_fp, "Lexical: ");
	first = TRUE;
	for (i = 0; i < Num_obj_sets; i++) {
		if (Is_Lexical(&Obj_set[i])) {
			if (!first) {
				fprintf(orc_fp, ", ");
				fprintf(ecf_fp, ", ");
			}
			fprintf(orc_fp, "%s", Obj_set[i].name);
			fprintf(ecf_fp, "%s", Obj_set[i].name);
			first = FALSE;
		}
	}
	fprintf(orc_fp, ";\n");
	fprintf(ecf_fp, ";\n");

	/* Now list nonlexical generalization/specializations. */
	for (i = 0; i < Num_obj_sets; i++) {
		if (Obj_set[i].parent != NULL && !Is_Lexical(&Obj_set[i])) {
			fprintf(orc_fp, "%s isa %s;\n", Obj_set[i].name,
				Obj_set[i].parent->name);
			fprintf(ecf_fp, "%s isa %s;\n", Obj_set[i].name,
				Obj_set[i].parent->name);
		}
	}

	/*
	* Now find an object set with a max participation constraint of 1
	* and build table from that object set.
	*/
	while (Exists_Unmarked_Rel_Set()) {
		NODE	*table = Find_Major_Obj_Set();
		Clear_Tmp_Marks();
		if (table == NULL)
			break;

		fprintf(sql_fp, "create table %s (\n    %s %s",
			table->name, table->name,
			Is_Lexical(table) ? Lex_type : Nonlex_type);
		if (Is_Lexical(table))
			fprintf(sql_fp, "(%d)", table->max_len == PC_STAR ?
DEFAULT_MAX_LEN : table->max_len);
		table->marked = TRUE;
		table->tmp_marked = TRUE;

		Emit_Table_SQL(sql_fp, orc_fp, ecf_fp, table, table);
		fprintf(sql_fp, ")\n");
	}

	/* Emit many-to-many tables for remaining unmarked relationship sets. */
	for (i = 0; i < Num_rel_sets; i++) {
		if (!Rel_set[i].marked) {
			int	changed,
				conn_ix[MAX_REL_SET_ARITY],
				res,
				temp;

			for (j = 0; j < Rel_set[i].num_conns; j++)
				conn_ix[j] = j;
			changed = TRUE;
			while (changed) {
				changed = FALSE;
				for (j = 1; j < Rel_set[i].num_conns; j++) {
					res = strcasecmp(Rel_set[i].conn[conn_ix[j - 1]].obj_set->name,
						Rel_set[i].conn[conn_ix[j]].obj_set->name);
					if (res > 0) {
						temp = conn_ix[j];
						conn_ix[j] = conn_ix[j - 1];
						conn_ix[j - 1] = temp;
						changed = TRUE;
					}
				}
			}

			fprintf(sql_fp, "create table ");
			for (j = 0; j < Rel_set[i].num_conns; j++) {
				fprintf(sql_fp, "%s", Rel_set[i].conn[conn_ix[j]].obj_set->name);
				fprintf(orc_fp, "%s", Rel_set[i].conn[conn_ix[j]].obj_set->name);
				fprintf(ecf_fp, "%s", Rel_set[i].conn[conn_ix[j]].obj_set->name);
			}
			fprintf(sql_fp, " (");
			fprintf(orc_fp, ":");
			fprintf(ecf_fp, ":");
			Emit_Structure(orc_fp, &Rel_set[i], PC_ONLY);
			Emit_Structure(ecf_fp, &Rel_set[i], PC_EXPECTED);
			for (j = 0; j < Rel_set[i].num_conns; j++) {
				if (j > 0)
					fprintf(sql_fp, ",");
				fprintf(sql_fp, "\n    %s %s",
					Rel_set[i].conn[conn_ix[j]].obj_set->name,
					Is_Lexical(Rel_set[i].conn[conn_ix[j]].obj_set) ? Lex_type :
				Nonlex_type);
				if (Is_Lexical(Rel_set[i].conn[conn_ix[j]].obj_set))
					fprintf(sql_fp, "(%d)",
					Rel_set[i].conn[conn_ix[j]].obj_set->max_len == PC_STAR ?
DEFAULT_MAX_LEN :
				Rel_set[i].conn[conn_ix[j]].obj_set->max_len);
				Rel_set[i].conn[conn_ix[j]].obj_set->marked = TRUE;
			}
			fprintf(sql_fp, ")\n");
			Rel_set[i].marked = TRUE;
		}
	}

	/* Emit standalone tables for remaining unmarked object sets. */
	for (i = 0; i < Num_obj_sets; i++) {
		if ( !Obj_set[i].marked &&
			(!Obj_set[i].has_spec || !Is_Lexical(&Obj_set[i])) ) {
				fprintf(sql_fp, "create table %s (\n    %s %s",
					Obj_set[i].name, Obj_set[i].name,
					Is_Lexical(&Obj_set[i]) ? Lex_type : Nonlex_type);
				if (Is_Lexical(&Obj_set[i]))
					fprintf(sql_fp, "(%d)", Obj_set[i].max_len == PC_STAR ?
DEFAULT_MAX_LEN : Obj_set[i].max_len);
				fprintf(sql_fp, ")\n");

				Obj_set[i].marked = TRUE;
			}
	}

} /* Emit_SQL */

/*------------------------------------------------------------------------
* FUNCTION Emit_Structure -- emit relationship set declaration
*/
void
Emit_Structure(FILE *fp, EDGE *rs, bool expected)
{
	bool	multiple,
		optional;

	if (rs->max_one) {
		fprintf(fp, " %s [", rs->gen_conn->obj_set->name);
		if (rs->obj_conn->replace) {
			optional = rs->obj_conn->rp_optional;
		} else {
			optional = rs->obj_conn->pc_optional;
		}
		if (optional) {
			fprintf(fp, "0:1");
		} else {
			fprintf(fp, "1");
		}
		if (expected) {
			fprintf(fp, " (%g,%g)", rs->obj_conn->pc_mean, rs->obj_conn->pc_stddev);
		}
		fprintf(fp, "]");
	} else {
		for (int i = 0; i < rs->num_conns; i++) {
			if (strlen(rs->conn[i].rs_name_frag) > 0)
				fprintf(fp, " %s", rs->conn[i].rs_name_frag);
			fprintf(fp, " %s [", rs->conn[i].obj_set->name);
			if (rs->conn[i].replace) {
				multiple = rs->conn[i].rp_multiple;
				optional = rs->conn[i].rp_optional;
			} else {
				multiple = rs->conn[i].pc_multiple;
				optional = rs->conn[i].pc_optional;
			}
			if (optional) {
				if (multiple)
					fprintf(fp, "0:*");
				else
					fprintf(fp, "0:1");
			} else {
				if (multiple)
					fprintf(fp, "1:*");
				else
					fprintf(fp, "1");
			}
			if (expected) {
				fprintf(fp, " (%g,%g)", rs->conn[i].pc_mean,
					rs->conn[i].pc_stddev);
			}
			fprintf(fp, "]");
		}
	}
	fprintf(fp, ";\n");

} /* Emit_Structure */


/*------------------------------------------------------------------------
* Emit_Table_SQL -- emit SQL for this table recursively
*/
void
Emit_Table_SQL(FILE *sql_fp, FILE *orc_fp, FILE *ecf_fp,
			   NODE *table, NODE *node)
{
	int	i,
		j;
	bool	has_multiple;

	/*
	* NEEDSWORK: If an object set is 1-1 with object, omit the
	*		intermediate relationship set stuff and just
	*		generate info about the object set.
	*/
	for (i = 0; i < Num_rel_sets; i++) {
		if (!Rel_set[i].marked) {
			for (j = 0; j < Rel_set[i].num_conns; j++)
				if (node == Rel_set[i].conn[j].obj_set)
					break;
			if (j < Rel_set[i].num_conns && !Rel_set[i].conn[j].pc_multiple) {
				has_multiple = FALSE;
				for (j = 0; j < Rel_set[i].num_conns; j++) {
					if (Rel_set[i].conn[j].pc_multiple)
						has_multiple = TRUE;
				}
				for (j = 0; j < Rel_set[i].num_conns; j++) {
					//if (!Rel_set[i].conn[j].obj_set->marked ||
					if (!Rel_set[i].conn[j].obj_set->tmp_marked ||
						Rel_set[i].conn[j].pc_multiple) {
							fprintf(sql_fp, ",\n    %s %s",
								Rel_set[i].conn[j].obj_set->name,
								Is_Lexical(Rel_set[i].conn[j].obj_set) ? Lex_type :
							Nonlex_type);
							if (Is_Lexical(Rel_set[i].conn[j].obj_set))
								fprintf(sql_fp, "(%d)",
								Rel_set[i].conn[j].obj_set->max_len == PC_STAR ?
DEFAULT_MAX_LEN :
							Rel_set[i].conn[j].obj_set->max_len);
							Rel_set[i].conn[j].obj_set->marked = TRUE;
							Rel_set[i].conn[j].obj_set->tmp_marked = TRUE;
							if (!Rel_set[i].marked) {
								Rel_set[i].marked = TRUE;
								fprintf(orc_fp, "%s:", table->name);
								fprintf(ecf_fp, "%s:", table->name);
								Emit_Structure(orc_fp, &Rel_set[i], PC_ONLY);
								Emit_Structure(ecf_fp, &Rel_set[i], PC_EXPECTED);
							}
							if (!has_multiple)
								Emit_Table_SQL(sql_fp, orc_fp, ecf_fp, table,
								Rel_set[i].conn[j].obj_set);
						}
				}
			}
		}
	}

} /* Emit_Table_SQL */

/*------------------------------------------------------------------------
* FUNCTION Exists_Unmarked_Rel_Set -- see if there is still an unmarked
*				       relationship set in the ORM graph
*/
bool
Exists_Unmarked_Rel_Set()
{
	for (int i = 0; i < Num_rel_sets; i++)
		if (!Rel_set[i].marked)
			return (TRUE);
	return (FALSE);

} /* Exists_Unmarked_Rel_Set */

/*------------------------------------------------------------------------
* FUNCTION Extract_String -- extract the contents of a given string
*/
char *
Extract_String(char *str)
{
	if (str == NULL) {
		return ("");
	} else {
		int	len = strlen(str);

		if (len >= 2)
			str = Dup_String(str + 1, len - 2);
	}
	return (str);

} /* Extract_String */


bool Is_Major_Obj_Set(object_set_node * obj_set, bool allow_marked)
{
	int i, j;
	if (!allow_marked && obj_set->marked) return false;
	for (i = 0; i < Num_rel_sets; i++) 
	{
		if (!Rel_set[i].marked) 
		{
			for (j = 0; j < Rel_set[i].num_conns; j++)
				if (obj_set == Rel_set[i].conn[j].obj_set)
					break;
			if (j < Rel_set[i].num_conns && 
				        !Rel_set[i].conn[j].pc_multiple) 
				return true;
		}
	}
	return false;
}


/*------------------------------------------------------------------------
* FUNCTION Find_Major_Obj_Set -- find an object set with maximum one
*				  participation in a relationship set
*/
NODE *
Find_Major_Obj_Set()
{
	int	i;

	/* First look for unmarked object set in unmarked rel set. */
	for (i = 0; i < Num_rel_sets; i++) {
		if (!Rel_set[i].marked) {
			for (int j = 0; j < Rel_set[i].num_conns; j++) {
				//if (!Rel_set[i].conn[j].obj_set->marked &&
				//	!Rel_set[i].conn[j].pc_multiple) {
				if (Is_Major_Obj_Set(Rel_set[i].conn[j].obj_set, false))
					return (Rel_set[i].conn[j].obj_set);
			}
		}
	}

	/* If not found, now look for marked object set in unmarked rel set. */
	for (i = 0; i < Num_rel_sets; i++) {
		if (!Rel_set[i].marked) {
			for (int j = 0; j < Rel_set[i].num_conns; j++) {
				//if (!Rel_set[i].conn[j].pc_multiple) {
				if (Is_Major_Obj_Set(Rel_set[i].conn[j].obj_set, true))
					return (Rel_set[i].conn[j].obj_set);
			}
		}
	}

	return (NULL);

} /* Find_Major_Obj_Set */


/*------------------------------------------------------------------------
* FUNCTION Harmonylex -- get next input token
*/
int
Harmonylex(void)
{
	return (Harmony_lexer->yylex());

} /* Harmonylex */

extern "C" {

	/*------------------------------------------------------------------------
	* FUNCTION Harmonywrap -- tell lexer there are no more files
	*/
	int
		Harmonywrap(void)
	{
		return (TRUE);

	} /* Harmonywrap */

} /* End "C" linkage */

/*------------------------------------------------------------------------
* FUNCTION Is_All_Lower -- see if word is all lowercase
*/
int
Is_All_Lower(const char *ptr)
{
	while (*ptr) {
		if (*ptr >= 'A' && *ptr <= 'Z')
			return (FALSE);
		++ptr;
	}
	return (TRUE);

} /* Is_All_Lower */

/*------------------------------------------------------------------------
* FUNCTION Is_All_Upper -- see if word is all uppercase
*/
int
Is_All_Upper(const char *ptr)
{
	while (*ptr) {
		if (*ptr >= 'a' && *ptr <= 'z')
			return (FALSE);
		++ptr;
	}
	return (TRUE);

} /* Is_All_Upper */

/*------------------------------------------------------------------------
* FUNCTION Is_Lexical -- determine whether object set is lexical
*/
bool
Is_Lexical(NODE *obj_set)
{
	if (obj_set->num_framec > 0)
		return (TRUE);
	if (obj_set->parent == NULL)
		return (FALSE);
	return (Is_Lexical(obj_set->parent));

} /* Is_Lexical */

/*------------------------------------------------------------------------
* FUNCTION Max_Length -- determine the max length of an object set
*/
int
Max_Length(NODE *obj_set)
{
	if (obj_set->num_framec > 0)
		return (obj_set->max_len);
	if (obj_set->parent == NULL)
		return (0);
	return (Max_Length(obj_set->parent));

} /* Max_Length */

/*------------------------------------------------------------------------
* FUNCTION Set_Node_Length -- add an object-set node to the graph
*/
void
Set_Node_Length(char *name, int max_len)
{
	for (int i = 0; i < Num_obj_sets; i++)
		if (!strcmp(Obj_set[i].name, name)) {
			Obj_set[i].max_len = max_len;
			break;
		}

} /* Set_Node_Length */

/*------------------------------------------------------------------------
* FUNCTION String_Or_Null -- format regular expression string for output
*/
char *
String_Or_Null(char *str, bool case_sens)
{
	char	output[MAX_NAME_LEN + 10];

	if (str == NULL)
		return ("");
	if (case_sens)
		return (str);
	sprintf(output, "/%s/i", str);
	return (Dup_String(output, strlen(output)));

} /* String_Or_Null */

/*------------------------------------------------------------------------
* FUNCTION Strip_Word -- remove leading tildes & underscores and useless
*			  backslashes
*/
char *
Strip_Word(char *str)
{
	char	*from,
		*new_str,
		*ptr = str,
		*to;

	while (*ptr == '~' || *ptr == '_')
		++ptr;

	new_str = Dup_String(ptr, strlen(ptr));
	if (strchr(new_str, '\\') == NULL)
		return (new_str);

	to = from = new_str;
	while (*from) {
		if (*from == '\\') {
			++from;
			if (!*from)
				break;
			*to = *from;
		} else {
			if (to != from)
				*to = *from;
		}
		++to;
		++from;
	}
	*to = '\0';
	ptr = Dup_String(new_str, strlen(new_str));
	delete [] new_str;

	return (ptr);

} /* Strip_Word */

/*------------------------------------------------------------------------
* FUNCTION Use_Framec -- get a frame constant structure from global array
*/
FRAMEC *
Use_Framec(NODE *obj_set)
{
	if (Num_framecs >= MAX_FRAMECS) {
		fprintf(stderr, "ontology: Out of frame constants.\n");
		exit (-101);
	}
	Add_Frame_Constant(obj_set, &Framec[Num_framecs]);
	return (&Framec[Num_framecs++]);

} /* Use_Framec */

/*------------------------------------------------------------------------
* FUNCTION Use_Lexicon -- get a lexicon structure from global array
*/
LEXICON_ST *
Use_Lexicon()
{
	if (Num_lexicons >= MAX_LEXICONS) {
		fprintf(stderr, "ontology: Out of lexicon slots.\n");
		exit (-102);
	}
	Lexicon[Num_lexicons].name = NULL;
	Lexicon[Num_lexicons].filename = NULL;
	Lexicon[Num_lexicons].separators = NULL;
	Lexicon[Num_lexicons].case_sens = FALSE;
	return (&Lexicon[Num_lexicons++]);

} /* Use_Lexicon */

/*------------------------------------------------------------------------
* FUNCTION Write_Expressions -- write constant and keyword expressions
*/
void
Write_Expressions()
{
	NODE	*os;
	int	i,
		j;

	Clear_Marks();

	for (i = 0; i < Num_obj_sets; i++) {
		if ( Is_Lexical(&Obj_set[i]) &&
			(Obj_set[i].parent != NULL || !Obj_set[i].has_spec) ) {
				os = &Obj_set[i];
				while (os != NULL) {
					for (j = 0; j < os->num_framec; j++) {
						fprintf(EXP_fp, "%s : %s : %d : %s : %d : %s : %d",
							Obj_set[i].name,
							os->framec[j]->extract,
							os->framec[j]->cs_extract,
							(os->framec[j]->context != NULL) ?
							os->framec[j]->context : os->framec[j]->extract,
							(os->framec[j]->context != NULL) ?
							os->framec[j]->cs_context : os->framec[j]->cs_extract,
							Extract_String(os->framec[j]->except),
							os->framec[j]->cs_except);
						if (os->framec[j]->filter != NULL)
							fprintf(EXP_fp, " : %s%s",
							Extract_String(os->framec[j]->filter),
							os->framec[j]->use_quote ? "\"" : "");
						if (os->framec[j]->subst1 != NULL &&
							os->framec[j]->subst2 != NULL) {
								if (os->framec[j]->filter == NULL)
									fprintf(EXP_fp, " : ");
								fprintf(EXP_fp, " : %s : %s",
									Extract_String(os->framec[j]->subst1),
									Extract_String(os->framec[j]->subst2));
							}
							fprintf(EXP_fp, " : %g\n", os->framec[j]->confidence);
					}
					for (j = 0; j < os->num_keywd; j++) {
						fprintf(EXP_fp, "KEYWORD(%s) : %s : %d : %g\n", Obj_set[i].name,
							Extract_String(os->keyword[j]),
							Obj_set[i].case_sens ? 1 : 0,
							Obj_set[i].keyconf[j]);
					}
					os = os->parent;
				}
			} else if (!Is_Lexical(&Obj_set[i]) && Obj_set[i].num_keywd > 0) {
				for (j = 0; j < Obj_set[i].num_keywd; j++) {
					fprintf(EXP_fp, "KEYWORD(%s) : %s : %d : %g\n", Obj_set[i].name,
						Extract_String(Obj_set[i].keyword[j]),
						Obj_set[i].case_sens ? 1 : 0,
						Obj_set[i].keyconf[j]);
				}
			}
	}
	for (i = 0; i < Num_lexicons; i++) {
		fprintf(EXP_fp, "LEXICON : %s : %d : %s : %s\n",
			Lexicon[i].name,
			Lexicon[i].case_sens,
			(Lexicon[i].filename != NULL) ?
			Lexicon[i].filename : "default.dict",
			(Lexicon[i].separators != NULL) ?
			Lexicon[i].separators : "default_separators");
	}

} /* Write_Expressions */

/*------------------------------------------------------------------------
* FUNCTION yyerror -- handle a parse error
*/
void
yyerror(char *msg)
{
	Dump_Text(stderr, Parse_str, Col_num, Line_num);
	fprintf(stderr, "ontology: Line %ld Col %ld: %s\n",
		Line_num, Col_num, msg);
	exit (-100);

} /* yyerror */

/*========================================================================
*                      END OF FILE $RCSfile: semantic.C,v $
*/
