/*========================================================================
 * $RCSfile: semantic.H,v $
 *
 * $Author: mja47 $
 * $Date: 2005/07/01 22:44:05 $
 * $Revision: 1.1.1.2 $
 *
 * $Description: 
 * Header for parser support routines. $
 * 
 * $Log: semantic.H,v $
 * Revision 1.1.1.2  2005/07/01 22:44:05  mja47
 * no message
 *
 * Revision 1.1.1.1  2003/08/09 20:48:43  pwinward
 * no message
 *
 * Revision 1.2  2003/02/26 21:41:16  tim
 * Added misc. bug fixes and minor features to demo,ontology editor, and ontos
 *
 * Revision 1.1.1.1  2002/04/25 23:00:01  tim
 * no message
 *
 * Revision 1.1.1.1  1999/05/11 15:16:49  swliddle
 * Ontos.rev2 stuff
 *
 * Revision 1.3  1998/02/05  22:20:14  liddl
 * Checkpoint of reasonably decent version.
 *
 * Revision 1.2  1998/02/05 18:24:02  liddl
 * Working version for VLDB Ontology paper.
 *
 * Revision 1.1  1996/04/24  13:46:10  liddl
 * Initial revision
 *
 */

#ifndef _SEMANTIC_H_
#define _SEMANTIC_H_

#ifndef FILE
#include <stdio.h>
#endif

/*------------------------------------------------------------------------
 *			CONSTANTS
 */
#define LEXICAL			TRUE		/* Mark object set type	*/
#define NONLEXICAL		FALSE

#define ALLOC_CHUNK		10		/* Allocation chunk size*/
#define	MAX_REL_SET_ARITY	10		/* Static table sizes	*/
#define MAX_NODES		200		/*   for quick'n'dirty	*/
#define MAX_EDGES		200		/*   implementation	*/
#define MAX_LEXICONS		200
#define MAX_FRAMECS		10000
#define MAX_SPEC_NAMES		50
#define MAX_NAME_LEN		150

#define PC_STAR			2147483647	/* Effectively infinite	*/
						/*   max upper bound	*/
#define DEFAULT_MAX_LEN		80		/* Maximum database	*/
						/*   field length if	*/
						/*   none or * given	*/

/*------------------------------------------------------------------------
 *			TYPE DEFINITIONS
 */
typedef enum {	ANYCASE, EITHERCASE, LOWERCASE, UPPERCASE
	     } CASE_TYPE;

typedef struct lexicon_st {
  char		*name;			/* Lexicon name			*/
  char		*filename;		/* Filename for dictionary	*/
  char		*separators;		/* Separator characters		*/
  bool		case_sens;		/* Case sensitivity flag	*/
} LEXICON_ST;

typedef struct frame_const {
  char		*context;
  char		*extract;
  char		*except;
  char		*filter;
  char		*subst1;
  char		*subst2;
  bool		cs_context;
  bool		cs_extract;
  bool		cs_except;
  bool		use_quote;
  float		confidence;		/* Confidence of this rule	*/
} FRAMEC;

typedef struct pc_values {
  bool		pc_optional;		/* Optional participation	*/
  bool		pc_multiple;		/* Multiple participation	*/
  float		pc_mean;		/* Expected mean participation	*/
  float		pc_stddev;		/* Expected standard deviation	*/
  bool		replace;		/* Do we replace for instance?	*/
  bool		rp_optional;		/* Replacement values		*/
  bool		rp_multiple;
  bool		rp_object;		/* Replace with object		*/
  bool		rp_header;		/* Is a header object		*/
} PCONSTR;

typedef struct object_set_node {
  char		*name;			/* Object-set name		*/
  int		max_len;		/* Maximum length of field	*/
  bool		marked;			/* Object set is marked		*/
  bool		tmp_marked;		/* Another more temporary mark used in emitting tables */
  bool		case_sens;		/* Default case sensitivity	*/
  bool		has_spec;		/* True if this is a genrlztn.	*/
  bool		rp_object;		/* Replace with object		*/
  bool		max_one;		/* Has max occurrence of 1	*/
					/*   w.r.t. an object		*/
  struct object_set_node
		*parent;		/* Generalization of this set	*/
  int		num_framec;		/* Number of frame constants	*/
  int		alloc_framec;		/* Number of framec's allocated	*/
  FRAMEC	**framec;		/* This object set's frame	*/
					/*   constants			*/
  int		num_keywd;		/* Number of keywords		*/
  int		alloc_keywd;		/* Number of keywords allocated */
  char		**keyword;		/* This object set's keywords	*/
  float		*keyconf;		/* Keyword confidence values	*/
} NODE;

typedef struct connection {
  char		*rs_name_frag;		/* Lower-word sequence that	*/
					/*   precedes this object-set	*/
					/*   name in rel-set decl	*/
  NODE		*obj_set;		/* Pointer to connected obj-set	*/
  bool		pc_optional;		/* Optional participation	*/
  bool		pc_multiple;		/* Multiple participation	*/
  float		pc_mean;		/* Expected mean participation	*/
  float		pc_stddev;		/* Expected standard deviation	*/
  bool		replace;		/* Do we replace for instance?	*/
  bool		rp_optional;		/* Replacement values		*/
  bool		rp_multiple;
} CONNECTION;

typedef struct rel_set_edge {
  CONNECTION	conn[MAX_REL_SET_ARITY];/* Connections of this rel-set	*/
  int		num_conns;		/* Number of connections	*/
  char		*name;			/* Relationship-set name	*/
  bool		marked;			/* Relationship set is marked	*/
  bool		max_one;		/* Has max. occurrence of 1	*/
					/*   w.r.t. an object		*/
  CONNECTION	*obj_conn;		/* Object-side connection	*/
  CONNECTION	*gen_conn;		/* Object set to generate	*/
} EDGE;

/*------------------------------------------------------------------------
 *			CONSTANTS
 */
#ifndef TRUE
#  define TRUE		1
#endif
#ifndef FALSE
#  define FALSE		0
#endif

/*------------------------------------------------------------------------
 *			EXTERNAL GLOBAL VARIABLES
 */
extern CASE_TYPE	Case_switch;
extern FILE		*EXP_fp,
			*ORC_fp,
			*SQL_fp;
extern int		Num_spec_names;
extern char		Spec_name[MAX_SPEC_NAMES][MAX_NAME_LEN];

/*------------------------------------------------------------------------
 *                      FUNCTION DECLARATIONS
 */
extern void	Add_Connection(		char *		rs_name_frag,
					NODE *		obj_set,
					bool		pc_optional,
					bool		pc_multiple,
					float		pc_mean,
					float		pc_stddev,
					bool		replace,
					bool		rp_optional,
					bool		rp_multiple,
					EDGE *		edge		);
extern EDGE *	Add_Edge(		char *		name		);
extern void	Add_Edge_Name(		const char *	name,
					EDGE *		edge		);
extern void	Add_Frame_Constant(	NODE *		obj_set,
					FRAMEC *	frame		);
extern void	Add_Keyword(		NODE *		obj_set,
					char *		keyword,
					float		conf		);
extern NODE *	Add_Node(		char *		name		);
extern char *	Append_String(		const char *	str1,
					const char *	str2,
					const char *	str3		);
extern int	Case_Is_Key(		void				);
extern void	Clear_Marks(		void				);
extern void	Dump_Error(		char 		*msg,
					int 		col = -1,
					int 		row = -1	);
extern void	Dump_Graph(		FILE		*out_fp		);
extern void	Dump_Text(		FILE 		*out_fp,
					char 		*str,
					int 		col,
					int 		row		);
extern char *	Dup_String(		const char	*str,
					int		len		);
extern void	Emit_SQL(		FILE *		sql_fp,
					FILE *		orc_fp,
					FILE *		ecf_fp		);
extern void	Emit_Structure(		FILE *		orc_fp,
					EDGE *		rs,
					bool		expected	);
extern void	Emit_Table_SQL(		FILE *		sql_fp,
					FILE *		orc_fp,
					FILE *		ecf_fp,
					NODE *		table,
					NODE *		node		);
extern bool	Exists_Unmarked_Rel_Set(void				);
extern char *	Extract_String(		char *		str		);
extern NODE *	Find_Major_Obj_Set(	void				);
extern int	Harmonylex(		void				);
extern int	Is_All_Lower(		const char 	*ptr		);
extern int	Is_All_Upper(		const char 	*ptr		);
extern bool	Is_Lexical(		NODE *		obj_set		);
extern int	Max_Length(		NODE *		obj_set		);
extern void	Set_Node_Length(	char		*name,
					int		max_len		);
extern FRAMEC *	Use_Framec(		NODE *		obj_set		);
extern LEXICON_ST *
		Use_Lexicon(		void				);
extern void	Write_Expressions(	void				);
extern void	yyerror(		char 		*msg		);

#endif
/*========================================================================
 *			END OF FILE $RCSfile: semantic.H,v $
 */
