/*
 * DocumentChangeEvent.java
 *
 * Created on June 3, 2003, 8:58 PM
 */

package edu.byu.deg.osmx;

/**
 * Encapsulates information about a change of state in an {@link OSMXDocument}.
 * @author Alan
 */
public class DocumentChangeEvent extends javax.swing.event.ChangeEvent {
  
    /**
     * Defines a general type of event that does not fit one of the other types of
     * events defined by this class.
     */    
  public static final int GENERAL_EVENT = 0;
  /**
   * Denotes that an element has been created and added to the document.
   */  
  public static final int CREATE_ELEMENT_EVENT = 1;
  /**
   * Denotes that an element has been removed from the document.
   */  
  public static final int DELETE_ELEMENT_EVENT = 2;
  
  private int eventType;
  
  private OSMXElement element;
  
  /**
   * Creates a new instance of DocumentChangeEvent
   * @param source The object causing the event to be generated.
   * @param evtType One of the event constants defined by this class, corresponding to the type of
   * event generated.
   */
  public DocumentChangeEvent(Object source, int evtType) {
    super(source);
    if (evtType >= GENERAL_EVENT && evtType <= DELETE_ELEMENT_EVENT) {
      eventType = evtType;
    } else {
      eventType = GENERAL_EVENT;
    }
  }
  
  /**
   * Returns the type of change the document experienced.
   * @return One of the document event constants, corresponding to the type of event that
   * occurred.
   */  
  public int getType() {
    return eventType;
  }
  
  /**
   * Returns an <CODE>OSMXElement</CODE> associated with this event.
   * @return The element associated with this event.
   */  
  public OSMXElement getElement() {
    return element;
  } // method getElement
  
  /**
   * Stores a reference to an <CODE>OSMXElement</CODE> associated with this event.
   * @param elem The element to associate with this event.
   */  
  public void setElement(OSMXElement elem) {
    element = elem;
  } // method setElement
  
}
