/*
 * MatchedTextComparator.java
 *
 * Created on March 31, 2004, 3:30 PM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.MatchedTextType;

/**
 * A utility class for defining an order upon a list of <CODE>MatchedTextType</CODE>
 * objects.
 * @author aew2
 */
public class MatchedTextComparator implements java.util.Comparator {
    
    /**
     * Defines a total ordering on a set of <CODE>MatchedTextType</CODE> objects.  This
     * method will throw {@link ClassCastException} if both parameters are not of this
     * type.
     * @param o1 One of the <CODE>MatchedTextType</CODE> objects to use in the comparison.
     * @param o2 The other <CODE>MatchedTextType</CODE> object to use in the comparison.
     * @return The result of the comparison: -1 if <CODE>o1</CODE> precedes <CODE>o2</CODE>; 1
     * if the opposite is true; and 0 if they can be considered equal.
     *
     * The function defines three levels of order:
     * <ol><li>The <CODE>document</CODE> attribute (dictionary order)</li>
     * <li>The start position of the match (ascending)</li>
     * <li>The end position of the match (ascending)</li>
     * </ol>
     */    
    public int compare(Object o1, Object o2) {
        MatchedTextType m1 = (MatchedTextType) o1;
        MatchedTextType m2 = (MatchedTextType) o2;
        
        if (!m1.getDocument().equals(m2.getDocument())) {
            return m1.getDocument().compareTo(m2.getDocument());
        }
        
        if (m1.getStartPos() < m2.getStartPos()) return -1;
        else if (m1.getStartPos() > m2.getStartPos()) return 1;
        else if (m1.getEndPos() < m2.getEndPos()) return -1;
        else if (m1.getEndPos() > m2.getEndPos()) return 1;
        return 0;
    }
    
}
