/*
 * OSMXAnchorType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Represents an <CODE>Anchor</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXAnchorType extends edu.byu.deg.osmx.binding.impl.AnchorTypeImpl
implements PositionedElement {
  
    /**
     * Keeps track of objects registered to listen for position changes by this
     * element.
     */    
  protected List positionListeners;
  
  /** Creates a new instance of OSMXAnchorType */
  public OSMXAnchorType() {
    positionListeners = new ArrayList();
  } // constructor
  
  /**
   * Registers an object to listen for position changes by this element.
   * @param pl The object to register for position change event notification.
   */  
  public void addPositionListener(PositionListener pl) {
    positionListeners.add(pl);
  } // method addPositionListener
  
  /**
   * Un-registers an object currently listening for position change events.
   * @param pl The object to remove from the notification list for position changes.
   */  
  public void removePositionListener(PositionListener pl) {
    positionListeners.remove(pl);
  } // method removePositionListener
  
  /**
   * Changes the x-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new x-coordinate position of this element.
   */  
  public void setX(int value) {
    super.setX(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateX(this, value);
    }
  } // method setX
  
  /**
   * Changes the y-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new y-coordinate position of this element.
   */  
  public void setY(int value) {
    super.setY(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateY(this, value);
    }
  } // method setY
  
  /**
   * Changes the value of the <CODE>order</CODE> attribute and notifies the position
   * change listeners.
   * @param value The new value for the z-order of this element.
   */  
  public void setOrder(int value) {
    super.setOrder(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateOrder(this, value);
    }
  } // method setOrder
  
  /**
   * Indicates whether this element has explicitly been given an x and y coordinate
   * position.
   * @return <CODE>true</CODE> if both the <CODE>x</CODE> and <CODE>y</CODE> coordinate
   * attributes have been set; <CODE>false</CODE> otherwise.
   */  
  public boolean isSetPosition() {
    return (isSetX() && isSetY());
  }
  
 
  
}
