/*
 * OSMXConjunctionConnectionType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.ConditionType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
/**
 * Represents a <CODE>ConjunctionConnection</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXConjunctionConnectionType extends edu.byu.deg.osmx.binding.impl.ConjunctionConnectionTypeImpl
implements StyledElement {

  /**
   * Creates a new instance of OSMXConjunctionConnectionType
   */  
  public  OSMXConjunctionConnectionType (){
    styleListener = null;
  }

  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf((OSMXElement) getStyle());
    this.setAsParentOf((OSMXElement) getCondition());
  }

  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  public void setStyle(StyleType value) {
    StyleType oldVal = getStyle();
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
    if (value instanceof OSMXStyleType) {
      if (styleListener == null) {
        styleListener = new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            firePropertyChange(OSMXStyleType.STYLE_PROPERTY,
                               evt.getOldValue(), evt.getNewValue());
          }
        };
          }
      ((OSMXStyleType)value).addPropertyChangeListener(styleListener);
    }
    firePropertyChange(OSMXStyleType.STYLE_PROPERTY, oldVal, value);

  }

  /**
   * Stores a new value for the <CODE>condition</CODE> attribute.
   * @param value The new value to store.
   */  
  public void setCondition(ConditionType value) {
    this.replaceChild((OSMXElement) getCondition(), (OSMXElement) value);
    super.setCondition(value);
  }
}
