/*
 * OSMXConjunctionType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.OSMType;
import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.PositionedText;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
/**
 * Represents a <CODE>Conjunction</CODE> element in OSMX.
 * @author Alan Wessman
 * @Modified by Muhammed
 */
public class OSMXConjunctionType extends edu.byu.deg.osmx.binding.impl.ConjunctionTypeImpl
implements StyledElement, PositionedElement {

  protected PropertyChangeListener styleListener;
  protected List positionListeners = new ArrayList();
  protected Set workingInstance = new HashSet();;

  /**
   * Creates a new instance of OSMXConjunctionType
   */  
  public  OSMXConjunctionType () {
    styleListener = null;
  }


  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf(getConjunctionConnection());
    this.setAsParentOf((OSMXElement) getStyle());
    this.setAsParentOf((OSMXElement) getName());
  }

  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  public void setStyle(StyleType value) {
    StyleType oldVal = getStyle();
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
    if (value instanceof OSMXStyleType) {
      if (styleListener == null) {
        styleListener = new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            firePropertyChange(OSMXStyleType.STYLE_PROPERTY,
                               evt.getOldValue(), evt.getNewValue());
          }
        };
          }
      ((OSMXStyleType)value).addPropertyChangeListener(styleListener);
    }
    firePropertyChange(OSMXStyleType.STYLE_PROPERTY, oldVal, value);


  }

  /**
   * Stores a new value for the <CODE>name</CODE> child element.
   * @param value The new element to store.
   */  
  public void setName(PositionedText value) {
    this.replaceChild((OSMXElement) getName(), (OSMXElement) value);
    super.setName(value);
  }
  
  /**
   * Assigns a sub-model to the state.  This does not depend on or affect the
   * condition of the state element's <CODE>highLevel</CODE> attribute.
   * @param value The <CODE>OSM</CODE> element that is the root of the sub-model to add to the
   * state.
   */  
 /* public void setOSM(OSMType value) {
    this.replaceChild((OSMXElement) getOSM(), (OSMXElement) value);
    super.setOSM(value);
  }*/
  
  public void addPositionListener(PositionListener pl) {
	  System.out.println("AddedListState"+pl);
      positionListeners.add(pl);
  }
  
  public boolean isSetPosition() {
      return (isSetX() && isSetY());
  }
  
  public void removePositionListener(PositionListener pl) {
      positionListeners.remove(pl);
  }
  
  /**
   * Changes the value of the <CODE>order</CODE> attribute and notifies the position
   * change listeners.
   * @param value The new value for the z-order of this element.
   */  
  public void setOrder(int value) {
    super.setOrder(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateOrder(this, value);
    }
  } // method setOrder

  
  /**
   * Changes the x-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new x-coordinate position of this element.
   */  
  public void setX(int value) {
    super.setX(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateX(this, value);
    }
  } // method setX
  
  /**
   * Changes the y-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new y-coordinate position of this element.
   */
  public void setY(int value) {
    super.setY(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateY(this, value);
    }
  } // method setY
  

}
