/*
 * OSMXDataFrameExpression.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import java.awt.Color;

/**
 * Represents a <CODE>DataFrameExpression</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXDataFrameExpression extends edu.byu.deg.osmx.binding.impl.DataFrameExpressionImpl {
    
    public static final String COLOR_PROPERTY = "color";
    public static final String EXPRESSION_TEXT_PROPERTY = "expressionText";
    
    public static final String ACCEPTED_MATCH = "accepted";
    public static final String PROPOSED_MATCH = "proposed";
    public static final String REJECTED_MATCH = "rejected";
    
    /** Creates a new instance of OSMXDataFrameExpression */
    public OSMXDataFrameExpression() {
        if (getExpressionText() == null) {
            setExpressionText("");
        }
    }
    
    /**
     * Stores a new value for the <CODE>color</CODE> attribute.
     * @param value The new value to store.
     */    
    public void setColor(Color value) {
        Color oldVal = getColor();
        Color newVal = value; // in case value == null
        if ((newVal == null && oldVal == null) || !newVal.equals(oldVal)) {
            super.setColor(value);
            firePropertyChange(COLOR_PROPERTY, oldVal, value);
        }
    }
    
    /**
     * Stores a new value for the <CODE>expressionText</CODE> attribute.
     * @param value The new value to store.
     */    
    public void setExpressionText(String value) {
        String oldVal = getExpressionText();
        String newVal = new String(value); // in case value == null
        if (!newVal.equals(oldVal)) {
            super.setExpressionText(value);
            firePropertyChange(EXPRESSION_TEXT_PROPERTY, oldVal, value);
        }
    }
    
}
