/*
 * OSMXElement.java
 *
 * Created on June 11, 2003, 9:45 AM
 */

package edu.byu.deg.osmx;

import java.beans.*;
import java.util.*;
import java.lang.reflect.Field;
import com.sun.xml.bind.util.ListImpl;
import edu.byu.deg.osmx.binding.StyleType;

/** Superclass of all element classes and element type classes for the OSMX schema. */
public abstract class OSMXElement implements java.io.Serializable {
    /** */
    private Map wrappedCollectionFields;
    private Map replacedCollectionFields;
    private String internalID;
    
    protected PropertyChangeSupport propertyChangeListeners;
    protected PropertyChangeSupport internalIDChangeListeners;
    protected List elementDeletionListeners;
    protected OSMXElement parent;
    protected OSMXElement.OSMXElementDeletionListener deleteListener;
    protected OSMXDocument parentDoc;
    
    /** Creates a new instance of OSMXElement */
    public OSMXElement() {
        parentDoc = null;
        wrappedCollectionFields = new HashMap();
        replacedCollectionFields = new HashMap();
        
        propertyChangeListeners = new PropertyChangeSupport(this);
        internalIDChangeListeners = new PropertyChangeSupport(this);
        elementDeletionListeners = new LinkedList();
        
        deleteListener = new OSMXElement.OSMXElementDeletionListener() {
            public void elementDeleted(OSMXElement elem) {
                handleDeletionEvent(elem);
            }
        };
        
        setInternalID(null);
    } // constructor
    
    
    /** Workaround for a problem with the JAXB list implementation.
     * JAXB generates beans with either singleton fields (where at most one occurrence
     * of a child element may exist) or collection fields (to support multiple
     * child occurrences) based on the dictates of the XML schema.  The reference
     * implementation of JAXB provides the
     * {@link com.sun.xml.bind.util.ListImpl ListImpl} class as the implementation for
     * the {@link java.util.List} interface required for collection fields by the JAXB
     * specification.
     *
     * The reference implementation also provides for a user to specify a different
     * {@link java.util.List } implementation by sending certain customizing
     * instructions to the JAXB generator.  However, in the generated classes the
     * custom {@link java.util.List } implementation is wrapped inside a
     * {@link com.sun.xml.bind.util.ListImpl } object, which does not expose its
     * internal list.
     *
     * In order to allow non-<CODE>List</CODE> methods to be invoked on the custom
     * <CODE>List</CODE> implementation, it is therefore necessary to unwrap the
     * outer <CODE>ListImpl</CODE> and expose the custom <CODE>List</CODE> via a
     * special function ({@link #getInnerList }).
     *
     * The method documented here performs the unwrapping step for all
     * <CODE>ListImpl</CODE> fields belonging to <CODE>this</CODE>.
     */
    private void unwrapCollectionFields(Class toUnwrap) {
        // see if the class is a type under our control (belongs in this package hierarchy)
        String packageName = toUnwrap.getName();  //fix for bug in tomcat getPackage() returns null
        packageName = packageName.substring(0, packageName.lastIndexOf('.'));
        String thisPackage = OSMXElement.class.getName();
        thisPackage = thisPackage.substring(0, thisPackage.lastIndexOf('.'));
        if (!packageName.startsWith(thisPackage)) return;
        try {
            // grab all fields via reflection
            Field[] fields = toUnwrap.getDeclaredFields();
            // make sure we can manipulate them
            java.lang.reflect.AccessibleObject.setAccessible(fields, true);
            // iterate over fields to find the ListImpl fields to unwrap
            for (int i = 0; i < fields.length; i++) {
                Class fieldType = fields[i].getType();
                if (fieldType.equals(ListImpl.class)) {
                    // grab handle to original ListImpl object
                    ListImpl oldOuterList = (ListImpl) fields[i].get(this);
                    // declare an OSMXElementList object that we know about
                    OSMXElementList newInnerList = new OSMXElementList();
                    if (oldOuterList != null) {
                        // populate the new list with the data from the old
                        /*   NOTE: We must iterate over each element in the old list
	                         and add() it to the new one (and remove() it from the old)
	                         in order to properly handle registration and de-registration
	                         of the deletion listeners in the old and new OSMXElementList
	                         objects.  Passing the old list into the constructor for the new,
	                         or calling addAll() and removeAll(), does not handle the listeners
	                         properly and a bug results where deleting an element does not 
	                         persist when a document is saved, closed, and then re-opened.*/
                        for (Iterator j = oldOuterList.iterator(); j.hasNext();) {
                            newInnerList.add(j.next());
                            j.remove();
                        }
                    }
                    newInnerList.setParent(this);
                    ListImpl outerList = new ListImpl(newInnerList);
                    fields[i].set(this, outerList);
                    wrappedCollectionFields.put(outerList, newInnerList);
                    if (oldOuterList != null) {
                        if (replacedCollectionFields != null) {
                            replacedCollectionFields.put(oldOuterList, outerList);
                        }
                    }
                }
            }
            java.lang.reflect.AccessibleObject.setAccessible(fields, false);
            unwrapCollectionFields(toUnwrap.getSuperclass());
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    } // method unwrapCollectionFields
    
    /** Provides access to the internal, "wrapped" {@link OSMXElementList} that
     * underlies the {@link java.util.List} property passed in.
     * When accessing a collection property (e.g.,
     * {@link edu.byu.deg.osmx.binding.OSMType#getAllModelElements}), the returned
     * {@link java.util.List} is not guaranteed to be an {@link OSMXElementList} due to
     * a limitation of the JAXB runtime implementation currently used.  Instead, the
     * {@link java.util.List} is a wrapper for the {@link OSMXElementList} that
     * actually implements the collection property.  The present method provides access
     * to the underlying {@link OSMXElementList} since it contains important custom
     * methods beyond the {@link java.util.List} interface.
     * @param outerList The original {@link java.util.List} returned by the collection property's getter
     * method.
     * @return The wrapped {@link OSMXElementList}.
     */
    public OSMXElementList getInnerList(List outerList) {
        // return null if param is not right type (REVISIT: should raise exception)
        if (!(outerList instanceof ListImpl)) return null;
        if (!wrappedCollectionFields.containsKey(outerList)) {
            unwrapCollectionFields(this.getClass());
            if (!wrappedCollectionFields.containsKey(outerList)) {
                if (replacedCollectionFields.containsKey(outerList)) {
                    OSMXElementList result = getInnerList((List) replacedCollectionFields.get(outerList));
                    return result;
                }
                // REVISIT: should raise exception?
                return null;
            }
            replacedCollectionFields.clear();
            replacedCollectionFields = null;
        }
        List innerList = (List) wrappedCollectionFields.get(outerList);
        return (OSMXElementList) innerList;
    } // method getInnerList
    
    /** Returns the element's internal ID, which all elements (even those without an
     * <CODE>id</CODE> attribute) possess.  This ID is guaranteed to be unique within a
     * document.
     * @return The element's internal ID.
     */
    public String getInternalID() {
        if (internalID == null) {
            setInternalID(null);
        }
        return internalID;
    } // method getInternalID
    
    /** Assigns a string as the internal ID for the element, replacing the current one
     * and causing any element that previously owned the assigned ID to relinquish it
     * and obtain a new generated ID.  This method also causes an internal ID change
     * notification to be sent to all {@link OSMXElement.InternalIDChangeListener}s.
     * @param s The string to assign as the new internal ID.
     */
    public void setInternalID(String s) {
        if (s != null && s.equals(internalID)) return;
        String oldID = internalID;
        internalID = s;
        
        // fire property change notifications
        internalIDChangeListeners.firePropertyChange(
        new PropertyChangeEvent(this, "internalID", oldID, internalID)
        );
    } // method setInternalID
    
    /** Responds to notification of the deletion of an external element (one that isn't
     * <CODE>this</CODE>).  This is a no-op method intended to be overridden by
     * subclasses so that any response to an external element's deletion may be
     * delegated to the subclass.
     * @param deletedElement The element that was deleted.
     */
    protected void handleDeletionEvent(OSMXElement deletedElement) {
        // do nothing; subclasses may override
    } // method handleDeletionEvent
    
    /** Registers an object as a listener for {@link java.beans.PropertyChangeEvent}s
     * coming from this element.  These events will generally be fired when the
     * element's attributes or child elements are changed.
     * @param pcl The object to register as a property change listener.
     */
    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        propertyChangeListeners.addPropertyChangeListener(pcl);
    } // method addPropertyChangeListener
    
    /** Un-registers an object as a property change listener on the element.
     * @param pcl The object that should no longer be listening for property change events.
     */
    public void removePropertyChangeListener(PropertyChangeListener pcl) {
        propertyChangeListeners.removePropertyChangeListener(pcl);
    } // method removePropertyChangeListener
    
    /** Accessor for the element's collection of
     * {@link java.beans.PropertyChangeListener}s.
     * @return The element's registered property change listeners.
     */
    public PropertyChangeListener[] getPropertyChangeListeners() {
        return propertyChangeListeners.getPropertyChangeListeners();
    } // method getPropertyChangeListeners
    
    /** Registers an object as a listener for any internal ID changes that happen for
     * this element.  This allows anything that references the element by its ID to
     * update its reference so things don't get out of synch.
     * @param iidcl The object to register as an internal ID change listener for the element.
     */
    public void addInternalIDChangeListener(InternalIDChangeListener iidcl) {
        internalIDChangeListeners.addPropertyChangeListener(iidcl);
    } // method addInternalIDChangeListener
    
    /** Un-registers an object so that it no longer receives notifications for changes to
     * the element's internal ID.
     * @param iidcl The object to un-register.
     */
    public void removeInternalIDChangeListener(InternalIDChangeListener iidcl) {
        internalIDChangeListeners.removePropertyChangeListener(iidcl);
    } // method removeInternalIDChangeListener
    
    /** Provides access to the element's collection of registered internal ID change
     * listeners.
     * @return The collection of internal ID change listeners registered on the element.
     */
    public InternalIDChangeListener[] getInternalIDChangeListeners() {
        return (InternalIDChangeListener[]) internalIDChangeListeners.getPropertyChangeListeners();
    } // method getInternalIDChangeListeners
    
    /** Convenience method for invoking
     * {@link java.beans.PropertyChangeSupport#firePropertyChange(java.beans.PropertyChangeEvent)} on the
     * {@link #propertyChangeListeners} field, with <CODE>this</CODE> as the source for
     * the resultant {@link java.beans.PropertyChangeEvent}.
     * @param propertyName Name of the property.
     * @param oldValue The previous value of the property, before it changed.
     * @param newValue The new value of the property, after it changed.
     * @see java.beans.PropertyChangeEvent
     * @see java.beans.PropertyChangeSupport
     */
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        propertyChangeListeners.firePropertyChange(
        new PropertyChangeEvent(this, propertyName, oldValue, newValue)
        );
    } // method firePropertyChange
    
    /** Invokes {@link OSMXElement.OSMXElementDeletionListener#elementDeleted} on each
     * of the {@link OSMXElement.OSMXElementDeletionListener}s registered on this
     * element, then disposes of all listener registries so that stale references may
     * be garbage-collected.
     */
    public void delete() {
        Object[] listeners = elementDeletionListeners.toArray();
        for (int i = 0; i < listeners.length; i++) {
            OSMXElement.OSMXElementDeletionListener l = (OSMXElement.OSMXElementDeletionListener) listeners[i];
            l.elementDeleted(this);
        }
        elementDeletionListeners.clear();
    } // method delete
    
    /** Registers an object as a listener for deletion notifications from this element.
     * @param l The object to register to receive deletion notifications.
     */
    public void addElementDeletionListener(OSMXElement.OSMXElementDeletionListener l) {
        elementDeletionListeners.add(l);
    } // method addElementDeletionListener
    
    /** Un-registers an object so it no longer listens for deletion events on this
     * element.
     * @param l The object to remove as a deletion listener.
     */
    public void removeElementDeletionListener(OSMXElement.OSMXElementDeletionListener l) {
        elementDeletionListeners.remove(l);
    } // method removeElementDeletionListener
    
    /** Accessor for the collection of element deletion listeners registered for this
     * element.
     * @return The collection of registered element deletion listeners.
     */
    public OSMXElement.OSMXElementDeletionListener[] getElementDeletionListeners() {
        return (OSMXElement.OSMXElementDeletionListener[]) elementDeletionListeners.toArray();
    } // method getElementDeletionListeners
    
    /**
     * Narrowed type of <CODE>PropertyChangeListener</CODE> used to receive
     * notification of a change to an element's internal ID.
     */    
    public interface InternalIDChangeListener extends PropertyChangeListener {
        /* marker interface; no new methods defined */
    } // interface InternalIDChangeListener
    
    /**
     * Receives notification from an element it has registered for that the element
     * will be deleted.
     */    
    public interface OSMXElementDeletionListener {
        /**
         * Defines an action to perform when an element deletion notification is received.
         * @param elem The element that will be deleted.
         */
        public void elementDeleted(OSMXElement elem);
    } // interface OSMXElementListener
    
    /** Links this element to its parent element so that upward traversal of the object
     * tree may be possible.  JAXB does not natively create links from child to parent
     * so this method must be invoked before child<PRE>-></PRE>parent links may be
     * traversed.
     * <br>
     * The method also invokes {@link #addToParentDocument}.
     * @param elem The element to which this element belongs.
     */
    void setParent(OSMXElement elem) {
        parent = elem;
        
        addToParentDocument();
    }
    
    /** Invokes {@link OSMXDocument#addElement} on the root {@link OSMXOSMType} ancestor
     * element's {@link OSMXOSMType#parentDoc}.
     */
    protected void addToParentDocument() {
        OSMXDocument doc = getParentDocument();
        if (doc != null) {
            doc.addElement(this);
        }
    }
    
    /**
     * Stores a reference to the <CODE>OSMXDocument</CODE> to which the element
     * belongs.
     * @param doc The ontology that contains this element.
     */    
    void setParentDocument(OSMXDocument doc) {
        parentDoc = doc;
    }
    
    /** Accessor for the element to which this element belongs.
     * @return The parent element of this element.
     */
    public OSMXElement getParent() {
        return parent;
    }
    
    /** Returns the closest ancestor {@link OSMXOSMType} element
     * that is not <CODE>this</CODE>.
     * @return The closest ancestor <CODE>OSMXOSMType</CODE> element.
     */
    public OSMXOSMType getParentOSM() {
        OSMXElement parentElem = getParent();
        if (parentElem == null) return null;
        if (parentElem instanceof OSMXOSMType) {
            return (OSMXOSMType) parentElem;
        }
        return parentElem.getParentOSM();
    }
    
    /** Accessor for the parent {@link OSMXDocument}.
     * @return The <CODE>OSMXDocument</CODE> to which this element belongs, or
     * <CODE>null</CODE> if the element does not belong to a document or
     * {@link #getParent} returns <CODE>null</CODE>.
     */
    public OSMXDocument getParentDocument() {
        if (parentDoc != null) return parentDoc;
        if (parent != null) return parent.getParentDocument();
        else return null;
    }
    
    /** Invokes {@link #setParent} on the specified child element.  Strictly speaking,
     * the input element does not have to belong to <CODE>this</CODE>, but problems may
     * arise if it does not.
     * @param child The child element to link to <CODE>this</CODE>, the parent element.
     */
    protected void setAsParentOf(OSMXElement child) {
        if (child != null) child.setParent(this);
    }
    
    /** Invokes {@link #setParent} on all of the <CODE>OSMXElement</CODE>s contained in
     * the specified list.
     * @param childList The list of <CODE>OSMXElement</CODE>s that should be linked as children to
     * <CODE>this</CODE>.
     */
    protected void setAsParentOf(List childList) {
        if (childList != null) {
            if (childList instanceof OSMXElementList)
                ((OSMXElementList) childList).setParent(this);
            else if (childList instanceof ListImpl) {
                setAsParentOf(getInnerList(childList));
            }
        }
    }
    
    /** Replaces an element that is a child of <CODE>this</CODE> with another element.
     * When this method has finished, {@link #getParent} returns <CODE>null</CODE> for
     * the old child element and returns <CODE>this</CODE> for the new element.
     * @param oldChild The child element to replace.
     * @param newChild The element to set in place of the former child element.
     */
    protected void replaceChild(OSMXElement oldChild, OSMXElement newChild) {
        if (oldChild != null) {
            oldChild.setParent(null);
        }
        setAsParentOf(newChild);
    }
    
    /**
     * Replaces a list of child elements with another list.  Example: if element
     * <CODE>A</CODE> is defined to contain possibly many <CODE>B</CODE> elements,
     * calling this method would allow replacing <CODE>(b1, b2, b3)</CODE> in <CODE>A</CODE>
     * with a new list such as <CODE>(b4, b5)</CODE>.
     * @param oldChild The old list to replace.
     * @param newChild The new list to store in the element.
     */    
    protected void replaceChild(List oldChild, List newChild) {
        if (oldChild != null) {
            if (oldChild instanceof OSMXElementList) {
                ((OSMXElementList) oldChild).setParent(null);
            } else if (oldChild instanceof ListImpl) {
                OSMXElementList inner = getInnerList(oldChild);
                inner.setParent(null);
            }
        }
        setAsParentOf(newChild);
    }
    
    /**
     * Accessor for the <CODE>Style</CODE> child element.
     * @return The currently set style element, or <CODE>null</CODE>.
     */    
    public StyleType getStyle() {
        // this method will be overridden if the element is a StyledElement
        if (parent != null) return getParent().getStyle();
        return null;
    }
    
}
