/*
 * OSMXFontType.java
 *
 * Created on December 12, 2003, 1:29 AM
 */

package edu.byu.deg.osmx;

import java.awt.Color;
import java.awt.Font;

/**
 * Represents a <CODE>Font</CODE> element in OSMX.
 * @author Alan
 */
public class OSMXFontType extends edu.byu.deg.osmx.binding.impl.FontTypeImpl {
  
  public static final String FAMILY_PROPERTY = "family";
  public static final String STYLE_PROPERTY = "style";
  public static final String SIZE_PROPERTY = "size";
  public static final String COLOR_PROPERTY = "color";
  
  public static final String BOLD_STYLE = "bold";
  public static final String ITALIC_STYLE = "italic";
  public static final String BOLD_ITALIC_STYLE = "bold-italic";
  
  /**
   * Transforms a font style value (used by the {@link java.awt.Font} class) to a
   * string representation of the style, used by the OSMX architecture.
   * @param value The value to transform.
   * @return The appropriate style constant (<CODE>OSMXFontType.BOLD_STYLE</CODE>,
   * <CODE>OSMXFontType.ITALIC_STYLE</CODE>, or <CODE>OSMXFontType.BOLD_ITALIC_STYLE</CODE>)
   * for the integer style value given.
   */  
  public static String intToStyle(int value) {
    if (value == Font.BOLD + Font.ITALIC) return BOLD_ITALIC_STYLE;
    if (value == Font.BOLD) return BOLD_STYLE;
    if (value == Font.ITALIC) return ITALIC_STYLE;
    return null;
  }
  
  /**
   * Converts a font style constant from the OSMX storage format to an integer value
   * consistent with the constants defined in {@link java.awt.Font}.
   * @param style The style value from OSMX.
   * @return The integer style value: either <CODE>Font.BOLD</CODE>,
   * <CODE>Font.ITALIC</CODE>, or <CODE>Font.BOLD + Font.ITALIC</CODE>.
   */  
  public static int styleToInt(String style) {
    if (BOLD_ITALIC_STYLE.equalsIgnoreCase(style)) return Font.BOLD + Font.ITALIC;
    if (BOLD_STYLE.equalsIgnoreCase(style)) return Font.BOLD;
    if (ITALIC_STYLE.equalsIgnoreCase(style)) return Font.ITALIC;
    return Font.PLAIN;
  }
  
  /**
   * Stores a new value for the font family (or font face) attribute.
   * @param value The new value to store.
   */  
  public void setFamily(String value) {
    String oldVal = getFamily();
    super.setFamily(value);
    firePropertyChange(FAMILY_PROPERTY, oldVal, value);
  }
  
  /**
   * Clears the current setting for the font family attribute.
   */  
  public void unsetFamily() {
    String oldVal = getFamily();
    super.unsetFamily();
    firePropertyChange(FAMILY_PROPERTY, oldVal, null);
  }
  
  /**
   * Stores a new value in the text style attribute of this element.
   * @param value The new value to store.
   */  
  public void setTextStyle(String value) {
    String oldVal = getTextStyle();
    super.setTextStyle(value);
    firePropertyChange(STYLE_PROPERTY, oldVal, value);
  }
  
  /**
   * Clears the current setting for the text style attribute.
   */  
  public void unsetTextStyle() {
    String oldVal = getTextStyle();
    super.unsetTextStyle();
    firePropertyChange(STYLE_PROPERTY, oldVal, null);
  }
  
  /**
   * Stores a new value for the font size attribute of this element.
   * @param value The new value to store.
   */  
  public void setSize(float value) {
    Float oldVal = (isSetSize() ? new Float(getSize()) : null);
    super.setSize(value);
    firePropertyChange(SIZE_PROPERTY, oldVal, new Float(value));
  }
  
  /**
   * Clears the current setting for the font size attribute.
   */  
  public void unsetSize() {
    Float oldVal = (isSetSize() ? new Float(getSize()) : null);
    super.unsetSize();
    firePropertyChange(SIZE_PROPERTY, oldVal, null);
  }
  
  /**
   * Stores a new value for the font color attribute of this element.
   * @param value The new value to store.
   */  
  public void setColor(Color value) {
    Color oldVal = getColor();
    super.setColor(value);
    firePropertyChange(COLOR_PROPERTY, oldVal, value);
  }
  
  /**
   * Clears the current setting for the text color attribute.
   */  
  public void unsetColor() {
    Color oldVal = getColor();
    super.unsetColor();
    firePropertyChange(COLOR_PROPERTY, oldVal, null);
  }
  
}
