/*
 * OSMXKeywordPhraseType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.DataFrameExpression;
import edu.byu.deg.osmx.binding.ObjectFactory;

/**
 * Represents a <CODE>KeywordPhrase</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXKeywordPhraseType extends edu.byu.deg.osmx.binding.impl.KeywordPhraseTypeImpl {
  
  public static final String HINT_PROPERTY = "hint";
  public static final String CONFIDENCE_TAG_PROPERTY = "confidenceTag";
  public static final String CASE_SENSITIVE_PROPERTY = "caseSensitive";
  
  /** Creates a new instance of OSMXKeywordPhraseType */
  public OSMXKeywordPhraseType() {
    if (getKeywordExpression() == null) {
      try {
        setKeywordExpression(createDataFrameExpression());
      } catch (javax.xml.bind.JAXBException e) {
        e.printStackTrace();
      }
    }
  }
  
  private DataFrameExpression createDataFrameExpression() throws javax.xml.bind.JAXBException {
    OSMXDocument doc = getParentDocument();
    ObjectFactory factory;
    if (doc != null) {
      factory = doc.getObjectFactory();
    } else {
      factory = new ObjectFactory();
    }
    return factory.createDataFrameExpression();
  }
  
  /**
   * Stores a new value for the hint (label) for this phrase.
   * @param value The new value to store.
   */  
  public void setHint(String value) {
    String oldVal = getHint();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setHint(value);
      firePropertyChange(HINT_PROPERTY, oldVal, value);
    }
  } // method setHint
  
  /**
   * Stores a new value for the confidence measure assigned to matches that belong to this
   * phrase.
   * @param value The new value to store.
   */  
  public void setConfidenceTag(String value) {
    String oldVal = getConfidenceTag();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setConfidenceTag(value);
      firePropertyChange(CONFIDENCE_TAG_PROPERTY, oldVal, value);
    }
  } // method setConfidenceTag
  
  /**
   * Stores a new value for the <CODE>caseSensitive</CODE> flag for the phrase.
   * @param value The new value to store; <CODE>true</CODE> means that the matching expressions
   * will be case-sensitive, and <CODE>false</CODE> means that case-insensitive
   * matching will be performed.
   */  
  public void setCaseSensitive(boolean value) {
    boolean oldVal = isCaseSensitive();
    if (oldVal != value) {
      super.setCaseSensitive(value);
      firePropertyChange(CASE_SENSITIVE_PROPERTY, new Boolean(oldVal), new Boolean(value));
    }
  } // method setCaseSensitive
  
  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf((OSMXElement) getKeywordExpression());
  }
  
  /**
   * Assigns a <CODE>DataFrameExpression</CODE> element to this phrase as the keyword
   * recognition expression.
   * @param value The <CODE>DataFrameExpression</CODE> to store.
   */  
  public void setKeywordExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getKeywordExpression(), (OSMXElement) value);
    super.setKeywordExpression(value);
  }
  
}
