/*
 * OSMXOSMType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import java.util.*;
import edu.byu.deg.osmx.binding.ObjectSetType;
import edu.byu.deg.osmx.binding.StateType;
import edu.byu.deg.osmx.binding.TransitionType;
import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.ObjectType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Represents an <CODE>OSM</CODE> element in OSMX.
 * @author Alan Wessman
 * @Modified by Muhammed
 */
public class OSMXOSMType extends edu.byu.deg.osmx.binding.impl.OSMTypeImpl
implements StyledElement {

  protected PropertyChangeListener styleListener;

  /**
   * Creates a new instance of OSMXOSMType
   */  
  public OSMXOSMType () {
    styleListener = null;
  }
  
  /**
  * Convenience method for accessing all States that belong to this model.
  * @return A list of all States that belong to this model.
  */  
 public List getStates() {
   List result = new LinkedList();
   List elements = getAllModelElements();
   if (elements == null) return result;
   for (Iterator i = elements.iterator(); i.hasNext();) {
     Object o = i.next();
     if (o instanceof StateType) {
       result.add(o);
     }
   }
   return result;
 } 
  
  /**
   * Locates and returns an state belonging to the model,
   * based on the states's ID attribute.
   * @param id The ID of the desired state.
   * @return The state corresponding to the given ID, or <CODE>null</CODE> if no such
   * state exists.
   */  
  public StateType findState(String id) {
    if (id == null) return null;
    for (Iterator i = getStates().iterator(); i.hasNext();) {
      OSMXStateType objSet = (OSMXStateType) i.next();
      if (id.equals(objSet.getId())) {
        return objSet;
      }
      if (objSet.getOSM() != null) {
        StateType result = ((OSMXOSMType) objSet.getOSM()).findState(id);
        if (result != null) return result;
      }
    }
    return null;
  } 
  
  /**
   * Convenience method for accessing all Transitions that belong to this model.
   * @return A list of all Transitions that belong to this model.
   */  
  public List getTransition() {
    List result = new LinkedList();
    List elements = getAllModelElements();
    if (elements == null) return result;
    for (Iterator i = elements.iterator(); i.hasNext();) {
      Object o = i.next();
      if (o instanceof TransitionType) {
        result.add(o);
      }
    }
    return result;
  } 
   
   /**
    * Locates and returns an transition belonging to the model,
    * based on the transition's ID attribute.
    * @param id The ID of the desired transition.
    * @return The transition corresponding to the given ID, or <CODE>null</CODE> if no such
    * transition exists.
    */  
   public TransitionType findTransition(String id) {
     if (id == null) return null;
     for (Iterator i = getTransition().iterator(); i.hasNext();) {
       OSMXTransitionType objSet = (OSMXTransitionType) i.next();
       if (id.equals(objSet.getId())) {
         return objSet;
       }
       if (objSet.getOSM() != null) {
         TransitionType result = ((OSMXOSMType) objSet.getOSM()).findTransition(id);
         if (result != null) return result;
       }
     }
     return null;
   } 


  
  /**
   * Convenience method for accessing all object sets that belong to this model.
   * @return A list of all object sets that belong to this model.
   */  
  public List getObjectSets() {
    List result = new LinkedList();
    List elements = getAllModelElements();
    if (elements == null) return result;
    for (Iterator i = elements.iterator(); i.hasNext();) {
      Object o = i.next();
      if (o instanceof ObjectSetType) {
        result.add(o);
      }
    }
    return result;
  } // method getObjectSets

  /**
   * Returns a list of object sets belonging to this model,
   * sorted based on the given <CODE>Comparator</CODE>.
   * @param c The <CODE>Comparator</CODE> used to sort the list.
   * @return The sorted list of object sets.
   */  
  public List getObjectSets(java.util.Comparator c) {
    List result = getObjectSets();
    java.util.Collections.sort(result, c);
    return result;
  } // method getObjectSets(Comparator)

  /**
   * Locates and returns an object set belonging to the model,
   * based on the object set's ID attribute.
   * @param id The ID of the desired object set.
   * @return The object set corresponding to the given ID, or <CODE>null</CODE> if no such
   * object set exists.
   */  
  public ObjectSetType findObjectSet(String id) {
    if (id == null) return null;
    for (Iterator i = getObjectSets().iterator(); i.hasNext();) {
      OSMXObjectSetType objSet = (OSMXObjectSetType) i.next();
      if (id.equals(objSet.getId())) {
        return objSet;
      }
      if (objSet.getOSM() != null) {
        ObjectSetType result = ((OSMXOSMType) objSet.getOSM()).findObjectSet(id);
        if (result != null) return result;
      }
    }
    return null;
  } // method findObjectSet

  /**
   * Convenience method for accessing all Object elements that directly belong to this model.
   * @return A list of the Object elements that are direct child elements of this model.
   */  
  public List getObjects() {
    List result = new LinkedList();
    List elements = getAllModelElements();
    if (elements == null) return result;
    for (Iterator i = elements.iterator(); i.hasNext();) {
      Object o = i.next();
      if (o instanceof OSMXObjectType) {
        result.add(o);
      }
    }
    return result;
  } // method getObjects

  /**
   * Returns a sorted list of the Object elements that directly belong to this model.
   * @param c The <CODE>Comparator</CODE> used to sort the list.
   * @return The list, whose Object elements are sorted according to the order defined by the
   * given <CODE>Comparator</CODE>.
   */  
  public List getObjects(java.util.Comparator c) {
    List result = getObjects();
    java.util.Collections.sort(result, c);
    return result;
  } // method getObjects(Comparator)

  /**
   * Locates and returns an Object element belonging directly to the model,
   * based on the object's ID attribute.
   * @param id The ID of the desired object.
   * @return The object with the given ID, or <CODE>null</CODE> if no such element exists.
   */  
  public ObjectType findObject(String id) {
    if (id == null) return null;
    for (Iterator i = getObjects().iterator(); i.hasNext();) {
      OSMXObjectType obj = (OSMXObjectType) i.next();
      if (id.equals(obj.getId())) {
        return obj;
      }
    }
    return null;
  } // method findObject

  /**
   * Returns a reference to the ontology that contains this element.
   * @return The ontology that contains this element.
   */  
  public OSMXDocument getParentDocument() {
    if (getParent() == null) return parentDoc;
    else return getParent().getParentDocument();
  }

  /**
   * Stores a reference to the containing ontology document
   * (<CODE>OSMXDocument</CODE>) so it can be accessed via this element.
   * @param doc The ontology to which this element belongs.
   */  
  public void setParentDocument(OSMXDocument doc) {
    parentDoc = doc;
    setParent(null);
  }

  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf(getAllModelElements());
    this.setAsParentOf((OSMXElement) getStyle());
  }

  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  public void setStyle(StyleType value) {
    StyleType oldVal = getStyle();
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
    if (value instanceof OSMXStyleType) {
      if (styleListener == null) {
        styleListener = new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            firePropertyChange(OSMXStyleType.STYLE_PROPERTY,
                               evt.getOldValue(), evt.getNewValue());
          }
        };
          }
      ((OSMXStyleType)value).addPropertyChangeListener(styleListener);
    }
    firePropertyChange(OSMXStyleType.STYLE_PROPERTY, oldVal, value);

  }
}
