/*
 * OSMXParameterType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.TypeSpecification;
import edu.byu.deg.osmx.binding.ObjectFactory;

/**
 * Represents a <CODE>Parameter</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXParameterType extends edu.byu.deg.osmx.binding.impl.ParameterTypeImpl
implements KeywordListOwner {
  
  public static final String NAME_PROPERTY = "name";
  public static final String DECORATOR_PROPERTY = "decorator";
  
  /** Creates a new instance of OSMXParameterType */
  public OSMXParameterType() {
    if (getType() == null) {
      try {
        setType(createTypeSpecification());
      } catch (javax.xml.bind.JAXBException e) {
        e.printStackTrace();
      }
    }
  }
  
  private TypeSpecification createTypeSpecification() throws javax.xml.bind.JAXBException {
    OSMXDocument doc = getParentDocument();
    ObjectFactory factory;
    if (doc != null) {
      factory = doc.getObjectFactory();
    } else {
      factory = new ObjectFactory();
    }
    return factory.createTypeSpecification();
  }
  
  /**
   * Represents the <CODE>Parameter</CODE> element as a string.
   * @return The string representation of the parameter, which is in the following format:
   * <CODE>decorator param_name: param_type</CODE>.
   */  
  public String toString() {
    return toString(getName());
  }
  
  /**
   * Represents the parameter as a string, using the input string in place of the
   * parameter name in the output.
   * @param altName The string to substitute for the parameter name in the output.
   * @return A string in the following format:
   *
   * <CODE>param_decorator alt-name: param_type</CODE>
   */  
  public String toString(String altName) {
    String decor = getDecorator();
    TypeSpecification ts = getType();
    String type = null;
    if (ts != null) type = ts.toString();
    String result = altName;
    if (decor != null && decor.length() > 0) {
      result = decor + " " + result;
    }
    if (type != null && type.length() > 0) {
      result += ": " + type;
    }
    return result;
  }
  
  /**
   * Assigns a new value to the <CODE>name</CODE> attribute of the parameter.
   * @param value The value to assign as the parameter name.
   */  
  public void setName(String value) {
    String oldVal = getName();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setName(value);
      firePropertyChange(NAME_PROPERTY, oldVal, value);
    }
  }
  
  /**
   * Stores a new value for the <CODE>decorator</CODE> attribute of the parameter.
   * @param value The value to store.
   */  
  public void setDecorator(String value) {
    String oldVal = getDecorator();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setDecorator(value);
      firePropertyChange(DECORATOR_PROPERTY, oldVal, value);
    }
  }
  
  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf((OSMXElement) getType());
    this.setAsParentOf(this.getInnerList(getKeywordPhrase()));
  }
  
  /**
   * Assigns the given <CODE>TypeSpecification</CODE> as the parameter type.
   * @param value The <CODE>TypeSpecification</CODE> element to set as the parameter type.
   */  
  public void setType(TypeSpecification value) {
    this.replaceChild((OSMXElement) getType(), (OSMXElement) value);
    super.setType(value);
  }
  
}
