/*
 * OSMXStateType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.RealTimeConstraintType;
import edu.byu.deg.osmx.binding.OSMType;
import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.RealTimeConstraintType;
import edu.byu.deg.osmx.binding.OSMType;
import java.util.*;

import edu.byu.deg.osmx.binding.OSMType;
import edu.byu.deg.osmx.binding.DataFrameType;
import edu.byu.deg.osmx.binding.StyleType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
/**
 * Represents a <CODE>State</CODE> element in OSMX.
 * @author Alan Wessman
 * @Modified by Muhammed
 */
public class OSMXStateType extends edu.byu.deg.osmx.binding.impl.StateTypeImpl
implements StyledElement, TextAlignableElement, ModelContainer, PositionedElement,ScalableElement {  
	 protected List positionListeners = new ArrayList();
	  protected PropertyChangeListener styleListener;
	  private DataFrameType suppressedDataFrame;
	  protected Set relSetConnections = new HashSet();
	  protected Set genConnections = new HashSet();
	  protected Set specConnections = new HashSet();
	  protected Set workingInstance = new HashSet();
	
    /**
     * Overrides the inherited method in order to pass the call recursively down to its
     * child elements.
     * @param elem The new parent of this element.
     */    
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf((OSMXElement) getStyle());
    this.setAsParentOf((OSMXElement) getRTC());
    this.setAsParentOf((OSMXElement) getOSM());
  }

  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  /*public void setStyle(StyleType value) {
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
  }*/
  
  
  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  public void setStyle(StyleType value) {
    StyleType oldVal = getStyle();
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
    if (value instanceof OSMXStyleType) {
      if (styleListener == null) {
        styleListener = new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            firePropertyChange(OSMXStyleType.STYLE_PROPERTY,
                               evt.getOldValue(), evt.getNewValue());
          }
        };
          }
      ((OSMXStyleType)value).addPropertyChangeListener(styleListener);
    }
    firePropertyChange(OSMXStyleType.STYLE_PROPERTY, oldVal, value);
  }
  
  /**
   * Stores a <CODE>RealTimeConstraint</CODE> element as a child of this element.
   * @param value The real-time constraint for this <CODE>State</CODE> element.
   */  
  public void setRTC(RealTimeConstraintType value) {
    this.replaceChild((OSMXElement) getRTC(), (OSMXElement) value);
    super.setRTC(value);
  }
  
  /**
   * Assigns a sub-model to the state.  This does not depend on or affect the
   * condition of the state element's <CODE>highLevel</CODE> attribute.
   * @param value The <CODE>OSM</CODE> element that is the root of the sub-model to add to the
   * state.
   */  
  public void setOSM(OSMType value) {
    this.replaceChild((OSMXElement) getOSM(), (OSMXElement) value);
    super.setOSM(value);
  }
  
  public void addPositionListener(PositionListener pl) {
	 // System.out.println("AddedListState"+pl);
      positionListeners.add(pl);
  }
  
  public boolean isSetPosition() {
      return (isSetX() && isSetY());
  }
  
  public void removePositionListener(PositionListener pl) {
      positionListeners.remove(pl);
  }
  
  public String getPrimaryName() {
	    String result = this.getId();
	    String name = getName();
	    if (name == null || name.length() == 0) {
	      setName(result);
	      return result;
	    }
	    return name;
	  }
	   public String toString() {
	    return getPrimaryName();
	  } // method toString
	    
	   private static final String NEWLINE = "\n";
	    private static final String ENCODED_NEWLINE = "\\\\n";
	   
	  public void setName(java.lang.String value) {
	      if (value != null) {
	          value = value.replaceAll(NEWLINE, ENCODED_NEWLINE);
	      }
	      super.setName(value);
	  }
	  
	   public void addConnection(OSMXBasicConnection conn) {
	        if (conn instanceof OSMXConjunctionConnectionType) {
	            relSetConnections.add(conn);
	        } 
	    }
	   
	    public void removeConnection(OSMXBasicConnection conn) {
	        if (conn instanceof OSMXConjunctionConnectionType) {
	            relSetConnections.remove(conn);
	        } 
	    }
	    
	    /**
	     * Changes the value of the <CODE>order</CODE> attribute and notifies the position
	     * change listeners.
	     * @param value The new value for the z-order of this element.
	     */  
	    public void setOrder(int value) {
	      super.setOrder(value);
	      for (Iterator i = positionListeners.iterator(); i.hasNext();) {
	        PositionListener pl = (PositionListener) i.next();
	        pl.updateOrder(this, value);
	      }
	    } // method setOrder
  
	    
	    /**
	     * Changes the x-coordinate position and causes the position listeners to be
	     * notified.
	     * @param value The new x-coordinate position of this element.
	     */  
	    public void setX(int value) {
	      super.setX(value);
	      for (Iterator i = positionListeners.iterator(); i.hasNext();) {
	        PositionListener pl = (PositionListener) i.next();
	        pl.updateX(this, value);
	      }
	    } // method setX
	    
	    /**
	     * Changes the y-coordinate position and causes the position listeners to be
	     * notified.
	     * @param value The new y-coordinate position of this element.
	     */
	    public void setY(int value) {
	      super.setY(value);
	      for (Iterator i = positionListeners.iterator(); i.hasNext();) {
	        PositionListener pl = (PositionListener) i.next();
	        pl.updateY(this, value);
	      }
	    } // method setY
}
