/*
 * OSMXTransitionType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import edu.byu.deg.osmx.binding.DataFrameType;
import edu.byu.deg.osmx.binding.StyleType;
import edu.byu.deg.osmx.binding.PositionedText;
import edu.byu.deg.osmx.binding.OSMType;

/**
 * Represents a <CODE>Transition</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXTransitionType extends edu.byu.deg.osmx.binding.impl.TransitionTypeImpl
implements  StyledElement, TextAlignableElement, ModelContainer, PositionedElement,ScalableElement {
  
	 protected List positionListeners = new ArrayList();
	  protected PropertyChangeListener styleListener;
	  private DataFrameType suppressedDataFrame;
	  protected Set relSetConnections = new HashSet();
	  protected Set genConnections = new HashSet();
	  protected Set specConnections = new HashSet();
	  protected Set workingInstance = new HashSet();
    /**
     * Overrides the inherited method in order to pass the call recursively down to its
     * child elements.
     * @param elem The new parent of this element.
     */    
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf(getRTC());
    this.setAsParentOf((OSMXElement) getStyle());
    this.setAsParentOf((OSMXElement) getIdentifier());
    this.setAsParentOf((OSMXElement) getOSM());
  }

  /**
   * Gives this element a new <CODE>Style</CODE> element and notifies listeners of a
   * style change.
   * @param value The new <CODE>Style</CODE> element.
   */  
  public void setStyle(StyleType value) {
    this.replaceChild((OSMXElement) getStyle(), (OSMXElement) value);
    super.setStyle(value);
  }
  
  /**
   * Sets a <CODE>PositionedText</CODE> element as the identifier for this transition
   * element.
   * @param value The <CODE>PositionedText</CODE> element that should serve as the identifier.
   */  
  public void setIdentifier(PositionedText value) {
    this.replaceChild((OSMXElement) getIdentifier(), (OSMXElement) value);
    super.setIdentifier(value);
  }
  
  /**
   * Assigns a sub-model to the transition.  This does not depend on or affect the
   * state of the transition's <CODE>highLevel</CODE> attribute.
   * @param value The <CODE>OSM</CODE> element that is the root of the sub-model to add to the
   * transition.
   */  
  public void setOSM(OSMType value) {
    this.replaceChild((OSMXElement) getOSM(), (OSMXElement) value);
    super.setOSM(value);
  }
  
  public void addPositionListener(PositionListener pl) {
		 // System.out.println("AddedListState"+pl);
	      positionListeners.add(pl);
	  }
	  
	  public boolean isSetPosition() {
	      return (isSetX() && isSetY());
	  }
	  
	  public void removePositionListener(PositionListener pl) {
	      positionListeners.remove(pl);
	  }
	  
	/*  public String getPrimaryName() {
		    String result = this.getId();
		    String name = getName();
		    if (name == null || name.length() == 0) {
		      setName(result);
		      return result;
		    }
		    return name;
		  }
		   public String toString() {
		    return getPrimaryName();
		  } // method toString
  
  /**
   * Changes the x-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new x-coordinate position of this element.
   */  
  public void setX(int value) {
    super.setX(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateX(this, value);
    }
  } // method setX
  
  /**
   * Changes the y-coordinate position and causes the position listeners to be
   * notified.
   * @param value The new y-coordinate position of this element.
   */
  public void setY(int value) {
    super.setY(value);
    for (Iterator i = positionListeners.iterator(); i.hasNext();) {
      PositionListener pl = (PositionListener) i.next();
      pl.updateY(this, value);
    }
  } // method setY
  
}
