/*
 * OSMXValuePhraseType.java
 *
 * Created on June 11, 2003, 11:43 AM
 */

package edu.byu.deg.osmx;

import edu.byu.deg.osmx.binding.DataFrameExpression;
import edu.byu.deg.osmx.binding.ObjectFactory;

/**
 * Represents a <CODE>ValuePhrase</CODE> element in OSMX.
 * @author Alan Wessman
 */
public class OSMXValuePhraseType extends edu.byu.deg.osmx.binding.impl.ValuePhraseTypeImpl
implements KeywordListOwner {
  
  public static final String HINT_PROPERTY = "hint";
  public static final String CONFIDENCE_TAG_PROPERTY = "confidenceTag";
  public static final String CASE_SENSITIVE_PROPERTY = "caseSensitive";
  public static final String CANONICALIZATION_PROPERTY = "canonicalizationMethod";
  
  /** Creates a new instance of OSMXValuePhraseType */
  public OSMXValuePhraseType() {
    if (getValueExpression() == null) {
      try {
        setValueExpression(createDataFrameExpression());
      } catch (javax.xml.bind.JAXBException e) {
        e.printStackTrace();
      }
    }
  }
  
  private DataFrameExpression createDataFrameExpression() throws javax.xml.bind.JAXBException {
    OSMXDocument doc = getParentDocument();
    ObjectFactory factory;
    if (doc != null) {
      factory = doc.getObjectFactory();
    } else {
      factory = new ObjectFactory();
    }
    return factory.createDataFrameExpression();
  }
  
  /**
   * Stores a new value for the hint (label) for this phrase.
   * @param value The new value to store.
   */  
  public void setHint(String value) {
    String oldVal = getHint();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setHint(value);
      firePropertyChange(HINT_PROPERTY, oldVal, value);
    }
  } // method setHint
  
  /**
   * Stores a new value for the confidence measure assigned to matches that belong to this
   * phrase.
   * @param value The new value to store.
   */  
  public void setConfidenceTag(String value) {
    String oldVal = getConfidenceTag();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setConfidenceTag(value);
      firePropertyChange(CONFIDENCE_TAG_PROPERTY, oldVal, value);
    }
  } // method setConfidenceTag
  
  /**
   * Stores a new value for the <CODE>caseSensitive</CODE> flag for the phrase.
   * @param value The new value to store; <CODE>true</CODE> means that the matching expressions
   * will be case-sensitive, and <CODE>false</CODE> means that case-insensitive
   * matching will be performed.
   */  
  public void setCaseSensitive(boolean value) {
    boolean oldVal = isCaseSensitive();
    if (oldVal != value) {
      super.setCaseSensitive(value);
      firePropertyChange(CASE_SENSITIVE_PROPERTY, new Boolean(oldVal), new Boolean(value));
    }
  } // method setCaseSensitive
  
  /**
   * Stores the name of the method used to represent the extracted value in a
   * canonical form.  The name is the only part of the method that needs to be
   * specified, as the formal parameters are necessarily just a single <CODE>String</CODE>
   * argument (because the extracted value is always a string), and the return type
   * is implied by the <CODE>TypeSpecification</CODE> given for this data type.
   * @param value The name of the canonicalization method.
   */  
  public void setCanonicalizationMethod(String value) {
    String oldVal = getCanonicalizationMethod();
    String newVal = new String(value); // in case value == null
    if (!newVal.equals(oldVal)) {
      super.setCanonicalizationMethod(value);
      firePropertyChange(CANONICALIZATION_PROPERTY, oldVal, value);
    }
  } // method setCanonicalizationMethod
  
  /**
   * Overrides the inherited method in order to pass the call recursively down to its
   * child elements.
   * @param elem The new parent of this element.
   */  
  void setParent(OSMXElement elem) {
    parent = elem;
    addToParentDocument();
    this.setAsParentOf((OSMXElement) getValueExpression());
    this.setAsParentOf((OSMXElement) getExceptionExpression());
    this.setAsParentOf((OSMXElement) getLeftContextExpression());
    this.setAsParentOf((OSMXElement) getRightContextExpression());
    this.setAsParentOf((OSMXElement) getRequiredContextExpression());
    this.setAsParentOf((OSMXElement) getSubFromExpression());
    this.setAsParentOf((OSMXElement) getSubToExpression());
    this.setAsParentOf(this.getInnerList(getKeywordPhrase()));
  }
  
  /**
   * Stores the expression used to extract value matches from text.
   * @param value The <CODE>DataFrameExpression</CODE> that matches extractable values.
   */  
  public void setValueExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getValueExpression(), (OSMXElement) value);
    super.setValueExpression(value);
  }
  
  /**
   * Stores the expression that disqualifies extracted values when it matches within
   * those values.
   * @param value The <CODE>DataFrameExpression</CODE> to set as the exception expression.
   */  
  public void setExceptionExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getExceptionExpression(), (OSMXElement) value);
    super.setExceptionExpression(value);
  }
  
  /**
   * Stores the expression that matches the required left-hand adjoining context for
   * the matched values.
   * @param value The <CODE>DataFrameExpression</CODE> to store as the left context expression.
   */  
  public void setLeftContextExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getLeftContextExpression(), (OSMXElement) value);
    super.setLeftContextExpression(value);
  }
  
  /**
   * Stores the expression that matches the required right-hand adjoining context for
   * the matched values.
   * @param value The <CODE>DataFrameExpression</CODE> to store as the right context expression.
   */  
  public void setRightContextExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getRightContextExpression(), (OSMXElement) value);
    super.setRightContextExpression(value);
  }
  
  /**
   * Stores the expression that matches the required surrounding context for
   * the matched values.
   * @param value The <CODE>DataFrameExpression</CODE> to store as the required context expression.
   */  
  public void setRequiredContextExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getRequiredContextExpression(), (OSMXElement) value);
    super.setRequiredContextExpression(value);
  }
  
  /**
   * Stores the expression that locates substrings of extracted values that should be
   * replaced by other substrings specified by the <CODE>subTo</CODE> expression.
   * @param value The <CODE>DataFrameExpression</CODE> that represents the subsitution source
   * expression.
   */  
  public void setSubFromExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getSubFromExpression(), (OSMXElement) value);
    super.setSubFromExpression(value);
  }
  
  /**
   * Stores the expression that will replace the substrings of the matched values
   * that were identified by the <CODE>subFrom</CODE> expression.
   * @param value The <CODE>DataFrameExpression</CODE> used to replace substrings of extracted
   * values.
   */  
  public void setSubToExpression(DataFrameExpression value) {
    this.replaceChild((OSMXElement) getSubToExpression(), (OSMXElement) value);
    super.setSubToExpression(value);
  }
  
}
