/*
 * PositionListener.java
 *
 * Created on June 23, 2003, 6:49 AM
 */

package edu.byu.deg.osmx;

/**
 * Interface for objects that need to be notified when a positioned element's
 * position changes.
 * @author Alan
 */
public interface PositionListener {
    /**
     * Invoked when an element's horizontal position changes.
     * @param src The element whose position changed.
     * @param newX The new <CODE>x</CODE>-coordinate position of the element.
     */    
  public void updateX(OSMXElement src, int newX);
  /**
   * Invoked when an element's vertical position changes.
   * @param src The element whose position changed.
   * @param newY The new <CODE>y</CODE>-coordinate of the element.
   */  
  public void updateY(OSMXElement src, int newY);
  /**
   * Invoked when the element's <CODE>z</CODE>-order changes.
   * @param src The element whose depth position has changed.
   * @param newOrder The new <CODE>z</CODE>-coordinate of the element.
   */  
  public void updateOrder(OSMXElement src, int newOrder);  
}
