/*
 * PositionedElement.java
 *
 * Created on June 19, 2003, 9:58 PM
 */

package edu.byu.deg.osmx;

/**
 * Represents an element that occupies a location on a graphical surface.
 * @author Alan
 */
public interface PositionedElement {
    /**
     * Returns the <CODE>x</CODE>-coordinate of the element.
     * @return The <CODE>x</CODE>-coordinate of the element.
     */
    public int getX();
    /**
     * Returns the <CODE>y</CODE>-coordinate of the element.
     * @return The <CODE>y</CODE>-coordinate of the element.
     */
    public int getY();
    /**
     * Returns the <CODE>z</CODE>-coordinate of the element.
     * @return The <CODE>z</CODE>-coordinate of the element.
     */
    public int getOrder();
    
    /**
     * Changes the x-coordinate position and causes any position listeners to be
     * notified.
     * @param value The new x-coordinate position of this element.
     */
    public void setX(int value);
    /**
     * Changes the y-coordinate position and causes any position listeners to be
     * notified.
     * @param value The new y-coordinate position of this element.
     */
    public void setY(int value);
    /**
     * Changes the value of the <CODE>order</CODE> attribute and notifies the position
     * change listeners.
     * @param value The new value for the z-order of this element.
     */
    public void setOrder(int value);
    /**
     * Registers an object to listen for position changes by this element.
     * @param pl The object to register for position change event notification.
     */
    public void addPositionListener(PositionListener pl);
    /**
     * Un-registers an object currently listening for position change events.
     * @param pl The object to remove from the notification list for position changes.
     */
    public void removePositionListener(PositionListener pl);
    /**
     * Indicates whether this element has explicitly been given an x and y coordinate
     * position.
     * @return <CODE>true</CODE> if both the <CODE>x</CODE> and <CODE>y</CODE> coordinate
     * attributes have been set; <CODE>false</CODE> otherwise.
     */
    public boolean isSetPosition();
    /**
     * Clears the current setting for the <CODE>x</CODE> attribute.
     */
    public void unsetX();
    /**
     * Clears the current setting for the <CODE>y</CODE> attribute.
     */
    public void unsetY();
}
