/*
 * DataPrinter.java
 *
 * Created on April 27, 2004, 2:53 PM
 */

package edu.byu.deg.query;
import java.io.*;
import java.util.*;
import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;

/**
 *
 * @author  troywalk
 */
public class DataPrinter {
  HashMap trees = new HashMap();
  OSMXDocument doc;
  OSMXObjectSetType primary;
  
  /** Creates a new instance of DataPrinter */
  public DataPrinter() {
  }
  
  public void printToFile(String filename, OSMXDocument ont) throws FileNotFoundException{
    PrintWriter out = new PrintWriter(new FileOutputStream(new File(filename)));
    printData(ont, out, null);
    out.close();
  }
  
  public void printData(OSMXDocument ont, PrintWriter out, Thread thread){
    doc = ont;
    out.println("<html>");
    out.println("  <head>");
    out.println("    <title>Query Results</title>");
    out.println("    <script language=javascript>");
    out.println("      function toggle(object){");
    out.println("        vis = object.style.display");
    out.println("        object.style.display = (vis=='none')?'':'none';");
    out.println("      }");
    out.println("    </script>");
    out.println("    <link rel=\"Stylesheet\" type=\"text/css\" href=\"http://www.deg.byu.edu/GeneTIQS/style.css\" />");
    out.println("  </head>");
    out.println("  <body>");
    out.print("    <h1>Query Results");
    out.print("<form action=\"edu.byu.deg.query.StopProcess\" method=post><input type=submit class=button value=\"Stop\">");
    out.println("</h1>");
    out.println("    <hr />");
    out.println("    <br />");

    primary = doc.getPrimaryObjectSet();
    ArrayList header = makeObjectSetTree();
    printObjects(out, header, thread);

    out.println("  </body>");
    out.println("</html>");
    out.close();
  }
  

  private void makeTableHeader(ArrayList attributes, PrintWriter out, boolean includeSource){
    String id;
    String name;
    out.println("      <tr>");
    Iterator i = attributes.iterator();
    i.next();
    while(i.hasNext()){
      Object o = i.next();
      id=(String)o;
      out.print("        <th>");
      out.print(((OSMXObjectSetType)doc.getElement(id)).getName());
      out.println("</th>");
    }
    if(includeSource)
      out.println("        <th>Source</th>");
    out.println("      </tr>");
  }
  
  private void addAttribute(OSMXObjectSetType obj, ArrayList result, OSMXObjectSetType parent){
    OSMXObjectSetType next;
    ArrayList temp = null;
    Object o;
    if(obj.isLexical())
      result.add(obj.getId());
    ArrayList relsets = getRelationshipSetsWith(obj, parent);
    Iterator i = relsets.iterator();
    while(i.hasNext()){
      OSMXRelationshipSetType relset = (OSMXRelationshipSetType)i.next();
      OSMXRelSetConnectionType c = relset.getConnectionFor(obj);
      Iterator j = relset.getRelSetConnection().iterator();
      temp = null;
      while(j.hasNext()) {
        o = j.next();
        if(c!=o){
          OSMXRelSetConnectionType con = (OSMXRelSetConnectionType)o;
          next = (OSMXObjectSetType)doc.getElement(con.getObjectSet());
          if(c.isOneMax())
            addAttribute(next, result, obj);
          else{
            if(temp==null){
              temp = new ArrayList();
              temp.add(next.getId());
              result.add(next.getId());
              trees.put(next.getId(), temp);
            }
            addAttribute(next, temp, obj);
          }
        }
      }
    }
  }

  private ArrayList makeObjectSetTree(){
    ArrayList result = new ArrayList();
    OSMXObjectSetType primary = doc.getPrimaryObjectSet();
    result.add(primary.getId());
    addAttribute(primary, result, null);
    trees.put(primary.getId(), result);
    return result;
  }
  
  private ArrayList getRelationshipSetsWith(OSMXObjectSetType obj, OSMXObjectSetType parent){
    ArrayList result = new ArrayList();
    List l = doc.getModelRoot().getAllModelElements();
    for (Iterator i = l.iterator(); i.hasNext();) {
      Object o = i.next();
      if(o instanceof OSMXDataInstanceType)
        break;
      if(o instanceof OSMXRelationshipSetType){
        OSMXRelationshipSetType relset = (OSMXRelationshipSetType)o;
        OSMXRelSetConnectionType c = relset.getConnectionFor(obj);
        if((c!=null) && (relset.getConnectionFor(parent)==null)){
          result.add(relset);
        }
      }
    }
    return result;
  }
  
  private void printObjects(PrintWriter out, ArrayList header, Thread thread){
    List l = doc.getModelRoot().getAllModelElements();
    int i=0;
    int count = 0;
    while(i<l.size()||(thread!=null&&thread.isAlive())){
      while(i<l.size()){
        Object o = l.get(i);
        if(o instanceof DataInstanceType){
          count++;
          DataInstanceType di = (DataInstanceType)o;
          double confidence = 0.0;
          try{
            confidence = Double.parseDouble(di.getConfidenceTag());
          } catch(NumberFormatException e){}
          if(confidence<.2){
            out.println("<br>Cosine measure shows this document to be unrelated to the ontology:<br>");
            out.println(di.getId() + "<br>");
            continue;
          } else if(confidence<.6){
            out.print("<br>Cosine measure shows this document unlikely to be related to the ontology:");
            out.println(di.getId() + "<br>");
            out.print("<input type=button value=Show onclick=\"toggle(document.getElementById('source");
            out.print(count);
            out.print("'))\"><br>");
            out.println("    <table style=\"{display:none}\">");
          } else {
            out.println("    <table>");
          }
          makeTableHeader(header, out, true);
          Iterator j = di.getDataInstanceElements().iterator();
          while(j.hasNext()){
            o = j.next();
            if(!(o instanceof SourceDocument))
              continue;
            SourceDocument src = (SourceDocument)o;
            Iterator k = src.getDataInstanceElements().iterator();
            while(k.hasNext()){
              o = k.next();
              if(o instanceof OSMObject){
                OSMObject obj = (OSMObject)o;
                if(obj.getObjectSet().equals(primary.getId())){
                  printObject(obj, src, out, null);
                }
              }
            }
          }
          out.println("    </table>");
        }
        i++;
      }
      out.flush();
      try{
        Thread.sleep(500);
      }catch(InterruptedException e){}
    }
  }
  
  private void printObject(OSMObject obj, SourceDocument source, PrintWriter out, String parentID){
    ArrayList order = (ArrayList)trees.get(obj.getObjectSet());
    OSMObject[] objs = new OSMObject[order.size()];
    ArrayList queue = new ArrayList();
    for(int i=0;i<order.size();i++)
      objs[i] = null;
    ArrayList multiples = new ArrayList();
    fillObjectArray(objs, order, obj, parentID, source, multiples);
    out.print("<tr>");
    for(int i=1;i<objs.length;i++){
      out.print("<td>"); 
      OSMObject object = objs[i];
      if(object==null)
        out.print("&nbsp;");
      else if(trees.get(object.getObjectSet())==null||object==obj){
        out.print(object.getValue());
      }
      else {
        out.print("<input type=button value=Show onclick=\"toggle(document.getElementById('");
        out.print(obj.getId());
        out.print(object.getObjectSet());
        out.print("'))\">");
        queue.add(object.getObjectSet()); 
      }
      out.print("</td>"); 
    }
    if(parentID==null){
      String src = source.getUri();
      String src2 = src.substring(src.lastIndexOf('/')+1);
      if(src2.length()>13){
        src2 = src2.substring(0, 13);
      }
      out.print("<td>"); 
      out.print("<a href=\"");
      out.print(src);
      out.print("\" target=\"source\">");
      out.print(src2);
      out.print("</a>");
      out.print("</td>"); 
    }
    out.print("</tr>");
    
    for(int i=0;i<queue.size();i++){
      String osID = (String)queue.get(i);
      out.print("<tr id=\"");
      out.print(obj.getId());
      out.print(osID);
      out.print("\" style=\"{display:none}\">");
      out.print("<td>&nbsp</td><td colspan=99>");
      out.println("    <table>");
      makeTableHeader((ArrayList)trees.get(osID), out, false);
      for(Iterator j=multiples.iterator();j.hasNext();){
        OSMObject object = (OSMObject)j.next();
        if(object.getObjectSet().equals(osID))
          printObject(object, source, out, obj.getId());
      }
      out.print("</table>");
      out.print("</td>");
      out.print("</tr>");
    }
  }
  private void fillObjectArray(OSMObject[] objs, ArrayList order, OSMObject obj, String parentID, SourceDocument source, ArrayList multiples){
    if(obj.getObjectSet().equals(order.get(1)))
      objs[1] = obj;
    ArrayList objects = getRelatedObjects(obj.getId(), parentID, source);
    for(int i=0;i<objects.size();i++){
      OSMObject object = (OSMObject)doc.getElement((String)objects.get(i));
      int index = order.indexOf(object.getObjectSet());
      if(index<0&&trees.get(object.getObjectSet())==null){
        fillObjectArray(objs, order, object, obj.getId(), source, multiples); 
      }
      else
        if(!multiples.contains(object))
          multiples.add(object);
      if(index>0){
        objs[index] = object;
      }
    }
  }
  
  private ArrayList getRelatedObjects(String id, String parentID, SourceDocument source){
    ArrayList result = new ArrayList();
    Iterator i = source.getDataInstanceElements().iterator();
    while(i.hasNext()){
      Object o = i.next();
      if(o instanceof Relationship){
        Relationship rel = (Relationship)o;
        Iterator j = rel.getObjectBinding().iterator();
        boolean hasID = false;
        //boolean hasParent = false;
        while(j.hasNext()){
          ObjectBindingType ob = (ObjectBindingType)j.next();
          String objid = ob.getObjectRef();
          if(objid.equals(id))
            hasID = true;
          //if(objid.equals(parentID))
            //hasParent = true;
        }
        if(hasID){//&&!hasParent){
          j = rel.getObjectBinding().iterator();
          while(j.hasNext()){
            ObjectBindingType ob = (ObjectBindingType)j.next();
            if(!ob.getObjectRef().equals(parentID)&&!ob.getObjectRef().equals(id)){
              result.add(ob.getObjectRef());
            }
          }
        }
      }
    }
    return result;
  }
}
