/*
 * OSMQuery.java
 *
 * Created on March 2, 2004, 1:10 PM
 */

package edu.byu.deg.query;


import java.util.*;

import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;
/**
 *
 * @author  troywalk
 */
public class OSMQuery {
  private OSMXObjectSetType objectSet;
  private String value;
  private ArrayList subqueries;
  private OSMXDocument scheme;
  /** Creates a new instance of OSMQuery */
  public OSMQuery(OSMXDocument scheme, OSMXObjectSetType os) {
    objectSet = os;
    this.scheme = scheme;
    value = null;
    subqueries = new ArrayList();
  }
  
  public String getValue(){
    return value;
  }
  public void setValue(String val){
    value = val;
  }
  public void addSubquery(OSMQuery sub){
    subqueries.add(sub);
  }
  
  public void removeSubquery(OSMQuery sub){
    subqueries.remove(sub); 
  }
  
  public boolean isEmpty(){
    return (value==null && subqueries.size()==0);
  }
  
  public boolean canHaveSubquery(OSMQuery q){
    Iterator i = scheme.getModelRoot().getAllModelElements().iterator();
    while(i.hasNext()) {
      Object o = i.next();
      if(o instanceof DataInstance)
        break;
      if(o instanceof OSMXRelationshipSetType){
        OSMXRelationshipSetType relset = (OSMXRelationshipSetType)o;
        OSMXRelSetConnectionType c = relset.getConnectionFor(objectSet);
        if(c!=null){
          boolean nomore = false;
          for(Iterator j = relset.getRelSetConnection().iterator(); j.hasNext(); ) {
            o = j.next();
            if((c!=o) && (o instanceof OSMXRelSetConnectionType)){
              OSMXRelSetConnectionType con = (OSMXRelSetConnectionType)o;
              if(con.getObjectSet().equals(q.objectSet.getId()))
                return true;
            }
          }
        }
      }
    }
    return false;
 }
  
  public void writeTo(StringBuffer b, int indent){
    for(int i=0;i<indent;i++)
      b.append(' ');
    b.append(objectSet.getName());
    if(value!=null){
      b.append(" = ");
      b.append(value);
    }
    b.append("\r\n");
    for(int i=0;i<subqueries.size();i++)
      ((OSMQuery)subqueries.get(i)).writeTo(b, indent+2);
  }
  
  public String toString(){
    StringBuffer b = new StringBuffer();
    writeTo(b, 0);
    return b.toString();
  }
  
  public boolean satisfiesQuery(SourceDocument instance){
    return satisfiesQuery(instance, null);
  }
  
  public boolean satisfiesQuery(SourceDocument instance, String parentID){
    Iterator i = instance.getDataInstanceElements().iterator();
    while(i.hasNext()){
      Object o = i.next();
      if(o instanceof OSMObject){
        OSMObject obj = (OSMObject)o;
        if(!objectSet.getId().equals(obj.getObjectSet()))
          continue;
        if(value!=null&&!obj.getValue().equals(value)){
           continue;
        }
        if(parentID!=null){
          ArrayList relatedObjects = getRelatedObjects(obj.getId(), "", instance);
          if(!relatedObjects.contains(parentID))
            continue;
        }
        boolean works = true;
        for(int j=0;j<subqueries.size();j++){
          OSMQuery sub = (OSMQuery)subqueries.get(j);
          if(!sub.satisfiesQuery(instance, obj.getId())){
            works = false;
            break;
          }
        }
        if(works==true){
          return true;
        }
      }
    }
    return false;
  }
  
  private ArrayList getRelatedObjects(String id, String parentID, SourceDocument source){
    ArrayList result = new ArrayList();
    Iterator i = source.getDataInstanceElements().iterator();
    while(i.hasNext()){
      Object o = i.next();
      if(o instanceof Relationship){
        Relationship rel = (Relationship)o;
        Iterator j = rel.getObjectBinding().iterator();
        boolean hasID = false;
        boolean hasParent = false;
        while(j.hasNext()){
          ObjectBindingType ob = (ObjectBindingType)j.next();
          String objid = ob.getObjectRef();
          if(objid.equals(id))
            hasID = true;
          if(objid.equals(parentID))
            hasParent = true;
        }
        if(hasID&&!hasParent){
          j = rel.getObjectBinding().iterator();
          while(j.hasNext()){
            ObjectBindingType ob = (ObjectBindingType)j.next();
            result.add(ob.getObjectRef());
          }
        }
      }
    }
    return result;
  }
}
