/*
 * FormProcessor.java
 *
 * Created on February 3, 2004, 1:21 PM
 */

package edu.byu.deg.query;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import edu.byu.deg.osmx.*;
import edu.byu.deg.osmx.binding.*;
import edu.byu.deg.extractionmodule.MetaExtractor;
/**
 *
 * @author  troywalk
 * @version
 */
public class QueryProcessor extends HttpServlet {
  private static final String DEFAULT_ENCODING = "Cp1252";
  public static final String WAIT = "Wait for it!";
  
  /** Initializes the servlet.
   */
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    
  }
  
  /** Destroys the servlet.
   */
  public void destroy() {
    
  }
  
  /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
   * @param request servlet request
   * @param response servlet response
   */
  protected void processRequest(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException {
    response.sendRedirect("../query.html");
  }
  
  /** Handles the HTTP <code>GET</code> method.
   * @param request servlet request
   * @param response servlet response
   */
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException {
    processRequest(request, response);
  }
  
  /** Handles the HTTP <code>POST</code> method.
   * @param request servlet request
   * @param response servlet response
   */
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException {
    OSMQuery query = null;
    boolean hassession = false;
    if(request.getSession(false)!=null){
      hassession = true;
    }
    HttpSession session = request.getSession();
    OSMXDocument scheme = null;
    try {
      
      BufferedReader reader = request.getReader();
      String body = reader.readLine();
      String[] pairs = body.split("&");
      
      String encoding = request.getCharacterEncoding();
      if(encoding==null)
        encoding = DEFAULT_ENCODING;
      String[] arr = pairs[0].split("=");
      String filename = decode(arr[1], encoding);
      if(hassession&&session.getAttribute("ontology")!=null){
        scheme = (OSMXDocument)session.getAttribute("ontology");
        String ontfilename = getFileName(scheme.getURI().toString());
        if(!ontfilename.equals(getFileName(filename))){
          System.out.println(scheme.getURI().toString());
          System.out.println(filename);
          session.invalidate();
          session = request.getSession();
          scheme = new OSMXDocument();
          scheme.openDocument(filename);
          session.setAttribute("ontology", scheme);
          hassession = false;
        }
      }
      else {
        scheme = new OSMXDocument();
        scheme.openDocument(filename);
        session.setAttribute("ontology", scheme);
        hassession = false;
      }

      OSMXObjectSetType primary = scheme.getPrimaryObjectSet();
      query = new OSMQuery(scheme, primary);
      arr = pairs[1].split("=");
        if(arr.length==2)
          query.setValue(decode(arr[1], encoding));
      
      Stack stack = new Stack();
      stack.push(query);
      OSMQuery top = query;
      
      
      for(int i=2;i<pairs.length;i++){
        arr = pairs[i].split("=");
        OSMQuery current = new OSMQuery(scheme, (OSMXObjectSetType)scheme.getElement(decode(arr[0], encoding)));
        if(arr.length==2)
          current.setValue(decode(arr[1], encoding));
        top = (OSMQuery)stack.peek();
        while(!top.canHaveSubquery(current)){
          OSMQuery last = (OSMQuery)stack.pop();
          top = (OSMQuery)stack.peek();
          if(last.isEmpty()){
            top.removeSubquery(last);
          }
        }
        top.addSubquery(current);
        stack.push(current);
      }
      while(top!=query){
        OSMQuery last = (OSMQuery)stack.pop();
        top = (OSMQuery)stack.peek();
        if(last.isEmpty())
          top.removeSubquery(last);
      }
      
      session.setAttribute("query", query);
    } catch(Exception e){ e.printStackTrace(); }
    response.sendRedirect(response.encodeRedirectURL("edu.byu.deg.query.SourceSelector"));
    if(!hassession){
      Thread t = new InitThread("init", session, scheme);
      t.run();
    }
  }
  
  /** Returns a short description of the servlet.
   */
  public String getServletInfo() {
    return "Processes the submission of a form generated by FormGenerator.";
  }
  
  private String decode(String str, String encoding) throws java.io.UnsupportedEncodingException{
    return URLDecoder.decode(str, encoding);
  }
  
  private String getFileName(String str){
    int loc = str.lastIndexOf('\\');
    int loc2 = str.lastIndexOf('/');
    if(loc2>loc)
      loc = loc2;
    if(loc>=0&&loc<str.length())
      return str.substring(loc+1);
    return str;
  }
  
  public class InitThread extends Thread {
    HttpSession session;
    OSMXDocument ontology;
    public InitThread(String str, HttpSession ses, OSMXDocument ont) {
      super(str);
      session = ses;
      ontology = ont;
    }
    
    
    public void run() {
      session.setAttribute("extractor", WAIT);
      MetaExtractor extractor = new MetaExtractor(ontology);
      session.setAttribute("extractor", extractor);
      System.out.println("done loading lexicons");System.out.flush();
    }
  }
  
}
