package edu.byu.deg.records;

import java.util.*;

public class ApplyCH {
   /* certainty factors for each individual heuristic */
   final static double[] om_cf = {0.845, 0.125, 0.02, 0.01, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00};
   final static double[] rp_cf = {0.775, 0.125, 0.09, 0.01, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00};
   final static double[] sd_cf = {0.655, 0.225, 0.12, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00};
   final static double[] it_cf = {0.96, 0.04, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00};
   final static double[] ht_cf = {0.49, 0.325, 0.165, 0.02, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00,
									0.00, 0.00, 0.00, 0.00, 0.00};

   Vector tag_list = new Vector();
   Vector oml = new Vector();
   Vector rpl = new Vector();
   Vector sdl = new Vector();
   Vector itl = new Vector();
   Vector htl = new Vector();

   /**
	*  Constructor
	*/
   ApplyCH(Vector tl, Vector om, Vector rp, Vector sd, Vector it, Vector ht) {
	  tag_list = tl;
	  oml = om;
	  rpl = rp;
	  sdl = sd;
	  itl = it;
	  htl = ht;
   }

   public Vector find_candidate() {

	  Vector candidate = new Vector();

	  for(int i=0; i<tag_list.size(); i++) {
		 String tag = new String(((GTI)tag_list.elementAt(i)).str);
		 double ocf = 0.0;
		 double rcf = 0.0;
		 double scf = 0.0;
		 double icf = 0.0;
		 double hcf = 0.0;

		 int rank = 0;

//         System.out.println(tag + "  ->");

		 if ( (oml.size() > 0) && ((rank=find_tag(oml, tag)) > 0)) {
			if(rank <= 15) {
			   ocf = om_cf[rank-1];
			}
//            System.out.print("\tOM Rank=" + rank + " CF=" + ocf);
		 }

		 if ( (rpl.size() > 0) && ((rank=find_tag(rpl, tag)) > 0)) {
			if(rank <= 15) {
			   rcf = rp_cf[rank-1];
			}
//            System.out.print("\tRP Rank=" + rank + " CF=" + rcf);
		 }

		 if ( (sdl.size() > 0) && ((rank=find_tag(sdl, tag)) > 0)) {
			if(rank <= 15) {
			   scf = sd_cf[rank-1];
			}
//            System.out.println("\tSD Rank=" + rank + " CF=" + scf);
		 }

		 if ( (itl.size() > 0) && ((rank=find_tag(itl, tag)) > 0)) {
			if(rank <= 15) {
			   icf = it_cf[rank-1];
			}
//            System.out.print("\tIT Rank=" + rank + " CF=" + icf);
		 }

		 if ( (htl.size() > 0) && ((rank=find_tag(htl, tag)) > 0)) {
			if(rank <= 15) {
			   hcf = rp_cf[rank-1];
			}
//            System.out.print("\tHT Rank=" + rank + " CF=" + hcf);
		 }

		 /* Apply Stanford certainty theory */
		 double rate = calculateCommpoundCertainty(ocf, rcf,scf,icf, hcf);
//         System.out.println("\tCCF=" + rate);

		 candidate.addElement(new GTD(tag, rate));
	  }


	  // Sorting
	  for(int i=0; i<candidate.size()-1; i++) {
		 for(int j=i+1; j<candidate.size(); j++) {
			double rate1 = ((GTD)candidate.elementAt(i)).num;
			double rate2 = ((GTD)candidate.elementAt(j)).num;
			if(rate1 < rate2) {
			   GTD tmp = new GTD((GTD)candidate.elementAt(j));
			   ((GTD)candidate.elementAt(j)).str = ((GTD)candidate.elementAt(i)).str;
			   ((GTD)candidate.elementAt(j)).num = ((GTD)candidate.elementAt(i)).num;
			   ((GTD)candidate.elementAt(i)).str = tmp.str;
			   ((GTD)candidate.elementAt(i)).num = tmp.num;
			}
		 }
	  }
/*
	  for(int i=0; i<candidate.size(); i++) {
		 System.out.println(((GTD)candidate.elementAt(i)).str + "\t" +
							((GTD)candidate.elementAt(i)).num);
	  }
*/
	  return candidate; //
   }

   private static int find_tag(Vector tag_list, String tag) {
	  for(int i=0; i<tag_list.size(); i++) {
		 String str = ((GTI)tag_list.elementAt(i)).str;
		 if(str.equals(tag)) {
			return ((GTI)tag_list.elementAt(i)).num;
		 }
	  }
	  return -1;
   }

   private static double calculateCommpoundCertainty(double a, double b, double c, double d, double e) {
	  double result;

	  result = a + b + c + d + e - a * b - a * c - a * d - a * e - b * c - b * d
			   - b * e - c * d - c * e - d * e + a * b * c + a * b * d + a * b * e
			   + a * c * d + a * c * e + a * d * e + b * c * d + b * c * e
			   + b * d * e + c * d * e - a * b * c * d - a * b * c * e
			   - a * b * d * e - a * c * d * e - b * c * d * e + a * b * c * d * e;
	  int tmp = (int)((result + 0.00005)*10000);
	  result = (double)tmp / 10000;

	  return result;
   }


}
