package edu.byu.deg.records;

import java.util.*;
import java.io.*;

/**
  * This class take a tag-tree as input
  * find the separators
  *
  * @version 1.0.0
  * @author  Yuan Jiang
  */
class ApplyOM {
   TTree  tree = new TTree();
   Vector tag_list  = new Vector();
   Vector type_list = new Vector();
   Vector data_record_table = new Vector();
   Vector entry = new Vector();
   Vector cnt = new Vector();
   Vector record_type = new Vector();
   String[][] data_entry = new String[10][1000];

   /**
	* Constructor
	*/
   ApplyOM() {
	  tree = new TTree();
	  tag_list  = new Vector();
	  type_list = new Vector();
	  data_record_table = new Vector();
   }

   /**
	* Constructor
	* @param sub
	*/
   ApplyOM(TTree sub) {
	  this();

	  tree = sub;
	  PreProcessTags pt = new PreProcessTags(sub);
	  pt.eliminateIrrelevantTags(2);
	  tag_list  = pt.tag_list;
	  type_list = pt.type_list;   // of type GTI
   }

   /**
	* Constructor
	* @param sub
	* @param tbl
	*/
   ApplyOM(TTree sub, File tbl) {
	  this(sub);

	  // Read data-record table and match to actual file
	  try {
		 InputStream in = new FileInputStream(tbl);
		 int i = in.read();
		 String row = new String();
		 while(i != -1) {
			if(i == 10 || i == 13) { // "\r" or "\n"
			   processTable(row);
			   row = new String();
			}
			else {
			   row += (char)i;
			}
			i = in.read();
		 }
		 processEntry();
	  } catch (IOException io) {
		   System.out.println(io.getMessage());
		}

   }

   /**
	*   Tokenize the data-record table
	*/
   private void processTable(String row) {
	  TABLE tb = new TABLE();

	  StringTokenizer st = new StringTokenizer(row, "|");
	  int ccnt = 0;

	  while (st.hasMoreTokens()) {
		 String token = st.nextToken();
		 if(token.equals("")) {
			break;
		 }
		 else {
			switch(ccnt) {
			   case 0:  tb.key = token;
						break;
			   case 1:  tb.content = token;
						break;
			}
			ccnt++;
		 }
		 if(ccnt == 4) {
			//System.out.println(tb.key + " " + tb.content);
			//if(tb.key.startsWith("KEYWORD(")) {
			   entry.addElement(new TABLE(tb));
			//}
		 }
	  }

   }


   /**
	*
	*/
   private void processEntry() {

	  for(int i=0; i<entry.size(); i++) {
		 String descriptor = new String(((TABLE)entry.elementAt(i)).key);
		 int pos = record_type.indexOf(descriptor);
		 if( pos == -1) {
			record_type.addElement(descriptor);
			data_entry[record_type.size()-1][0] =
							 new String(((TABLE)entry.elementAt(i)).content);
			cnt.addElement(new Integer(1));
		 }
		 else {

			int cntpos = ((Integer)cnt.elementAt(pos)).intValue();
			data_entry[pos][cntpos] =
							 new String(((TABLE)entry.elementAt(i)).content);
			cntpos++;
			cnt.removeElementAt(pos);
			cnt.insertElementAt(new Integer(cntpos), pos);
		 }
	  }

	  //testing
/*
	  for(int i=0; i<record_type.size(); i++) {
		 System.out.print((String)record_type.elementAt(i) + " " +
						  ((Integer)cnt.elementAt(i)).intValue() + "-> ");
		 for(int j=0; j< ((Integer)cnt.elementAt(i)).intValue(); j++) {
			System.out.print(data_entry[i][j] + " ");
		 }
		 System.out.println("");

	  }
*/
   }

   public Vector find_candidate() {
	  int rank = 1;
	  Vector candidate_list = new Vector();
	  Vector tag_list = new Vector();
	  int sum = 0;
	  int min = 32767;
	  for(int i=0; i<record_type.size(); i++) {
		 int m = ((Integer)cnt.elementAt(i)).intValue();
		 if(m < min) {
			min = m;
		 }
		 sum +=m;
	  }
	  int ave = (int) (((double)sum / (double)record_type.size() + 0.5));

//      System.out.println("sum = " + sum + "\tave = " + ave);

	  int [] dis = new int[type_list.size()];
	  for(int k=0; k<type_list.size(); k++) {
		 int d = ((GTI)type_list.elementAt(k)).num;

		 if (d < (50*min)) {
			dis[k] = d -ave;
		 }
		 else {
			dis[k] = Math.abs(d - ave);
		 }
	  }

	  tag_list.addElement(new GTI(((GTI)type_list.elementAt(0)).str, dis[0]));


	  for(int k=1; k<type_list.size(); k++) {
		 String tg = ((GTI)type_list.elementAt(k)).str;

		 for(int i=0; i< tag_list.size(); i++) {
			int appl = ((GTI)tag_list.elementAt(i)).num;

			if(dis[k] < appl && dis[k] >= 0) {
			   tag_list.insertElementAt(new GTI(tg, dis[k]), i);
			   break;
			}
		 }

		 if(tag_list.size() <= k) {
			tag_list.addElement(new GTI(tg, dis[k]));
		 }
	  }
//      System.out.println("\n");

/*
	  for(int k=0; k<tag_list.size(); k++) {
		 String tg = ((GTI)tag_list.elementAt(k)).str;
//         System.out.println(tg + "\t->\t" + ((GTI)tag_list.elementAt(k)).num);
	  }
*/
	  candidate_list.addElement(new GTI(((GTI)tag_list.elementAt(0)).str, rank));

	  for(int i=1; i<tag_list.size(); i++) {
		 int app1 = ((GTI)tag_list.elementAt(i-1)).num;
		 int app2 = ((GTI)tag_list.elementAt(i)).num;

		 if(app1 < app2) {
			rank++;
		 }

		 candidate_list.addElement(new GTI(((GTI)tag_list.elementAt(i)).str, rank));

	  }
/*
	  System.out.println("\nCandidate_list");
	  for(int i=0; i<candidate_list.size(); i++) {
		 System.out.println(((GTI)candidate_list.elementAt(i)).str + "\t" +
							((GTI)candidate_list.elementAt(i)).num);
	  }
*/
	  return candidate_list;

   }
}

class TABLE {
   protected String key = new String();
   protected String content = new String();

   TABLE() {
	  key = new String();
	  content = new String();
   }

   TABLE(String k, String c) {
	  key = new String(k);
	  content = new String(c);
   }

   TABLE(TABLE tb) {
	  key = new String(tb.key);
	  content = new String(tb.content);
   }
}

