package edu.byu.deg.records;

import java.util.*;

/**
  * This class take a tag-tree as input
  * find the separators
  *
  * @version 1.1.1
  * @author  Yuan Jiang
  */
class ApplySD {
   TTree tree = new TTree();
   Vector tag_list = new Vector();
   Vector type_list = new Vector();

   /**
	* Constructor
	*/
   ApplySD() {
	  tree = new TTree();
	  tag_list = new Vector();
	  type_list = new Vector();

   }

   /**
	* Constructor
	* @param sub
	*/
   ApplySD(TTree sub) {
	  this();
	  tree = sub;

	  PreProcessTags pt = new PreProcessTags(sub);
	  pt.eliminateIrrelevantTags(2);
	  tag_list = pt.tag_list;
	  type_list = pt.type_list;
   }

   /**
	* Constructor
	* @param sub
	* @param tglist
	* @param tylist
	*/
   ApplySD(TTree sub, Vector tglist, Vector tylist) {
	  this();
	  tree = sub;
	  for(int i=0; i< sub.children.size(); i++) {
		 String tg = new String((String)((TTree)sub.children.elementAt(i)).current.getTag());
		 tag_list.addElement(new String(tg));
	  }

	  type_list = tylist;


   }
   /**
	*   find the tag with samllest standard deviation
	*/
   public Vector find_candidate() {
	  int rank = 1;
	  Vector candidate_list = new Vector();
	  Vector temp_list = new Vector();
	  Vector sdl = new Vector();

	  if(type_list.size() == 0) {
		 return candidate_list;
	  }

	  for (int j=0; j<type_list.size(); j++) {
		 String test_tag = new String(((GTI)type_list.elementAt(j)).str);
		 double temp = calculateSD(test_tag);
		 int pos = 0;


		 if(sdl.size() == 0) {
			sdl.addElement(new Double(temp));
			pos = 0;
		 }
		 else {
			for(int i=0; i<sdl.size(); i++) {
			   pos = 0;
			   double sdv = ((Double)sdl.elementAt(i)).doubleValue();

			   if (temp < sdv) {
				  sdl.insertElementAt(new Double(temp), i);
				  pos = i;
				  break;
			   }
			   if(i == (sdl.size()-1) && temp >= sdv) {
				  sdl.addElement(new Double(temp));
				  pos = i + 1;
				  break;
			   }
			}
		 }

		 temp_list.insertElementAt(new String(test_tag), pos);
	  }


/*
	  System.out.println("Standard Deviation:");
	  for(int i=0; i<temp_list.size(); i++) {
		 System.out.print((String)temp_list.elementAt(i) + "\t");
		 System.out.println(((Double)sdl.elementAt(i)).doubleValue());
	  }
	  System.out.println("\n");
*/
	  candidate_list.addElement(new GTI((String)temp_list.elementAt(0), rank));

	  for(int i=1; i<temp_list.size(); i++) {
		 double app1 = ((Double)sdl.elementAt(i-1)).doubleValue();
		 double app2 = ((Double)sdl.elementAt(i)).doubleValue();

		 if(app1 < app2) {
			rank++;
		 }
		 candidate_list.addElement(new GTI((String)temp_list.elementAt(i), rank));
	  }

/*
	  System.out.println("Candidate_list");
	  for(int i=0; i<candidate_list.size(); i++) {
		 System.out.println(((GTI)candidate_list.elementAt(i)).str + "\t" +
							((GTI)candidate_list.elementAt(i)).num);
	  }
*/

	  return candidate_list;
   }

   /**
	*   calculate the standard deviation between the input tag
	*/
   private double calculateSD(String tag) {
	  Vector distance = new Vector();
	  double sd = 0;
	  int sum = 0;
	  //int sum2 = 0;

	  distance.removeAllElements();

	  int len = 0;
	  int firstpos = tag_list.indexOf(tag);

	  if (firstpos < 0) {
		 return -1;
	  }


	  for(int i=firstpos; i< tag_list.size(); i++) {
		 String tg = new String((String)tag_list.elementAt(i));
		 if(tg.equals(tag) && i != firstpos) {
			sum += len;
			//sum2 += (len * len);

			distance.addElement(new Integer(len));
			len = 0;
			len += ((TTree)tree.children.elementAt(i)).getPlainText().length();
		 }
		 else {
			len += ((TTree)tree.children.elementAt(i)).getPlainText().length();
		 }
	  }

	  int size = distance.size();

	  // remove outlier
	  int outlier = 0;
	  if (outlier > 0) {
		 double ave = ((double)sum) / ((double)size);
		 double[] diff = new double[outlier];
		 int[] diffv = new int[outlier];
		 for(int j=0; j< diff.length; j++) {
			diff[j] = 0.0;
			diffv[j] = 0;
		 }

		 for(int i=0; i<size; i++) {
			int e = ((Integer)distance.elementAt(i)).intValue();
			double dis = Math.abs((double)e - ave);

			for(int j=0; j<diff.length; j++) {
			   if(dis > diff[j]) {
				  diff[j] = dis;
				  diffv[j] = e;
				  break;
			   }
			}
		 }
		 for(int j=0; j< diffv.length; j++) {
			sum -= diffv[j];
			//sum2 -= (diffv[j] * diffv[j]);
		 }

		 size -= outlier;

	  }


	  double ave = ((double)sum) / ((double)size);
	  double add_e = 0;

	  //Calculate Standard Deviation
	  for(int j=0; j< size; j++) {
		 int tt = ((Integer)distance.elementAt(j)).intValue();
		 add_e += ((double)tt - ave) * ((double)tt - ave);
	  }

	  sd = Math.sqrt(add_e / (double)size);

	  //calculate standard deviation
	  //double variance = ((double)(size*sum2-(sum*sum)))/((double)(size*size));
	  //sd = Math.sqrt(variance);
	  //System.out.println(tag + "\t" + variance + "\t" + sd);

	  return sd;
   }
}

