package edu.byu.deg.records;

import java.util.*;
import java.io.*;
import java.net.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.tidy.Tidy;
import com.sun.xml.tree.XmlDocument;
import org.apache.xpath.XPathAPI;


class DOMSeparator {

	private static boolean F = false;
	private static boolean T = true;
	private static String BL = "";
	private static String NL = "\n";
	private static String SP = " ";


	String tag = new String();
	String parent_xpath;
	String inputDocURI = null;
	String ws_regex = "[ \\u00A0\t\\x09\n\\x0A\f\\x0c\r\\x0D]+";

	HashMap tags = new HashMap();

	public static Tidy tidy;

	Document doc;
	Node container_node;

	DOMSeparator(String filename, String tag, String parent_node_xpath) throws FileNotFoundException
	{
		this(new File(filename), tag, parent_node_xpath);
	}

	DOMSeparator(File file, String tag, String parent_node_xpath) throws FileNotFoundException
	{
		this(new FileInputStream(file), tag, parent_node_xpath);
	}

	DOMSeparator(URL url, String tag, String parent_node_xpath)  throws IOException
	{
		this(url.openStream(), tag, parent_node_xpath);
	}


	DOMSeparator(InputStream instream, String tag, String parent_node_xpath)
	{
		this(DOMSeparator.getTidy().parseDOM(instream, null), tag, parent_node_xpath);

	}

	DOMSeparator(Document doc, String tag, String parent_node_xpath)
	{
		this.doc = doc;
		this.tag = tag;
		this.parent_xpath = parent_node_xpath;
		build_tags();
		try{
			this.container_node = XPathAPI.selectSingleNode(doc, parent_node_xpath);
		}catch (javax.xml.transform.TransformerException e)
		{
			System.err.println("Error in XPath Evaluation: "+e.toString());
		}
		if (container_node == null)
			System.out.println(parent_node_xpath + " not found");

		//trim(doc, container_node);
		//DOMSeparator.getTidy().pprint(doc, System.out);
	}

	public static Tidy getTidy()
	{
		if (tidy == null)
		{
			tidy = new Tidy();
			tidy.setShowWarnings(false);
			tidy.setMakeClean(false);
			tidy.setXmlTags(false);
		}
		return tidy;
	}

/*
	private void trim(Node node, Node target_node)
	{
		if (node == target_node) return;
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			if (!isAncestor(child, target_node))
			{
			   child.getParentNode().removeChild(child);
			   i--;
			}
			else if (child != target_node)
			{
				trim(child, target_node);
			}
		}
	}

	private boolean isAncestor(Node ancestor, Node descendent)
	{
		if (ancestor == descendent) return true;
		NodeList children = ancestor.getChildNodes();
		Node child = null;
		for (int i = 0; i < children.getLength(); i++)
		{
			child = children.item(i);
			if (child == descendent) return true;
			else if (isAncestor(child, descendent)) return true;
		}
		return false;
	}
*/

	// provided for legacy systems that still use plain text records
	// separated by five # signs
	public String getTextRecords()
	{
		XmlDocument	recs_doc = getXMLRecords();
		StringBuffer recs_text = new StringBuffer("\n\n#####\n\n");
		Node root = recs_doc.getFirstChild();
		for (int i = 0; i < root.getChildNodes().getLength(); i++)
		{
			Node record = root.getChildNodes().item(i);
			recs_text.append(record.getFirstChild().getNodeValue());
			recs_text.append("\n\n#####\n\n");
		}
		return recs_text.toString();
	}

	public void setInputDocURI(String uri)
	{
		this.inputDocURI = uri;
	}

	public XmlDocument getXMLRecords()
	{
		XmlDocument	recs_doc = new XmlDocument ();
		Element recs = recs_doc.createElement("records");
		if (this.inputDocURI != null)
		   recs.setAttribute("IputDocumentURI", this.inputDocURI);
		recs_doc.appendChild(recs);

		NodeList node_list;
		try{
			String child_xpath = parent_xpath + "/" + this.tag;
			node_list = XPathAPI.selectNodeList(doc, child_xpath);
			Element rec;
//			int loc, offset_in_record;

			// if this is a self-closing tag, we'll use the parent of the separator
			// tags for the xpath and calculate a character offset for the node in
			// the parent node.  Then we'll use the text nodes between this occurence
			// of the separator tag and the next as the text for the record
			if (((Tag)tags.get(this.tag)).isEmpty())
			{
				Node curr_node;
				StringBuffer rec_text;
				int rec_offset = 0;
				for (int i = -1; i < node_list.getLength(); i++)
				{
					// the first record will probably be before the first occurrance
					// of the separator tag, so get all the text in the parent
					// node up to the first separator tag node
					if (i == -1)
					{
						curr_node = node_list.item(0).getParentNode().getFirstChild();
					}
					else
					{
						curr_node = node_list.item(i);
						curr_node = curr_node.getNextSibling();
					}
					rec_text = new StringBuffer();
					int num_nodes = node_list.getLength();
					while (curr_node != null && i+1 < num_nodes &&
						   curr_node != node_list.item(i+1))
					{
//						offset_in_record = rec_text.length();
						rec_text.append(string_value(curr_node, true));
						curr_node = curr_node.getNextSibling();
					}

					String rec_str = rec_text.toString();


					if (rec_str.length() > 0)
					{
						rec = recs_doc.createElement("record");
						recs.appendChild(rec);
						rec.appendChild(recs_doc.createCDATASection(rec_str));
						rec.setAttribute("XPath", parent_xpath);
						rec.setAttribute("charOffsetInParent", Integer.toString(rec_offset));
						rec_offset += getNumNonWhiteCharsBefore(rec_str, -1);
					}
				}

			}
			// otherwise, we'll simply use the tag node for the xpath and use its text
			// content as the text for the record
			else
			{
				for (int i = 0; i < node_list.getLength(); i++)
				{
					Node curr_node = node_list.item(i);
					String rec_str = string_value(curr_node, true).trim();

					if (rec_str.length() > 0)
					{
						rec = recs_doc.createElement("record");
						recs.appendChild(rec);
						rec.appendChild(recs_doc.createCDATASection(rec_str));
						rec.setAttribute("XPath", getXPath(curr_node));
						rec.setAttribute("charOffsetInParent", "0");
					}
				}
			}
		}catch (javax.xml.transform.TransformerException e)
		{
			System.err.println("Error in XPath Evaluation: "+e.toString());
		}
		try{

		}catch(Exception e){System.out.println(e.toString());}
		return recs_doc;
	}


	private void append_start_tag_whitespace(StringBuffer str, Tag tag)
	{
		if (tag != null)
			str.append(tag.getStartSpace());
	}

	private void append_end_tag_whitespace(StringBuffer str, Tag tag)
	{
		if (tag != null)
			str.append(tag.getEndSpace());
	}


	private String collapseWhiteSpace(String str)
	{
		//System.out.println("str = -->"+str+"<--");
		str = str.replaceAll(ws_regex, " ");
		//System.out.println("str = -->"+str+"<--");
		return str;
	}

	/* This is a utility method to get the non-whitespace length of a string.
	 * If the boundary param is not -1, it returns the number of non-whitespace
	 * characters in a string before a given position.  This is used to get
	 * around browser whitespace issues with xpointers.
	 *
	 * Note: a matching method is found in edu.byu.deg.Ontos
	 */
	private int getNumNonWhiteCharsBefore(String str, int boundary)
	{
		if (boundary != -1)
			str = str.substring(0, boundary);

		str = str.replaceAll(ws_regex, "");
		int num = str.length();
		return num;
	}


	private String string_value(Node node, boolean add_whitespace)
	{
		StringBuffer str = new StringBuffer();

		if (node.getNodeType() == Node.TEXT_NODE)
		{
			str.append(collapseWhiteSpace(node.getNodeValue()));
		}
		else
		{
			Tag tag = (Tag)tags.get(node.getNodeName().toLowerCase());
			if (add_whitespace)
				append_start_tag_whitespace(str, tag);

			NodeList children = node.getChildNodes();
			for (int i = 0; i < children.getLength(); i++)
			{
				Node child = children.item(i);
				str.append(string_value(child, add_whitespace));
			}

			if (add_whitespace)
			   append_end_tag_whitespace(str, tag);
		}
		return str.toString();
	}

	public static String getXPath(Node node)
	{
		Node curr_node = node;
		String xpath = "";
		while (curr_node.getNodeType() != Node.DOCUMENT_NODE)
		{
			xpath = "/"+curr_node.getNodeName()+"["+getPositionInParent(curr_node)+"]" + xpath;
			curr_node = curr_node.getParentNode();
		}

		return xpath;
	}


	private static int getPositionInParent(Node node)
	{
		Node parent = node.getParentNode();
		NodeList children = parent.getChildNodes();
		int cnt = 1;
		for (int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			if (child == node)
			   return cnt;
			else if (child.getNodeName().equalsIgnoreCase(node.getNodeName()))
			   cnt ++;
		}
		return -1;
	}

	public static void main(String args[])
	{
		if(args.length < 3) {
			System.out.println("Usage: java edu.byu.deg.DOMSeparator <filename> <tag_name> <xpath_to_container> (optional [output_xml_file])");
			System.out.println("		OR");
			System.out.println("Usage: java edu.byu.deg.DOMSeparator <url> <tag_name>  <xpath_to_container> (optional [output_xml_file])");
			/*Sampe Command Line
			 * java -cp d:\cvs\libs\Tidy.jar;d:\tim\semanticweb\jars\xml.jar;.
			 *      edu.byu.deg.records.DOMSeparator d:\test\software.html
			 *      tr
			 *      /html[1]/body[1]/table[1]
			 *      d:\test\records.xml
			 */
		}
		else {
			// Read the file

			try{
				//String filename = "d:/test/software.html";
				//String xpath = "/html[1]/body[1]/table[1]";
				//String tag = "tr";

				//String filename = "d:/test/software_hr.html";
				//String xpath = "/html[1]/body[1]/div[1]/table[3]/tr[1]/td[5]/table[3]";
				//String tag = "tr";
				String filename = args[0];
				String tag = args[1];
				String xpath = args[2];

				DOMSeparator sep;

				//for (int i = 25; i <= 40; i++)
				//{
					//String filename = "d:\\test\\in\\cnet_html_files\\cnet_"+i+".html";
					//args[3] = "d:\\test\\in\\cnet_xml_files\\cnet_"+i+".xml";
					//System.out.println("Processing: "+filename);
					if (filename.indexOf("://") != -1)
						sep = new DOMSeparator(new URL(filename), tag, xpath);
					else
						sep = new DOMSeparator(filename, tag, xpath);

					XmlDocument xdoc = sep.getXMLRecords();

					if (args.length >= 4)
						xdoc.write(new FileOutputStream(args[3]));
					else
						xdoc.write(System.out);
				//}
			}catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	private void build_tags()
	{
		tags.put("a",       new Tag("a",           F, BL, F, BL, F));
		tags.put("abbr",    new Tag("abbr",        F, BL, F, BL, F));
		tags.put("acronym", new Tag("acronym",     F, BL, F, BL, F));
		tags.put("address", new Tag("address",     F, BL, F, BL, F));
		tags.put("applet",  new Tag("applet",      T, SP, T, BL, F));
		tags.put("area",    new Tag("area",        T, BL, F, BL, F));
		tags.put("b",       new Tag("b",           F, BL, F, BL, F));
		tags.put("base",    new Tag("base",        T, BL, F, BL, F));
		tags.put("basefont",new Tag("basefont",    T, BL, F, BL, F));
		tags.put("bdo",     new Tag("bdo",         F, BL, F, BL, F));
		tags.put("big",     new Tag("big",         F, BL, F, BL, F));
		tags.put("blockquote",new Tag("blockquote",F, NL, F, NL, F));
		tags.put("body",    new Tag("body",        F, BL, F, BL, F));
		tags.put("br",      new Tag("br",          T, NL, T, BL, F));
		tags.put("button",  new Tag("button",      T, SP, T, BL, F));
		tags.put("caption", new Tag("caption",     F, BL, F, BL, F));
		tags.put("center",  new Tag("center",      F, NL, F, NL, F));
		tags.put("cite",    new Tag("cite",        F, BL, F, BL, F));
		tags.put("code",    new Tag("code",        F, BL, F, BL, F));
		tags.put("col",     new Tag("col",         T, BL, F, SP, F));
		tags.put("colgroup",new Tag("colgroup",    F, BL, F, BL, F));
		tags.put("dd",      new Tag("dd",          F, BL, F, BL, F));
		tags.put("del",     new Tag("del",         F, BL, F, BL, F));
		tags.put("dfn",     new Tag("dfn",         F, BL, F, BL, F));
		tags.put("dir",     new Tag("dir",         F, NL, F, NL, F));
		tags.put("div",     new Tag("div",         F, NL, F, NL, F));
		tags.put("dl",      new Tag("dl",          F, NL, F, NL, F));
		tags.put("dt",      new Tag("dt",          F, BL, F, NL, F));
		tags.put("em",      new Tag("em",          F, BL, F, BL, F));
		tags.put("frame",   new Tag("frame",       T, BL, F, SP, F));
		tags.put("frameset",new Tag("frameset",    F, BL, F, BL, F));
		tags.put("fieldset",new Tag("fieldset",    F, BL, F, BL, F));
		tags.put("font",    new Tag("font",        F, BL, F, BL, F));
		tags.put("form",    new Tag("form",        F, BL, F, BL, F));
		tags.put("h1",      new Tag("h1",          F, NL, F, NL, F));
		tags.put("h2",      new Tag("h2",          F, NL, F, NL, F));
		tags.put("h3",      new Tag("h3",          F, NL, F, NL, F));
		tags.put("h4",      new Tag("h4",          F, NL, F, NL, F));
		tags.put("h5",      new Tag("h5",          F, NL, F, NL, F));
		tags.put("h6",      new Tag("h6",          F, NL, F, NL, F));
		tags.put("hr",      new Tag("hr",          T, NL, T, BL, F));
		tags.put("iframe",  new Tag("iframe",      T, NL, T, BL, F));
		tags.put("isindex", new Tag("isindex",     T, BL, F, SP, F));
		tags.put("i",       new Tag("i",           F, BL, F, BL, F));
		tags.put("img",     new Tag("img",         T, SP, T, BL, F));
		tags.put("input",   new Tag("input",       T, SP, T, BL, F));
		tags.put("ins",     new Tag("ins",         F, BL, F, BL, F));
		tags.put("kbd",     new Tag("kbd",         F, BL, F, BL, F));
		tags.put("label",   new Tag("label",       F, SP, F, SP, F));
		tags.put("legend",  new Tag("legend",      F, SP, F, SP, F));
		tags.put("li",      new Tag("li",          F, NL, F, NL, F));
		tags.put("link",    new Tag("link",        T, BL, F, BL, F));
		tags.put("menu",    new Tag("menu",        F, NL, F, NL, F));
		tags.put("map",     new Tag("map",         F, BL, F, BL, F));
		tags.put("meta",    new Tag("meta",        T, BL, F, BL, F));
		tags.put("noscript",new Tag("noscript",    F, BL, F, BL, F));
		tags.put("noframes",new Tag("noframes",    F, BL, F, BL, F));
		tags.put("object",  new Tag("object",      F, BL, F, BL, F));
		tags.put("ol",      new Tag("ol",          F, NL, F, NL, F));
		tags.put("option",  new Tag("option",      F, BL, F, NL, F));
		tags.put("optgroup",new Tag("optgroup",    F, BL, F, BL, F));
		tags.put("p",       new Tag("p",           F, NL, F, NL, F));
		tags.put("param",   new Tag("param",       T, BL, F, BL, F));
		tags.put("pre",     new Tag("pre",         F, NL, F, NL, F));
		tags.put("q",       new Tag("q",           F, SP, F, SP, F));
		tags.put("s",       new Tag("s",           F, BL, F, BL, F));
		tags.put("samp",    new Tag("samp",        F, SP, F, SP, F));
		tags.put("script",  new Tag("script",      F, BL, F, BL, F));
		tags.put("select",  new Tag("select",      T, SP, T, BL, F));
		tags.put("small",   new Tag("small",       F, BL, F, BL, F));
		tags.put("span",    new Tag("span",        F, BL, F, BL, F));
		tags.put("strike",  new Tag("strike",      F, BL, F, BL, F));
		tags.put("style",   new Tag("style",       F, BL, F, BL, F));
		tags.put("sub",     new Tag("sub",         F, BL, F, BL, F));
		tags.put("sup",     new Tag("sup",         F, BL, F, BL, F));
		tags.put("strong",  new Tag("strong",      F, BL, F, BL, F));
		tags.put("table",   new Tag("table",       F, NL, T, NL, T));
		tags.put("tbody",   new Tag("tbody",       F, BL, F, BL, F));
		tags.put("td",      new Tag("td",          F, BL, F, SP, F));
		tags.put("tfoot",   new Tag("tfoot",       F, BL, F, NL, F));
		tags.put("th",      new Tag("th",          F, BL, F, SP, F));
		tags.put("thead",   new Tag("thead",       F, BL, F, NL, F));
		tags.put("tr",      new Tag("tr",          F, BL, F, NL, F));
		tags.put("tt",      new Tag("tt",          F, BL, F, BL, F));
		tags.put("textarea",new Tag("textarea",    T, SP, T, BL, F));
		tags.put("ul",      new Tag("ul",          F, NL, F, NL, F));
		tags.put("u",       new Tag("u",           F, BL, F, BL, F));
		tags.put("var",     new Tag("var",         F, BL, F, BL, F));
	}




	class Tag
	{
		String name;
		boolean empty, force_start, force_end;
		String start_whitespace;
		String end_whitespace;


		Tag(String name, boolean empty, String start_whitespace, boolean force_start,
										String end_whitespace, boolean force_end)
		{
			this.name = name;
			this.empty = empty;
			this.start_whitespace = start_whitespace;
			this.end_whitespace = end_whitespace;
		}
		public String getName() {return this.name;}
		public boolean isEmpty() {return this.empty;}
		public String getStartSpace() {return this.start_whitespace;}
		public boolean getForceStart() {return this.force_start;}
		public String getEndSpace() {return this.end_whitespace;}
		public boolean getForceEnd() {return this.force_end;}
	}
}



