package edu.byu.deg.records;

import java.io.*;

/**
  * This class takes in a .dat file and also an .orc file
  * and produces as output the information for three programs
  * that can  check an HTML Web page, cleaned by TagRemover and parsed
  * by Ontos, to see whether it satisfies the given input ontology
  * in the sense that it contains multiple records that each
  * individually satisfy the ontology.
  *
  * version 1.0
  * author  David W. Embley
  */
class PgmGen2 {
   String datFileText = new String();
   String orcFileText = new String();

   String objectSetOrKeyword[] = new String[500];
   String extractedString[] = new String[500];
   int startPos[] = new int[500];
   int endPos[] = new int[500];
// float confFactor[] = new float[500];
   boolean keyword[] = new boolean[500];
   int totalLines;

   String objectSet[] = new String[50];
   int max[] = new int[50];
// float expectedNr[] = new float[50];
// double recogPrecision[] = new double[50];
   int totalObjSets;

   PgmGen2(File filename1, File filename2) {
	  try {
		 InputStream in = new FileInputStream(filename1);
		 int i = in.read();
		 while(i != -1) {
			datFileText = datFileText+(char)i;
		i = in.read();
		 }
	  }
	  catch (IOException io) {
		 System.out.println(io.getMessage());
	  }
	  try {
		 InputStream in = new FileInputStream(filename2);
		 int i = in.read();
		 while(i != -1) {
			orcFileText = orcFileText+(char)i;
		i = in.read();
		 }
	  }
	  catch (IOException io) {
		 System.out.println(io.getMessage());
	  }
   }

   public void getDatFacts(String datText) {
	  // This reads the four values on each line: (1) object-set name
	  // (or KEYWORD(object-set name), (2) extracted string, (3) start
	  // position, and (4) end position.
	  // It converts these values respectively to string, string, int,
	  // and int.  It also sets a boolean: keyword = true if the
	  // object-set name is for a keyword and = false otherwise.
	  // For now, it simply outputs the converted values as facts --
	  // "value(os, es, sp, ep)" or "keyword(os, es, sp, ep)".

//    System.out.println("(* getDatFacts *)");

	  String stringToConvert;
	  int pos = 0;
	  int line = 1;
	  boolean morelines;
	  morelines = pos < datText.length();
	  while (morelines) {

	 // Get object-set name or KEYWORD(object-set name).
	 keyword[line] = false;
		 objectSetOrKeyword[line] = new String();
	 while (datText.charAt(pos) != '|') {
		objectSetOrKeyword[line]
		   = objectSetOrKeyword[line]+datText.charAt(pos);
		pos = pos+1;
	 }
		 if (objectSetOrKeyword[line].length() > "KEYWORD()".length()) {
			if (objectSetOrKeyword[line].charAt(0) == 'K'
			  && objectSetOrKeyword[line].charAt(1) == 'E'
			  && objectSetOrKeyword[line].charAt(2) == 'Y'
			  && objectSetOrKeyword[line].charAt(3) == 'W'
			  && objectSetOrKeyword[line].charAt(4) == 'O'
			  && objectSetOrKeyword[line].charAt(5) == 'R'
			  && objectSetOrKeyword[line].charAt(6) == 'D'
			  && objectSetOrKeyword[line].charAt(7) == '('
			  && objectSetOrKeyword[line].charAt(
			 objectSetOrKeyword[line].length()-1)
		  ==')') {
		   keyword[line] = true;
		   objectSetOrKeyword[line] =
				 objectSetOrKeyword[line].substring("KEYWORD(".length(),
					 objectSetOrKeyword[line].length()-1);
		}
	 }
		 pos = pos+1;

	 // Get extracted string.
		 extractedString[line]  = new String();
	 while (datText.charAt(pos) != '|') {
		extractedString[line] = extractedString[line]+datText.charAt(pos);
		pos = pos+1;
	 }
		 pos = pos+1;

	 // Get start position.
		 stringToConvert = new String();
	 while (datText.charAt(pos) != '|') {
		stringToConvert = stringToConvert+datText.charAt(pos);
		pos = pos+1;
	 }
		 startPos[line] = Integer.parseInt(stringToConvert);
		 pos = pos+1;

	 // Get end position.
		 stringToConvert = new String();
	 while ('0' <= datText.charAt(pos) && datText.charAt(pos) <= '9') {
		stringToConvert = stringToConvert+datText.charAt(pos);
		pos = pos+1;
	 }
		 endPos[line] = Integer.parseInt(stringToConvert);

	 /*
		 pos = pos+1;

	 // Get confidence factor.
	 stringToConvert = new String();
	 while (datText.charAt(pos) == '.' ||
		   ('0' <= datText.charAt(pos) && datText.charAt(pos) <= '9')) {
		stringToConvert = stringToConvert+datText.charAt(pos);
		pos = pos+1;
	 }
		 confFactor[line] = Float.valueOf(stringToConvert).floatValue();
	 */

	 // Set up for next iteration, if any.
	 while (pos < datText.length() &&
		   !('A' <= datText.charAt(pos)
		   && datText.charAt(pos) <= 'Z')) {
		pos = pos+1;
	 }
	 if (pos >= datText.length()) {
		morelines = false;
		totalLines = line;
		 }
	 else {
		line = line+1;
	 }
	  }
   }

   public void ResolveOverlaps() {
	  // If there are overlaps in extracted values and/or keywords, we keep
	  // the first and discard any subsequent overlaping extracted strings.
	  // Exceptions for exact matches: (1) values have precedence over
	  // keywords and (2) value sets are resolved by the closest previous
	  // or subsequent (non-overlapping) keyword.  We close up the array of
	  // records by actually discarding the record.
	  int firstCharPos = 0;
	  int lastCharPos = 0;
	  int newLineNr = 0;
	  int groupStart = 0;
	  int groupEnd = 0;

//    System.out.println();
//    System.out.println("(* Resolve-Overlaps *)");

	  /*
	  System.out.println();
	  System.out.println("Table before resolving overlaps:");
	  for (int line = 1; line <= totalLines; line++) {
		 System.out.print(objectSetOrKeyword[line]);
	 System.out.print(", ");
		 System.out.print(extractedString[line]);
	 System.out.print(", ");
		 System.out.print(startPos[line]);
	 System.out.print(", ");
		 System.out.print(endPos[line]);
	 System.out.print(", ");
//       System.out.print(confFactor[line]);
	 System.out.print(", ");
		 System.out.print(keyword[line]);
	 System.out.println();
	  }
	  */

	  // Resolve all overlaps except exact-match overlaps.
	  // For exact-match overlaps, reduce the problem to guaranteeing
	  // that the only overlaps are exact-match value overlaps.
	  for (int line = 1; line <= totalLines; line++) {
		 if (startPos[line] > lastCharPos /* no overlap */) {
		newLineNr = newLineNr+1;
			objectSetOrKeyword[newLineNr] = objectSetOrKeyword[line];
			extractedString[newLineNr] = extractedString[line];
			startPos[newLineNr] = startPos[line];
			endPos[newLineNr] = endPos[line];
//          confFactor[newLineNr] = confFactor[line];
			keyword[newLineNr] = keyword[line];
	 }
	 else if (startPos[line] != firstCharPos || endPos[line] < lastCharPos
		   /* overlap not exact, 1st starts before or 1st subsumes */) {
			// keep first -- already have it (do nothing)
	 }
	 else if (startPos[line] == firstCharPos && endPos[line] > lastCharPos
		   /* overlap not exact, 2nd properly subsumes 1st */) {
		// overwrite the record previously loaded -- the new one subsumes it
			objectSetOrKeyword[newLineNr] = objectSetOrKeyword[line];
			extractedString[newLineNr] = extractedString[line];
			startPos[newLineNr] = startPos[line];
			endPos[newLineNr] = endPos[line];
//          confFactor[newLineNr] = confFactor[line];
			keyword[newLineNr] = keyword[line];
	 }
	 else if (startPos[line] == firstCharPos
		   && endPos[line] == lastCharPos /* exact match */) {
			if (keyword[line]) {
		   // The second is a keyword -- we thus ignore it because
		   // we either already have a keyword (and we keep the first)
		   // or we already have one or more values (and we discard all
		   // exact match keywords).  So we do nothing.
		}
		else if (keyword[newLineNr]) {
		   // In this case we now have a value, having only previously
		   // seen a keyword.  We thus overwrite the keyword with the value.
			   objectSetOrKeyword[newLineNr] = objectSetOrKeyword[line];
			   extractedString[newLineNr] = extractedString[line];
			   startPos[newLineNr] = startPos[line];
			   endPos[newLineNr] = endPos[line];
//             confFactor[newLineNr] = confFactor[line];
			   keyword[newLineNr] = keyword[line];
		}
		else {
		   // Both are values, so we add it to the group.
		   newLineNr = newLineNr+1;
			   objectSetOrKeyword[newLineNr] = objectSetOrKeyword[line];
			   extractedString[newLineNr] = extractedString[line];
			   startPos[newLineNr] = startPos[line];
			   endPos[newLineNr] = endPos[line];
//             confFactor[newLineNr] = confFactor[line];
			   keyword[newLineNr] = keyword[line];
		}
	 }
	 else {
		System.out.println("error: This state should be impossible.");
	 }
	 // get set up for next iteration
	 firstCharPos = startPos[line];
	 lastCharPos = endPos[line];
	  }
	  totalLines = newLineNr;

	  /*
	  System.out.println();
	  System.out.println("Table after resolving inexact overlaps:");
	  for (int line = 1; line <= totalLines; line++) {
		 System.out.print(objectSetOrKeyword[line]);
	 System.out.print(", ");
		 System.out.print(extractedString[line]);
	 System.out.print(", ");
		 System.out.print(startPos[line]);
	 System.out.print(", ");
		 System.out.print(endPos[line]);
	 System.out.print(", ");
//       System.out.print(confFactor[line]);
	 System.out.print(", ");
		 System.out.print(keyword[line]);
	 System.out.println();
	  }
	  */

	  // Resolve exact-match value overlaps.
	  groupStart = 1;
	  groupEnd = 2;
	  int prevKeyword;
	  int nextKeyword;
	  int theOne;
	  int newTotalLines;
	  newTotalLines = 1;
	  while (groupStart <= totalLines) {
		 if (groupStart == totalLines) {
			objectSetOrKeyword[newTotalLines] = objectSetOrKeyword[groupStart];
			extractedString[newTotalLines] = extractedString[groupStart];
			startPos[newTotalLines] = startPos[groupStart];
			endPos[newTotalLines] = endPos[groupStart];
//          confFactor[newTotalLines] = confFactor[groupStart];
			keyword[newTotalLines] = keyword[groupStart];

		// Set up for next iteration.
		groupStart = groupEnd;
		groupEnd = groupEnd+1;
	 }
		 else if (startPos[groupStart] == startPos[groupEnd]) {
			// Find group: groupStart -- groupEnd.
		while (startPos[groupStart] == startPos[groupEnd]) {
		   groupEnd = groupEnd+1;
		   if (groupEnd > totalLines) break;
		}
		groupEnd = groupEnd-1;

			// Find previous keyword, if any.
		prevKeyword = groupStart-1;
		while (prevKeyword > 0) {
		   if (keyword[prevKeyword]) break;
		   prevKeyword = prevKeyword-1;
		}

			// Find next keyword, if any.
		nextKeyword = groupEnd+1;
		while (nextKeyword <= totalLines) {
		   if (keyword[nextKeyword]) break;
		   nextKeyword = nextKeyword+1;
		}

			// Use closest keyword to match, if any
		// (from top if previous keyword and
			// from bottom if subsequent keyword).
			// If no match, use the other keyword to match.
			// If still no match, take the first in group.
		if (prevKeyword > 0 && nextKeyword <= totalLines) {
		   if (startPos[groupStart]-endPos[prevKeyword]
				 <= startPos[nextKeyword]-endPos[groupEnd]) {
			  theOne = groupStart;
			  while (theOne <= groupEnd) {
				 if (objectSetOrKeyword[theOne]
				.equals(objectSetOrKeyword[prevKeyword])) break;
			theOne = theOne+1;
			  }
			  if (theOne > groupEnd) {
				 theOne = groupEnd;
				 while (theOne > groupStart) {
					if (objectSetOrKeyword[theOne]
				   .equals(objectSetOrKeyword[nextKeyword])) break;
				theOne = theOne-1;
				 }
			  }
		   }
		   else {
			  theOne = groupEnd;
			  while (theOne >= groupStart) {
				 if (objectSetOrKeyword[theOne]
				.equals(objectSetOrKeyword[nextKeyword])) break;
			 theOne = theOne-1;
			  }
		  if (theOne < groupStart) {
				 theOne = groupStart;
				 while (theOne <= groupEnd) {
					if (objectSetOrKeyword[theOne]
			   .equals(objectSetOrKeyword[prevKeyword])) break;
				theOne = theOne+1;
				 }
				 if (theOne > groupEnd) {
					theOne = groupStart;
				 }
				  }
		   }
		}
		else if (prevKeyword > 0) {
		   theOne = groupStart;
		   while (theOne <= groupEnd) {
			  if (objectSetOrKeyword[theOne]
			 .equals(objectSetOrKeyword[prevKeyword])) break;
		  theOne = theOne+1;
		   }
		   if (theOne > groupEnd) {
			  theOne = groupStart;
		   }
		}
		else if (nextKeyword <= totalLines) {
		   theOne = groupEnd;
		   while (theOne > groupStart) {
			  if (objectSetOrKeyword[theOne]
			 .equals(objectSetOrKeyword[nextKeyword])) break;
		  theOne = theOne-1;
		   }
		}
		else {
		   theOne = groupStart;
		}

		// Remove all in the group except theOne;
			objectSetOrKeyword[newTotalLines] = objectSetOrKeyword[theOne];
			extractedString[newTotalLines] = extractedString[theOne];
			startPos[newTotalLines] = startPos[theOne];
			endPos[newTotalLines] = endPos[theOne];
//          confFactor[newTotalLines] = confFactor[theOne];
			keyword[newTotalLines] = keyword[theOne];

		// Set up for next iteration.
		groupStart = groupEnd+1;
		groupEnd = groupEnd+2;
	 }
	 else {
			objectSetOrKeyword[newTotalLines] = objectSetOrKeyword[groupStart];
			extractedString[newTotalLines] = extractedString[groupStart];
			startPos[newTotalLines] = startPos[groupStart];
			endPos[newTotalLines] = endPos[groupStart];
//          confFactor[newTotalLines] = confFactor[groupStart];
			keyword[newTotalLines] = keyword[groupStart];

		// Set up for next iteration.
		groupStart = groupEnd;
		groupEnd = groupEnd+1;
	 }
	 newTotalLines = newTotalLines+1;
	  }
	  totalLines = newTotalLines-1;

	  /*
	  System.out.println();
	  System.out.println("Table after resolving overlaps:");
	  for (int line = 1; line <= totalLines; line++) {
		 System.out.print(objectSetOrKeyword[line]);
	 System.out.print(", ");
		 System.out.print(extractedString[line]);
	 System.out.print(", ");
		 System.out.print(startPos[line]);
	 System.out.print(", ");
		 System.out.print(endPos[line]);
	 System.out.print(", ");
//       System.out.print(confFactor[line]);
//       System.out.print(", ");
		 System.out.print(keyword[line]);
	 System.out.println();
	  }
	  */

   }

   public void DensityPgm(String datText) {
	  // This computes the density of matched characters.
	  // It assumes that overlaps have been resolved, and then
	  // computes the density as (NrMatchedChars / TotNrChars).
	  // This heuristic also judges the applicability: if the
	  // density exceeds the THRESHOLD, it reports "YES" for
	  // applicability; otherwise it reports "NO".

	  final double THRESHOLD = 0.1;
	  double density;

//    System.out.println();
	  System.out.println("(* Density Information *)");

	  // Get total number of characters fact.
	  System.out.println();
	  System.out.print("Total Number of Characters: ");
	  System.out.println(datText.length());

	  // Get total number of matched characters
	  // and the average certainty of matched characters fact.
	  int totalMatchedChars = 0;
	  int totalMatchedFacts = 0;
//    double sumOfConfidenceFactors = 0.0;
//	   int lastCharPos = 0;
	  for (int line = 1; line <= totalLines; line++) {
	 totalMatchedChars = totalMatchedChars+endPos[line]-startPos[line]+1;
	 totalMatchedFacts = totalMatchedFacts+1;
//       sumOfConfidenceFactors = sumOfConfidenceFactors+confFactor[line];
	  }
	  System.out.println();
	  System.out.print("Number of Matched Characters: ");
	  System.out.println(Integer.toString(totalMatchedChars));
	  System.out.println();
	  System.out.print("Density: ");
	  density = (float)totalMatchedChars/(float)datText.length();
	  System.out.println(density);
//    System.out.println();
//    System.out.print("Average Matched Certainty: ");
//    System.out.println(
//       Double.toString(sumOfConfidenceFactors/totalMatchedFacts));

	  // Report applicability;
	  System.out.println();
	  System.out.print("Using a threshold of "+THRESHOLD+", \n"+
		 "the judged applicability according to the Density Heuristic is: ");
	  if (density > THRESHOLD) {
		 System.out.println("YES");
	  }
	  else {
		 System.out.println("NO");
	  }
   }

   public void getOrcFacts(String orcText) {
	  // This searches the .orc file looking for object sets associated
	  // with the specified object in a binary relationship set.
	  // For each object set it finds, it loads a table with the maximum
	  // number of occurrences (MAX_VALUE if *) from the dat table.
	  //
	  // We will eventually also want the expected number of occurrences.

//    System.out.println();
//    System.out.println("(* getOrcFacts *)");
	  String object;
	  int objectStartLoc = "Object: ".length();
	  int objectEndLoc;
	  int objectLoc;
	  int objSetStartLoc;
	  int objSetEndLoc;
	  int maxNrStartLoc;
	  int maxNrEndLoc;
//    int expNrStartLoc;
//    int expNrEndLoc;
	  int searchStartLoc;
//	  String extractedAttr;
//    String extractedExpNr;
//    double sumRecogPrecision;
//    int cntRecogPrecision;
//	  int objectSetLoc;
	  String probableObjSet;

	  // Get object of interest.
	  objectEndLoc = objectStartLoc;
	  while (orcText.charAt(objectEndLoc) != ';') {
		 objectEndLoc = objectEndLoc+1;
	  }
	  object = orcText.substring(objectStartLoc, objectEndLoc);

	  // For each object set attached to the object of interest,
	  // extract the max.
	  //
	  // (Later) and expected number of occurrences and
	  // compute the average recognition precision.
	  totalObjSets = 0;
	  searchStartLoc = objectEndLoc;
	  while (true) {
		 objectLoc = orcText.indexOf(object, searchStartLoc);
		 if (objectLoc == -1) break;
		 if (orcText.charAt(objectLoc-1) == '\n'
		   && orcText.charAt(objectLoc+object.length()) == ':') {
		   // Case 1: 1-max object set
		totalObjSets = totalObjSets+1;
		objSetStartLoc = objectLoc+object.length()+2; // 2 = ": "
		objSetEndLoc = objSetStartLoc;
		while (orcText.charAt(objSetEndLoc) != ' ') {
		   objSetEndLoc = objSetEndLoc+1;
		}
			objectSet[totalObjSets]
		   = orcText.substring(objSetStartLoc, objSetEndLoc);
		max[totalObjSets] = 1;
		/*
			expNrStartLoc = objSetEndLoc+7; // 7 = " [0:n ("
			expNrEndLoc = expNrStartLoc;
		while (orcText.charAt(expNrEndLoc) != ',') {
		   expNrEndLoc = expNrEndLoc+1;
		}
		expectedNr[totalObjSets]
		   = (Float.valueOf(orcText.substring(expNrStartLoc,
		   expNrEndLoc)).floatValue());
			sumRecogPrecision = 0.0;
			cntRecogPrecision = 0;
			for (int line = 1; line <= totalLines; line++) {
		   if (objectSet[totalObjSets].equals(objectSetOrKeyword[line])
				 && !keyword[line]) {
				  sumRecogPrecision = sumRecogPrecision+confFactor[line];
			  cntRecogPrecision = cntRecogPrecision+1;
		   }
		   if (cntRecogPrecision > 0) {
				  recogPrecision[totalObjSets]
			 = sumRecogPrecision/cntRecogPrecision;
		   }
		   else {
				  recogPrecision[totalObjSets] = 0.0;
		   }
		}
		*/
		 }
		 else if (orcText.charAt(objectLoc-1) == '\n') {
		// Case 2: >1-max object set, object 1st
		objSetStartLoc = objectLoc+object.length();
		objSetEndLoc = objSetStartLoc;
		while (orcText.charAt(objSetEndLoc) != ':') {
		   objSetEndLoc = objSetEndLoc+1;
		}
		probableObjSet
		   = orcText.substring(objSetStartLoc, objSetEndLoc);
			objSetStartLoc = orcText.indexOf(probableObjSet, objSetEndLoc);
		if (objSetStartLoc > 0) {
			   // The above determines whether the rel set is binary -- i.e.,
		   // determines whether there is at most one other obj-set name
		   // by checking for the probable object-set name in the
		   // relationship set declaration.

		   totalObjSets = totalObjSets+1;
			   objectSet[totalObjSets] = probableObjSet;

			   // Find the object name in the relationship set.
		   objectStartLoc = orcText.indexOf(object, objSetEndLoc);
		   objectEndLoc = objectStartLoc+object.length();

		   // Extract the max value.
			   maxNrStartLoc = objectEndLoc+1;
		   while (orcText.charAt(maxNrStartLoc) != ':') {
			  maxNrStartLoc = maxNrStartLoc+1;
		   }
		   maxNrStartLoc = maxNrStartLoc+1;
		   if (orcText.charAt(maxNrStartLoc) == '*') {
			  max[totalObjSets] = Integer.MAX_VALUE;
// not needed?    maxNrEndLoc = maxNrStartLoc+1;
		   }
		   else {
			  maxNrEndLoc = maxNrStartLoc;
		  while (orcText.charAt(maxNrEndLoc) != ']') {
			 maxNrEndLoc = maxNrEndLoc+1;
		  }
			  max[totalObjSets]
				 = (Integer.valueOf(orcText.substring(maxNrStartLoc,
				 maxNrEndLoc)).intValue());
		   }

		   /*
		   // Extract the expected value.
			   expNrStartLoc = maxNrEndLoc+2; // 2 = " ("
		   expNrEndLoc = expNrStartLoc;
		   while (orcText.charAt(expNrEndLoc) != ',') {
			  expNrEndLoc = expNrEndLoc+1;
		   }
		   expectedNr[totalObjSets]
			  = (Float.valueOf(orcText.substring(expNrStartLoc,
			  expNrEndLoc)).floatValue());

			   // Compute the recognition precision.
			   sumRecogPrecision = 0.0;
			   cntRecogPrecision = 0;
			   for (int line = 1; line <= totalLines; line++) {
			  if (objectSet[totalObjSets]
				.equals(objectSetOrKeyword[line])
					&& !keyword[line]) {
					 sumRecogPrecision = sumRecogPrecision+confFactor[line];
				 cntRecogPrecision = cntRecogPrecision+1;
			  }
		   }
		   if (cntRecogPrecision > 0) {
				  recogPrecision[totalObjSets]
				 = sumRecogPrecision/cntRecogPrecision;
		   }
		   else {
				  recogPrecision[totalObjSets] = 0.0;
		   }
		   */
		}
		else {
		   // not binary -- do nothing
		}
		 }
	 else if (orcText.charAt(objectLoc+object.length()) == ':') {
		// Case 3: >1-max object set, object 2nd
		objSetEndLoc = objectLoc;
			objSetStartLoc = objectLoc;
		while (orcText.charAt(objSetStartLoc-1) != '\n') {
		   objSetStartLoc = objSetStartLoc-1;
		}
		probableObjSet
		   = orcText.substring(objSetStartLoc, objSetEndLoc);
			objSetStartLoc = orcText.indexOf(probableObjSet, objSetEndLoc);
		if (objSetStartLoc > 0) {
			   // The above determines whether the rel set is binary -- i.e.,
		   // determines whether there is at most one other obj-set name
		   // by checking for the probable object-set name in the
		   // relationship set declaration.

		   totalObjSets = totalObjSets+1;
			   objectSet[totalObjSets] = probableObjSet;

			   // Find the object name in the relationship set.
		   objectStartLoc
			  = orcText.indexOf(object, objectLoc+object.length());
		   objectEndLoc = objectStartLoc+object.length();

		   // Extract the max value.  (From here, the code is as above.)
			   maxNrStartLoc = objectEndLoc+1;
		   while (orcText.charAt(maxNrStartLoc) != ':') {
			  maxNrStartLoc = maxNrStartLoc+1;
		   }
		   maxNrStartLoc = maxNrStartLoc+1;
		   if (orcText.charAt(maxNrStartLoc) == '*') {
			  max[totalObjSets] = Integer.MAX_VALUE;
// not needed?    maxNrEndLoc = maxNrStartLoc+1;
		   }
		   else {
			  maxNrEndLoc = maxNrStartLoc;
		  while (orcText.charAt(maxNrEndLoc) != ']') {
			 maxNrEndLoc = maxNrEndLoc+1;
		  }
			  max[totalObjSets]
				 = (Integer.valueOf(orcText.substring(maxNrStartLoc,
				 maxNrEndLoc)).intValue());
		   }

		   /*
		   // Extract the expected value.
			   expNrStartLoc = maxNrEndLoc+2; // 2 = " ("
		   expNrEndLoc = expNrStartLoc;
		   while (orcText.charAt(expNrEndLoc) != ',') {
			  expNrEndLoc = expNrEndLoc+1;
		   }
		   expectedNr[totalObjSets]
			  = (Float.valueOf(orcText.substring(expNrStartLoc,
			  expNrEndLoc)).floatValue());

			   // Compute the recognition precision.
			   sumRecogPrecision = 0.0;
			   cntRecogPrecision = 0;
			   for (int line = 1; line <= totalLines; line++) {
			  if (objectSet[totalObjSets]
				.equals(objectSetOrKeyword[line])
					&& !keyword[line]) {
					 sumRecogPrecision = sumRecogPrecision+confFactor[line];
				 cntRecogPrecision = cntRecogPrecision+1;
			  }
		   }
		   if (cntRecogPrecision > 0) {
				  recogPrecision[totalObjSets]
				 = sumRecogPrecision/cntRecogPrecision;
		   }
		   else {
				  recogPrecision[totalObjSets] = 0.0;
		   }
		   */
		}
		else {
		   // not binary -- do nothing
		}
		 }
		 else {
		// Case 4: object between; ternary -- ignore
		// ternary -- ignore this case
	 }

	 // get set up for next iteration
	 searchStartLoc = objectLoc;
	 while (orcText.charAt(searchStartLoc) != ';') {
		searchStartLoc = searchStartLoc+1;
	 }
	  }
   }

   public void NrValuesPgm() {
	  // Get the expected number of values for every object set
	  // attached to the object of interest for the record.

	  int NrOccurrences;

	  System.out.println();
	  System.out.println();
	  System.out.println("(* Right-Nr-of-Values Information *)");
	  System.out.println();
	  System.out.println("\"one\" Object Sets and Number of Occurrences");
//       +"Probability of Occurrence & Recognition Precision");
	  for (int objSet = 1; objSet <= totalObjSets; objSet++) {
	 if (max[objSet] == 1) {
			System.out.print(objectSet[objSet]);
		System.out.print(", ");
//          System.out.print(expectedNr[objSet]);
//          System.out.print(", ");
//          System.out.println(recogPrecision[objSet]);
			NrOccurrences = 0;
		for (int line = 1; line <= totalLines; line++) {
		   if (objectSet[objSet].equals(objectSetOrKeyword[line])
				 && keyword[line] == false) {
			  NrOccurrences = NrOccurrences+1;
		   }
		}
		System.out.println(NrOccurrences);
	 }
	  }
	  System.out.println();
	  System.out.println("\"many\" Object Sets and Number of Occurences");
//       +"Probability of Occurrence & Recognition Precision");
	  for (int objSet = 1; objSet <= totalObjSets; objSet++) {
	 if (max[objSet] != 1) {
			System.out.print(objectSet[objSet]);
		System.out.print(", ");
//          System.out.print(expectedNr[objSet]);
//          System.out.print(", ");
//          System.out.println(recogPrecision[objSet]);
			NrOccurrences = 0;
		for (int line = 1; line <= totalLines; line++) {
		   if (objectSet[objSet].equals(objectSetOrKeyword[line])
				 && keyword[line] == false) {
			  NrOccurrences = NrOccurrences+1;
		   }
		}
		System.out.println(NrOccurrences);
	 }
	  }
   }

   public void RecGrpPgm() {
	  // Count the number of one-max related object sets (=n).
	  //
	  // (Later) Sum the
	  // expected numbers for these object sets and take the floor (=m).
	  // For now m = n.
	  //
	  // If m < 2, this heuristic is not applicable.
	  // Divide the sequence of one-max object sets into groups of m,
	  // discarding the last group if not m object sets.
	  // Count the number of DISTINCT object sets in each set.
	  System.out.println();
	  System.out.println();
	  System.out.println("(* Record-Group Information *)");
	  System.out.println();

	  int totalOneMaxObjSets;  // n, above
//    float expectedOneMaxObjSets;
//    int nrExpectedOneMaxObjSets;  // m, above
	  int totalNrGroups;
	  int groupStart;
	  int objSetNr;
	  int objSetDistinctCounter;
	  int nrConsideredForGroup;
	  String oneMaxObjSets[] = new String[50];
	  int groupSetSize[] = new int[50];
	  String currentGroup[] = new String[50];

	  totalOneMaxObjSets = 0;
//    expectedOneMaxObjSets = (float)0;
	  for (int objSet = 1; objSet <= totalObjSets; objSet++) {
	 if (max[objSet] == 1) {
	 totalOneMaxObjSets = totalOneMaxObjSets+1;
	 oneMaxObjSets[totalOneMaxObjSets] = objectSet[objSet];
//       expectedOneMaxObjSets = expectedOneMaxObjSets+expectedNr[objSet];
		 }
	  }
	  /*
	  nrExpectedOneMaxObjSets = (int)expectedOneMaxObjSets;
	  if (nrExpectedOneMaxObjSets < 2) {
		 System.out.println("This heuristic is not applicable: "
		+"< 2 expected \"one-max\" object sets");
	  }
	  */
	  if (totalOneMaxObjSets < 2) {
		 System.out.println("This heuristic is not applicable: "
		+"< 2 \"one-max\" object sets");
	  }
	  else {
	 totalNrGroups = 0;
		 groupStart = 1;
	 objSetNr = groupStart;
	 outer: while (groupStart <= totalLines) {
		objSetDistinctCounter = 0;
		nrConsideredForGroup = 0;
//       while (nrConsideredForGroup < nrExpectedOneMaxObjSets) {
		 while (nrConsideredForGroup < totalOneMaxObjSets) {
		   if (objSetNr > totalLines) {
			  // Last group partial -- ignore the group
		  break outer;
		   }
		   else {
			  if (!keyword[objSetNr]) {

					 // Determine whether objectSetOrKeyword[objSetNr] is
			 // on the list of "one-max" object sets.
					 int i = 1;
			 while (i <= totalOneMaxObjSets) {
			if (objectSetOrKeyword[objSetNr]
				  .equals(oneMaxObjSets[i])) {
			   System.out.print
				  (objectSetOrKeyword[objSetNr]);
			   System.out.println
				  (": "+extractedString[objSetNr]);
			   break;
				}
			i = i+1;
			 }
			 if (i <= totalOneMaxObjSets /* it's on the list */) {

			// Determine if it is in the current group.
				int j = 1;
			while (j <= objSetDistinctCounter) {
			   if (objectSetOrKeyword[objSetNr]
					 .equals(currentGroup[j])) {
				  break;
			   }
			   else {
				  j = j+1;
			   }
			}
			if (j > objSetDistinctCounter /* it is not */ ) {

			   // Put it in the current group.
			   objSetDistinctCounter = objSetDistinctCounter+1;
			   currentGroup[objSetDistinctCounter]
				  = objectSetOrKeyword[objSetNr];
			}
					nrConsideredForGroup = nrConsideredForGroup+1;
			 }
		  }
		   }
		   objSetNr = objSetNr+1;
		}
		// Initialize for next group.
			groupStart = objSetNr;
		objSetNr = groupStart;

		totalNrGroups = totalNrGroups+1;
		groupSetSize[totalNrGroups] = objSetDistinctCounter;

		System.out.print("------ Nr of Distinct \"One Max\" Object Sets: ");
		System.out.println(groupSetSize[totalNrGroups]);
		System.out.println();
	 }
	  }
   }

   public static void main(String args[]) {

	  if(args.length < 2) {
		 System.out.println("Usage: java PgmGen2 <.dat file> <.orc file>");
	  }
	  else {
		 // Initialize
		 PgmGen2 gen
		= new PgmGen2(new File(args[0]), new File(args[1]));

	 // Get dat facts.
	 gen.getDatFacts(gen.datFileText);

	 // Resolve the overlaps.
	 gen.ResolveOverlaps();

	 // Generate the Density Information.
	 gen.DensityPgm(gen.datFileText);

	 // Get orc facts.
	 gen.getOrcFacts(gen.orcFileText);

	 // Generate the Right-Nr-of-Values Information
	 gen.NrValuesPgm();

	 // Generate the PgmGen2 Record-Group Information
	 gen.RecGrpPgm();
	  }
   }
}
