/**
 * class recordBoundDiscovery This class is written for discoverying record
 * boundary. Usage: java recordSeparator <filename> <dataRecordTable>
 * <separatorListFile> [<outputFile>] or java recordSeparator <URL>
 * <dataRecordTable> <separatorListFile> [<outputFile>] Examples: java
 * recordSeparator http://students.cs.byu.edu/~jiang/obits320.html obits320.dat
 * ist.txt java recordSeparator ~jiang/thesis/star318.html obits320.dat ist.txt
 *
 * @version   1.2
 * @author    Yuan Jiang @date: 3-5-1998
 */

package edu.byu.deg.records;


import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.Document;
import com.sun.xml.tree.XmlDocument;


/**
 * Description of the Class
 *
 * @author    tim
 * @created   July 29, 2002
 */
public class RecordSeparator
{
	/**
	 * The main program for the RecordSeparator class
	 *
	 * @param args  The command line arguments
	 */
	public static void main(String[] args)
	{
		//args[0] = "d:\\test\\software_hr.html";
		//args[0] = "d:\\test\\software.html";
		//args[0] = "d:\\cvs\\demo\\user_data\\customhtml\\st_ebay.car.htm";
		//args[1] = "none";
		//args[2] = "d:\\test\\records.xml";
		//args[3] = "http://www.thechartrands.com/index.html";


		/*
		 *  for (int i = 0; i < args.length; i++)
		 *  {
		 *  System.err.println(i + " --> " + args[i]);
		 *  }
		 */
		/*
		 *  Check input
		 *  Build TagTree
		 *  Find largest-fan-out sub-tree
		 */
		TTree tt = null;
		//      File out_file = null;
		//     FileWriter outf = null;

		if (args.length < 2)
		{
			System.out.println("Usage: java RecordSeparator <html filename|url> <dataRecordTable> <outFileName.(txt|xml)> [<pageURI(for RDF)>]");
			System.out.println("\tNote: output is xml unless the output file uses the .txt extension");
			System.out.println("\tNote: If a URI is specified as the last parameter, it will be embedded into the generated XML document for use in the RDF viewer");

			/* SAMPLE COMMAND LINE:
			 * java edu.byu.deg.records.RecordSeparator d:\test\software.html
			 * none
			 * d:\test\records.xml
			 */
		}
		else
		{
			if (args[0].indexOf("://") > 0)
			{
				try
				{
					tt = new TTree(new URL(args[0]));
				}
				catch (IOException e)
				{
					System.err.println("Cannot open URL.");
					System.exit(1);
				}
			}
			else if (args[0] != null)
			{
				try
				{
					tt = new TTree(new File(args[0]));
				}
				catch (Exception e)
				{
					System.err.println("File not found.");
					System.exit(1);
				}
			}


			// Print out the tag-tree
			//System.out.println("\n\nPrint out the tag-tree:\n");
			//tt.printT();

			TTree sub = new TTree();

			sub = tt.findBiggestBranch();
			String tg = sub.current.getTag();
			tg = tg.substring(tg.indexOf("<")+1, tg.indexOf(">"));
			System.out.println("Biggest Branch: "+tg);

			// Print out the subtree with the largest fanout
			// System.out.println("\n\nPrint out the subtree with the largest fanout:\n");
			//	 sub.printT();

			PreProcessTags pt = new PreProcessTags(sub);
			pt.eliminateIrrelevantTags(2);

			Vector tl = pt.get_list();

			/*
			 *  System.out.println("\n-------------------------------------------\n\n");
			 */
			/*
			 *  Apply five invidual heuristics
			 */
			/*
			 *  System.out.println("\n Start OM \n");
			 */
			Vector oml = null;
			File record_table_file = new File(args[1]);
			if (record_table_file.exists())
			{
				ApplyOM om = new ApplyOM(sub, new File(args[1]));
				oml = om.find_candidate();
			}
			else
			{
				/*
				 *  In case the file doesn't exist, make a dummy vector to send into ApplyCH
				 */
				oml = new Vector();
			}

			/*
			 *  System.out.println("\n Start RP \n");
			 */
			ApplyRP rp = new ApplyRP(sub);
			Vector rpl = rp.find_candidate();

			/*
			 *  System.out.println("\n Start SD \n");
			 */
			ApplySD sd = new ApplySD(sub);
			Vector sdl = sd.find_candidate();

			/*
			 *  System.out.println("\n Start IT \n");
			 */
			Vector ist = new Vector();
			ist.add("<hr>");
			ist.add("<td>");
			ist.add("<tr>");
			ist.add("<table>");
			ist.add("<a>");
			ist.add("<p>");
			ist.add("<br>");
			ist.add("<h4>");
			ist.add("<h1>");
			ist.add("<strong>");
			ist.add("<b>");
			ApplyIT it = new ApplyIT(tl, ist);
			Vector itl = it.find_candidate();
			// RetrieveRecord rtr = new RetrieveRecord();
			String rectag = new String();

			/*
			 *  System.out.println("\n Start HT \n");
			 */
			ApplyMA ht = new ApplyMA(tl);
			Vector htl = ht.find_candidate();

			/*
			 *  System.out.println("\n\n -------------------------------\n\n");
			 */
			/*
			 *  Apply Combined heuristic
			 */
			ApplyCH compound = new ApplyCH(tl, oml, rpl, sdl, itl, htl);

			Vector candidate = compound.find_candidate();

			/*
			 *  System.out.println("\n\n -------------------------------\n\n");
			 */
			rectag = ((GTD) candidate.elementAt(0)).str;
			//         System.out.println("Record Separator: " +
			//                           rectag + " " + rectag.length());



			Document xml_doc =  sub.current.getDomNode().getOwnerDocument();
			String container_xpath = DOMSeparator.getXPath(sub.current.getDomNode());
			// remove the < > from the tag
			String tag = rectag.substring(1, rectag.length()-1);
			System.out.println("TAG FOUND: "+tag);
			DOMSeparator separator = new DOMSeparator(xml_doc, tag, container_xpath);


			try{
				if (args.length >= 3)
				{
					if (args.length >= 4)
					{
						separator.setInputDocURI(args[3]);
					}
					FileWriter outfile = new FileWriter(args[2]);
					if (args[2].indexOf(".txt") != -1)
					{
						String records = separator.getTextRecords();
						outfile.write(records);
						outfile.flush();
						outfile.close();
						//System.out.println(records);
					}
					else
					{
						XmlDocument xml_records = separator.getXMLRecords();
						xml_records.write(new FileOutputStream(args[2]));
						//xml_records.write(System.out);
					}
				}
				else
				{
					XmlDocument xml_records = separator.getXMLRecords();
					xml_records.write(System.out);
				}
			}catch (Exception e){e.printStackTrace();}
		}
	}
}
