package edu.byu.deg.records;

import java.util.*;
import java.io.*;
import java.net.*;

import org.w3c.dom.Node;

/**
	* This class takes a Web document as INPUT
	* Use the recordboundary tag as reference
	* Remove all the tags.
	* OUTPUT only the plain text for the record
	* bounded by the recordboundary tag and insert
	* 5 '#' signs between each record text.
	*
	* @version 1.0.0
	* @author Sai Ho Yau (Tony)
	*/
class Separator {
	String text = new String();
	String rectag = new String();

	HashMap tag_replacements = new HashMap();

	Node domNode;
	Separator(Node domNode)
	{
		this.domNode = domNode;
	}

	/**
	* Constructor
	*/
	Separator() {
		text = new String();
		build_tag_replacements();
	}

	/**
	* Constructor
	* @param hypertext
	*/
	Separator(String hypertext, String rtag) {
		this();
		text = hypertext;
		rectag = rtag;
	}

	/**
	* Constructor
	* @param filename
	*/
	Separator(File filename, String rtag) {
		this();
		rectag = rtag;
		try {
			InputStream in = new FileInputStream(filename);
			int i = in.read();

			while (i != -1) {
				if(!(i<=14)) {	// huh?	what's this for?
					text += (char)i;
				}
				else if ((i == 10) || (i == 13))
				{
					text += " ";
				}
				i = in.read();
			}
			int ctlidx = text.indexOf("&nbsp;");	// replace &nbsp; with ' '.	TODO:	other ctl codes
			while (ctlidx != -1)
			{
				String temp = text.substring(0,ctlidx);
				temp = temp + " " + text.substring(ctlidx + 6);
				text = temp;
				ctlidx = text.indexOf("&nbsp;");
			}
		} catch (IOException io) {
			System.out.println(io.getMessage());
		}
	}

	/**
	* Constructor
	* @param url
	*/
	Separator(URL url, String rtag) {
		this();
		rectag = rtag;
		try {
			InputStream in = url.openStream();
			int i = in.read();

			while (i != -1) {
				if(!(i<=14)) {	// huh?	what's this for?
					text += (char)i;
				}
				else if ((i == 10) || (i == 13))
				{
					text += " ";
				}
				i = in.read();
			}
			int ctlidx = text.indexOf("&nbsp;");	// replace &nbsp; with ' '.	TODO:	other ctl codes
			while (ctlidx != -1)
			{
				String temp = text.substring(0,ctlidx);
				temp = temp + " " + text.substring(ctlidx + 6);
				text = temp;
				ctlidx = text.indexOf("&nbsp;");
			}
		} catch (IOException io) {
			System.out.println(io.getMessage());
		}
	}

	private void build_tag_replacements()
	{
		tag_replacements.put("br",		"\r\n");
		tag_replacements.put("tr",		"\r\n");
		tag_replacements.put("address",		" ");
		tag_replacements.put("applet",		" ");
		tag_replacements.put("blockquote",	"\r\n");
		tag_replacements.put("br",		"\r\n");
		tag_replacements.put("button",		" ");
		tag_replacements.put("caption",		" ");
		tag_replacements.put("center",		" ");
		tag_replacements.put("cite",		" ");
		tag_replacements.put("code",		" ");
		tag_replacements.put("dd",		" ");
		tag_replacements.put("del",		" ");
		tag_replacements.put("dfn",		" ");
		tag_replacements.put("dir",		" ");
		tag_replacements.put("div",		"\r\n");
		tag_replacements.put("dl",		"\r\n");
		tag_replacements.put("dt",		" ");
		tag_replacements.put("fieldset",	" ");
		tag_replacements.put("h1",		"\r\n");
		tag_replacements.put("h2",		"\r\n");
		tag_replacements.put("h3",		"\r\n");
		tag_replacements.put("h4",		"\r\n");
		tag_replacements.put("h5",		"\r\n");
		tag_replacements.put("h6",		"\r\n");
		tag_replacements.put("hr",		"\r\n");
		tag_replacements.put("iframe",		" ");
		tag_replacements.put("img",		" ");
		tag_replacements.put("input",		" ");
		tag_replacements.put("li",		"\r\n");
		tag_replacements.put("object",		" ");
		tag_replacements.put("ol",		"\r\n");
		tag_replacements.put("p",		"\r\n");
		tag_replacements.put("pre",		"\r\n");
		tag_replacements.put("samp",		" ");
		tag_replacements.put("select",		" ");
		tag_replacements.put("table",		"\r\n");
		tag_replacements.put("td",		" ");
		tag_replacements.put("th",		" ");
		tag_replacements.put("tr",		"\r\n");
		tag_replacements.put("textarea",	"\r\n");
		tag_replacements.put("ul",		"\r\n");
	}

	/**
	*	get PlainText
	*/
	public String getRecordText() {
		String plain = new String();
		String lctext = new String();
		String newRecText = new String();
		String tag;
		String tag_name;
		String place_holder;	//if a br tag for example is removed, replace it with \r\n

		int start_tag_pos = 0;
		int end_tag_pos = 0;
		int start_rec_pos = 0;
		int end_rec_pos = 0;
		int startcount = 0;


		lctext = text.toLowerCase();
		do {
			end_rec_pos = -1;
			if(startcount == 0) {
				start_rec_pos = lctext.indexOf(rectag, start_rec_pos);
			}
			startcount++;

			if(start_rec_pos != -1) {
				end_rec_pos = lctext.indexOf(rectag, start_rec_pos+rectag.length());
			}
			else break;


			if(end_rec_pos != -1) {
				if(startcount > 1) {
					//newRecText = newRecText.append("\n\n#####\n");
					newRecText = newRecText.concat("\n\n#####\n\n");
				}
				plain = text.substring(start_rec_pos+rectag.length(), end_rec_pos);
			} else break;

			start_tag_pos = 0;

			do
			{
				end_tag_pos = -1;

				start_tag_pos = plain.indexOf("<", start_tag_pos);
				if(start_tag_pos != -1)
				{
					end_tag_pos = plain.indexOf(">", start_tag_pos);
				}
				else break;

				if(end_tag_pos >= end_rec_pos) break;

				if((end_tag_pos != (start_tag_pos+1)) || (end_tag_pos != -1))
				{
					if(plain.charAt(start_tag_pos+1) <= 14 ||
						plain.charAt(start_tag_pos+1) == ' ')
					{
							start_tag_pos++;
					}
					else
					{
						if(start_tag_pos != 0)
						{
							//We need to remove the html tag from the record, but
							//we may need to replace it with some white space character
							//if its a tag that would move its contents around
							//on the page

							tag = plain.substring(start_tag_pos, end_tag_pos);
							int slash	= tag.indexOf('/');
							int space	= tag.indexOf(' ');
							int start	= 1;
							int end		= tag.length();


							if (slash == 1) //it's a normal close-tag
							{
								start = 2;
								if (space != -1)
									end = Math.min(end, space);
							}
							else
							{
								if (slash != -1) //it's a self-closing tag
									end = slash;
								if (space != -1)
									end = Math.min(end, space);
							}

							tag_name = tag.substring(start, end);
							//System.out.println(tag + " --> " + tag_name);
							place_holder = (String)tag_replacements.get(tag_name.toLowerCase());
							if (place_holder == null)
								place_holder = "";
							plain = plain.substring(0, start_tag_pos) + place_holder + plain.substring(end_tag_pos+1);
						}
						else
						{
							plain = plain.substring(end_tag_pos+1);
							start_tag_pos = 0;
						}
					}
				}
			} while ((start_tag_pos != -1) || (end_tag_pos != -1));

			newRecText = newRecText.concat(plain.trim());
			start_rec_pos = end_rec_pos;
		} while (end_rec_pos != -1);
		return newRecText;
	}

	public static void main(String args[]) {

		RetrieveRecord rtr = new RetrieveRecord();

		if(args.length <= 1) {
			System.out.println("Usage: java RetrieveRecord [filename] ['<tag>']");
			System.out.println("		OR");
			System.out.println("Usage: java RetrieveRecord [url] ['<tag>']");
		}
		else {
			// Read the file
			if(args[0].indexOf("://")>0) {
				try {
					rtr = new RetrieveRecord(new URL(args[0]), args[1]);
				}
				catch (MalformedURLException e) {
					System.out.println("MalformedURL...check the input parameter.");
				}
			}
			else {
				rtr = new RetrieveRecord(new File(args[0]), args[1]);
			}

			// Print out plain text
			System.out.println(rtr.getRecordText());
		}
	}
}



