package edu.byu.deg.records;

import java.util.*;
import java.io.*;
import java.net.*;

import org.w3c.dom.Node;
import org.w3c.tidy.Tidy;


/**
  * This class take a HTML/XML document as an input
  * build a tag tree for that document
  *
  * @version 1.0.1b
  * @author  Yuan Jiang
  */
class TTree {
   TTNode current = new TTNode();
   int current_level = 0;
   Vector children = new Vector();

   /**
	* Constructor
	*/
   TTree() {
	  current = new TTNode();
	  children = new Vector();
	  current_level = 0;
   }



   TTree(File file)
   {
	   this();
	   try {
		   BufferedReader br = new BufferedReader(new FileReader(file));
		   StringBuffer sb = new StringBuffer();
		   String line;
		   while ((line = br.readLine()) != null)
				sb.append(line+"\r\n");
		   br.close();
		   //System.out.println(sb.toString());
		   StringBufferInputStream in = new StringBufferInputStream(sb.toString());

		   file.delete();
		   file.createNewFile();
		   FileOutputStream out = new FileOutputStream(file);
		   Tidy tidy = new Tidy();
		   tidy.setShowWarnings(false);
		   tidy.setMakeClean(false);
		   tidy.setXmlTags(false);
		   tidy.setWraplen(0);
		   Node doc = tidy.parseDOM(in, out);

		   TTree tmp = createTree(doc);
		   this.children  = tmp.children;
		   this.current   = tmp.current;
	   } catch (IOException io) {
		   System.out.println(io.getMessage());
	   }
   }

   TTree(String hypertext)
   {
	   this();
	   StringBufferInputStream in = new StringBufferInputStream(hypertext);
	   Tidy tidy = new Tidy();
	   tidy.setShowWarnings(false);
	   tidy.setMakeClean(false);
	   tidy.setXmlTags(false);
	   Node doc = tidy.parseDOM(in, null);

	   TTree tmp = createTree(doc);
	   this.children  = tmp.children;
	   this.current   = tmp.current;
   }


   TTree(URL url)
   {
	   this();
	   try {
		   InputStream in = url.openStream();
		   Tidy tidy = new Tidy();
		   tidy.setShowWarnings(false);
		   tidy.setMakeClean(false);
		   tidy.setXmlTags(false);
		   Node doc = tidy.parseDOM(in, null);

		   TTree tmp = createTree(doc);
		   this.children  = tmp.children;
		   this.current   = tmp.current;
	   } catch (IOException io) {
		   System.out.println(io.getMessage());
	   }
   }

   private TTree createTree(Node domNode)
   {
		TTree subTree = new TTree();
		if (domNode.getNodeType() == Node.DOCUMENT_NODE)
			subTree.current.setTagName("root");
		else if (domNode.getNodeType() == Node.ELEMENT_NODE)
			subTree.current.setTagName(domNode.getNodeName());
		subTree.current.setDomNode(domNode);

		for (int i = 0; i < domNode.getChildNodes().getLength(); i++)
		{
			Node child = domNode.getChildNodes().item(i);
			if (child.getNodeType() == Node.ELEMENT_NODE)
				subTree.children.add(createTree(child));
		}
		return subTree;
   }


   /**
	*  find subtree with largest fanout
	*/
   public TTree findBiggestBranch() {
	  int[] num_of_children = {0};

	  TTree branch = new TTree();

	  branch = this;

	  int sz = children.size();
	  num_of_children[0] = sz;

	  int count = 0;
	  while(count < sz && children != null) {
		TTree branch2 = findBiggestBranchAux((TTree)children.elementAt(count),
									 branch, num_of_children);

		if(branch2.children.size() >= branch.children.size()) {
		   branch = branch2;
		}
		count++;
	  }
	  return branch;
   }


   /**
	*  Aux func for TagTree findBiggestBranch()
	*/
   public TTree findBiggestBranchAux(TTree sub, TTree bra, int[] num) {
	  int count = 0;
	  int sz = sub.children.size();
	  TTree branch = new TTree();

	  if (sz >= num[0]) {
		 num[0] = sz;
		 branch = sub;
	  }

	  while(count < sz && sub.children != null) {
		 TTree branch2 =
			findBiggestBranchAux((TTree)sub.children.elementAt(count),
								 branch, num);

		 if(branch2.children.size() >= branch.children.size()) {
		   branch = branch2;
		 }
		 count++;
	  }
	  return branch;
   }

   /**
	*   print out the tree structure
	*/
   public void printT() {
	  int count = 0;
	  int level = 0;

	  System.out.println("********************Print Tree*********************");

	  current.printTag();

	  int sz = children.size();
	  while (count < sz && children != null) {
		 for(int i=0; i < (level + 1); i++) {
			System.out.print("\t");
		 }

		 printTAux((TTree)children.elementAt(count), level+1);
		 count++;
	  }
	  System.out.println("********************End Tree*********************");
   }


   /**
	*  print out the subtree structure
	*/
   public void printTAux(TTree sub, int level) {
	  int count = 0;
	  sub.current.printTag();
	  int sz = sub.children.size();

	  while (count < sz && sub.children != null) {
		 for(int i=0; i < (level + 1); i++) {
			System.out.print("\t");
		 }
		 printTAux((TTree)sub.children.elementAt(count), level+1);
		 count++;
	  }
   }



   /**
	*  print out the TagTree
	*/
   public void printTree() {
	  int count = 0;
	  int level = 0;

	  System.out.println("********************Print Tree*********************");
	  current.printNode();

	  int sz = children.size();
	  while (count < sz && children != null) {

		 System.out.println("Level " + (level+1) + " : \tParent: "
							+ current.getTag()
							+ "\t * Child No " + count + " *");
		 printSubtree((TTree)children.elementAt(count), level+1);
		 count++;
	  }
	  System.out.println("********************End Tree*********************");
   }


   /**
	*  print out the subtree
	*/
   public void printSubtree(TTree sub, int level) {
	  int count = 0;
	  sub.current.printNode();
	  int sz = sub.children.size();

	  while (count < sz && sub.children != null) {

		 System.out.println("Level " + (level+1) + " : \tParent: "
							+ sub.current.getTag()
							+ "\t * Child No " + count + " *");
		 printSubtree((TTree)sub.children.elementAt(count), level+1);
		 count++;
	  }

   }

   /**
	*   get the size of the TagTree
	*/
   public long size() {

	  long treesize = 0;
	  int count = 0;

	  treesize += current.size();

	  int sz = children.size();
	  while (count < sz && children != null) {
		 treesize += sizeSubtree((TTree)children.elementAt(count));
		 count++;
	  }
	  return treesize;
   }


   /**
	*  get the size of the subtree
	*/
   public long sizeSubtree(TTree sub) {

	  long treesize = 0;
	  int count = 0;

	  treesize += sub.current.size();

	  int sz = sub.children.size();

	  while (count < sz && sub.children != null) {
		 treesize += sizeSubtree((TTree)sub.children.elementAt(count));
		 count++;
	  }
	  return treesize;
   }


   /**
	*   get plain text
	*/
   public String getPlainText() {

	  String plain = new String();
	  int count = 0;

	  plain += current.getRtext();
	  plain += current.getItext();

	  int sz = children.size();

	  while (count < sz && children != null) {
		 plain += getPlainTextAux((TTree)children.elementAt(count));
		 count++;
	  }
	  plain += current.getOtext();
	  return plain;
   }

   /**
	*  get plain text of subtrees
	*/
   public String getPlainTextAux(TTree sub) {

	  String plain = new String();
	  int count = 0;

	  plain += sub.current.getItext();
	  int sz = sub.children.size();

	  while (count < sz && sub.children != null) {
		 plain += getPlainTextAux((TTree)sub.children.elementAt(count));
		 count++;
	  }
	  plain += sub.current.getOtext();
	  return plain;
   }



   /**
	*  print out plaintext
	*/
   public void printPlainText() {
	  System.out.println(getPlainText());
   }

   /**
	*   get hypertext
	*/
   public String getHypertext() {
	  //return current.getHypertext();
	  String hyper = new String();
	  int count = 0;

	  hyper += current.getRtext();
	  if(!(current.getStartag().equals("<root>"))) {
		 hyper += current.getStartag();
	  }
	  hyper += current.getItext();

	  int sz = children.size();

	  while (count < sz && children != null) {
		 hyper += getHypertextAux((TTree)children.elementAt(count));
		 count++;
	  }
	  if(!(current.getEndtag().equals("</root>"))) {
		 hyper += current.getEndtag();
	  }
	  hyper += current.getOtext();
	  return hyper;
   }

   /**
	*  get hypertext of subtrees
	*/
   public String getHypertextAux(TTree sub) {

	  String hyper = new String();
	  int count = 0;

	  hyper += sub.current.getStartag();
	  hyper += sub.current.getItext();
	  int sz = sub.children.size();

	  while (count < sz && sub.children != null) {
		 hyper += getHypertextAux((TTree)sub.children.elementAt(count));
		 count++;
	  }
	  hyper += sub.current.getEndtag();
	  hyper += sub.current.getOtext();
	  return hyper;
   }

   /**
	*  print out hypertext
	*/
   public void printHypertext() {
	  System.out.println(getHypertext());
   }

   /**
	*  get the number of descendants of TTree
	*/
   public int getNumOfChildren() {
	  return children.size();
   }

}

class TTNode {
   protected String startag = new String();
   protected String endtag = new String();
   protected String rtext = new String();
   protected String itext = new String();
   protected String otext = new String();
   private Node domNode = null;

   TTNode() {
   }

   TTNode(String txt) {
	  rtext = new String(txt);
   }

   TTNode(String tok, String txt) {
	  startag = new String(tok);
	  otext = new String(txt);
   }

   TTNode(String stok, String etok, String txt) {
	  startag = new String(stok);
	  endtag = new String(etok);
	  itext = new String(txt);
   }

   TTNode(String stok, String etok, String itxt, String otxt) {
	  startag = new String(stok);
	  endtag = new String(etok);
	  itext = new String(itxt);
	  otext = new String(otxt);
   }

   TTNode(String stok, String etok, String itxt, String otxt, String rtxt) {
	  startag = new String(stok);
	  endtag = new String(etok);
	  itext = new String(itxt);
	  otext = new String(otxt);
	  rtext = new String(rtxt);
   }

   TTNode(TTNode ttnode) {
	  startag = new String(ttnode.startag);
	  endtag = new String(ttnode.endtag);
	  rtext = new String(ttnode.rtext);
	  itext = new String(ttnode.itext);
	  otext = new String(ttnode.otext);
   }

   final String getHypertext() {
	  if(startag.equals("<root>")) {
		 return (rtext);
	  }
	  else {
	  return (rtext + startag + itext + endtag + otext);
	  }
   }

   final String getText() {
	  return (rtext + itext + otext);
   }


   final String getStartag() {
	  return startag;
   }

   final String getEndtag() {
	  return endtag;
   }

   final String getRtext() {
	  return rtext;
   }

   final String getItext() {
	  return itext;
   }
   final String getOtext() {
	  return otext;
   }

	public void setTagName(String name)
	{
		this.startag = "<"+name+">";
		this.endtag  = "</"+name+">";
	}

	public void setDomNode(Node node)
	{
		this.domNode = node;
	}

	public Node getDomNode()
	{
		return this.domNode;
	}

   final String getTag() {
	  StringTokenizer subtok = new StringTokenizer(startag," ",true);
	  String tag = subtok.nextToken();
	  if (!tag.endsWith(">"))
		 tag += ">";
	  tag = tag.toLowerCase();
	  return tag;
   }

   final long size() {
	  return ((long)(getHypertext().length() * 2));
   }

   void printTag() {
	  System.out.println(getTag());
   }

   void printTagList() {
	  System.out.println(startag + " " + endtag);
   }

   void printNode() {
	  System.out.println("%%%%%%%%%%% Node %%%%%%%%%%%%%%%%%");
	  System.out.print("Print Tags: \n\t");
	  System.out.println(startag + " " + endtag);
	  System.out.println("\nPrint Text: \n" + getText());
	  System.out.println("%%%%%%%%%%%  End  %%%%%%%%%%%%%%%%%\n");
   }
}
