/**
 * class recordBoundDiscovery
 *   This class is written for discoverying record boundary.
 *
 * Usage: java recordBoundDiscovery <filename> <dataRecordTable> <separatorListFile> [<outputFile>] or
 *        java recordBoundDiscovery <URL> <dataRecordTable> <separatorListFile> [<outputFile>]
 *
 * Examples: java recordBoundDiscovery http://students.cs.byu.edu/~jiang/obits320.html obits320.dat ist.txt
 *           java recordBoundDiscovery ~jiang/thesis/star318.html obits320.dat ist.txt
 *
 * @version 1.2
 * @author  Yuan Jiang
 * @date:   3-5-1998
 *
 */


package edu.byu.deg.records;


import java.io.*;
import java.util.*;
import java.net.*;

public class recordBoundDiscovery
{
	public static void main (String[] args)
	{

		/*
		for (int i = 0; i < args.length; i++)
		{
			System.err.println(i + " --> " + args[i]);
		}
		*/

		/* Check input
		Build TagTree
		Find largest-fan-out sub-tree */

		TTree tt = new TTree();
		//      File out_file = null;
		//     FileWriter outf = null;

		if(args.length < 3)
		{
			System.out.println("Usage: java recordBoundDiscovery <filename> <dataRecordTable> <separatorListFile> [<outputFile>]");
			System.out.println("          OR                  ");
			System.out.println("Usage: java recordBoundDiscovery <url> <dataRecordTable> <separatorListFile> [<outputFile>]");
		}
		else
		{
			if(args[0].indexOf("://")>0)
			{
				try
				{
					tt = new TTree (new URL(args[0]));
				} catch (IOException e) {
					System.out.println("Cannot open URL.");
				}
			}
			else if(args[0] != null)
			{
				try {
					tt = new TTree(new File(args[0]));
					} catch (Exception e) {
					System.out.println("File not found.");
				}
			}

			File out_file = null;
			FileWriter outf = null;

			if(args.length < 4)
			{
				out_file = new File("recfile.txt");
			} else
			{
				out_file = new File(args[3]);
			}


			try {
				outf = new FileWriter(out_file);
			} catch (IOException e) {
				System.out.println("Bad Out File"+args[3]);
			}


			// Print out the tag-tree
			//System.out.println("\n\nPrint out the tag-tree:\n");
			//tt.printT();

			TTree sub = new TTree();

			sub = tt.findBiggestBranch();
			System.out.println("Biggest Branch: "+sub.current.getTag());

			 // Print out the subtree with the largest fanout
			//  System.out.println("\n\nPrint out the subtree with the largest fanout:\n");
			//	 sub.printT();

			PreProcessTags pt = new PreProcessTags(sub);
			pt.eliminateIrrelevantTags(2);

			Vector tl = pt.get_list();


			/* System.out.println("\n-------------------------------------------\n\n");
			*/
			/* Apply five invidual heuristics */

			/* System.out.println("\n Start OM \n"); */
			Vector oml = null;
			File record_table_file = new File(args[1]);
			if (record_table_file.exists())
			{
				ApplyOM om = new ApplyOM(sub, new File(args[1]));
				oml = om.find_candidate();
			}
			else
			{
				/* In case the file doesn't exist, make a dummy vector to send into ApplyCH */
				oml = new Vector();
			}

			/* System.out.println("\n Start RP \n"); */
			ApplyRP rp = new ApplyRP(sub);
			Vector rpl = rp.find_candidate();

			/* System.out.println("\n Start SD \n"); */
			ApplySD sd = new ApplySD(sub);
			Vector sdl = sd.find_candidate();


			/* System.out.println("\n Start IT \n"); */
			Vector ist = new Vector();
			try {
				InputStream in = new FileInputStream(args[2]);
				int i = in.read();
				String ist_str = new String();

				while(i != -1)
				{
				   ist_str += (char)i;
				   i = in.read();
				}

				int start_tag_pos = 0;
				int end_tag_pos = 0;

				do
				{
					start_tag_pos = ist_str.indexOf("<", start_tag_pos);
					if(start_tag_pos != -1)
					{
						end_tag_pos = ist_str.indexOf(">", start_tag_pos);
					}
					else
					{
						break;
					}

					String str = ist_str.substring(start_tag_pos, end_tag_pos+1);
					ist.addElement(str);
					start_tag_pos = end_tag_pos + 1;

				}  while ((start_tag_pos != -1) || (end_tag_pos != -1));

			} catch (IOException io) {
				System.out.println(io.getMessage());
			}


			ApplyIT it = new ApplyIT(tl, ist);
			Vector itl = it.find_candidate();
			RetrieveRecord rtr = new RetrieveRecord();
			String rectag = new String();

			/* System.out.println("\n Start HT \n"); */
			ApplyMA ht = new ApplyMA(tl);
			Vector htl = ht.find_candidate();

			/* System.out.println("\n\n -------------------------------\n\n"); */

			/* Apply Combined heuristic */

			ApplyCH compound = new ApplyCH(tl, oml,rpl,sdl,itl,htl);

			Vector candidate = compound.find_candidate();

			/* System.out.println("\n\n -------------------------------\n\n"); */

			rectag = ((GTD)candidate.elementAt(0)).str;
			//         System.out.println("Record Separator: " +
			//                           rectag + " " + rectag.length());

			//          System.out.println("Record Separator: " +
			//	      ((GTD)candidate.elementAt(0)).str +
			//		((GTD)candidate.elementAt(0)).str.length());


			System.out.println("TAG FOUND: "+rectag);
			if(args[0].indexOf("://")>0)
			{
			   try
			   {
					rtr = new RetrieveRecord(new URL(args[0]), rectag);
			   }
			   catch (MalformedURLException e)
			   {
					System.out.println("MalformedURL...check the input parameter.");
			   }
			}
			else
			{
					rtr = new RetrieveRecord(new File(args[0]), rectag);
			}

			// Print out plain text
			System.out.println(rtr.getRecordText());

			try
			{
			   outf.write(rtr.getRecordText());
			   outf.close();
			} catch (IOException e)
			{
				System.out.print(rtr.getRecordText());
			}
		}
   }
}
