package edu.byu.deg.util;

/** <p>Removes the HTML markup and scripts from a string.</p>
 * <p>HTMLStripper is intended as a quick and memory efficient filter to extract
 * human-readable text from a string marked up with HTML.</p>
 * @author Troy Walker
 * @version 1.0
 */

public class HTMLStripper {
  private int index;
  private char currentChar;
  private String currentString;
  private boolean whitespacePrevious;

  private char getNext(){
    return currentChar = currentString.charAt(index++);
  }
  private boolean isAlpha(char c){
    if((currentChar<='z')&&(currentChar>='a'))
        return true;
    if((currentChar<='Z')&&(currentChar>='A'))
        return true;
    return false;
  }
  private boolean isSpace(char c){
      return (currentChar==' ')||(currentChar=='\t')||(currentChar=='\r')||(currentChar=='\n');
  }
  
  private char getNextLCase(){
      currentChar = getNext();
      if((currentChar<='Z')&&(currentChar>='A'))
        currentChar = (char)(currentChar | 32);
      return currentChar;
  }

  private void eatToLT(){
    if(currentChar=='>')
      return;
    getNext();
    while ((index<currentString.length()) && (currentChar != '>')){
      getNext();
    }
  }

  /** Filters a string.
   * @param str The string to be stripped.
   * @return A string stripped of HTML markup and script with HTML entities (for
   * example &amp;amp;) replaced.
   */  
  public String stripHTML(String str){
    StringBuffer buffer = new StringBuffer(str.length());
    currentString = str;
    index = 0;
    whitespacePrevious = false;
    while (index<str.length()){
      getNext();
      if(currentChar=='<') {
        getNextLCase();
        //remove html comments <!--
        if((currentChar=='!') && ((getNext())=='-') && ((getNext())=='-'))
          while(index < str.length()) {
            if((getNext()=='-') && (getNext()=='-') && (getNext()=='>'))
              break;
          }
        else if(currentChar=='s'){
          getNextLCase();
          //remove everything from <script to </script>
          if((currentChar=='c') && (getNextLCase()=='r') &&
                (getNextLCase()=='i') && (getNextLCase()=='p') &&
                (getNextLCase()=='t')){
            while(index < str.length()) {
              if((getNext() == '<') && (getNext() == '/') &&
                  (getNextLCase() == 's') && (getNextLCase() == 'c') &&
                  (getNextLCase() == 'r') && (getNextLCase() == 'i') &&
                  (getNextLCase() == 'p') && (getNextLCase() == 't')) {
                 eatToLT();
                 break;
              }
            }
          }
          //remove everything from <style to </style>
          else if((currentChar=='t') && (getNextLCase()=='y') &&
                  (getNextLCase()=='l') && (getNextLCase()=='e')){
            while(index < str.length()) {
              if((getNext() == '<') && (getNext() == '/') &&
                  (getNextLCase() == 's') && (getNextLCase() == 't') &&
                  (getNextLCase() == 'y') && (getNextLCase() == 'l') &&
                  (getNextLCase() == 'e')) {
                 eatToLT();
                 break;
              }
            }
          }
        }
        //p, hr, h1-6, tr, td, title, blockquote and br 
        else if(((currentChar=='p') && !isAlpha(getNext())) ||
                (currentChar=='h') ||
                ((currentChar=='t') && ((getNextLCase()=='r')||currentChar=='d')) ||
                ((currentChar=='b') && ((getNextLCase()=='r')||currentChar=='l')) ||
                (currentChar=='/') && ((getNextLCase()=='h') ||
                                      ((currentChar=='t')&&(getNextLCase()=='i')))){
          if(!whitespacePrevious){
            buffer.append("\n");
            whitespacePrevious = true;
          } 
          eatToLT();
        }
        //all other tags
        else{
          if(!whitespacePrevious){
            buffer.append(" ");
            whitespacePrevious = true;
          } 
          eatToLT();
        }
      }
      //handle some HTML entities (others could be added)
      else if(currentChar=='&'){
        getNextLCase();
        //&lt;
        if((currentChar=='l') && (getNextLCase()=='t') && (getNext()==';')){
          buffer.append('<');
          whitespacePrevious = false;
        }
        //&gt;
        else if((currentChar=='g') && (getNextLCase()=='t') && (getNext()==';')){
          buffer.append('>');
          whitespacePrevious = false;
        }
        //&amp;
        else if((currentChar=='a') && (getNextLCase()=='m') &&
                (getNextLCase()=='p') && (getNext()==';')){
          buffer.append('&');
          whitespacePrevious = false;
        }
        //&nbsp;
        else if((currentChar=='n') && (getNextLCase()=='b') &&
                (getNextLCase()=='s') && (getNextLCase()=='p') &&
                (getNext()==';')){
          buffer.append(' ');
          whitespacePrevious = false;
        }
        //&quot;
        else if((currentChar=='q') && (getNextLCase()=='u') &&
                (getNextLCase()=='o') && (getNextLCase()=='t') &&
                (getNext()==';')){
          buffer.append('\'');
          whitespacePrevious = false;
        }
        else
          buffer.append('&');
          whitespacePrevious = false;
      }
      else{
        if(!isSpace(currentChar)||!whitespacePrevious)
          buffer.append(currentChar);
        whitespacePrevious = (currentChar==' ')||(currentChar=='\t')||(currentChar=='\r')||(currentChar=='\n');
      }
    }
    return buffer.toString();
  }
}