/*
 * Timer.java
 *
 * Created on August 1, 2003, 2:54 PM
 */

package edu.byu.deg.util;

import java.util.HashMap;
/** Timer is a utility class to aid in performance testing.  It simply keeps track
 * of when tasks were started and returns the time taken when a task is completed.
 * Turn on the verbose flag if messages to standard out are wanted when starting
 * and ending tasks.  To use timing in your classes, do something like the
 * following:
 * <PRE>
 *      if(Timer.time) Timer.timer.startTask("Doing something");
 *      //code to do something
 *      if(Timer.time) Timer.timer.endTask("Doing something");
 * </PRE>
 * <CODE>Timer</CODE> can handle any number of tasks (memory permitting).  Tasks
 * can be nested within other tasks and can overlap.
 * @author troywalk
 */
public class Timer {
  /** Setting this flag to false indicates that no timing is desired.  This effectivly
   * disables this class.
   */  
  public static final boolean time = true;
  /** Whether <CODE>Timer</CODE> should print starting and ending task messages to
   * standard out.
   */  
  public static final boolean verbose = true;
  
  /** Instance of timer for all to use. */  
  public static Timer timer = new Timer();
  
  HashMap times;
  /** Creates a new instance of Timer */
  private Timer() {
    times = new HashMap();
  }
  
  /** Records the beginning of the specified task
   * @param task string identifying task
   */  
  public void startTask(String task){
    times.put(task, new Long(System.currentTimeMillis()));
    if(verbose)
      System.out.println(task + "...");
  }
  
  /** Records the ending of the specified task and returns the time taken to complete
   * task.
   * @param task string identifying task
   * @return duration of the task.
   */  
  public long endTask(String task){
    Long start = (Long)times.remove(task);
    if(start==null)
      return 0;
    long duration = System.currentTimeMillis() - start.longValue();
    if(verbose)
      System.out.println(task + " took " + duration + " ms");
    return duration;
  }
  
}

